/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.auth;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashSet;
import java.util.Set;

import org.apache.commons.dbcp2.BasicDataSource;
import org.apache.shiro.authc.AccountException;
import org.apache.shiro.authc.AuthenticationException;
import org.apache.shiro.authc.AuthenticationInfo;
import org.apache.shiro.authc.AuthenticationToken;
import org.apache.shiro.authc.SimpleAuthenticationInfo;
import org.apache.shiro.authc.UnknownAccountException;
import org.apache.shiro.authc.UsernamePasswordToken;
import org.apache.shiro.authc.credential.PasswordMatcher;
import org.apache.shiro.authc.credential.PasswordService;
import org.apache.shiro.authz.AuthorizationException;
import org.apache.shiro.authz.AuthorizationInfo;
import org.apache.shiro.authz.SimpleAuthorizationInfo;
import org.apache.shiro.cache.MemoryConstrainedCacheManager;
import org.apache.shiro.realm.jdbc.JdbcRealm;
import org.apache.shiro.subject.PrincipalCollection;
import org.apache.shiro.subject.Subject;
import org.apache.shiro.util.JdbcUtils;
import org.apache.shiro.web.mgt.DefaultWebSecurityManager;
import org.eclipse.rap.rwt.RWT;

import de.eoda.qtafi.surveymanager.model.Role;

public class SecurityManager extends DefaultWebSecurityManager {

	
	public static final PasswordService passwordService = new org.apache.shiro.authc.credential.DefaultPasswordService();

	static class Realm extends JdbcRealm {
		
		boolean tableChecked = false;
		
		public Realm() {
			PasswordMatcher matcher = new PasswordMatcher();
			matcher.setPasswordService(passwordService);	
			/*
			 * for plain passwords
			 */
			//setCredentialsMatcher(new SimpleCredentialsMatcher());
			setCredentialsMatcher(matcher);
			setPermissionsLookupEnabled(false);
			setAuthenticationQuery("SELECT password FROM `__accounts` WHERE name=?");
			setUserRolesQuery("SELECT __role.role FROM `__accountroles` LEFT JOIN `__accounts` ON __accountroles.accountid=__accounts.id LEFT JOIN `__role` ON __accountroles.roleid=__role.id WHERE __accounts.name=?");
		}
		
		
		@Override
		protected AuthenticationInfo doGetAuthenticationInfo(AuthenticationToken token) throws AuthenticationException {
			if(! tableChecked){
				checkTables("qtafi");
				//append initial account
				if(! adminExists()){
					String pass = passwordService.encryptPassword("12345");
					tableCreate("INSERT INTO `__accounts` (id, name, password, status, administrator) VALUES (1, \"admin\", \""+pass+"\", 1, 1)");
					tableCreate("INSERT INTO `__role` (id, role) VALUES (1, \""+Role.AS_ADMIN+"\")");
					tableCreate("INSERT INTO `__role` (id, role) VALUES (2, \""+Role.AS_MANAGER+"\")");
					tableCreate("INSERT INTO `__role` (id, role) VALUES (3, \""+Role.AS_REPORTER+"\")");
					tableCreate("INSERT INTO `__accountroles` (id, roleid, accountid) VALUES (1, 1, 1)");
				}
				tableChecked = true;
			}
			
			
			BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
	        UsernamePasswordToken upToken = (UsernamePasswordToken) token;
	        String name = upToken.getUsername();
	        if (name == null) {
	            throw new AccountException("Null usernames are not allowed by this realm.");
	        }

	        Connection conn = null;
	        AuthenticationInfo info = null;
	        try {
	            conn = ds.getConnection();
	            Account account = getAccount(conn, name);
	            if(account == null){
	            	throw new UnknownAccountException("No account found for user [" + name + "]");
	            }
	            String password = getPasswordForUser(conn, name);
	            if (password == null) {
	                throw new UnknownAccountException("No account found for user [" + name + "]");
	            }
	            info = new SimpleAuthenticationInfo(account, password.toCharArray(), getName());
	        }
	        catch (SQLException e) {
	            final String message = "There was a SQL error while authenticating user [" + name + "]";
	            throw new AuthenticationException(message, e);
	        } 
	        finally {
	            JdbcUtils.closeConnection(conn);
	        }
	        return info;
	    }
		
		private String getPasswordForUser(Connection conn, String name) throws SQLException {
			PreparedStatement ps = null;
			ResultSet rs = null;
			String password = null;
			try {
				ps = conn.prepareStatement(authenticationQuery);
				ps.setString(1, name);
				rs = ps.executeQuery();
				boolean foundResult = false;
				while (rs.next()) {
					if (foundResult) {
						throw new AuthenticationException("More than one user row found for user [" + name + "]. Usernames must be unique.");
					}
					password = rs.getString(1);
					foundResult = true;
				}
			} 
			finally {
				JdbcUtils.closeResultSet(rs);
				JdbcUtils.closeStatement(ps);
			}
			return password;
		}
		
		private Account getAccount(Connection conn, String name) throws SQLException {
			PreparedStatement ps = null;
			ResultSet rs = null;
			Account account = null;
			try {
				ps = conn.prepareStatement("SELECT id, administrator FROM `__accounts` WHERE name=?");
				ps.setString(1, name);
				rs = ps.executeQuery();
				if (rs.next()) {
					account = new Account(rs.getInt(1), name);
					account.setAdministrator(rs.getInt("administrator") == 1);
				}
			} 
			finally {
				JdbcUtils.closeResultSet(rs);
				JdbcUtils.closeStatement(ps);
			}
			return account;
		}
		
		@Override
		protected AuthorizationInfo doGetAuthorizationInfo(PrincipalCollection principals) {
			BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
			if (principals == null) {
				throw new AuthorizationException("PrincipalCollection method argument cannot be null.");
			}
			Account account = (Account) principals.getPrimaryPrincipal();
			
			String username = account.name;
			Connection conn = null;
			Set<String> roleNames = null;
			Set<String> projects = new HashSet<String>();
			try {
				conn = ds.getConnection();
				roleNames = getRoleNamesForUser(conn, username);
				projects = getProjectsForUser(conn, account.id);
				account.setAccessibleProjects(projects);
				//
				//if(roleNames.contains(Account.ADMINSTRATOR_ROLE)) account.setAdministrator(true);
				if(account.isAdministrator()) roleNames.add("admin");
			}
			catch (SQLException e) {
				final String message = "There was a SQL error while authorizing user [" + username + "]";
				throw new AuthorizationException(message, e);
			} 
			finally {
				JdbcUtils.closeConnection(conn);
			}
			SimpleAuthorizationInfo info = new SimpleAuthorizationInfo(roleNames);
			return info;
		}
		
		Set<String> getProjectsForUser(Connection conn, int userid){
			PreparedStatement ps = null;
			ResultSet rs = null;
			Set<String> pjs = new HashSet<String>();
			try {
				ps = conn.prepareStatement("SELECT pr.name as name FROM `__projects` pr, `__projectassignment` ass WHERE pr.id=ass.projectid AND ass.accountid=?");
				ps.setInt(1, userid);
				rs = ps.executeQuery();
				while(rs.next()) {
					pjs.add(rs.getString("name"));
				}
			}
			catch (SQLException e) {
				e.printStackTrace();
			} 
			finally {
				JdbcUtils.closeResultSet(rs);
				JdbcUtils.closeStatement(ps);
			}
			return pjs;
		}	
	}
	
	
	public SecurityManager() {
		setRealm(new Realm());
		setCacheManager(new MemoryConstrainedCacheManager());
	}
	
	@Override
	protected void rememberMeLogout(Subject arg0) {
	}
	
	@Override
	public void logout(Subject arg0) {
		try{
			super.logout(arg0);
		}
		catch(Exception ex){}
	}
	
	/*
	@Override
	public void logout(Subject subject) {
		DefaultSessionManager sessionManager = (DefaultSessionManager) getSessionManager();
	    Collection<Session> activeSessions = sessionManager.getSessionDAO().getActiveSessions();
	    for (Session session: activeSessions){
	        if(subject.getSession().getId().equals(session.getId())){
	            session.stop();
	        }
	    }
	}
	*/
	
	
	static void checkTables(String schemaName){
		if(! tableExists("__accounts", schemaName)){
			tableCreate("CREATE TABLE `__accounts` (id INT NOT NULL AUTO_INCREMENT PRIMARY KEY, name VARCHAR(200), password varchar(500), email varchar(200), status INT, administrator INT) DEFAULT CHARSET=utf8;");
		}
		
		if(! tableExists("__role", schemaName)){
			tableCreate("CREATE TABLE `__role` (id INT NOT NULL AUTO_INCREMENT PRIMARY KEY, role varchar(200) NOT NULL) DEFAULT CHARSET=utf8;");
		}
		
		if(!tableExists("__accountroles", schemaName)){
			tableCreate("CREATE TABLE `__accountroles` (id INT NOT NULL AUTO_INCREMENT PRIMARY KEY, accountid INT NOT NULL, roleid INT NOT NULL ) DEFAULT CHARSET=utf8;");
		}
		
		if(!tableExists("__projects", schemaName)){
			tableCreate("CREATE TABLE `__projects` (id INT NOT NULL AUTO_INCREMENT PRIMARY KEY, name VARCHAR(100) NOT NULL, start DATETIME, end DATETIME, status INT, owner INT, authentication VARCHAR(50), log VARCHAR(20)) DEFAULT CHARSET=utf8;");
		}
		
		if(!tableExists("__projectassignment", schemaName)){
			tableCreate("CREATE TABLE `__projectassignment` (id INT NOT NULL AUTO_INCREMENT PRIMARY KEY, projectid INT, accountid INT, roleid INT) DEFAULT CHARSET=utf8;");
		}
		
		if(!tableExists("__library", schemaName)){
			tableCreate("CREATE TABLE `__library` (id INT NOT NULL AUTO_INCREMENT PRIMARY KEY, quid VARCHAR(200), type VARCHAR(50), accountid INT, public INT, subject VARCHAR(200), qml TEXT, dateadded DATETIME) DEFAULT CHARSET=utf8;");
		}
	}
	
	static void tableCreate(String sql){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		Connection conn = null;
		Statement st = null;
		try {
			conn = ds.getConnection();
			st = conn.createStatement();
			st.execute(sql);
			st.close();
		} 
		catch (SQLException e) {
			e.printStackTrace();
		}
		finally{
			try {
				st.close();
				conn.close();
			}
			catch (SQLException e) {
				e.printStackTrace();
			}	
		}
	}
	
	static boolean tableExists(String name, String schemaName){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		Statement st = null;
		Connection conn = null;
		boolean exists = false;
		try {
			conn = ds.getConnection();
			st = conn.createStatement();
			ResultSet rs = st.executeQuery("SELECT COUNT(*) FROM information_schema.tables WHERE TABLE_NAME=\""+name+"\" AND TABLE_SCHEMA=\""+schemaName+"\";");
			if(rs.next()){
				int temp = rs.getInt(1);
				if(temp == 1){
					exists = true;
				}
			}
			rs.close();
			st.close();
		} 
		catch (SQLException e) {
			e.printStackTrace();
		}
		finally{
			try {
				conn.close();
			}
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return exists;
	}
	
	

	static boolean adminExists(){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		Statement st = null;
		Connection conn = null;
		boolean exists = false;
		try {
			conn = ds.getConnection();
			st = conn.createStatement();
			ResultSet rs = st.executeQuery("SELECT COUNT(*) FROM `__accounts` WHERE name=\"admin\"");
			if(rs.next()){
				int temp = rs.getInt(1);
				if(temp == 1){
					exists = true;
				}
			}
			rs.close();
			st.close();
		} 
		catch (SQLException e) {
			e.printStackTrace();
		}
		finally{
			try {
				conn.close();
			}
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return exists;
	}
}
