/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import org.apache.commons.dbcp2.BasicDataSource;
import org.eclipse.rap.rwt.RWT;
import org.eclipse.rap.rwt.application.Application;
import org.eclipse.rap.rwt.application.Application.OperationMode;
import org.eclipse.rap.rwt.application.ApplicationConfiguration;
import org.eclipse.rap.rwt.client.WebClient;

import de.eoda.qtafi.surveymanager.util.DataDownloadService;
import de.eoda.qtafi.surveymanager.util.DblClickHandler;
import de.eoda.qtafi.surveymanager.util.FileResourceLoader;
import de.eoda.qtafi.surveymanager.util.Resources;


public class SurveyManagerConfiguration implements ApplicationConfiguration {

	@Override
	public void configure(Application application) {
		Map<String, String> properties = new HashMap<String, String>();
	    properties.put(WebClient.PAGE_TITLE, "Surveymanager");
	    properties.put(WebClient.BODY_HTML, readTextFromResource("resources/body.html", "UTF-8"));
	    properties.put(WebClient.HEAD_HTML, readTextFromResource("resources/head.html", "UTF-8"));
	    
	    application.setOperationMode(OperationMode.SWT_COMPATIBILITY);
	    application.addStyleSheet(RWT.DEFAULT_THEME_ID, "theme/theme.css");
	    
	    application.addResource("resources/d3.min.js", new FileResourceLoader());
	    application.addResource("resources/c3.min.js", new FileResourceLoader());
	    application.addResource("resources/c3.css", new FileResourceLoader());
	    
	    application.addResource("resources/bootstrap.js", new FileResourceLoader());
	    application.addResource("resources/bootstrap-datepicker.js", new FileResourceLoader());
	    application.addResource("resources/jquery.js", new FileResourceLoader());
	    application.addResource("resources/css/bootstrap.css", new FileResourceLoader());
	    application.addResource("resources/css/bootstrap-datepicker.css", new FileResourceLoader());
	    
	    application.addResource("resources/fonts/glyphicons-halflings-regular.ttf", new FileResourceLoader());
	    application.addResource("resources/fonts/glyphicons-halflings-regular.eot", new FileResourceLoader());
	    application.addResource("resources/fonts/glyphicons-halflings-regular.svg", new FileResourceLoader());
	    application.addResource("resources/fonts/glyphicons-halflings-regular.woff", new FileResourceLoader());
	    
	    application.addServiceHandler(DataDownloadService.DOWNLOAD_SERVICE, DataDownloadService.getInstance());
	    application.addServiceHandler(DblClickHandler.DBLCLICK, DblClickHandler.getInstance());
	    
	    application.addEntryPoint("/manager", UIEntryPoint.class, properties);
	    
	    Properties p = new Properties();
	    try {
			p.load(new FileInputStream(Resources.ResolveFile("/WEB-INF/db.properties")));
		} 
	    catch (IOException e) {
			e.printStackTrace();
		}
	    
	    BasicDataSource dataSource = new BasicDataSource();
	    dataSource.setUrl(p.getProperty("url")); 
	    dataSource.setDriverClassName(p.getProperty("driver"));
	    dataSource.setUsername(p.getProperty("user"));
	    dataSource.setPassword(p.getProperty("password"));
	    dataSource.setDefaultCatalog(p.getProperty("defaultCatalog"));
	    if(p.getProperty("connectionProperties") != null) dataSource.setConnectionProperties(p.getProperty("connectionProperties"));
	    //if(p.getProperty("connectionPropperties") != null) dataSource.setConnectionProperties(p.getProperty("connectionPropperties"));

	    application.setAttribute("datasource", dataSource);
	}

	private static String readTextFromResource(String resourceName, String charset) {
		String result;
		try {
			ClassLoader classLoader = SurveyManagerConfiguration.class.getClassLoader();
			InputStream inputStream = classLoader.getResourceAsStream(resourceName);
			if (inputStream == null) {
				throw new RuntimeException("Resource not found: "+ resourceName);
			}
			try {
				BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream, charset));
				StringBuilder stringBuilder = new StringBuilder();
				String line = reader.readLine();
				while (line != null) {
					stringBuilder.append(line);
					stringBuilder.append('\n');
					line = reader.readLine();
				}
				result = stringBuilder.toString();
			} 
			finally {
				inputStream.close();
			}
		} 
		catch (IOException e) {
			throw new RuntimeException("Failed to read text from resource: "+ resourceName);
		}
		return result;
	}
	
}
