/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap;


import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.util.Locale;

import org.apache.shiro.SecurityUtils;
import org.apache.shiro.subject.Subject;
import org.eclipse.rap.rwt.RWT;
import org.eclipse.rap.rwt.application.AbstractEntryPoint;
import org.eclipse.rap.rwt.client.service.BrowserNavigation;
import org.eclipse.rap.rwt.client.service.BrowserNavigationEvent;
import org.eclipse.rap.rwt.client.service.BrowserNavigationListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Device;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import de.eoda.qtafi.surveymanager.auth.Account;
import de.eoda.qtafi.surveymanager.lang.Messages;
import de.eoda.qtafi.surveymanager.model.Breadcrumb;
import de.eoda.qtafi.surveymanager.rap.pages.AccountContribution;
import de.eoda.qtafi.surveymanager.rap.pages.AdministrationContribution;
import de.eoda.qtafi.surveymanager.rap.pages.DataViewContribution;
import de.eoda.qtafi.surveymanager.rap.pages.EditorContribution;
import de.eoda.qtafi.surveymanager.rap.pages.HomeContribution;
import de.eoda.qtafi.surveymanager.rap.pages.LogPageContribution;
import de.eoda.qtafi.surveymanager.rap.pages.LoginContribution;
import de.eoda.qtafi.surveymanager.rap.pages.ProjectContribution;
import de.eoda.qtafi.surveymanager.rap.pages.ReportContribution;
import de.eoda.qtafi.surveymanager.rap.pages.UsersContribution;

public class UIEntryPoint extends AbstractEntryPoint implements Serializable {

	private static final long serialVersionUID = 1L;

	private static final int HEADER_HEIGHT = 88;
	// private static final int CENTER_AREA_WIDTH = 800;
	
	private Composite centerArea;
	private Composite headerArea;

	private String contributionTitle;
	private String contributionArgs;

	private Label crumblist, title;
	private Language[] languages;
	private int langIdx;
	// private Label headlineLabel;
	private Label accountinfo;

	UIPages pages = new UIPages();

	public UIEntryPoint() {
		pages.registerPage(new HomeContribution());
		pages.registerPage(new AccountContribution());
		pages.registerPage(new LoginContribution());
		pages.registerPage(new EditorContribution());
		pages.registerPage(new ProjectContribution());
		pages.registerPage(new DataViewContribution());
		pages.registerPage(new ReportContribution());
		pages.registerPage(new UsersContribution());
		pages.registerPage(new AdministrationContribution());
		pages.registerPage(new LogPageContribution());
		langIdx = 0;
	}

	@Override
	protected Shell createShell(Display display) {
		Shell shell = super.createShell(display);
		FillLayout baseLayout = new FillLayout();
		baseLayout.marginHeight = 150;
		shell.setLayout(baseLayout);
		return shell;
	}

	@Override
	protected void createContents(Composite parent) {
		// parent = Background for main content area
		parent.setLayout(new FillLayout(SWT.VERTICAL));
		ScrolledComposite scrolledArea = createScrolledArea(parent);
		Composite content = createContent(scrolledArea);
		scrolledArea.setContent(content);
		// scrolled area = behind scrollbars and behind main content/parent
		attachHistoryListener();
		selectContribution(pages.getContribution("account"), "");
	}

	private void attachHistoryListener() {
		BrowserNavigation history = RWT.getClient().getService(BrowserNavigation.class);
		if (history != null) {
			history.addBrowserNavigationListener(new BrowserNavigationListener() {
				public void navigated(BrowserNavigationEvent event) {
					String evet = event.getState();
					int arg = evet.indexOf("|");
					String cont, args;
					if (arg < 0) {
						cont = evet;
						args = "";
					} else {
						cont = evet.substring(0, arg);
						args = evet.substring(arg + 1);
					}
					UIPageContribution contribution = pages.getContribution(cont);
					selectContribution(contribution, args);
				}
			});
		}
	}

	private void selectContribution(UIPageContribution contribution, String args) {
		Subject sub = SecurityUtils.getSubject();
		// if(! sub.isRemembered()){
		if (!sub.isAuthenticated()) {
			UIPageContribution login = pages.getContribution("login");
			activate(login, "");
			return;
		}
		// }
		/*
		 * if(! sub.isAuthenticated()) { UIPageContribution login =
		 * pages.getContribution("login"); activate(login); return; }
		 */
		this.contributionTitle = contribution.getId();
		this.contributionArgs = args;
		
		if (contribution == null) {
			System.out.println("contribution not found ");
			return;
		}
		activate(contribution, args);
	}

	private void activate(UIPageContribution contribution, String args) {
		UIPage page = contribution.createPage();
		if (page != null) {
			BrowserNavigation history = RWT.getClient().getService(BrowserNavigation.class);
			if (history != null) {
				if (args.length() > 0)
					history.pushState(contribution.getId() + "|" + args, contribution.getTitle());
				else
					history.pushState(contribution.getId(), contribution.getTitle());
			}
			
			UIPage prev = (UIPage) centerArea.getData();
			if(prev != null) prev.dispose();
			
			Control[] children = centerArea.getChildren();
			for (Control child : children) {
				child.dispose();
			}
			Composite contentComp = UIPages.initPage(contribution.getTitle(), centerArea);
			centerArea.setData(page);

			page.createControl(contentComp, args);
			updateBreadCrumb(page.getCrumbs());
			title.setText("<i>" + contribution.getTitle().replace("&", "&&") + "</i>");
			updateAccountInfo();
			centerArea.layout(true, true);
		}
	}

	private ScrolledComposite createScrolledArea(Composite parent) {
		ScrolledComposite scrolledComp = new ScrolledComposite(parent, SWT.V_SCROLL | SWT.H_SCROLL);
		scrolledComp.setMinHeight(Display.getCurrent().getPrimaryMonitor().getBounds().y);
		scrolledComp.setMinWidth(Display.getCurrent().getPrimaryMonitor().getBounds().x);
		scrolledComp.setExpandVertical(true);
		scrolledComp.setExpandHorizontal(true);
		return scrolledComp;
	}

	private Composite createContent(ScrolledComposite scrolledArea) {
		Composite comp = new Composite(scrolledArea, SWT.NONE);
		comp.setData(RWT.CUSTOM_VARIANT, "bg-dark");
		GridLayout mainLayout = new GridLayout();
		mainLayout.numColumns = 1;

		GridData headerData = new GridData(GridData.FILL_HORIZONTAL);
		headerData.widthHint = HEADER_HEIGHT;
		comp.setLayout(mainLayout);
		Composite header = createHeader(comp);
		header.setSize(20, HEADER_HEIGHT);
		header.setLayoutData(headerData);

		GridData contentData = new GridData(GridData.FILL_BOTH);

		// Everything around the main content area
		Composite content = new Composite(comp, SWT.BORDER);
		content.setLayout(mainLayout);
		content.setLayoutData(contentData);
		// content.setBackground(Display.getDefault().getSystemColor(SWT.COLOR_DARK_BLUE));
		centerArea = createCenterArea(content);

		return comp;
	}

	private Composite createHeader(Composite parent) {
		Composite comp = new Composite(parent, SWT.BORDER | SWT.BOLD);
		initLanguages(parent.getDisplay());
		comp.setBackgroundMode(SWT.INHERIT_DEFAULT);
		comp.setLayout(new GridLayout(1, false));

		headerArea = new Composite(comp, SWT.NONE);
		headerArea.setLayout(new GridLayout(4, false));
		headerArea.setLayoutData(new GridData(GridData.FILL_BOTH));
		createLogo(headerArea);
		createHeadContent(headerArea);
		Composite headerAreaRight = new Composite(headerArea, SWT.NONE);
		headerAreaRight.setLayout(new GridLayout(1, false));
		headerAreaRight.setLayoutData(new GridData(SWT.RIGHT, SWT.TOP, false, true));
		createAccountInfo(headerAreaRight);
		createLanguageCombo(headerAreaRight);
		return comp;
	}

	void createAccountInfo(Composite parent) {
		accountinfo = new Label(parent, SWT.NONE);
		//GridData gd = new GridData();
		//gd.verticalAlignment = GridData.BEGINNING;
		//gd.grabExcessVerticalSpace = true;
		accountinfo.setLayoutData(new GridData(SWT.RIGHT, SWT.TOP, false, true));
		accountinfo.setData(RWT.MARKUP_ENABLED, true);
		accountinfo.setText("");
	}

	void updateBreadCrumb(Breadcrumb[] crumbs) {
		StringBuffer buf = new StringBuffer();
		buf.append("<a href=\"/admin/manager#home\" >" + Messages.get().Home_Title + "</a> ");
		for (Breadcrumb c : crumbs)
			buf.append("> <a href=\"/admin/manager#" + c.link + "\" >" + c.name + "</a> ");
		crumblist.setText(buf.toString());
	}

	void updateAccountInfo() {
		Subject subject = SecurityUtils.getSubject();
		if (subject.isAuthenticated()) {
			Account account = (Account) SecurityUtils.getSubject().getPrincipal();
			accountinfo.setText(Messages.get().Account_Title + ": <a href=\"/admin/manager#account\">" + account.toString() + "</a>" + (account.isAdministrator() ? " | <a href=\"/admin/manager#administration\" >Admin panel</a>" : ""));
		} else
			accountinfo.setText("");
	}

	private Composite createCenterArea(Composite parent) {
		Composite centerArea = new Composite(parent, SWT.NONE);
		centerArea.setLayout(new GridLayout());
		centerArea.setLayoutData(new GridData(GridData.FILL_BOTH));
		return centerArea;
	}

	private void createLogo(final Composite headerComp) {
		Label logoLabel = new Label(headerComp, SWT.NONE);
		GridData gridData = new GridData(SWT.LEFT, SWT.TOP, false, false);
		gridData.widthHint = 250;
		gridData.verticalSpan = 2;
		logoLabel.setLayoutData(gridData);
		Image logo = getImage(headerComp.getDisplay(), "etf-logo.gif");
		logoLabel.setImage(logo);
	}

	private void createHeadContent(final Composite headerComp) {
		Composite middleHead = new Composite(headerComp, SWT.NONE);
		middleHead.setLayoutData(new GridData(GridData.FILL_BOTH));
		middleHead.setLayout(new GridLayout(1, true));
		crumblist = new Label(middleHead, SWT.NONE);
		crumblist.setData(RWT.MARKUP_ENABLED, true);
		GridData gdc = new GridData(GridData.FILL_HORIZONTAL);
		gdc.heightHint = 40;
		gdc.grabExcessVerticalSpace = true;
		crumblist.setLayoutData(gdc);
		crumblist.setText("<a href=\"/admin/manager#home\">" + Messages.get().Home_Title + "</a>");

		title = new Label(middleHead, SWT.NONE);
		title.setData(RWT.MARKUP_ENABLED, true);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.widthHint = 300;
		title.setLayoutData(gd);
		title.setData(RWT.CUSTOM_VARIANT, "pageHeadline");
	}

	private void createLanguageCombo(final Composite headerComp) {

		final Combo languageSelect = new Combo(headerComp, SWT.READ_ONLY);
		//GridData gd = new GridData();
		//gd.horizontalAlignment = SWT.RIGHT;
		languageSelect.setLayoutData(new GridData(SWT.RIGHT, SWT.TOP, true, true));
		
		for (int i = 0; i < languages.length; i++) {
			if (languages[i].locale != null)
				languageSelect.add(languages[i].locale.getLanguage());
			else
				languageSelect.add(languages[i].name);
			// item.addListener( SWT.Selection, new MenuItemSelectionListener(
			// dropDown ) );
		}
		
		languageSelect.select(langIdx);
		languageSelect.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				langIdx = languageSelect.getSelectionIndex();
				RWT.setLocale(languages[languageSelect.getSelectionIndex()].locale);
				selectContribution(pages.getContribution(contributionTitle), contributionArgs);
			}
		});
	}
	
	public static Image getImage(Display display, String path) {
		ClassLoader classLoader = UIEntryPoint.class.getClassLoader();
		InputStream inputStream = classLoader.getResourceAsStream("resources/" + path);
		Image result = null;
		if (inputStream != null) {
			try {
				result = new Image(display, inputStream);
			} finally {
				try {
					inputStream.close();
				} 
				catch (IOException e) { }
			}
		}
		return result;
	}
	
	private void initLanguages(Device device) {
		languages = new Language[] { new Language(null), new Language(Locale.US), new Language(Locale.GERMANY),	new Language(new Locale("ru")), new Language(new Locale("ar")) // ,
		};
	}

	private final class Language {

		public final String name;
		public final Locale locale;
		
		public Language(Locale locale) {
			if (locale == null) {
				this.name = "default";
			} else {
				// this.name = locale.getDisplayLanguage();
				this.name = locale.getDisplayName(locale);
			}
			this.locale = locale;
		}

	}
}
