/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap;

import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.util.HashMap;

import org.eclipse.rap.rwt.RWT;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;

import de.eoda.qtafi.surveymanager.model.Breadcrumb;
import de.eoda.qtafi.surveymanager.rap.pages.UsersPage;

public class UIPages implements Serializable {

	private static final long serialVersionUID = 1L;

	private static final int DEFAULT_SPACE = 10;

	HashMap<String, UIPageContribution> pages = new HashMap<String, UIPageContribution>();

	private Breadcrumb breadcrumb;
	
	public Breadcrumb getBreadcrumbStart() {
		return breadcrumb;
	}

	public UIPages() {
		breadcrumb = null;
	}
	
	public void registerPage(UIPageContribution page) {
		pages.put(page.getId(), page);
	}

	public UIPageContribution getContribution(String id) {
		return pages.get(id);
	}
	
	/*
	public void UpdateBreadcrumb(String id, String title, String suffix) {
		if (breadcrumb == null) {
			breadcrumb = new Breadcrumb(id, pages.get(id).getTitle());
			breadcrumb.setSuffix(suffix);
			System.out.println("First Breadcrumb " + id + ":" + pages.get(id).getTitle());
		} else {
			Breadcrumb tmpCrumb = breadcrumb;
			while (tmpCrumb!= null) {
				if (tmpCrumb.getId().equals(id)) {
					tmpCrumb.setNext(null);
					System.out.println("Jump to Breadcrumb " + id + ":" + pages.get(id).getTitle());
					break;
				} else if (tmpCrumb.getNext() == null) {
					Breadcrumb newCrumb = new Breadcrumb(id, pages.get(id).getTitle());
					newCrumb.setSuffix(suffix);
					tmpCrumb.setNext(newCrumb);
					System.out.println("Added Breadcrumb " + id + ":" + pages.get(id).getTitle());
				}
				tmpCrumb = tmpCrumb.getNext();
			}
		}
	}
	*/
	
	public static Composite initPage(String title, Composite parent) {
		Composite pageComp = new Composite(parent, SWT.NONE);
		pageComp.setLayout(new GridLayout());
		pageComp.setLayoutData(new GridData(GridData.FILL_BOTH));		
		Composite contentComp = new Composite(pageComp, SWT.NONE);
		//contentComp.setLayout(new FillLayout());
		contentComp.setLayout(new GridLayout());
		contentComp.setLayoutData(new GridData(GridData.FILL_BOTH));
		//contentComp.setBackground(Display.getDefault().getSystemColor(SWT.COLOR_CYAN));
		contentComp.setLayoutData(UIPages.createFillData());
		return contentComp;
	}

	public static void createHeading(Composite parent, String text, int horizontalSpan) {
		Label label = new Label(parent, SWT.NONE);
		label.setText(text);
		label.setData(RWT.CUSTOM_VARIANT, "heading");
		GridData labelLayoutData = new GridData();
		labelLayoutData.horizontalSpan = horizontalSpan;
		
		label.setLayoutData(labelLayoutData);
	}

	public static GridLayout createMainLayout(int numColumns) {
		GridLayout result = new GridLayout(numColumns, true);
		result.marginWidth = 0;
		result.marginHeight = 0;
		result.marginTop = 0;
		result.verticalSpacing = 0;
		result.horizontalSpacing = 50;
		return result;
	}

	public static GridLayout createMainLayout(int numColumns, int horzSpacing) {
		GridLayout result = new GridLayout(numColumns, true);
		result.marginWidth = 0;
		result.marginHeight = 0;
		result.marginTop = 0;
		result.verticalSpacing = 0;
		result.horizontalSpacing = horzSpacing;
		return result;
	}

	public static GridLayout createGridLayout(int numColumns,
			boolean makeColsEqualWidth, boolean setTopMargin,
			boolean setVertSpacing) {
		GridLayout result = new GridLayout(numColumns, makeColsEqualWidth);
		result.marginWidth = DEFAULT_SPACE;
		result.marginHeight = 0;
		result.marginBottom = DEFAULT_SPACE;
		result.horizontalSpacing = DEFAULT_SPACE;
		if (setTopMargin) {
			result.marginTop = DEFAULT_SPACE;
		}
		if (setVertSpacing) {
			result.verticalSpacing = DEFAULT_SPACE;
		}
		return result;
	}

	public static GridLayout createGridLayoutWithoutMargin(int numColumns,
			boolean makeColsEqualWidth) {
		GridLayout result = new GridLayout(numColumns, makeColsEqualWidth);
		result.marginHeight = 0;
		result.marginWidth = 0;
		return result;
	}

	public static RowLayout createRowLayout(int type, boolean setMargin) {
		RowLayout result = new RowLayout(type);
		result.marginTop = 0;
		result.marginLeft = 0;
		result.marginHeight = 0;
		if (setMargin) {
			result.marginBottom = DEFAULT_SPACE;
			result.marginWidth = DEFAULT_SPACE;
		} else {
			result.marginBottom = 0;
			result.marginWidth = 0;
		}
		return result;
	}

	public static FillLayout createFillLayout(boolean setMargin) {
		FillLayout result = new FillLayout();
		if (setMargin) {
			result.marginWidth = DEFAULT_SPACE;
			result.marginHeight = DEFAULT_SPACE;
		}
		return result;
	}

	public static GridData createHorzFillData() {
		return new GridData(SWT.FILL, SWT.TOP, true, false);
	}

	public static GridData createFillData() {
		return new GridData(SWT.FILL, SWT.FILL, true, true);
	}

	private static Label createHeadlineLabel(Composite parent, String text) {
		Label label = new Label(parent, SWT.NONE);
		label.setText(text.replace("&", "&&"));
		label.setData(RWT.CUSTOM_VARIANT, "pageHeadline");
		return label;
	}

	private static GridData createHeadlineLayoutData() {
		GridData layoutData = new GridData();
		layoutData.verticalIndent = 30;
		layoutData.horizontalAlignment = SWT.CENTER;
		layoutData.horizontalIndent = DEFAULT_SPACE;
		return layoutData;
	}
	
	public static Image loadImage(Display display, String name ) {
	    Image result = null;
	    InputStream stream = UsersPage.class.getClassLoader().getResourceAsStream( name );
	    if(stream != null) {
	      try {
	        result = new Image( display, stream );
	      } 
	      finally {
	        try {
	          stream.close();
	        } 
	        catch( IOException unexpected ) {
	          throw new RuntimeException( "Failed to close image input stream", unexpected );
	        }
	      }
	    }
	    return result;
	  }
}
