/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.composites;

import java.io.IOException;
import java.io.InputStream;

import org.eclipse.rap.rwt.RWT;
import org.eclipse.rap.rwt.client.service.JavaScriptLoader;
import org.eclipse.rap.rwt.remote.Connection;
import org.eclipse.rap.rwt.remote.RemoteObject;
import org.eclipse.rap.rwt.service.ResourceManager;
import org.eclipse.rap.rwt.widgets.WidgetUtil;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Layout;

public class ChartsComposite extends Composite {

	private static final long serialVersionUID = 1L;

	private static final String REMOTE_TYPE = "Charts";
	private static final String REGISTER_PATH = "charts/";
	
	private final RemoteObject remoteObject;
	private String chartdata = "[2,4,5]";
	private String title = "Chart title";
	
	
	public ChartsComposite(Composite parent) {
		 super(parent, SWT.NONE);
		 registerResources();
		 loadJavaScript();
		 Connection connection = RWT.getUISession().getConnection();
		 remoteObject = connection.createRemoteObject(REMOTE_TYPE);
		 //remoteObject.setHandler(operationHandler);
		 remoteObject.set("parent", WidgetUtil.getId(this));
		 remoteObject.set("chartdata", this.chartdata);
	}
	
	public void setChartData(String data){
		this.chartdata = data;
		remoteObject.set("chartdata", this.chartdata);
	}
	
	public void setTitle(String title){
		this.title = title;
		remoteObject.set("title", this.title);
	}
	
	private void registerResources() {
		ResourceManager resourceManager = RWT.getResourceManager();
		boolean isRegistered = resourceManager.isRegistered(REGISTER_PATH + "charts.js");
		if(! isRegistered){
			try {
				 InputStream inputStream = ChartsComposite.class.getClassLoader().getResourceAsStream("de/eoda/qtafi/surveymanager/rap/composites/charts.js");
				 try {
					 resourceManager.register(REGISTER_PATH + "charts.js", inputStream);
				 } 
				 finally {
					 inputStream.close();
				 }
			} 
			catch(IOException ioe) {
				throw new IllegalArgumentException("Failed to load resources", ioe);
			}
		}
	}
	
	private void loadJavaScript() {
		JavaScriptLoader jsLoader = RWT.getClient().getService(JavaScriptLoader.class);
		ResourceManager resourceManager = RWT.getResourceManager();
		jsLoader.require(resourceManager.getLocation(REGISTER_PATH + "charts.js"));
	}
	
	@Override
	public void setLayout(Layout layout) {
		throw new UnsupportedOperationException("Cannot change internal layout");
	}
	
	@Override
	public void dispose() {
		remoteObject.destroy();
		super.dispose();
	}
	
	
}
