/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.composites;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.Vector;

import org.eclipse.rap.json.JsonObject;
import org.eclipse.rap.json.JsonValue;
import org.eclipse.rap.rwt.RWT;
import org.eclipse.rap.rwt.client.service.JavaScriptLoader;
import org.eclipse.rap.rwt.remote.Connection;
import org.eclipse.rap.rwt.remote.OperationHandler;
import org.eclipse.rap.rwt.remote.RemoteObject;
import org.eclipse.rap.rwt.service.ResourceManager;
import org.eclipse.rap.rwt.widgets.WidgetUtil;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Layout;
import org.eclipse.swt.widgets.Listener;

public class EditorHTMLComposite extends Composite {

	private static final long serialVersionUID = 1L;
	private static final String REMOTE_TYPE = "Editorhtmlcomposite";
	private static final String REGISTER_PATH = "editorhtmlcomposite/";
	
	protected final RemoteObject remoteObject;
	protected String html = "<h1></h1>";
	protected List<ClickListener> listeners = new Vector<ClickListener>();
	
	public EditorHTMLComposite(Composite parent, int parentStyle) {
		super(parent, parentStyle);
		registerResources();
		loadJavaScript();
		Connection connection = RWT.getUISession().getConnection();
		remoteObject = connection.createRemoteObject(REMOTE_TYPE);

		remoteObject.setHandler(new OperationHandler() {

			@Override
			public void handleSet(JsonObject arg0) {
			}

			@Override
			public void handleNotify(String event, JsonObject json) {
				if (event.equals("dblclick")) {
					for (ClickListener l: listeners){
						JsonValue uid = json.get("uid");
						if(uid != null) l.dblclick(uid.asString());
					}
				}
				else if(event.equals("singleclick")) {
					for (ClickListener l: listeners){
						JsonValue uid = json.get("uid");
						if(uid != null) l.singleclick(uid.asString());
					}
				}
			}

			@Override
			public void handleCall(String method, JsonObject json) {
			}
		});
		remoteObject.set("parent", WidgetUtil.getId(this));
		remoteObject.set("html", this.html);
		
		this.addListener(SWT.Selection, new Listener() {
			@Override
			public void handleEvent(Event arg0) {}
		});
		
		this.addListener(SWT.DefaultSelection, new Listener() {
			@Override
			public void handleEvent(Event arg0) {}
		});
	}

	public void addClickListener(ClickListener listener){
		this.listeners.add(listener);
	}
	
	public void removeClickListener(ClickListener listener){
		this.listeners.remove(listener);
	}
	
	@Override
	public void addListener(int eventType, Listener listener) {
		boolean waslistening = isListening(eventType);
		super.addListener(eventType, listener);
		boolean islistening = isListening(eventType);
		String remoteType = eventTypeToString(eventType);
		if(remoteType != null && !waslistening && islistening){
			remoteObject.listen(remoteType, true);
		}
	}
	
	String eventTypeToString(int event){
		if(event == SWT.Selection) return "dblclick";
		return "singleclick";
	}
	
	public String getHtml(){
		return html;
	}
	
	public void setHtml(String html){
		this.html = html;
		remoteObject.set("html", this.html);
	}
	
	public void scrollTo(String uid){
		remoteObject.set("scrollTo", uid);
	}
	
	protected void registerResources() {
		ResourceManager resourceManager = RWT.getResourceManager();
		boolean isRegistered = resourceManager.isRegistered(REGISTER_PATH + "editorhtmlcomposite.js");
		if(! isRegistered){
			try {
				 InputStream inputStream = HTMLComposite.class.getClassLoader().getResourceAsStream("de/eoda/qtafi/surveymanager/rap/composites/editorhtmlcomposite.js");
				 try {
					 resourceManager.register(REGISTER_PATH + "editorhtmlcomposite.js", inputStream);
				 } 
				 finally {
					 inputStream.close();
				 }
			} 
			catch(IOException ioe) {
				throw new IllegalArgumentException("Failed to load resources", ioe);
			}
		}
	}

	protected void loadJavaScript() {
		JavaScriptLoader jsLoader = RWT.getClient().getService(JavaScriptLoader.class);
		ResourceManager resourceManager = RWT.getResourceManager();
		jsLoader.require(resourceManager.getLocation(REGISTER_PATH + "editorhtmlcomposite.js"));
	}
	
	@Override
	public void setLayout(Layout layout) {
		throw new UnsupportedOperationException("Cannot change internal layout");
	}
	
	@Override
	public void dispose() {
		remoteObject.destroy();
		super.dispose();
	}
	
}
