/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.composites;

import java.io.IOException;
import java.io.InputStream;

import org.eclipse.rap.rwt.RWT;
import org.eclipse.rap.rwt.client.service.JavaScriptLoader;
import org.eclipse.rap.rwt.remote.Connection;
import org.eclipse.rap.rwt.remote.RemoteObject;
import org.eclipse.rap.rwt.service.ResourceManager;
import org.eclipse.rap.rwt.widgets.WidgetUtil;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;

public class StockChartComposite extends Composite {

	
	private static final long serialVersionUID = 1L;
	
	private static final String REMOTE_TYPE = "StockCharts";
	private static final String REGISTER_PATH = "stockcharts/";
	
	private final RemoteObject remoteObject;
	private String chartdata = "['data', 2, 5]";
	private String title = "Chart title";
	
	public StockChartComposite(Composite parent) {
		super(parent, SWT.NONE);
		registerResources();
		loadJavaScript();
		Connection connection = RWT.getUISession().getConnection();
		remoteObject = connection.createRemoteObject(REMOTE_TYPE);
		//remoteObject.setHandler(operationHandler);
		remoteObject.set("parent", WidgetUtil.getId(this));
		remoteObject.set("chartdata", this.chartdata);
	}
	
	public void setChartData(String data){
		this.chartdata = data;
		remoteObject.set("chartdata", this.chartdata);
	}
	
	public void setTitle(String title){
		this.title = title;
		remoteObject.set("title", this.title);
	}
	
	private void registerResources() {
		ResourceManager resourceManager = RWT.getResourceManager();
		boolean isRegistered = resourceManager.isRegistered(REGISTER_PATH + "stockcharts.js");
		if(! isRegistered){
			try {
				 InputStream inputStream = StockChartComposite.class.getClassLoader().getResourceAsStream("de/eoda/qtafi/surveymanager/rap/composites/stockcharts.js");
				 try {
					 resourceManager.register(REGISTER_PATH + "stockcharts.js", inputStream);
				 } 
				 finally {
					 inputStream.close();
				 }
			} 
			catch(IOException ioe) {
				throw new IllegalArgumentException("Failed to load resources", ioe);
			}
		}
	}
	
	private void loadJavaScript() {
		JavaScriptLoader jsLoader = RWT.getClient().getService(JavaScriptLoader.class);
		ResourceManager resourceManager = RWT.getResourceManager();
		jsLoader.require(resourceManager.getLocation(REGISTER_PATH + "stockcharts.js"));
	}

}
