/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/

/**
 * 
 */
(function(){
	'use strict';
	
	rap.registerTypeHandler("Charts", {
	    factory: function(properties) {
	    	return new Charts(properties);
	    },
	    
	    destructor: "destroy",
	    properties: ["chartdata", "title"]
	});
	
	
	var Charts = function(properties) {
		bindAll(this, ["layout", "onReady", "onSend", "onRender", "setData", "setTitle"]);
		this.parent = rap.getObject(properties.parent);
		this.element = document.createElement("div");
		this.parent.append(this.element); 
		//this.parent.addListener("Resize", this.layout);
		rap.on("render", this.onRender);
	};
	
	Charts.prototype = {

	    ready: false,
	    chartdata: [1,2,3,5],
	    title: 'Chart title',
	    chart: null,
	    
	    onRender: function(){
	    	var _this = this;
	    	if(this.element){
	    		rap.off("render", this.onRender);
	    		console.log('render ');
	    		this.element.innerHTML = "<div id='charts-container' style='width:100%; height:300px;'></div>";
	    		
	    		$(function () {
	    			_this.chart = new Highcharts.Chart({
	    				chart: {
	    		            type: 'line',
	    		            renderTo: 'charts-container'
	    		        },
	    		        title: {
	    		            text: _this.title
	    		        },
	    		        xAxis: {
	    		        },
	    		        series: [{
	    		        	name: 'xxx',
	    		        	data: _this.chartdata
	    		        }]
	    			});
	    		});
	    		rap.on("send", this.onSend);
	    	}
	    },
	
	    onSend: function() {
	    	//rap.getRemoteObject(this).set("html", this.html);
	    },
	    
	    setChartdata: function(_chartdata){
	    	//console.log(_chartdata);
	    	if(_chartdata.length > 3){
	    		this.chartdata = JSON.parse(_chartdata);
	    		//console.log(this.chartdata);
	    		if(this.chart){
	    			this.chart.series[0].setData(this.chartdata);
	    		}
	    	}
	    },
	    
	    setTitle: function(_title){
	    	this.title = _title;
	    	if(this.chart){
	    		this.chart.setTitle(_title);
	    	}
	    },
	    
	    
		destroy: function() {
			rap.off("send", this.onSend);
		    this.element.parentNode.removeChild(this.element);
		},

		layout: function() {
			if(this.ready) {
				var area = this.parent.getClientArea();
		        this.element.style.left = area[0] + "px";
		        this.element.style.top = area[1] + "px";
		    }
		},
	
		onReady: function(){
			console.log('onReady');
			this.ready = true;
			this.layout();
		}
	};
	
	var bind = function(context, method) {
		return function(){
			return method.apply(context, arguments);
		};
	};
	
	var bindAll = function(context, methodNames) {
		for(var i = 0; i < methodNames.length; i++) {
			var method = context[methodNames[i]];
			context[methodNames[i]] = bind(context, method);
		}
	};
	
}());