/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/

/**
 * 
 */
(function(){
	'use strict';
	
	rap.registerTypeHandler("StockCharts", {
	    factory: function(properties) {
	    	return new StockCharts(properties);
	    },
	    
	    destructor: "destroy",
	    properties: ["chartdata", "title"]
	});
	
	
	var StockCharts = function(properties) {
		bindAll(this, ["layout", "onReady", "onSend", "onRender", "setData", "setTitle"]);
		this.parent = rap.getObject(properties.parent);
		this.element = document.createElement("div");
		this.parent.append(this.element); 
		//this.parent.addListener("Resize", this.layout);
		rap.on("render", this.onRender);
	};
	
	StockCharts.prototype = {

		
	    ready: false,
	    chartdata: { 
	    	x: 'x',
	    	columns: [
	    	          ['x', 1, 2],
	    	          ['data', 1, 2]
	    	          ],
	    	     type: 'spline',
	    },
	    
	    title: 'Chart title',
	    //stockchart: null,
	    
	    onRender: function(){
	    	var _this = this;
	    	var date = new Date();
	    	if(this.element){
	    		rap.off("render", this.onRender);
	    		this.element.innerHTML = "<div id='stockcharts-container' style='width:100%; height:100%;'></div>";
	    		var chart = c3.generate(
	    			{
	    				bindto: '#stockcharts-container',
	    				data: _this.chartdata,
	    			
					    padding: {
					    	right: 40
					    },
					    point: {
					    	show: false
					    },
					    tooltip: {
					        show: true
					    },
					    legend: {
					    	show: false
					    },
					    size: {
					        //height: 200,
					        //width: 800
					    },
					    transition: {
					        duration: 400
					    },
					    grid: {
					        x: {
					            show: false
					        },
					        y: {
					            show: false
					        }
					    },
					    axis: {
					        x: {
					            type: 'timeseries',
					            //height: 70,
					            tick: {
					                //rotate: -65,
					                fit: true,
					                format: '%Y-%m-%d',
					                values: [_this.chartdata.columns[0][1], _this.chartdata.columns[0][ _this.chartdata.columns[0].length-1]]
					            }
					        },
					        y: {
					        	show: true,
					        	tick: {
					            },
					        	min: 0,
					        	padding: {
					        		top: 0, 
					        		bottom: 0
					        	}
					        }
					    }
		    		});

	    		rap.on("send", this.onSend);
	    	}
	    },
	
	    onSend: function() {
	    },
	    
	    setChartdata: function(_chartdata){
	    	if(_chartdata.length > 3){
	    		this.chartdata = JSON.parse(_chartdata);
	    		
	    		console.log(this.chartdata.columns[0][1]);
	    		console.log(this.chartdata.columns[0][this.chartdata.columns[0].length-1]);
	    	}
	    },
	    
	    setTitle: function(_title){
	    	this.title = _title;
	    	if(this.chart){
	    		this.chart.setTitle(_title);
	    	}
	    },
	    
	    
		destroy: function() {
			rap.off("send", this.onSend);
		    this.element.parentNode.removeChild(this.element);
		},

		layout: function() {
			if(this.ready) {
				var area = this.parent.getClientArea();
		        this.element.style.left = area[0] + "px";
		        this.element.style.top = area[1] + "px";
		    }
		},
	
		onReady: function(){
			console.log('onReady');
			this.ready = true;
			this.layout();
		}
	};
	
	var bind = function(context, method) {
		return function(){
			return method.apply(context, arguments);
		};
	};
	
	var bindAll = function(context, methodNames) {
		for(var i = 0; i < methodNames.length; i++) {
			var method = context[methodNames[i]];
			context[methodNames[i]] = bind(context, method);
		}
	};
	
}());