/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.export;

import java.util.Iterator;

import org.apache.commons.lang3.StringEscapeUtils;

import de.eoda.qtafi.surveymanager.lang.Messages;
import de.eoda.qtafi.surveymanager.rap.export.TabularReportData.Row;


public class ChartData {

	protected static final int CHART_ROW_ITEM_HEIGHT = 28;
	protected static final int CHART_OFFSET_HEIGHT = 120;
	protected static final int CHART_LEFT_LEGEND_WIDTH = 250;
	
	public int width;
	public int height;
	public double scaleMax = 5;
	public double scaleMin = 1;
	final TabularReportData tabledata;
	
	public ChartData(TabularReportData tabledata) {
		this.tabledata = tabledata;
		this.width = 700;
		
		if(tabledata.tableHead && ((tabledata.options & Report.OPTION_MEAN) != Report.OPTION_MEAN)){
			int c = tabledata.colcount();
			this.height = (int) ((tabledata.rowcount() * CHART_ROW_ITEM_HEIGHT * 0.5 * c) + CHART_OFFSET_HEIGHT);
		}
		else {
			this.height = (tabledata.rowcount() * CHART_ROW_ITEM_HEIGHT) + CHART_OFFSET_HEIGHT;
		}
	}
	
	public void setScale(double min, double max){
		scaleMin = min;
		scaleMax = max;
	}
	
	public String getC3Config(String elid){
		StringBuffer buf = new StringBuffer();
		buf.append("{\n");
		buf.append("bindto: '#" + elid + "',\n");
		buf.append("data: {\n");
		buf.append("columns:[\n");
		
		if(tabledata.tableHead){
			int c = 0;
			for(Iterator<String> it = tabledata.colnames.iterator(); it.hasNext(); c++){
				String col = it.next();
				buf.append("[\""+ col + "\", ");
				for(Iterator<Row> rit = tabledata.rows.iterator(); rit.hasNext();){
					Row v = rit.next();
					if((tabledata.options & Report.OPTION_MEAN) == Report.OPTION_MEAN){
						if(Double.isNaN(v.cells[c].value)) buf.append("null");
						else buf.append(v.cells[c].value);
					}
					else if((tabledata.options & Report.OPTION_COUNT) == Report.OPTION_COUNT){
						buf.append(v.cells[c].count);
					}
					else if((tabledata.options & Report.OPTION_PERCENT) == Report.OPTION_PERCENT){
						if(Double.isNaN(v.cells[c].value)) buf.append("null");
						else buf.append(v.cells[c].value);
					}
					if(rit.hasNext()) buf.append(",");
				}
				buf.append("]");
				if(it.hasNext()) buf.append(",\n");
			}
		}
		else {
			buf.append("['null', " );
			for(Iterator<Row> it = tabledata.rows.iterator(); it.hasNext();){
				Row v = it.next();
				if((tabledata.options & Report.OPTION_MEAN) == Report.OPTION_MEAN){
					if(Double.isNaN(v.cells[0].value)) buf.append("null");
					else buf.append(v.cells[0].value);
				}
				else if((tabledata.options & Report.OPTION_COUNT) == Report.OPTION_COUNT){
					buf.append(v.cells[0].count);
				}
				else if((tabledata.options & Report.OPTION_PERCENT) == Report.OPTION_PERCENT){
					if(Double.isNaN(v.cells[0].value)) buf.append("null");
					else buf.append(v.cells[0].value);
				}
				if(it.hasNext()) buf.append(",");
			}
			buf.append("]\n");
		}
		buf.append("],\n");
		
		if((tabledata.options & Report.OPTION_MEAN) == Report.OPTION_MEAN){
			buf.append("type: 'line',\n");
		}
		else {
			buf.append("type: 'bar',\n");
		}
		buf.append("labels: false\n");
		buf.append("},\n");
		buf.append("padding: {\n");
		buf.append("left: "+CHART_LEFT_LEGEND_WIDTH+"\n");
		buf.append("},\n");
		if(! ((tabledata.options & Report.OPTION_MEAN) == Report.OPTION_MEAN)){
			buf.append("bar: {\n");
			buf.append("width: { ratio: 0.5 }\n");
			buf.append("},\n");
		}
		buf.append("tooltip: {\n");
		buf.append("show: false\n");
		buf.append("},\n");
		buf.append("legend: {\n");
		if(tabledata.tableHead){
			buf.append("show: true\n");
		}
		else {
			buf.append("show: false\n");
		}
		buf.append("},\n");
		
		buf.append("color: {\n");
		buf.append("pattern: ['#7cb5ec','#434348', '#90ed7d','#f7a35c','#8085e9','#f15c80', '#e4d354', '#2b908f', '#f45b5b', '#91e8e1']\n");
		buf.append("},\n");
		buf.append("size: {\n");
		buf.append("height:" + height + ",\n");
		buf.append("width:" +width + "\n");
		buf.append("},\n");
		
		//Points
		buf.append("point: {\n");
		buf.append("show: true,\n");
		buf.append("r: 4\n");
		buf.append("},\n");
		
		//Grid
		buf.append("grid: {\n");
		buf.append("x: { show: false },\n");
		buf.append("y: { show: true },\n");
		buf.append("},\n");
		
		buf.append("axis: {\n");
		buf.append("rotated: true,\n");
		buf.append("x: {\n");
		buf.append("tick: {\n");
		buf.append("width: "+(CHART_LEFT_LEGEND_WIDTH-10)+"\n");
		buf.append("},\n");
		buf.append("type: 'category',\n");
		buf.append("categories:[\n");
		for(Iterator<Row> it = tabledata.rows.iterator(); it.hasNext();){
			Row v = it.next();
			String l = StringEscapeUtils.escapeJson(v.label);
			buf.append("\"" + l + "\"");
			if(it.hasNext()) buf.append(",");
		}
		buf.append("]\n");
		
		buf.append("},\n");
		buf.append("y: {\n");
		
		if((tabledata.options & Report.OPTION_MEAN) == Report.OPTION_MEAN){
			buf.append("min: " + scaleMin + ",\n");
			buf.append("max: " + scaleMax + ",\n");
		}
		buf.append("padding: {top: 0, bottom: 0, left:0, right:0 },\n");
		if((tabledata.options & Report.OPTION_MEAN) == Report.OPTION_MEAN){
			buf.append("label: '"+ Messages.get().ReportPage_Mean +"'\n");
		}
		else if((tabledata.options & Report.OPTION_COUNT) == Report.OPTION_COUNT){
			buf.append("label: '"+ Messages.get().ReportPage_Count +"'\n");
		}
		else if((tabledata.options & Report.OPTION_PERCENT) == Report.OPTION_PERCENT){
			buf.append("label: '"+ Messages.get().ReportPage_Percent +"'\n");
		}
		buf.append("}\n");
		buf.append("}\n");
		
		buf.append("}\n");
		
		//System.out.println(buf);
		return buf.toString();
	}
	
	
	/*
	public String getHighchartConfig(){
		StringBuffer buf = new StringBuffer();
		buf.append("{");
		if((tabledata.options & Report.OPTION_MEAN) == Report.OPTION_MEAN){
			buf.append("chart: { type: \"line\", inverted: true, marginLeft: "+CHART_LEFT_LEGEND_WIDTH+", width: 700, height:"+height+" },");
		}
		else {
			buf.append("chart: { type: \"bar\", marginLeft: "+CHART_LEFT_LEGEND_WIDTH + ", width: 700, height:"+height+"},");
		}
		buf.append("title: { text: null },");
		if((tabledata.options & Report.OPTION_MEAN) == Report.OPTION_MEAN){
			buf.append("yAxis: { min: "+ scaleMin + ", max: " + scaleMax + ", tickInterval: 1 },");
		}
		else buf.append("yAxis: { },"); 
		
		//Tooltip 
		buf.append("tooltip: {\n");
		buf.append("formatter: function() {\n");
		buf.append("return '<b>' + this.x + '</b>:' + Highcharts.numberFormat(this.y, 1);");
		buf.append("}");
		buf.append("},");
		
		buf.append("xAxis: {"); 
		buf.append("categories: [");
		for(Iterator<Row> it = tabledata.rows.iterator(); it.hasNext();){
			Row v = it.next();
			String l = StringEscapeUtils.escapeJson(v.label);
			buf.append("\"" + l + "\"");
			if(it.hasNext()) buf.append(",");
		}
		buf.append("]},");
		if(tabledata.tableHead){
			buf.append("legend: { enabled: true },");
		}
		else buf.append("legend: { enabled: false },");
		
		buf.append("series:[");
		if(tabledata.tableHead){
			int c = 0;
			for(Iterator<String> it = tabledata.colnames.iterator(); it.hasNext(); c++){
				String col = it.next();
				buf.append("{name:\""+ col + "\", data: [");
				for(Iterator<Row> rit = tabledata.rows.iterator(); rit.hasNext();){
					Row v = rit.next();
					if((tabledata.options & Report.OPTION_MEAN) == Report.OPTION_MEAN){
						//buf.append(v.cells[c].value);
						if(Double.isNaN(v.cells[c].value)) buf.append(0.0);
						else buf.append(v.cells[c].value);
					}
					else if((tabledata.options & Report.OPTION_COUNT) == Report.OPTION_COUNT){
						buf.append(v.cells[c].count);
					}
					else if((tabledata.options & Report.OPTION_PERCENT) == Report.OPTION_PERCENT){
						if(Double.isNaN(v.cells[c].value)) buf.append(0.0);
						else buf.append(v.cells[c].value);
					}
					if(rit.hasNext()) buf.append(",");
				}
				buf.append("] }");
				if(it.hasNext()) buf.append(",");
			}
		}
		else {
			buf.append("{ name: null, data: [" );
			for(Iterator<Row> it = tabledata.rows.iterator(); it.hasNext();){
				Row v = it.next();
				if((tabledata.options & Report.OPTION_MEAN) == Report.OPTION_MEAN){
					//buf.append(v.cells[0].value);
					if(Double.isNaN(v.cells[0].value)) buf.append(0.0);
					else buf.append(v.cells[0].value);
				}
				else if((tabledata.options & Report.OPTION_COUNT) == Report.OPTION_COUNT){
					buf.append(v.cells[0].count);
				}
				else if((tabledata.options & Report.OPTION_PERCENT) == Report.OPTION_PERCENT){
					//buf.append(v.cells[0].value);
					if(Double.isNaN(v.cells[0].value)) buf.append(0.0);
					else buf.append(v.cells[0].value);
				}
				if(it.hasNext()) buf.append(",");
			}
			buf.append("] }");
		}
		
		buf.append("]\n");
		buf.append("}");
		//System.out.println(buf.toString());
		return buf.toString();
	}
	*/
}
