/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.export;

import java.io.InputStream;
import java.text.DecimalFormat;
import java.util.UUID;

import de.eoda.qtafi.surveymanager.lang.Messages;
import de.eoda.qtafi.surveymanager.rap.export.TabularReportData.Row;
import de.eoda.qtafi.surveymanager.rap.report.MissingData;
import de.eoda.qtafi.surveymanager.rap.report.ReportItem;


public class HtmlDocument implements Report {

	final static DecimalFormat format0 = new DecimalFormat("#0");
	final static DecimalFormat format1 = new DecimalFormat("#0.0");
	
	StringBuffer buf = new StringBuffer();
	int height = 0;
	
	protected String formatNumber(double num, int options){
		if(Double.isNaN(num)) return "-";
		if(Double.isInfinite(num)) return "-";
		if((options & Report.OPTION_PERCENT) == Report.OPTION_PERCENT) return format1.format(num) + "%";
		if((options & Report.OPTION_MEAN) == Report.OPTION_MEAN) return format1.format(num);
		if((options & Report.OPTION_COUNT) == Report.OPTION_COUNT) return format0.format(num);
		else return format0.format(num);
	}
	
	public void appendHead(String tit, String...strings){
		if(tit == null) tit = "--";
		if(tit.length() == 0) tit = "--";
		buf.append("<h1 style=\"line-height:29px;font-size:22px;margin-top:20px;\">" + tit);
		if(strings.length > 0){
			buf.append("  <small>");
			for(String s: strings){
				buf.append(s + "; &#160;");
			}
			buf.append("</small>");
		}
		buf.append("</h1>");
		buf.append("<hr style=\"margin-top:0;margin-bottom:10px;\"/>");
		height += 70;
	}
	
	public void appendHead(ReportItem item, String...strings){
		buf.append("<h1 style=\"line-height:29px;font-size:22px;margin-top:20px;\">" + item.q.id + " - " + item.q.text);
		if(strings.length > 0){
			buf.append("  <small>");
			for(String s: strings){
				buf.append(s + "; &#160;");
			}
			buf.append("</small>");
		}
		buf.append("</h1>");
		buf.append("<hr style=\"margin-top:0;margin-bottom:10px;\"/>");
		height += 70;
	}
	
	public void appendSubHead(String tit, String...strings){
		if(tit == null) tit = "--";
		if(tit.length() == 0) tit = "--";
		buf.append("<h3 style=\"line-height:20px;font-size:12px;margin-top:20px;font-weight:bold;\">" + tit);
		if(strings.length > 0){
			buf.append("  <small>");
			for(String s: strings){
				buf.append(s + "; &#160;");
			}
			buf.append("</small>");
		}
		buf.append("</h3>");
		height += 45;
	}
	
	public void appendFoot(String foot, String...strings){
		if(foot == null) foot = "--";
		if(foot.length() == 0) foot = "--";
		buf.append("<hr style=\"margin-top:20px; margin-bottom:5px;\" />");
		buf.append("<div style=\"line-height:18px;font-size:12px;font-style:italic;\">Question: " + foot + "</div>");
		this.height += 60;
	}
	
	public void appendTable(TabularReportData table, int options){
		int px = 0;
		buf.append("<table style=\"width:100%;margin-bottom:10px;\">");
		if(! table.tableHead){
			/*
			buf.append("<thead>");
			buf.append("<tr>");
			buf.append("<td style=\"width:400px;line-height:21px;height:23px;white-space: nowrap;\"></td>");
			if((options & Report.OPTION_COUNT) == Report.OPTION_COUNT){
				buf.append("<th style=\"line-height:21px;height:23px;text-align:right;\">Count</th>");
			}
			if((options & Report.OPTION_PERCENT) == Report.OPTION_PERCENT){
				buf.append("<th style=\"line-height:21px;height:23px;text-align:right;\">Percent</th>");
			}
			if((options & Report.OPTION_MEAN) == Report.OPTION_MEAN){
				buf.append("<th style=\"line-height:21px;height:23px;text-align:right;\">Mean</th>");
			}
			buf.append("</tr>");
			buf.append("</thead>");
			px += 25;
			*/
			buf.append("<tbody>");
			for(int row = 0; row < table.rowcount(); row++){
				Row r = table.rows.get(row);
				buf.append("<tr>");
				buf.append("<td style=\"width:400px;line-height:21px;height:23px;white-space: nowrap;\">"+ r.label +"</td>");
				if((options & Report.OPTION_COUNT) == Report.OPTION_COUNT){
					buf.append("<td style=\"line-height:21px;height:23px;text-align:right;\">"+ formatNumber(r.cells[0].count, Report.OPTION_COUNT) + "</td>");
				}
				if((options & Report.OPTION_PERCENT) == Report.OPTION_PERCENT){
					buf.append("<td style=\"line-height:21px;height:23px;text-align:right;\">"+ formatNumber(r.cells[0].value, Report.OPTION_PERCENT) + "</td>");
				}
				if((options & Report.OPTION_MEAN) == Report.OPTION_MEAN){
					buf.append("<td style=\"line-height:21px;height:23px;text-align:right;\">"+ formatNumber(r.cells[0].value, Report.OPTION_MEAN) + "</td>");
				}
				buf.append("</tr>");
				px += 23;
			}
		}
		else { //break
			if(table.tableHead){
				buf.append("<thead>");
				buf.append("<th style=\"line-height:21px;height:23px;white-space: nowrap;\"></th>");
				for(String col: table.colnames){
					buf.append("<th style=\"line-height:21px;height:23px;text-align:right;\">"+col+"</th>");
				}
				buf.append("</thead>");
				px += 25;
			}
			
			buf.append("<tbody>");
			
			for(int row = 0; row < table.rowcount(); row++){
				Row r = table.rows.get(row);
				buf.append("<tr>");
				buf.append("<td style=\"width:400px;line-height:21px;height:23px;white-space: nowrap;\">"+ r.label +"</td>");
		
				int i = 0;
				for(String col: table.colnames){
					if((options & Report.OPTION_COUNT) == Report.OPTION_COUNT){
						buf.append("<td style=\"line-height:21px;height:23px;text-align:right;\">"+ formatNumber(r.cells[i].count, Report.OPTION_COUNT) + "</td>");
					}
					if((options & Report.OPTION_PERCENT) == Report.OPTION_PERCENT){
						buf.append("<td style=\"line-height:21px;height:23px;text-align:right;\">"+ formatNumber(r.cells[i].value, Report.OPTION_PERCENT) + "</td>");
					}
					if((options & Report.OPTION_MEAN) == Report.OPTION_MEAN){
						buf.append("<td style=\"line-height:21px;height:23px;text-align:right;\">"+ formatNumber(r.cells[i].value, Report.OPTION_MEAN) + "</td>");
					}
					i++;
				}
				
				buf.append("</tr>");
				px += 23;
			}
		}
		
		//Totals
		if((options & Report.OPTION_TOTALS) == Report.OPTION_TOTALS){
			buf.append("<tr>");
			buf.append("<td style=\"width:400px;line-height:21px;height:23px;white-space: nowrap;\">Total</td>");
			if(table.tableHead){
				int i = 0;
				for(String col: table.colnames){
					if((options & Report.OPTION_COUNT) == Report.OPTION_COUNT){
						buf.append("<td style=\"line-height:21px;height:23px;text-align:right;\">"+ formatNumber(table.totals.cells[i].count, Report.OPTION_COUNT) + "</td>");
					}
					if((options & Report.OPTION_PERCENT) == Report.OPTION_PERCENT){
						buf.append("<td style=\"line-height:21px;height:23px;text-align:right;\">"+ formatNumber(table.totals.cells[i].value, Report.OPTION_PERCENT) + "</td>");
					}
					i++;
				}
			}
			else {
				if((options & Report.OPTION_COUNT) == Report.OPTION_COUNT){
					buf.append("<td style=\"line-height:21px;height:23px;text-align:right;\">"+ formatNumber(table.totals.cells[0].count, Report.OPTION_COUNT) + "</td>");
				}
				if((options & Report.OPTION_PERCENT) == Report.OPTION_PERCENT){
					buf.append("<td style=\"line-height:21px;height:23px;text-align:right;\">"+ formatNumber(table.totals.cells[0].value, Report.OPTION_PERCENT) + "</td>");
				}
			}
			buf.append("</tr>");
			px += 23;
		}
		buf.append("</tbody>");
		buf.append("</table>");
		px += 10;
		this.height += px;
	}
	
	public void appendChart(ChartData data, int options){
		String cid = "container-" + UUID.randomUUID().toString();
		buf.append("<div id=\"" + cid + "\" style=\"width:100%;height:" + data.height + "px;\" ></div>\n");
		buf.append("<script type=\"text/javascript\">\n");
		buf.append("$(function() {\n");
		
		buf.append("var chart = c3.generate(" + data.getC3Config(cid) + ");\n");
		buf.append("console.log(chart);\n");
		buf.append("});\n");
		buf.append("</script>");
		
		/*
		buf.append("<script type=\"text/javascript\">");
		buf.append("$(function () {\n");
		buf.append("$('#"+cid+"').highcharts(\n"+ data.getHighchartConfig() + "\n);");
		buf.append("});");
		buf.append("</script>");
		*/
		
		this.height += data.height;
	}
	
	@Override
	public void appendImage(InputStream is, int options) {
	}
	
	@Override
	public void appendMissingInfo(MissingData data, int options) {
		int px = 0;
		buf.append("<table style=\"margin-top:14px;\">");
		px += 15;
		buf.append("<tbody>");
		buf.append("<tr>");
		buf.append("<td style=\"line-height:21px;height:23px;white-space: nowrap; padding-right: 10px;font-style:italic;\">" + Messages.get().ReportPage_MissingCases + "</td>");
		buf.append("<td style=\"text-align:right;\">"+ formatNumber(data.missing, Report.OPTION_NONE)+ "</td>");
		buf.append("</tr>");
		px += 23;
		
		buf.append("<tr>");
		buf.append("<td style=\"line-height:21px;height:23px;white-space: nowrap;padding-right: 10px;font-style:italic;\">"+Messages.get().ReportPage_TotalAnswers+"</td>");
		buf.append("<td style=\"text-align:right;\">" + formatNumber(data.responses, Report.OPTION_NONE) + "</td>");
		buf.append("</tr>");
		px += 23;
		
		buf.append("<tr>");
		buf.append("<td style=\"line-height:21px;height:23px;white-space: nowrap;padding-right: 10px;font-style:italic;\">" + Messages.get().ReportPage_QuestionType + "</td>");
		buf.append("<td style=\"text-align:right;\">" + data.type + "</td>");
		buf.append("</tr>");
		px += 23;
		
		buf.append("</tbody>");
		buf.append("</table>");
		this.height += px;
	}
	
	public String getHtml(){
		return buf.toString();
	}
	
	public int getHeight(){
		return height;
	}
	
}
