/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.export;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.math.BigInteger;
import java.text.DecimalFormat;
import java.util.Properties;
import java.util.UUID;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.poi.util.Units;
import org.apache.poi.xwpf.usermodel.Borders;
import org.apache.poi.xwpf.usermodel.ParagraphAlignment;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.apache.poi.xwpf.usermodel.XWPFParagraph;
import org.apache.poi.xwpf.usermodel.XWPFRun;
import org.apache.poi.xwpf.usermodel.XWPFTable;
import org.apache.poi.xwpf.usermodel.XWPFTableCell;
import org.apache.poi.xwpf.usermodel.XWPFTableRow;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTBorder;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTTbl;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTTblPr;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTTblWidth;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTTcBorders;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTTcPr;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.STBorder;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.STTblWidth;

import de.eoda.qtafi.surveymanager.lang.Messages;
import de.eoda.qtafi.surveymanager.rap.export.TabularReportData.Cell;
import de.eoda.qtafi.surveymanager.rap.report.MissingData;
import de.eoda.qtafi.surveymanager.rap.report.ReportItem;
import de.eoda.qtafi.surveymanager.util.Resources;

public class POIDocument implements Report {

	final static DecimalFormat format0 = new DecimalFormat("#0");
	final static DecimalFormat format1 = new DecimalFormat("#0.0");

	String exportdirectory;
	XWPFDocument doc;


	public POIDocument() {
		try {
			doc = new XWPFDocument(POIDocument.class.getClassLoader().getResourceAsStream("de/eoda/qtafi/surveymanager/rap/export/empty.docx"));
		} 
		catch (IOException e1) {
			e1.printStackTrace();
			doc = new XWPFDocument();
		}

		Properties pp = new Properties();
		try {
			pp.load(new FileInputStream(Resources.ResolveFile("/WEB-INF/projects.properties")));
			exportdirectory = pp.getProperty("exportdirectory");
		} 
		catch (Exception e) {
			e.printStackTrace();
		}
		
		checkResources();
	}
	
	
	protected void checkResources() {
		File exportdir = new File(exportdirectory);
		File c3_css = new File(exportdirectory + "/c3.css");
		if(! c3_css.exists()){
			storeFile("c3.css");
		}
		File c3_js = new File(exportdirectory + "/c3.min.js");
		if(! c3_js.exists()){
			storeFile("c3.min.js");
		}
		File d3_js = new File(exportdirectory + "/d3.min.js");
		if(! d3_js.exists()){
			storeFile("d3.min.js");
		}
		File export_html = new File(exportdirectory + "/export.html");
		if(! export_html.exists()){
			storeFile("export.html");
		}
		File render_js = new File(exportdirectory + "/render.js");
		if(! render_js.exists()){
			storeFile("render.js");
		}
	}
	
	protected void storeFile(String file){
		try{
			InputStream in = POIDocument.class.getClassLoader().getResourceAsStream("/resources/export/" + file);
			OutputStream out = new FileOutputStream(exportdirectory + "/" + file);
			IOUtils.copy(in, out);
			in.close();
			out.close();
		}
		catch(IOException ex){
			ex.printStackTrace();
		}
	}
	
	
	protected String formatNumber(double num, int options){
		if(Double.isNaN(num)) return "-";
		if(Double.isInfinite(num)) return "-";
		if((options & Report.OPTION_PERCENT)== Report.OPTION_PERCENT) return format1.format(num) + "%";
		if((options & Report.OPTION_MEAN)== Report.OPTION_MEAN) return format1.format(num);
		if((options & Report.OPTION_COUNT)== Report.OPTION_COUNT) return format0.format(num);
		return format0.format(num);
	}

	public void appendHead(String tit, String...args){
		if(tit == null) tit = "--";
		if(tit.length() == 0) tit = "--";
		tit += " ";
		//tit += " " + Messages.get().ReportPage_BreakBy + " ";
		for(String string : args){
			tit += string + "; ";
		}
		XWPFParagraph h1 = doc.createParagraph();
		h1.setStyle("Heading1");
		h1.createRun().setText(tit);
		h1.setBorderBottom(Borders.SINGLE);
	}

	public void appendHead(ReportItem item, String...args){
		appendHead(item.q.id + " - " + item.q.text, args);
	}

	public void appendSubHead(String tit, String...args){
		if(tit == null) tit = "--";
		if(tit.length() == 0) tit = "--";
		XWPFParagraph h1 = doc.createParagraph();
		h1.setStyle("Heading3");
		XWPFRun r = h1.createRun();
		r.setBold(true);
		r.setText(tit);
	}

	public void appendFoot(String foot, String...args){
		XWPFParagraph h1 = doc.createParagraph();
		XWPFRun r = h1.createRun();
		r.setItalic(true);
		r.setText("Question: " + foot);
	}

	public void appendTable(TabularReportData tabledata, int options) {
		boolean hasTotalRow = (options & Report.OPTION_TOTALS) == Report.OPTION_TOTALS;
		int rows = tabledata.rowcount();
		rows = (tabledata.tableHead)? rows + 1: rows;
		rows = (hasTotalRow)? rows + 1 : rows;
		
		XWPFTable table = doc.createTable(
				rows,
				tabledata.colcount());
		{
			CTTbl cttable = table.getCTTbl();
			CTTblPr pr = cttable.getTblPr();
			CTTblWidth tblW = pr.getTblW();
			tblW.setW(BigInteger.valueOf(5000));
			tblW.setType(STTblWidth.PCT);
			pr.setTblW(tblW);
			cttable.setTblPr(pr);
		}

		int row = 0;

		if(tabledata.tableHead){
			XWPFTableCell cell1 = table.getRow(row).getCell(0);
			int c = 1;
			for(String col: tabledata.colnames){
				XWPFTableCell cell = table.getRows().get(row).getTableCells().get(c);
				XWPFParagraph paragraph = cell.addParagraph();
				XWPFRun run = paragraph.createRun();				
				paragraph.setAlignment(ParagraphAlignment.RIGHT);
				run.setText(col);
				run.setBold(true);
				cell.getCTTc().addNewTcPr().addNewTcW().setW(BigInteger.valueOf(2000));				
				c++;
			}
			row++;
		}

		for(int tdRowCounter = 0; row < ((hasTotalRow) ? rows-1 : rows); row++, tdRowCounter++){
			XWPFTableCell lcell = table.getRows().get(row).getTableCells().get(0);
			lcell.setText(tabledata.rows.get(tdRowCounter).label);
			for(int c = 1; c < tabledata.colcount(); c++){
				XWPFTableCell cell = table.getRows().get(row).getTableCells().get(c);
				cell.getCTTc().addNewTcPr().addNewTcW().setW(BigInteger.valueOf(2000));
				XWPFParagraph cp = cell.getParagraphs().get(0);
				cp.setAlignment(ParagraphAlignment.RIGHT);
				Cell currentCell = tabledata.rows.get(tdRowCounter).cells[(c-1)];
				String text = formatNumber(((tabledata.options & Report.OPTION_COUNT)== Report.OPTION_COUNT) ? currentCell.count : currentCell.value, tabledata.options);
				cp.createRun().setText(text /*formatNumber(tabledata.rows.get(row).cells[(c-1)].count, tabledata.options)*/);

			}
		}

		if(hasTotalRow){
			XWPFTableCell lcell = table.getRows().get(row).getTableCells().get(0);
			lcell.setText("Total");
			for(int c = 1; c < tabledata.colcount(); c++){
				XWPFTableCell cell = table.getRows().get(row).getTableCells().get(c);
				cell.getCTTc().addNewTcPr().addNewTcW().setW(BigInteger.valueOf(2000));
				XWPFParagraph cp = cell.getParagraphs().get(0);
				cp.setAlignment(ParagraphAlignment.RIGHT);
				Cell currentCell = tabledata.totals.cells[(c-1)];
				String text = formatNumber(((tabledata.options & Report.OPTION_COUNT)== Report.OPTION_COUNT) ? currentCell.count : currentCell.value, tabledata.options);
				cp.createRun().setText(text/*formatNumber(tabledata.totals.cells[(c-1)].count, tabledata.options)*/);
			}
			row++;

		}

		for(XWPFTableRow xrow : table.getRows()){
			for(XWPFTableCell xcell : xrow.getTableCells()){
				setCellStyle(xcell);
			}
		}
		/*
		table.getRow(0).getCell(0).setText("EXAMPLE OF TABLE");
		table.getRow(0).getCell(1).setText("cell 2");
		table.getRow(0).getCell(2).setText("cell 3");
		XWPFParagraph p1 = table.getRow(1).getCell(0).getParagraphs().get(0);
		XWPFRun r1 = p1.createRun();
		r1.setText("The quick brown fox");
		r1.setTextPosition(100);
		table.getRow(2).getCell(2).setText("only text");
		 */
	}



	private void setCellStyle(XWPFTableCell xcell) {
		CTTcPr cttcpr = xcell.getCTTc().getTcPr();

		if(cttcpr == null){
			CTTcPr newTcPr = xcell.getCTTc().addNewTcPr();
			CTTcBorders newTcBorders = newTcPr.addNewTcBorders();
			createBorders(newTcBorders);
		}
		else if(cttcpr.getTcBorders() == null){
			CTTcBorders tcBorders = cttcpr.addNewTcBorders();
			createBorders(tcBorders);
		}
		else {
			CTTcBorders tcBorders = cttcpr.getTcBorders();

			CTBorder bottom = tcBorders.getBottom();
			CTBorder left = tcBorders.getLeft();
			CTBorder right = tcBorders.getRight();
			CTBorder top = tcBorders.getTop();
		}
	}

	private void createBorders(CTTcBorders newTcBorders) {
		CTBorder bottom = null;
		CTBorder top = null;
		CTBorder right = null;
		CTBorder left = null;

		if(newTcBorders.getBottom() == null){
			bottom = newTcBorders.addNewBottom();			
		}
		else{
			bottom = newTcBorders.getBottom();
		}

		if(newTcBorders.getLeft() == null){
			left = newTcBorders.addNewLeft();			
		}
		else{
			left = newTcBorders.getLeft();			
		}

		if(newTcBorders.getRight() == null){
			right = newTcBorders.addNewRight();
		}
		else{
			right = newTcBorders.getRight();
		}

		if(newTcBorders.getTop() == null){
			top = newTcBorders.addNewTop();
		}
		else{
			top = newTcBorders.getTop();
		}

		setBorderStyle(bottom);
		setBorderStyle(left);
		setBorderStyle(right);
		setBorderStyle(top);
	}

	private void setBorderStyle(CTBorder border) {
		if(border != null){
			border.setColor("ffffff");
			border.setSz(BigInteger.valueOf(6));
			border.setSpace(BigInteger.ZERO);
			border.setVal(STBorder.SINGLE);
		}else{
			System.err.println("CTBorder not defined");
		}
	}



	final int renderwidth = 1500;//800;

	public synchronized void appendChart(ChartData chartdata, int options) {
		
		String taskid = "chart_" + UUID.randomUUID().toString();

		try{
			BufferedWriter wr = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(exportdirectory + "/" + taskid)));
			//wr.append(chartdata.getHighchartConfig());
			
			wr.append(chartdata.getC3Config(taskid));
			
			wr.flush();
			wr.close();

			Runtime rt = Runtime.getRuntime();
			
			Process p = rt.exec("/usr/local/lib/node_modules/phantomjs/lib/phantom/bin/phantomjs render.js " + taskid,
					null,
					new File(exportdirectory));
			
			
			/*
			Process p = rt.exec(phantomjs + " highcharts-convert.js -infile "+ taskid + " -outfile "+ taskid + ".png -scale 3 -width " + renderwidth,
					null,
					new File(exportdirectory));
			*/
			BufferedReader reader = new BufferedReader(new InputStreamReader(p.getInputStream()));

			String line;
			while ((line = reader.readLine()) != null) System.out.println("phantomjs: " + line);
			p.waitFor();

			appendImage(new FileInputStream(exportdirectory + "/" + taskid + ".png"), chartdata.width/2, chartdata.height/2);

			FileUtils.deleteQuietly(new File(exportdirectory + "/" + taskid));
			FileUtils.deleteQuietly(new File(exportdirectory + "/" + taskid + ".png"));
		}
		catch(Exception ex){
			ex.printStackTrace();
		}
	}

	@Override
	public void appendImage(InputStream is, int options) {	
	}

	/*
	public synchronized void appendChart(String chartdata, int width, int height) {
		String taskid = "chart_" + UUID.randomUUID().toString();

		try{
			BufferedWriter wr = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(exportdirectory + "/" + taskid)));
			wr.append(chartdata);
			wr.flush();
			wr.close();

			Runtime rt = Runtime.getRuntime();
			Process p = rt.exec(phantomjs + " highcharts-convert.js -infile "+ taskid + " -outfile "+ taskid + ".png -scale 2 -width " + renderwidth,
					null,
					new File(exportdirectory));

			BufferedReader reader = new BufferedReader(new InputStreamReader(p.getInputStream()));

			String line;
			while ((line = reader.readLine()) != null) System.out.println("phantomjs: " + line);
			p.waitFor();

			appendImage(new FileInputStream(exportdirectory + "/" + taskid + ".png"), width/2, height/2);

			FileUtils.deleteQuietly(new File(exportdirectory + "/" + taskid));
			FileUtils.deleteQuietly(new File(exportdirectory + "/" + taskid + ".png"));
		}
		catch(Exception ex){
			ex.printStackTrace();
		}
	}
	*/


	public synchronized void appendImage(InputStream ins, int width, int height) {
		XWPFParagraph p = doc.createParagraph();
		XWPFRun r = p.createRun();
		//r.setText("chart.png");
		r.addBreak();
		try {
			r.addPicture(ins, XWPFDocument.PICTURE_TYPE_PNG, "chart.png",
					Units.toEMU(width), Units.toEMU(height));
		} 
		catch (Exception e) {
			e.printStackTrace();
		}
	}

	@Override
	public void appendMissingInfo(MissingData data, int options) {
				
		XWPFTable table = doc.createTable(3, 2);
		
		XWPFParagraph p11 = table.getRow(0).getCell(0).getParagraphs().get(0);
		p11.createRun().setText(Messages.get().ReportPage_MissingCases);

		XWPFParagraph p12 = table.getRow(0).getCell(1).getParagraphs().get(0);
		p12.createRun().setText(formatNumber(data.missing, Report.OPTION_NONE));

		XWPFParagraph p21 = table.getRow(1).getCell(0).getParagraphs().get(0);
		p21.createRun().setText(Messages.get().ReportPage_TotalAnswers);

		XWPFParagraph p22 = table.getRow(1).getCell(1).getParagraphs().get(0);
		p22.createRun().setText(formatNumber(data.responses, Report.OPTION_NONE));

		XWPFParagraph p31 = table.getRow(2).getCell(0).getParagraphs().get(0);
		p31.createRun().setText(Messages.get().ReportPage_QuestionType);

		XWPFParagraph p32 = table.getRow(2).getCell(1).getParagraphs().get(0);
		p32.createRun().setText(data.type);
		
		for (XWPFTableRow row: table.getRows()) {
			for(XWPFTableCell cell : row.getTableCells()){
				cell.getParagraphs().get(0).getRuns().get(0).setItalic(true);
				setCellStyle(cell);
			}
		}
	}

	public void write(OutputStream os){
		try {
			doc.write(os);
			os.close();
		} 
		catch (IOException e) {
			e.printStackTrace();
		}
	}


	public static void main(String[] args) throws Exception {
		int n = OPTION_TOTALS | OPTION_MEAN;
		System.out.println("totals -> " + ((n & OPTION_TOTALS) == OPTION_TOTALS));
		System.out.println("mean -> " + ((n & OPTION_MEAN) == OPTION_MEAN));
		System.out.println("count -> " + ((n & OPTION_COUNT) == OPTION_COUNT));

		for(int i = 0; i< 10; i++){
			System.out.println(i + " -> " +  (i & OPTION_TOTALS));
		}
		/*
		POIDocument d = new POIDocument();
		d.appendHead("sfsfsfsd f");

		TableData td = new TableData(TableData.TYPE_CV_PERCENT);
		td.addRow("xxx", 1.0);
		td.addRow("yyyy", 3.2);
		d.appendTable(td, Document.OPTION_TOTALS);

		d.appendChart(
				"{ chart: { type: 'line', width: 400, height: 250 }, " +	
						"xAxis:{categories: ['1','2,','3','4','5','6', '7', '8','9','10','11','12'] },"+
						"series:[{ data: [29.9, 71.5, 106.4, 129.2, 144.0, 176.0, 135.6, 148.5, 216.4, 194.1, 95.6, 54.4]}]"+
			"};", 400, 250);

		d.appendFoot("Question: gglpo s okos sdfsf okfo s?");

		d.write(new FileOutputStream("/home/maks/java/qtafi/simple.docx"));
		 */
	}

}
