/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.export;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import de.eoda.qtafi.surveymanager.rap.report.MissingData;
import de.eoda.qtafi.surveymanager.rap.report.ReportItem;

public class XLSXDocument implements Report {

	Workbook doc;
	Sheet sheet;
	int row = 0;
	
	
	public XLSXDocument() {
		doc = new XSSFWorkbook();
		sheet = doc.createSheet();
	}
	
	protected double formatNumber(double num, int options){
		if(Double.isNaN(num)) return 0.;
		if(Double.isInfinite(num)) return 0.;
		return num;
	}
	
	
	@Override
	public void appendHead(String tit, String... strings) {
		Row sheetrow = sheet.createRow(row);
		int ce = 0;
		Cell sheetcell = sheetrow.createCell(ce++);
		sheetcell.setCellValue(tit);
		
		for(String s: strings){
			Cell sheetcelll = sheetrow.createCell(ce++);
			sheetcelll.setCellValue(s);
		}
		
	}

	@Override
	public void appendHead(ReportItem item, String... strings) {
		Row sheetrow = sheet.createRow(row++);
		int ce = 0;
		Cell sheetcell = sheetrow.createCell(ce++);
		sheetcell.setCellValue(item.title);
		
		for(String s: strings){
			Cell sheetcelll = sheetrow.createCell(ce++);
			sheetcelll.setCellValue(s);
		}
	}

	@Override
	public void appendSubHead(String tit, String... strings) {
		Row sheetrow = sheet.createRow(row++);
		int ce = 0;
		Cell sheetcell = sheetrow.createCell(ce++);
		sheetcell.setCellValue(tit);
		
		for(String s: strings){
			Cell sheetcelll = sheetrow.createCell(ce++);
			sheetcelll.setCellValue(s);
		}
	}

	@Override
	public void appendFoot(String foot, String... strings) {
	}

	@Override
	public void appendTable(TabularReportData tabledata, int options) {
		if(tabledata.tableHead){
			Row sheetrow = sheet.createRow(row++);
			
			Cell labelcell = sheetrow.createCell(0);
			labelcell.setCellValue("");
			
			int i = 0;
			for(String col: tabledata.colnames){
				Cell sheetcell = sheetrow.createCell(i+1);
				sheetcell.setCellValue(col);
				i++;
			}
		}
		
		
		for(int r = 0; r < tabledata.rowcount(); r++){
			Row sheetrow = sheet.createRow(row++);
			
			Cell labelcell = sheetrow.createCell(0);
			labelcell.setCellValue(tabledata.rows.get(r).label);
			
			for(int c = 1; c < tabledata.colcount(); c++){
				Cell sheetcell = sheetrow.createCell(c);
				if((options & Report.OPTION_COUNT) == Report.OPTION_COUNT){
					sheetcell.setCellValue(formatNumber(tabledata.rows.get(r).cells[(c-1)].count, tabledata.options));
				}
				else if((options & Report.OPTION_PERCENT) == Report.OPTION_PERCENT){
					sheetcell.setCellValue(formatNumber(tabledata.rows.get(r).cells[(c-1)].value, tabledata.options));
				}
				else if((options & Report.OPTION_MEAN) == Report.OPTION_MEAN){
					sheetcell.setCellValue(formatNumber(tabledata.rows.get(r).cells[(c-1)].value, tabledata.options));
				}
			}
		}
		
		if((options & Report.OPTION_TOTALS) == Report.OPTION_TOTALS){
			Row sheetrow = sheet.createRow(row++);
			Cell labelcell = sheetrow.createCell(0);
			labelcell.setCellValue("Total");
			
			if(tabledata.tableHead){
				int i = 0;
				for(String col: tabledata.colnames){
					if((options & Report.OPTION_COUNT) == Report.OPTION_COUNT){
						Cell cell = sheetrow.createCell(i+1);
						cell.setCellValue(formatNumber(tabledata.totals.cells[i].count, Report.OPTION_COUNT));
					}
					else if((options & Report.OPTION_PERCENT) == Report.OPTION_PERCENT){
						Cell cell = sheetrow.createCell(i+1);
						cell.setCellValue(formatNumber(tabledata.totals.cells[i].value, Report.OPTION_PERCENT));
					}
					i++;
				}
			}
			else {
				if((options & Report.OPTION_COUNT) == Report.OPTION_COUNT){
					Cell cell = sheetrow.createCell(1);
					cell.setCellValue(formatNumber(tabledata.totals.cells[0].count, Report.OPTION_COUNT));
				}
				else if((options & Report.OPTION_PERCENT) == Report.OPTION_PERCENT){
					Cell cell = sheetrow.createCell(1);
					cell.setCellValue(formatNumber(tabledata.totals.cells[0].value, Report.OPTION_PERCENT));
				}
			}
		}
		Row sheetrow = sheet.createRow(row++);
	}

	@Override
	public void appendChart(ChartData data, int options) {
	}

	@Override
	public void appendImage(InputStream is, int options) {
	}

	@Override
	public void appendMissingInfo(MissingData data, int options) {
		
	}

	public void write(OutputStream os){
		try {
			doc.write(os);
			os.close();
		} 
		catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	
}
