/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.pages;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStream;

import org.apache.commons.io.IOUtils;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.subject.Subject;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.rap.rwt.RWT;
import org.eclipse.rap.rwt.client.service.JavaScriptExecutor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import de.eoda.qtafi.surveymanager.auth.Account;
import de.eoda.qtafi.surveymanager.lang.Messages;
import de.eoda.qtafi.surveymanager.model.Breadcrumb;
import de.eoda.qtafi.surveymanager.rap.UIPage;
import de.eoda.qtafi.surveymanager.rap.UIPageContribution;
import de.eoda.qtafi.surveymanager.util.DataDownloadService;
import de.eoda.qtafi.surveymanager.util.DataDownloadService.DownloadCallback;
import de.eoda.qtafi.surveymanager.util.DbHelpers;


public class AccountContribution implements UIPageContribution {

	private static final long serialVersionUID = 1L;

	public class PasswordChangeDialog extends Dialog {

		private static final long serialVersionUID = 1L;
		
		Text password1, password2;
		String password = null;

		protected PasswordChangeDialog(Shell parentShell) {
			super(parentShell);
		}

		protected void configureShell(Shell shell) {
			super.configureShell(shell);
			shell.setText("Change password");
		}
		
		@Override
		protected Control createDialogArea(final Composite parent) {
			Composite container = (Composite) super.createDialogArea(parent);
			container.setLayout(new GridLayout(2, true));
			
			Label label1 = new Label(container, SWT.NONE);
			label1.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			label1.setText(Messages.get().Dialog_EditUser_Password);
			
			password1 = new Text(container, SWT.PASSWORD | SWT.BORDER);
			password1.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			
			Label label2 = new Label(container, SWT.NONE);
			label2.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			label2.setText("Retype password");
			
			password2 = new Text(container, SWT.PASSWORD | SWT.BORDER);
			password2.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			
			
			return container;
		}
		
		@Override
		protected void okPressed() {
			if(password1.getText().equals(password2.getText())){
				if(password1.getText().length() >= 5){
					this.password = password1.getText();
				}
			}
			
			super.okPressed();
		}
		
		
		public String getPassword(){
			return password;
		}
	}

	
	

	@Override
	public String getId() {
		return "account";
	}

	@Override
	public String getTitle() {
		return "Account";
	}
	
	@Override
	public UIPage createPage() {
		return new UIPage() {
			
			Shell shell;
			
			@Override
			public Breadcrumb[] getCrumbs() {
				return new Breadcrumb[]{ new Breadcrumb("Account", "account") };
			}
			
			@Override
			public void dispose() {	
			}
			
			private void addDownload(Composite parent, final String version){
				Group release = new Group(parent, SWT.NONE);
				release.setLayout(new GridLayout(1, false));
				GridData releaselayout = new GridData();
				releaselayout.grabExcessHorizontalSpace = true;
				releaselayout.horizontalAlignment = GridData.FILL;
				releaselayout.heightHint = 150;
				release.setLayoutData(releaselayout);
				release.setText("Release " + version);
				
				Link desktopApp = new Link(release, SWT.NONE);
				desktopApp.setData(RWT.MARKUP_ENABLED, Boolean.TRUE);
				desktopApp.setText("<a>Download windows desktop application</a>");
				desktopApp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
				
				desktopApp.addSelectionListener(new SelectionAdapter(){
					final String appdownload = "release_"+RWT.getRequest().getSession().getId();
					
					@Override
					public void widgetSelected(SelectionEvent e) {
						DataDownloadService.getInstance().addDownload(appdownload, new DownloadCallback() {
							@Override
							public String filename() {
								return "desktop_" + version + ".zip";
							}
							@Override
							public void service(OutputStream os) {
								File f = new File("/home/martin/Projekte/qtafi/release/" + version + "/desktop.zip");
								try {
									IOUtils.copy(new FileInputStream(f), os);
								} 
								catch (IOException e) {
									e.printStackTrace();
								} 
							}
						});
						StringBuilder url = new StringBuilder();
						url.append(RWT.getServiceManager().getServiceHandlerUrl(DataDownloadService.DOWNLOAD_SERVICE));
						JavaScriptExecutor executor = RWT.getClient().getService(JavaScriptExecutor.class);
						if(executor != null) {
							url.append('&').append(DataDownloadService.DATA_ID).append('=').append(appdownload);
							executor.execute("window.location.href='" + url.toString() + "'");
						}
					}
				});
				
				Link serverApp = new Link(release, SWT.NONE);
				serverApp.setData(RWT.MARKUP_ENABLED, Boolean.TRUE);
				serverApp.setText("<a>Download server application</a>");
				serverApp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
				serverApp.addSelectionListener(new SelectionAdapter(){
					final String appdownload = "serverrelease_"+RWT.getRequest().getSession().getId();
					@Override
					public void widgetSelected(SelectionEvent e) {
						DataDownloadService.getInstance().addDownload(appdownload, new DownloadCallback() {
							@Override
							public String filename() {
								return "server_" + version + ".zip";
							}
							
							@Override
							public void service(OutputStream os) {
								File f = new File("/home/martin/Projekte/qtafi/release/" + version + "/server.zip");
								try {
									IOUtils.copy(new FileInputStream(f), os);
								} 
								catch (IOException e) {
									e.printStackTrace();
								} 
							}
						});
						StringBuilder url = new StringBuilder();
						url.append(RWT.getServiceManager().getServiceHandlerUrl(DataDownloadService.DOWNLOAD_SERVICE));
						JavaScriptExecutor executor = RWT.getClient().getService(JavaScriptExecutor.class);
						if(executor != null) {
							url.append('&').append(DataDownloadService.DATA_ID).append('=').append(appdownload);
							executor.execute("window.location.href='" + url.toString() + "'");
						}
					}
				});
				
				Link src = new Link(release, SWT.NONE);
				src.setData(RWT.MARKUP_ENABLED, Boolean.TRUE);
				src.setText("<a>Download source code</a>");
				src.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
				src.addSelectionListener(new SelectionAdapter(){
					final String appdownload = "srcrelease_"+RWT.getRequest().getSession().getId();
					@Override
					public void widgetSelected(SelectionEvent e) {
						DataDownloadService.getInstance().addDownload(appdownload, new DownloadCallback() {
							@Override
							public String filename() {
								return "src_" + version + ".zip";
							}
							
							@Override
							public void service(OutputStream os) {
								File f = new File("/home/martin/Projekte/qtafi/release/" + version + "/src.zip");
								try {
									IOUtils.copy(new FileInputStream(f), os);
								} 
								catch (IOException e) {
									e.printStackTrace();
								} 
							}
						});
						StringBuilder url = new StringBuilder();
						url.append(RWT.getServiceManager().getServiceHandlerUrl(DataDownloadService.DOWNLOAD_SERVICE));
						JavaScriptExecutor executor = RWT.getClient().getService(JavaScriptExecutor.class);
						if(executor != null) {
							url.append('&').append(DataDownloadService.DATA_ID).append('=').append(appdownload);
							executor.execute("window.location.href='" + url.toString() + "'");
						}
					}
				});
			}
			
			
			@Override
			public void createControl(final Composite parent, String args) {
				parent.setLayout(new GridLayout(1, true));
				final Subject subject = SecurityUtils.getSubject();
				final Account acc = (Account) subject.getPrincipal();
				
				Label accountname = new Label(parent, SWT.NONE);
				accountname.setText("<span style=\"font-size:18pt;\">Account name: " + acc.name + (acc.isAdministrator()? " <small>(admin)</small>":"") + "</span>");
				accountname.setData(RWT.MARKUP_ENABLED, true);
		
				
				Button changePassword = new Button(parent, SWT.PUSH);
				changePassword.setText("Change password");
				changePassword.addSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(SelectionEvent e) {
						PasswordChangeDialog pwsd = new PasswordChangeDialog(parent.getShell());
						int res = pwsd.open();
						if(res == Dialog.OK){
							String p = pwsd.getPassword();
							if(p != null){
								DbHelpers.AccountSetPassword(acc, p);
							}
						}
					}
				});
				
				
				Button button = new Button(parent, SWT.PUSH);
				button.setText("Logout");
				button.addSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(SelectionEvent e) {
						System.out.println("logout " + subject);
						if(subject.isAuthenticated() )	subject.logout();
						
						JavaScriptExecutor executor = RWT.getClient().getService(JavaScriptExecutor.class);
						System.out.println(executor);
						executor.execute("window.location.href = '/admin/manager#login'");
					}
				});
				
				if(acc.isAdministrator()){
					File releases = new File("/home/martin/Projekte/qtafi/release/");
					if(releases.exists() && releases.isDirectory()){
						for(File rel: releases.listFiles()){
							if(rel.isDirectory()){
								addDownload(parent, rel.getName());
							}
						}
					}
				}
			}
		};
	}

	@Override
	public String getIdentifier() {
		return null;
	}
	

}
