/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.pages;



import java.util.ArrayList;

import org.apache.shiro.SecurityUtils;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.rap.rwt.RWT;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Text;

import de.eoda.qtafi.surveymanager.auth.Account;
import de.eoda.qtafi.surveymanager.lang.Messages;
import de.eoda.qtafi.surveymanager.model.Breadcrumb;
import de.eoda.qtafi.surveymanager.model.Project;
import de.eoda.qtafi.surveymanager.model.ProjectInfo;
import de.eoda.qtafi.surveymanager.rap.UIPage;
import de.eoda.qtafi.surveymanager.rap.UIPageContribution;
import de.eoda.qtafi.surveymanager.rap.UIPages;
import de.eoda.qtafi.surveymanager.util.DbHelpers;
import de.eoda.qtafi.surveymanager.util.Resources;

public class AdministrationContribution implements UIPageContribution {

	private static final long serialVersionUID = 1L;

	@Override
	public String getId() {
		return "administration";
	}

	@Override
	public String getTitle() {
		return Messages.get().Admin_Title;
	}

	@Override
	public String getIdentifier() {
		return getId();
	}

	@Override
	public UIPage createPage() {
		return new UIPage() {
			
			java.util.List<ProjectInfo> allprojects;
			java.util.List<Account> allaccounts;
			List projects, accounts;
			Button removeProject;
			String accountFilter = "", projectFilter = "";
			
			@Override
			public Breadcrumb[] getCrumbs() {
				return new Breadcrumb[]{ new Breadcrumb(Messages.get().Admin_Title, "administration") };
			}
			
			@Override
			public void dispose() {}
			
			@Override
			public void createControl(final Composite parent, String args) {
				Account account = (Account) SecurityUtils.getSubject().getPrincipal();
				if(! account.isAdministrator()){
					MessageDialog.openError(parent.getShell(), Messages.get().Admin_Error_Access_Title, Messages.get().Admin_Error_Access);
					return;
				}
				
				parent.setLayout(new GridLayout(2, true));
				
				{
					Label accountslabel = new Label(parent, SWT.NONE);
					accountslabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
					accountslabel.setData(RWT.MARKUP_ENABLED, true);
					accountslabel.setText("<span style=\"font-size:18pt;\">"+Messages.get().Admin_Account + "</span>");
					
					Label projectslabel = new Label(parent, SWT.NONE);
					projectslabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
					projectslabel.setData(RWT.MARKUP_ENABLED, true);
					projectslabel.setText("<span style=\"font-size:18pt;\">" + Messages.get().Admin_Projects + "</span>");
				}
				{
					Composite accountActions = new Composite(parent, SWT.NONE);
					accountActions.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
					accountActions.setLayout(new GridLayout(2, false));
					
					Label accountFilterLabel = new Label(accountActions, SWT.NONE);
					accountFilterLabel.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0348-filter.png"));
					//accountFilterLabel.setText("Filter");
					
					final Text accountFilterInput = new Text(accountActions, SWT.BORDER);
					accountFilterInput.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
					accountFilterInput.addModifyListener(new ModifyListener() {
						@Override
						public void modifyText(ModifyEvent me) {
							accountFilter = accountFilterInput.getText();
							loadAccounts();
						}
					});
					
					Composite projectActions = new Composite(parent, SWT.NONE);
					projectActions.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
					projectActions.setLayout(new GridLayout(2, false));
					
					Label projectFilterLabel = new Label(projectActions, SWT.NONE);
					projectFilterLabel.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0348-filter.png"));
					//projectFilterLabel.setText("Filter");
					
					final Text projectFilterInput = new Text(projectActions, SWT.BORDER);
					projectFilterInput.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
					projectFilterInput.addModifyListener(new ModifyListener() {
						@Override
						public void modifyText(ModifyEvent me) {
							projectFilter = projectFilterInput.getText();
							loadProjects();
						}
					});
				}
				
				{
					accounts = new List(parent, SWT.BORDER | SWT.V_SCROLL);
					GridData toplayout = new GridData(GridData.FILL_BOTH);
					toplayout.verticalAlignment = GridData.BEGINNING;
					//toplayout.heightHint = 200;
					toplayout.grabExcessVerticalSpace = true;
					toplayout.minimumHeight = 200;
					accounts.setLayoutData(toplayout);
					accounts.addSelectionListener(new SelectionAdapter() {
						@Override
						public void widgetSelected(SelectionEvent e) {
						}
					});
					
					accounts.addMouseListener(new MouseAdapter() {
						@Override
						public void mouseDoubleClick(MouseEvent e) {
							if(accounts.getSelectionCount() > 0){
								Account selected = getAccountByName(accounts.getSelection()[0]);
								EditAccountDialog editUser = new EditAccountDialog(parent.getShell(), selected);
								int res = editUser.open();
								if(res == Dialog.OK){
									if(editUser.getPassword().length() > 4){
										DbHelpers.AccountSetPassword(selected, editUser.getPassword());
									}
									DbHelpers.AccountUpdate(editUser.getAccount());
								}	
							}
						}
					});
					loadAccounts();
				}
				{
					projects = new List(parent, SWT.BORDER | SWT.V_SCROLL);
					GridData toplayout = new GridData(GridData.FILL_BOTH);
					toplayout.verticalAlignment = GridData.BEGINNING;
					//toplayout.heightHint = 200;
					toplayout.grabExcessVerticalSpace = true;
					toplayout.minimumHeight = 200;
					projects.setLayoutData(toplayout);
					
					projects.addSelectionListener(new SelectionAdapter() {
						@Override
						public void widgetSelected(SelectionEvent e) {
							removeProject.setEnabled(projects.getSelectionCount() > 0);
						}
					});
					projects.addMouseListener(new MouseAdapter() {
						@Override
						public void mouseDoubleClick(MouseEvent e) {
							
							if(projects.getSelectionCount() > 0){
								ProjectInfo selected = getProjectByName(projects.getSelection()[0]);
								ProjectInfoDialog editUser = new ProjectInfoDialog(parent.getShell(), selected);
								int res = editUser.open();
							}
						}
					});
					
					loadProjects();
				}
				{
					
					Composite accountActions = new Composite(parent, SWT.NONE);
					accountActions.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
					accountActions.setLayout(new GridLayout(1, false));
					
					Button addAccount = new Button(accountActions, SWT.PUSH);
					addAccount.setText(Messages.get().Admin_AddAccount);
					addAccount.addSelectionListener(new SelectionAdapter() {
						@Override
						public void widgetSelected(SelectionEvent e) {
							EditAccountDialog createUser = new EditAccountDialog(parent.getShell(), null);
							int res = createUser.open();
							if(res == Dialog.OK){
								if(! DbHelpers.AccountNameExists(createUser.getName())){
									DbHelpers.AccountCreate(createUser.getName(), createUser.getPassword(), createUser.getAccount().isAdministrator());
									loadAccounts();
								}
								else {
									MessageDialog.openError(parent.getShell(), Messages.get().Admin_Exists_Error_Title, 
											Messages.get().Admin_Exists_Error_Pre + " " + createUser.getName() + Messages.get().Admin_Exists_Error_Suf);
								}
							}
						}
					});
				}
				{
					
					Composite projectActions = new Composite(parent, SWT.NONE);
					projectActions.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
					projectActions.setLayout(new GridLayout(2, false));
				
					Button addProject = new Button(projectActions, SWT.PUSH);
					addProject.setText(Messages.get().Admin_CreateProject);
					addProject.addSelectionListener(new SelectionAdapter() {
						@Override
						public void widgetSelected(SelectionEvent e) {
							CreateProjectDialog projectDialog = new CreateProjectDialog(parent.getShell());
			    			int res = projectDialog.open();
			    			if(res == Dialog.OK){
			    				Project project = projectDialog.getProject();
			    				DbHelpers.ProjectCreate(project);
			    				loadProjects();
			    			}
						}
					});
					
					removeProject = new Button(projectActions, SWT.PUSH);
					removeProject.setEnabled(false);
					removeProject.setText(Messages.get().Admin_RemoveProject);
					removeProject.addSelectionListener(new SelectionAdapter() {
						@Override
						public void widgetSelected(SelectionEvent e) {
							if(projects.getSelectionCount() > 0){
								String projectToRemove = projects.getSelection()[0];
								ProjectInfo pr = getProject(projectToRemove);
								if(pr != null){
									synchronized (this) {
										//make dump to resources and drop
										DbHelpers.ResponseDumpTable(pr.name);
										DbHelpers.ResponseDropTable(pr.name);
										
										//dump log??
										DbHelpers.LogDropTable(pr.name);
										
										//dump users and drop
										DbHelpers.UsersDumpTable(pr.name);
										DbHelpers.UsersDropTable(pr.name);
										
										//create zip of resources in archiv
										String zipfile = Resources.ZIPResources(pr.name, Resources.Archivpath() + "/" + pr.name + "_" + System.currentTimeMillis() + ".zip");
										
										//Remove from database and assingment
										DbHelpers.ProjectRemove(pr);
										
										//Remove folder
										Resources.Remove(pr.name);
										
										//reload list
										loadProjects();
									}
								}
							}
						}
					});
				}
			}
			
			Account getAccountByName(String acc){
				for(Account a: allaccounts){
					if(a.name.equals(acc)) return a;
				}
				return null;
			}
			
			ProjectInfo getProjectByName(String ps){
				for(ProjectInfo a: allprojects){
					if(a.name.equals(ps)) return a;
				}
				return null;
			}
			
			void loadProjects(){
				allprojects = DbHelpers.ProjectsAll();
				java.util.List<String> ps = new ArrayList<String>();
				for(int i = 0; i < allprojects.size(); i++){
					if(projectFilter.trim().length() == 0){
						ps.add(allprojects.get(i).name);
					}
					else if(allprojects.get(i).name.toLowerCase().contains(projectFilter.toLowerCase())){
						ps.add(allprojects.get(i).name);
					}
				}
				//String[] prjs = new String[allprojects.size()];
				projects.setItems(ps.toArray(new String[]{}));
			}
			
			ProjectInfo getProject(String name){
				for(ProjectInfo i: allprojects){
					if(i.name.equals(name)) return i;
				}
				return null;
			}
			
			void loadAccounts(){
				allaccounts = DbHelpers.AccountsAll();
				java.util.List<String> as = new ArrayList<String>();
				for(int i = 0; i < allaccounts.size(); i++){
					if(accountFilter.trim().length() == 0){
						as.add(allaccounts.get(i).name);
					}
					else if(allaccounts.get(i).name.toLowerCase().contains(accountFilter.toLowerCase())){
						as.add(allaccounts.get(i).name);
					}
				}
				accounts.setItems(as.toArray(new String[]{}));
			}
	
		};
	}
}
