/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.pages;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;

import de.eoda.qtafi.surveymanager.auth.Account;
import de.eoda.qtafi.surveymanager.lang.Messages;
import de.eoda.qtafi.surveymanager.model.Project;
import de.eoda.qtafi.surveymanager.model.ProjectInfo;
import de.eoda.qtafi.surveymanager.model.Role;
import de.eoda.qtafi.surveymanager.util.DbHelpers;

public class AssignProjectDialog extends Dialog {

	private static final long serialVersionUID = 1L;
	final Account account;
	
	ProjectInfo project = null;
	Role role = null;
	
	protected AssignProjectDialog(Shell parentShell, Account account) {
		super(parentShell);
		this.account = account;
	}

	
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setText(Messages.get().Dialog_AssignProject_Title);
	}
	
	
	@Override
	protected Control createDialogArea(final Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
	
		container.setLayout(new GridLayout(2, false));
		
		{
			Label listl = new Label(container, SWT.NONE);
			GridData labellayout = new GridData();
			labellayout.widthHint = 100;
			labellayout.verticalAlignment = GridData.BEGINNING;
			labellayout.horizontalAlignment = GridData.FILL;
			
			listl.setLayoutData(labellayout);
			listl.setText(Messages.get().Dialog_AssignProject_Project);
			
			
			final java.util.List<ProjectInfo> allprojects = DbHelpers.ProjectsAll();
			java.util.List<Project> alreadyasigned = DbHelpers.ProjectsForAccount(account);
			
			
			final List projectlist = new List(container, SWT.BORDER | SWT.V_SCROLL);
			GridData listlayout = new GridData();
			listlayout.horizontalAlignment = GridData.FILL;
			listlayout.heightHint = 200;
			listlayout.widthHint = 300;
			listlayout.verticalAlignment = GridData.BEGINNING;
			projectlist.setLayoutData(listlayout);
			
			String[] listitems = new String[allprojects.size() - alreadyasigned.size()];
	
			for(int i = 0, n = 0; i < allprojects.size() ; i++) {
				String pn = allprojects.get(i).name;
				if(! contains(alreadyasigned, pn)){
					listitems[n] = pn;
					n++;
				}
			}
			projectlist.setItems(listitems);
			projectlist.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					String[] sel = projectlist.getSelection();
					if(sel.length > 0){
						String na = sel[0];
						for(ProjectInfo i : allprojects){
							if(i.name.equals(na)) {
								project = i;
							}
						}
					}
					else project = null;
					canClickOK();
				}
			});
		}
		{
			Label rolel = new Label(container, SWT.NONE);
			GridData labellayout = new GridData();
			labellayout.widthHint = 100;
			labellayout.verticalAlignment = GridData.BEGINNING;
			labellayout.horizontalAlignment = GridData.FILL;
			rolel.setLayoutData(labellayout);
			rolel.setText(Messages.get().Dialog_AssignProject_Role);
		
			final java.util.List<Role> allroles = DbHelpers.RoleAll();
			final Combo roleslist = new Combo(container, SWT.READ_ONLY);
			GridData rolelayout = new GridData();
			rolelayout.horizontalAlignment = GridData.FILL;
			rolelayout.widthHint = 300;
			rolelayout.verticalAlignment = GridData.BEGINNING;
			roleslist.setLayoutData(rolelayout);
			
			String[] roleitems = new String[allroles.size()+1];
			roleitems[0] = Messages.get().Dialog_AssignProject_Select;
			for(int i = 0; i < allroles.size() ; i++) roleitems[i+1] = allroles.get(i).name;
			roleslist.setItems(roleitems);
			roleslist.select(0);
			
			roleslist.addSelectionListener(new SelectionAdapter(){
				@Override
				public void widgetSelected(SelectionEvent e) {
					int sel = roleslist.getSelectionIndex();
					if(sel > 0){
						role = allroles.get(sel - 1);
					}
					else role = null;
					canClickOK();
				}
			});
		}
		
		return container;	
	}
	
	
	boolean contains(java.util.List<Project> alreadyasigned, String name){
		for(Project p : alreadyasigned){
			if(p.name.equals(name)) return true;
		}
		return false;
	}
	
	void canClickOK(){
		if(project != null && role != null)
			getButton(IDialogConstants.OK_ID).setEnabled(true);
		else 
			getButton(IDialogConstants.OK_ID).setEnabled(false);
	}
	
	
	@Override
	protected Control createButtonBar(Composite parent) {
		Control bb =  super.createButtonBar(parent);
		getButton(IDialogConstants.OK_ID).setEnabled(false);
		return bb;
	}
	
	@Override
	protected void okPressed() {
		if(account != null && project != null && role != null){
			DbHelpers.AssignProject(account, project, role);
		}
		super.okPressed();
	}
	
	
	@Override
	protected Point getInitialSize() {
		return new Point(420, 400);
	}
	
}
