/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.pages;

import java.util.Date;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import de.eoda.qtafi.surveymanager.lang.Messages;
import de.eoda.qtafi.surveymanager.model.Project;
import de.eoda.qtafi.surveymanager.util.DbHelpers;

/**
 * 
 * Check whether project already exists
 * 
 * 
 * @author martin
 *
 */
public class CreateProjectDialog extends Dialog {

	
	private static final long serialVersionUID = 1L;
	Text input;
	String name = null;
	
	protected CreateProjectDialog(Shell parentShell) {
		super(parentShell);
	}
	
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setText(Messages.get().Dialog_CreateProject_Title);
	}
	
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		container.setLayout(new GridLayout());
		
		Label projectname = new Label(container, SWT.NONE);
		projectname.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		projectname.setText(Messages.get().Dialog_CreateProject_ProjectName);
		
		input = new Text(container, SWT.BORDER );
		input.addModifyListener(new ModifyListener() {
			
			@Override
			public void modifyText(ModifyEvent arg0) {
				getButton(IDialogConstants.OK_ID).setEnabled(validName());
			}
		});
		
		input.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		return container;
	}
	
	@Override
	protected Control createButtonBar(Composite parent) {
		Control bb =  super.createButtonBar(parent);
		getButton(IDialogConstants.OK_ID).setEnabled(false);
		return bb;
	}
	
	public Project getProject(){
		Project project = new Project(-1, name);
		project.startdate = new Date();
		project.enddate = new Date();
		project.authenticationmode = "OPEN";
		project.logmode = "OFF";
		return project;
	}
	
	boolean validName(){
		String name = input.getText();
		if(name.length() < 3) return false;
		if(name.contains(" ")) return false;
		if(name.contains("?")) return false;
		if(name.contains("#")) return false;
		return !DbHelpers.ProjectExists(name);
	}
	
	
	@Override
	protected void okPressed() {
		name = input.getText();
		super.okPressed();
	}
	
	
	@Override
	protected Point getInitialSize() {
		return new Point(450, 300);
	}
}
