/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.pages;


import java.io.OutputStream;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.rap.rwt.RWT;
import org.eclipse.rap.rwt.client.service.JavaScriptExecutor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

import de.eoda.qtafi.surveymanager.lang.Messages;
import de.eoda.qtafi.surveymanager.model.Breadcrumb;
import de.eoda.qtafi.surveymanager.model.Project;
import de.eoda.qtafi.surveymanager.model.Role;
import de.eoda.qtafi.surveymanager.rap.UIPage;
import de.eoda.qtafi.surveymanager.rap.UIPageContribution;
import de.eoda.qtafi.surveymanager.rap.UIPages;
import de.eoda.qtafi.surveymanager.util.DataDownloadService;
import de.eoda.qtafi.surveymanager.util.DataDownloadService.DownloadCallback;
import de.eoda.qtafi.surveymanager.util.DbColumn;
import de.eoda.qtafi.surveymanager.util.DbHelpers;
import de.eoda.qtafi.surveymanager.util.DbRow;
import de.eoda.qtafi.surveymanager.util.SessionResource;

public class DataViewContribution implements UIPageContribution {
	
	
	private static final long serialVersionUID = 1L;
	
	@Override
	public String getId() {
		return "data";
	}

	@Override
	public String getTitle() {
		return Messages.get().DataView_Title;
	}

	@Override
	public String getIdentifier() {
		return "data";
	}

	@Override
	public UIPage createPage() {
		return new UIPage() {
			
			private static final long serialVersionUID = 1L;
			Project project;
			Table tableview;
			Button order;
			Composite parent;
			List<DbColumn> cols;
			
			@Override
			public Breadcrumb[] getCrumbs() {
				String pn = SessionResource.Project().name;
				return new Breadcrumb[]{ 
						new Breadcrumb(Messages.get().DataView_Project + " " + pn , "project|" + project.name), 
						new Breadcrumb(Messages.get().DataView_ResponseData, "data") };
			}
			
			@Override
			public void dispose() {}
			
			@Override
			public void createControl(final Composite parent, String args) {
				this.parent = parent;
				//ServiceManager manager = RWT.getServiceManager();
				project = SessionResource.Project();
				parent.setLayout(new GridLayout(2, false));
				
				{
					Composite toolbar = new Composite(parent, SWT.BORDER);
					GridData toolbardata = new GridData();
					toolbardata.widthHint = 200;
					
					toolbardata.grabExcessVerticalSpace = true;
					toolbardata.verticalAlignment = GridData.FILL;
					toolbar.setLayoutData(toolbardata);
					toolbar.setLayout(new GridLayout(1, true));
					
					order = new Button(toolbar, SWT.TOGGLE);
					order.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0333-sort-amount-asc.png"));
					order.setText("Order by questionnaire");
					
					order.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
					order.addSelectionListener(new SelectionAdapter() {
						@Override
						public void widgetSelected(SelectionEvent e) {
							setTableData(tableview);
						}
					});
					
					Button export = new Button(toolbar, SWT.PUSH);
					export.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0097-download.png"));
					export.setText(Messages.get().DataView_Button_Export);
					export.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
					export.addSelectionListener(new SelectionAdapter() {
						
						@Override
						public void widgetSelected(SelectionEvent e) {
							final boolean exportorder = order.getSelection();
							
							StringBuilder url = new StringBuilder();
							DataDownloadService.getInstance().addDownload("data" + RWT.getRequest().getSession().getId(), new DownloadCallback() {
								
								@Override
								public String filename() {
									return "data.csv";
								}
								
								@Override
								public void service(OutputStream os) {
									DbHelpers.ResponseExport(project.name, os, exportorder);
								}
							});
							url.append(RWT.getServiceManager().getServiceHandlerUrl(DataDownloadService.DOWNLOAD_SERVICE));
							JavaScriptExecutor executor = RWT.getClient().getService(JavaScriptExecutor.class);
							if(executor != null) {
								url.append('&').append(DataDownloadService.DATA_ID).append('=').append("data" + RWT.getRequest().getSession().getId());
								executor.execute("window.location.href='" + url.toString() + "'");
							}
						}
					});
					
					Button delete = new Button(toolbar, SWT.PUSH);
					delete.setText(Messages.get().DataView_Button_Delect);
					if(! project.asRole.equals(Role.AS_ADMIN)){
						delete.setEnabled(false);
					}
					delete.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0270-cancel-circle.png"));
					delete.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
					delete.addSelectionListener(new SelectionAdapter() {
						@Override
						public void widgetSelected(SelectionEvent e) {
							if(! project.asRole.equals(Role.AS_ADMIN)) return;
							boolean res = MessageDialog.openQuestion(parent.getShell(), Messages.get().DataView_ErrorBox_Title, Messages.get().DataView_ErrorBox_Text);
							if(res){
								DbHelpers.ResponseDropTable(project.name);
								tableview.setItemCount(DbHelpers.ResponseCount(project.name));
							}
						}
					});
				}
				
				ScrolledComposite scrolled = new ScrolledComposite(parent, SWT.H_SCROLL | SWT.V_SCROLL);
				GridData tablelayout = new GridData();
				tablelayout.grabExcessHorizontalSpace = true;
				tablelayout.grabExcessVerticalSpace = true;
				tablelayout.horizontalAlignment = GridData.FILL;
				tablelayout.verticalAlignment = GridData.FILL;
				scrolled.setLayoutData(tablelayout);
				scrolled.setExpandHorizontal(true);
				scrolled.setExpandVertical(true);
				
				tableview = new Table(scrolled, SWT.VIRTUAL | SWT.BORDER);
				tableview.setHeaderVisible(true);
				tableview.setLinesVisible(true);
				tableview.setData(RWT.MARKUP_ENABLED, true);
				scrolled.setMinSize(tableview.computeSize(SWT.DEFAULT, SWT.DEFAULT));
				tableview.setItemCount(DbHelpers.ResponseCount(project.name));
				tableview.addListener (SWT.SetData, new Listener () {
					public void handleEvent (Event event) {
						TableItem item = (TableItem) event.item;
				        int index = tableview.indexOf (item);
				        DbRow row = DbHelpers.ResponseRow(SessionResource.Project().name, cols, index+1);
				        if(row != null) item.setText(row.content);
					}
				}); 
				
				setTableData(tableview);
				scrolled.setContent(tableview);
			}
			
			void setTableData(final Table tableview){
				cols = DbHelpers.ResponseColumns(project.name, order.getSelection());
				for(TableColumn tc: tableview.getColumns()) tc.dispose();
				
				for(final DbColumn col: cols){
					final TableColumn column = new TableColumn(tableview, SWT.NONE);
					column.setText(col.name);
					column.setWidth(100);
					column.addSelectionListener(new SelectionAdapter() {
						@Override
						public void widgetSelected(SelectionEvent e) {
							VariableFrequencyDialog dialog = new VariableFrequencyDialog(parent.getShell(), project.name, col.name, VariableFrequencyDialog.TYPE_RESPONSE_DATA);
							dialog.open();
						}
					});
				}
				tableview.setItemCount(0);
				tableview.redraw();
				tableview.setItemCount(DbHelpers.ResponseCount(project.name));
			}
		};
	}

}
