/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.pages;

import java.util.List;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;

import de.eoda.qtafi.surveymanager.auth.Account;
import de.eoda.qtafi.surveymanager.lang.Messages;
import de.eoda.qtafi.surveymanager.model.Project;
import de.eoda.qtafi.surveymanager.rap.UIPages;
import de.eoda.qtafi.surveymanager.util.DbHelpers;

public class EditAccountDialog extends Dialog {

	
	private static final long serialVersionUID = 1L;
	String name;
	String password;
	Text namefield, passwordfield, passwordfield2;
	Button admin;
	Account account;
	Table projects;
	
	public EditAccountDialog(Shell shell, Account account) {
		super(shell);
		this.account = account;
		if(this.account == null) this.account = new Account(-1, "");
	}
	

	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		if(this.account.id < 0) shell.setText(Messages.get().Dialog_EditUser_Create);
		else shell.setText(Messages.get().Dialog_EditUser_Account + " " + this.account.name);
	}
	
	
	@Override
	protected Control createDialogArea(final Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
	
		container.setLayout(new GridLayout(2, false));
		
		Label namel = new Label(container, SWT.NONE);
		namel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		namel.setText("Name");
		
		if(account.id >= 0) namefield = new Text(container, SWT.READ_ONLY | SWT.BORDER);
		else namefield = new Text(container, SWT.BORDER);
		namefield.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		if(account != null) namefield.setText(account.name);
		
		{	
			Label assignl = new Label(container, SWT.NONE);
			assignl.setLayoutData(new GridData());
			assignl.setText(Messages.get().Dialog_EditUser_AssignedProjects);
			
			projects = new Table(container, SWT.BORDER);
			if(account.id < 0) projects.setEnabled(false);
			GridData griddata = new GridData();
			griddata.horizontalAlignment = GridData.FILL;
			griddata.verticalAlignment = GridData.FILL;
			griddata.heightHint = 200;
			
			projects.setLayoutData(griddata);
			projects.setLinesVisible(true);
			projects.setHeaderVisible(true);
			TableColumn name = new TableColumn(projects, SWT.None);
			name.setText(Messages.get().Dialog_EditUser_Name);
			name.setWidth(300);
			
			TableColumn role = new TableColumn(projects, SWT.None);
			role.setText(Messages.get().Dialog_EditUser_Role);
			role.setWidth(100);
			
			TableColumn remove = new TableColumn(projects, SWT.None);
			remove.setText(Messages.get().Dialog_EditUser_Remove);
			remove.setWidth(100);
			
			loadProjects();
			
			
			Label empty = new Label(container, SWT.NONE);
			empty.setLayoutData(new GridData());
			
			Button add = new Button(container, SWT.PUSH);
			if(account.id < 0) add.setEnabled(false);
			add.setLayoutData(new GridData());
			add.setText(Messages.get().Dialog_AssignProject_Title);
			add.addSelectionListener(new SelectionAdapter(){
				@Override
				public void widgetSelected(SelectionEvent e) {
					AssignProjectDialog apd = new AssignProjectDialog(parent.getShell(), account);
					int res = apd.open();
					if(res == Dialog.OK) {
						loadProjects();
					}
				}
			});
		}	
		{
			Label passwordl = new Label(container, SWT.NONE);
			passwordl.setText(Messages.get().Dialog_EditUser_Password);
			passwordl.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			
			passwordfield = new Text(container, SWT.BORDER | SWT.PASSWORD);
			passwordfield.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			
			Label passwordl2 = new Label(container, SWT.NONE);
			passwordl2.setText(Messages.get().Dialog_EditUser_RetypePassword);
			passwordl2.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			
			passwordfield2 = new Text(container, SWT.BORDER | SWT.PASSWORD);
			passwordfield2.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		}
		{
			Label adminl = new Label(container, SWT.NONE);
			adminl.setText("Administrator");
			adminl.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			
			admin = new Button(container, SWT.CHECK);
			admin.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			admin.setSelection(account.isAdministrator());
		}
		return container;
	}
	
	void loadProjects(){
		for(TableItem t: projects.getItems()) {
			TableEditor te = (TableEditor) t.getData();
			te.getEditor().dispose();
			te.dispose();
			t.dispose();
		}
		
		List<Project> ps =  DbHelpers.ProjectsForAccount(account);
		for(final Project p: ps){
			TableItem tableItem = new TableItem(projects, SWT.NONE);
			tableItem.setText(new String[]{p.name, p.asRole});
			{
				TableEditor te = new TableEditor(projects);
				te.grabHorizontal = true;
				te.grabVertical = true;
				
				tableItem.setData(te);
				
				Button bb = new Button(projects, SWT.PUSH);
				bb.setImage(UIPages.loadImage(this.getParentShell().getDisplay() , "resources/icons/0173-bin.png"));
				bb.computeSize(SWT.DEFAULT, projects.getItemHeight() - 3);
				
				bb.addSelectionListener(new SelectionAdapter(){
					@Override
					public void widgetSelected(SelectionEvent e) {
						DbHelpers.UnassignProject(account, p);
						loadProjects();
					}
				});
				
				te.setEditor(bb, tableItem, 2);
			}
		}
	}
	
	@Override
	protected void okPressed() {
		this.name = namefield.getText().trim();
		account.setAdministrator(admin.getSelection());
		this.password = "";
		if(this.passwordfield.getText().trim().length() > 0){
			if(this.passwordfield.getText().equals(this.passwordfield2.getText())){
				this.password = passwordfield.getText().trim();
			}
			else {
				MessageDialog.openError(getShell(), Messages.get().Dialog_EditUser_Error_Title, Messages.get().Dialog_EditUser_Error_Text);
			}
		}
		super.okPressed();
	}
	
	public String getName(){
		return this.name;
	}
	
	public Account getAccount(){
		return account;
	}
	
	public String getPassword(){
		return password;
	}
	
	@Override
	protected Point getInitialSize() {
		return new Point(750, 650);
	}
	
}
