/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.pages;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.Writer;
import java.util.Enumeration;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.io.IOUtils;
import org.apache.shiro.SecurityUtils;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.rap.fileupload.DiskFileUploadReceiver;
import org.eclipse.rap.fileupload.FileDetails;
import org.eclipse.rap.fileupload.FileUploadEvent;
import org.eclipse.rap.fileupload.FileUploadHandler;
import org.eclipse.rap.fileupload.FileUploadListener;
import org.eclipse.rap.rwt.RWT;
import org.eclipse.rap.rwt.client.service.JavaScriptExecutor;
import org.eclipse.rap.rwt.service.ServerPushSession;
import org.eclipse.rap.rwt.service.ServiceHandler;
import org.eclipse.rap.rwt.widgets.FileUpload;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSource;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.DragSourceListener;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

import de.eoda.qtafi.surveymanager.auth.Account;
import de.eoda.qtafi.surveymanager.dnd.CustomTextTransfer;
import de.eoda.qtafi.surveymanager.lang.Messages;
import de.eoda.qtafi.surveymanager.model.Breadcrumb;
import de.eoda.qtafi.surveymanager.model.LibQuestion;
import de.eoda.qtafi.surveymanager.model.Project;
import de.eoda.qtafi.surveymanager.model.Role;
import de.eoda.qtafi.surveymanager.rap.UIPage;
import de.eoda.qtafi.surveymanager.rap.UIPages;
import de.eoda.qtafi.surveymanager.rap.composites.ClickListener;
import de.eoda.qtafi.surveymanager.rap.composites.EditorHTMLComposite;
import de.eoda.qtafi.surveymanager.util.DataDownloadService;
import de.eoda.qtafi.surveymanager.util.DataDownloadService.DownloadCallback;
import de.eoda.qtafi.surveymanager.util.DbHelpers;
import de.eoda.qtafi.surveymanager.util.LibraryUtils;
import de.eoda.qtafi.surveymanager.util.Resources;
import de.eoda.qtafi.surveymanager.util.SessionResource;
import de.tinu.qtafi4.db.QuestionDB;
import de.tinu.qtafi4.model.Question;
import de.tinu.qtafi4.model.QuestionnaireOptions;
import de.tinu.qtafi4.model.UIDInfo;
import de.tinu.qtafi4.tree.PageNode;
import de.tinu.qtafi4.tree.QuestionNode;
import de.tinu.qtafi4.tree.TreeNode;

public class EditorPage implements UIPage {

	private static final long serialVersionUID = 1L;
	Project project;
	private EditorHTMLComposite html;
	private TreeNode root;
	private QuestionTree questionnaire;
	private Composite parent;

	private Tree tools;
	TreeItem libraryNode;
	TreeItem dropNode;
	
	private String fileName;
	
	boolean readonly = false;
	
	String previewHandler;

	static class NewQuestionNode {
		public final String label;
		public final String type;

		public NewQuestionNode(String type, String label) {
			this.type = type;
			this.label = label;
		}
	}

	public static Transfer[] types = new Transfer[] { CustomTextTransfer.getInstance() };


	private NewQuestionNode[] question_types = { 
		new NewQuestionNode("HL", Messages.get().Editor_Questions_HL_long), 
		new NewQuestionNode("CV", Messages.get().Editor_Questions_CV_long), 
		new NewQuestionNode("MD", Messages.get().Editor_Questions_MD_long), 
		new NewQuestionNode("OR", Messages.get().Editor_Questions_OR_long),
		new NewQuestionNode("TE", Messages.get().Editor_Questions_TE_long), 
		new NewQuestionNode("ME", Messages.get().Editor_Questions_ME_long), 
		new NewQuestionNode("DATE", Messages.get().Editor_AdvancedQuestions_DATE_long) };


	private NewQuestionNode[] advanced_types = { 
		new NewQuestionNode("IMAGE", Messages.get().Editor_AdvancedQuestions_IMG_long), 
		new NewQuestionNode("NavigationButton", Messages.get().Editor_AdvancedQuestions_Nav_long), 
		new NewQuestionNode("ProgressBar", Messages.get().Editor_AdvancedQuestions_Prog_long),
		new NewQuestionNode("LOGIN", Messages.get().Editor_AdvancedQuestions_Log_long), 
		new NewQuestionNode("XHTML", Messages.get().Editor_AdvancedQuestions_HTML_long), 
		new NewQuestionNode("RAW", Messages.get().Editor_AdvancedQuestions_RAW_long),
		new NewQuestionNode("LANG", Messages.get().Editor_AdvancedQuestions_LANG_long) };
	

	public EditorPage() {
	}
	
	@Override
	public Breadcrumb[] getCrumbs() {
		String pn = SessionResource.Project().name;
		return new Breadcrumb[]{ 
				new Breadcrumb(Messages.get().Account_ProjectLinkPrefix + " " + pn , "project|" + SessionResource.Project().name),
				new Breadcrumb(Messages.get().Editor_Title + " " + this.fileName , "editor|" + this.fileName)};
	}
	
	@Override
	public void dispose() { 
		RWT.getServiceManager().unregisterServiceHandler(previewHandler);
	}
	
	@Override
	public void createControl(final Composite parent, String args) {
		this.fileName = args;
		this.parent = parent;
		this.parent.setData(RWT.CUSTOM_VARIANT, "bg-light");
		
		previewHandler = "preview_" + RWT.getRequest().getSession().getId();
		
		RWT.getServiceManager().unregisterServiceHandler(previewHandler);
		RWT.getServiceManager().registerServiceHandler(previewHandler, new ServiceHandler() {
			
			@Override
			public void service(HttpServletRequest request, HttpServletResponse response)
					throws IOException, ServletException {
				
				QuestionnaireOptions options = SessionResource.QuestionDB(EditorPage.this.fileName).getOptions();
				response.setCharacterEncoding("utf-8");
				Writer wr = response.getWriter();
				wr.append("<html dir=\""+ options.pagedir + "\"><head>");
				wr.append("<title>Questionnaire preview</title>");
				wr.append("<meta http-equiv=\"content-type\" content=\"text/html; charset=UTF-8\"></meta>");
				wr.append("<script type=\"text/javascript\" src=\"rwt-resources/resources/jquery.js\" ></script>");
				wr.append("<script type=\"text/javascript\" src=\"rwt-resources/resources/bootstrap.js\" ></script>");
				wr.append("<script type=\"text/javascript\" src=\"rwt-resources/resources/bootstrap-datepicker.js\" ></script>");
				if(options.pagedir.equals("rtl")){
					wr.append("<link type=\"text/css\" href=\"rwt-resources/resources/css/bootstrap.css\" rel=\"stylesheet\"></link>");
				}
				else wr.append("<link type=\"text/css\" href=\"rwt-resources/resources/css/bootstrap.css\" rel=\"stylesheet\"></link>");
				wr.append("<link type=\"text/css\" href=\"rwt-resources/resources/css/bootstrap-datepicker.css\" rel=\"stylesheet\"></link>");
				
				
				File f = Resources.ProjectResource(project.name, "style.css");
				if(f.exists()){
					wr.append("<style type=\"text/css\">\n");
					BufferedReader reader = new BufferedReader(new InputStreamReader(new FileInputStream(f)));
					String line = null;
					while((line = reader.readLine()) != null){
						wr.append(line + "\n");
					}
					reader.close();
					wr.append("</style>\n");
				}
				
				
				wr.append("<style type=\"text/css\">\n");
				wr.append("body * { font-size: 0.7em important!; }\n");
				//wr.append("body { font-size: 0.7em; }\n");
				wr.append(".question1 { margin-bottom:20px; }\n");
				wr.append(".onlypaper { visibility: visible; font-style:italic; }\n");
				wr.append("</style>");
				
				wr.append("</head><body>");
				StringBuffer result = new StringBuffer();
				InputStream file = null;
				if(options.pagedir.equals("rtl")){
					file = EditorPage.class.getResourceAsStream("/de/eoda/qtafi/surveymanager/rap/pages/main-rtl.xslt");
				}
				else file = EditorPage.class.getResourceAsStream("/de/eoda/qtafi/surveymanager/rap/pages/main.xslt");
				
				SessionResource.QuestionDB(EditorPage.this.fileName).transform(result, "/", file, "cid", RWT.getRequest().getParameter("cid"));
				wr.append(result.toString());
				wr.append("</body></html>");
				wr.flush();
				wr.close();
			}
		});
		
		
		parent.setLayout(new GridLayout());
		parent.setLayoutData(new GridData(GridData.FILL_BOTH));
		project = SessionResource.Project();
		if(project == null){
			MessageDialog.openError(parent.getShell(), Messages.get().Editor_Illegal_Access_Short, Messages.get().Editor_Illegal_Access_Long);
			return;
		}
		readonly = project.isOnline();
		root = SessionResource.QuestionDB(this.fileName).questionnaireInfo();

		SashForm mainForm = new SashForm(parent, SWT.HORIZONTAL);
		mainForm.setData(RWT.CUSTOM_VARIANT, "bg-light");
		mainForm.setLayoutData(new GridData(GridData.FILL_BOTH));

		createTooltree(mainForm);

		questionnaire = createQuestionnaireTree(mainForm);
		this.html = new EditorHTMLComposite(mainForm, SWT.BORDER);

		mainForm.setWeights(new int[] { 2, 2, 8 });
		
		updateQuestionnairePreview();
		
		this.html.addClickListener(new ClickListener() {
			
			@Override
			public void dblclick(String uid) {
				Question q = SessionResource.QuestionDB(EditorPage.this.fileName).get(new UIDInfo(uid));
				if(q != null) editQuestion(q);
			}
			
			@Override
			public void singleclick(String uid) {
				questionnaire.selectNode(uid);
			}
		});
	}
	
	//private void resetTreeLabelLang() {
	//}
	
	
	void createTooltree(SashForm mainForm) {
		//resetTreeLabelLang();
		SashForm toolSash = new SashForm(mainForm, SWT.VERTICAL);
		toolSash.setData(RWT.CUSTOM_VARIANT, "bg-light");
		toolSash.setLayout(new GridLayout());

		final ScrolledComposite toolScrollComp = new ScrolledComposite(toolSash, SWT.BORDER | SWT.V_SCROLL);
		toolScrollComp.setEnabled(true);
		toolScrollComp.setLayoutData( new GridData( GridData.FILL, GridData.FILL, true, true ) );
		
		final Composite toolComp = new Composite(toolScrollComp, SWT.NONE);
		toolComp.setLayout(new GridLayout());
		
		Button options = new Button(toolComp, SWT.PUSH);
		options.setText(Messages.get().Editor_Options);
		options.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0150-cogs.png"));
		options.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		options.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				QuestionnaireOptionsDialog qod = new QuestionnaireOptionsDialog(toolComp.getShell(), fileName);
				qod.open();
			}
		});
		
		
		Button export = new Button(toolComp, SWT.PUSH);
		export.setText(Messages.get().Editor_Export);
		export.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0097-download.png"));
		export.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		export.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				DataDownloadService.getInstance().addDownload("codebook" + RWT.getRequest().getSession().getId(), new DownloadCallback() {
					@Override
					public String filename() {
						return "codebook.xml";
					}

					@Override
					public void service(OutputStream os) {
						File f = Resources.ProjectResource(project.name, EditorPage.this.fileName);
						try {
							IOUtils.copy(new FileInputStream(f), os);
						} 
						catch (Exception e) {
							e.printStackTrace();
						}
					}
				});
				StringBuilder url = new StringBuilder();
				url.append(RWT.getServiceManager().getServiceHandlerUrl(DataDownloadService.DOWNLOAD_SERVICE));
				JavaScriptExecutor executor = RWT.getClient().getService(JavaScriptExecutor.class);
				
				System.out.println(url);
				String connectionId = RWT.getRequest().getParameter("cid");
				System.out.println("connectionId " + connectionId);
				
				for(Enumeration<String> en = RWT.getRequest().getSession().getAttributeNames(); en.hasMoreElements(); ){
					String name = en.nextElement();
					System.out.println("attribute " + name + " = " + RWT.getRequest().getSession().getAttribute(name));
				};
				
				
				if (executor != null) {
					url.append('&').append(DataDownloadService.DATA_ID).append('=').append("codebook" + RWT.getRequest().getSession().getId());
					executor.execute("window.location.href='" + url.toString() + "'");
				}
			}

		});

		final ServerPushSession pushSession = new ServerPushSession();
		final FileUpload importb = new FileUpload(toolComp, SWT.PUSH);
		
		importb.setEnabled(project.asRole.equals(Role.AS_ADMIN));
		importb.setText(Messages.get().Editor_Import);
		importb.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0098-upload.png"));
		importb.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		importb.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				pushSession.start();
				importb.submit(startUploadReceiver());
			}
		});
		
		Button copyfrom = new Button(toolComp, SWT.PUSH);
		if(EditorPage.this.fileName.equalsIgnoreCase("index.qml")){
			copyfrom.setEnabled(false);
		}
		copyfrom.setText(Messages.get().Editor_CopyFromDefault);
		copyfrom.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0046-paste.png"));
		copyfrom.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		copyfrom.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				try {
					File sourcefile = Resources.ProjectResource(project.name, "index.qml");
					File targetfile = Resources.ProjectResource(project.name, EditorPage.this.fileName);
					FileOutputStream output = new FileOutputStream(targetfile);
					IOUtils.copy(new FileInputStream(sourcefile.getAbsolutePath()), output);
					output.flush();
					output.close();
	
					Display.getDefault().asyncExec(new Runnable() {
						public void run() {
							root = SessionResource.QuestionDB(EditorPage.this.fileName).questionnaireInfo();
							questionnaire.setItems(root);
							updateQuestionnairePreview();
						}
					});
				}
				catch(Exception ex){
					ex.printStackTrace();
				}
			}
		});
		
		
/*
		Button trash = new Button(toolComp, SWT.PUSH);
		trash.setText(Messages.get().Editor_Trash);
		trash.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0173-bin.png"));
		trash.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		DropTarget target = new DropTarget(toolComp, DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_LINK);
		target.setTransfer(types);
		target.addDropListener(new DropTargetAdapter() {
			@Override
			public void dropAccept(DropTargetEvent event) {
			}

			@Override
			public void drop(DropTargetEvent event) {
				if (event.data.toString().startsWith("importQuestion_")){
					String id = event.data.toString();
					String[] xid = id.split("_");
					int intid = Integer.parseInt(xid[1]);
					LibraryUtils.DeleteQuestion(intid);
					updateLibrary();
					return;
				}
				if(readonly) return;
				if (event.data.toString().startsWith("newQuestion_")) return;
				String uid = event.data.toString();
				Question q = SessionResource.QuestionDB(EditorPage.this.fileName).get(new UIDInfo(uid));
				if(maySaveChange(q)){
					SessionResource.QuestionDB(EditorPage.this.fileName).delete(q);
					root = SessionResource.QuestionDB(EditorPage.this.fileName).questionnaireInfo();
					questionnaire.setItems(root);
					updateQuestionnairePreview();
				}
				else {
					MessageDialog.openError(parent.getShell(), Messages.get().Editor_Not_allowed, Messages.get().Editor_Not_allowed);
				}
			}
		});
		*/

		Button printable = new Button(toolComp, SWT.PUSH);
		printable.setText(Messages.get().Editor_Printable);
		printable.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0085-printer.png"));
		printable.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		printable.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				JavaScriptExecutor executor = RWT.getClient().getService(JavaScriptExecutor.class);
				String url = RWT.getServiceManager().getServiceHandlerUrl(previewHandler);
				
				if(executor != null) {
					executor.execute("window.open('"+url+"');");
				}
			}
		});
		
		
		Button appendPage = new Button(toolComp, SWT.PUSH);
		appendPage.setText(Messages.get().Editor_AppendPage);
		appendPage.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0361-page-break.png"));

		appendPage.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		appendPage.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				if(readonly) return;
				SessionResource.QuestionDB(fileName).appendPage();
				root = SessionResource.QuestionDB(fileName).questionnaireInfo();
				System.out.println("root::" + root);
				System.out.println("root:children::" + root.children.size());
				update(root.children.get(root.children.size() - 1));
			}
		});
		toolScrollComp.setContent(toolComp);
		toolScrollComp.setExpandHorizontal(true);
		toolScrollComp.setExpandVertical(true);
		toolScrollComp.setSize(toolComp.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		toolScrollComp.setMinSize(toolComp.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		
		toolScrollComp.addControlListener(new ControlAdapter() {
			@Override
			public void controlResized(ControlEvent e) {
				toolScrollComp.setMinSize(toolComp.computeSize(SWT.DEFAULT, SWT.DEFAULT));
			}
		});
		
		toolScrollComp.setOrigin(toolComp.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		
		tools = new Tree(toolSash, SWT.VERTICAL | SWT.BORDER);
		tools.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		tools.addMouseListener(new MouseAdapter(){
			@Override
			public void mouseDoubleClick(MouseEvent event) {
				if (tools.getSelectionCount() > 0 && tools.getSelection()[0].getData() != null) {
					String xuid = tools.getSelection()[0].getData().toString();
					String[] xid = xuid.split("_");
					if(xid.length == 2){
						try {
							int libid = Integer.parseInt(xid[1]);
							LibQuestion libq = LibraryUtils.GetQuestion(libid);
							LibraryQuestionDialog importdialog = new LibraryQuestionDialog(parent.getShell(), libq);
							int res = importdialog.open();
							if(res == Dialog.OK){
								Account acc = (Account) SecurityUtils.getSubject().getPrincipal();
								LibraryUtils.UpdateQuestion(libid, importdialog.getQuestion(), acc.id, importdialog.getPublicity());
								updateLibrary();
							}
						}
						catch(NumberFormatException ex){};
					}
				}
			}
		});
		
		
		TreeItem questionNode = new TreeItem(tools, SWT.NONE);
		questionNode.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0112-bubbles3.png"));
		questionNode.setText(Messages.get().Editor_Questions);

		final DragSource source = new DragSource(tools, DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_LINK);
		source.setTransfer(types);
		source.addDragListener(new DragSourceListener() {
			@Override
			public void dragStart(DragSourceEvent event) {
				event.doit = true;
			}

			@Override
			public void dragSetData(DragSourceEvent event) {
				if (tools.getSelectionCount() > 0) {
					event.data = tools.getSelection()[0].getData();
				} else {
					event.doit = false;
				}
			}

			@Override
			public void dragFinished(DragSourceEvent event) {
			}
		});

		for (final NewQuestionNode questiontype : question_types) {
			final TreeItem questionTi = new TreeItem(questionNode, SWT.BORDER);
			questionTi.setText(questiontype.label);
			questionTi.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0382-share.png"));
			questionTi.setData("newQuestion_" + questiontype.type);
		}

		TreeItem containerNode = new TreeItem(tools, SWT.NONE);
		containerNode.setText(Messages.get().Editor_AdvancedComponents);
		containerNode.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0190-menu.png"));
		for (final NewQuestionNode advancedtype: advanced_types) {
			TreeItem containerTi = new TreeItem(containerNode, SWT.NONE);
			containerTi.setText(advancedtype.label);
			containerTi.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0382-share.png"));
			containerTi.setData("newQuestion_"+advancedtype.type);
		}

		libraryNode = new TreeItem(tools, SWT.NONE);
		libraryNode.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0032-book.png"));
		libraryNode.setText(Messages.get().Editor_Library);
		libraryNode.setData("library");

		dropNode = new TreeItem(tools, SWT.NONE);
		dropNode.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0173-bin.png"));
		dropNode.setText(Messages.get().Editor_Trash);
		dropNode.setData("trash");
		
		DropTarget tooldroptarget = new DropTarget(tools, DND.DROP_COPY | DND.DROP_MOVE | DND.DROP_LINK);
		tooldroptarget.setTransfer(types);
		tooldroptarget.addDropListener(new DropTargetAdapter(){
			@Override
			public void dragEnter(DropTargetEvent event) {
				event.feedback = DND.FEEDBACK_EXPAND | DND.FEEDBACK_SELECT | DND.FEEDBACK_SCROLL;
			}

			@Override
			public void dragOver(DropTargetEvent event) {
				event.feedback = DND.FEEDBACK_EXPAND | DND.FEEDBACK_SCROLL;
				if (event.item != null) {
					TreeItem item = (TreeItem) event.item;
					Point pt = event.display.map(null, tools, event.x, event.y);
					Rectangle bounds = item.getBounds();
					if (pt.y < bounds.y + bounds.height / 3) {
						event.feedback |= DND.FEEDBACK_INSERT_BEFORE;
					} 
					else if (pt.y > bounds.y + 2 * bounds.height / 3) {
						event.feedback |= DND.FEEDBACK_INSERT_AFTER;
					} 
					else {
						event.feedback |= DND.FEEDBACK_SELECT;
					}
				}
			}
			
			public void drop(DropTargetEvent event) {
				if (event.item != null) {
					String dragtransfer = event.data.toString();
					TreeItem target = (TreeItem) event.item;
					if(target == null) return;
					if(target.getData() == null) return;
					
					Question im = SessionResource.QuestionDB(fileName).get(new UIDInfo(dragtransfer));
					if(target.getData().equals("library") && im != null){
						System.out.println("insert " + im);
						Account acc = (Account) SecurityUtils.getSubject().getPrincipal();
						
						LibraryQuestionDialog importdialog = new LibraryQuestionDialog(parent.getShell(), new LibQuestion(im, LibraryUtils.PRIVATE));
						int res = importdialog.open();
						if(res == Dialog.OK){
							LibraryUtils.InsertQuestion(importdialog.getQuestion(), acc.id, importdialog.getPublicity());
							updateLibrary();
						}
					}
					else if(target.getData().equals("trash")){
						if (event.data.toString().startsWith("importQuestion_")){
							String id = event.data.toString();
							String[] xid = id.split("_");
							int intid = Integer.parseInt(xid[1]);
							LibraryUtils.DeleteQuestion(intid);
							updateLibrary();
							return;
						}
						if(readonly) return;
						if (event.data.toString().startsWith("newQuestion_")) return;
						String uid = event.data.toString();
						Question q = SessionResource.QuestionDB(EditorPage.this.fileName).get(new UIDInfo(uid));
						if(maySaveChange(q)){
							SessionResource.QuestionDB(EditorPage.this.fileName).delete(q);
							root = SessionResource.QuestionDB(EditorPage.this.fileName).questionnaireInfo();
							questionnaire.setItems(root);
							updateQuestionnairePreview();
						}
						else {
							MessageDialog.openError(parent.getShell(), Messages.get().Editor_Not_allowed, Messages.get().Editor_Not_allowed);
						}
					}
					else { //trash
						if (event.data.toString().startsWith("importQuestion_")){
							String id = event.data.toString();
							String[] xid = id.split("_");
							int intid = Integer.parseInt(xid[1]);
							LibraryUtils.DeleteQuestion(intid);
							updateLibrary();
							return;
						}
						if(readonly) return;
						if (event.data.toString().startsWith("newQuestion_")) return;
						String uid = event.data.toString();
						Question q = SessionResource.QuestionDB(EditorPage.this.fileName).get(new UIDInfo(uid));
						if(maySaveChange(q)){
							SessionResource.QuestionDB(EditorPage.this.fileName).delete(q);
							root = SessionResource.QuestionDB(EditorPage.this.fileName).questionnaireInfo();
							questionnaire.setItems(root);
							updateQuestionnairePreview();
						}
						else {
							MessageDialog.openError(parent.getShell(), Messages.get().Editor_Not_allowed, Messages.get().Editor_Not_allowed);
						}
					}
				}
				else {
					if (event.data.toString().startsWith("importQuestion_")){
						String id = event.data.toString();
						String[] xid = id.split("_");
						int intid = Integer.parseInt(xid[1]);
						LibraryUtils.DeleteQuestion(intid);
						updateLibrary();
						return;
					}
					if(readonly) return;
					if (event.data.toString().startsWith("newQuestion_")) return;
					String uid = event.data.toString();
					Question q = SessionResource.QuestionDB(EditorPage.this.fileName).get(new UIDInfo(uid));
					if(maySaveChange(q)){
						SessionResource.QuestionDB(EditorPage.this.fileName).delete(q);
						root = SessionResource.QuestionDB(EditorPage.this.fileName).questionnaireInfo();
						questionnaire.setItems(root);
						updateQuestionnairePreview();
					}
					else {
						MessageDialog.openError(parent.getShell(), Messages.get().Editor_Not_allowed, Messages.get().Editor_Not_allowed);
					}
				}
			}
		});
		updateLibrary();
		
		toolSash.setWeights(new int[]{toolComp.computeSize(SWT.DEFAULT, SWT.DEFAULT).y, tools.getParent().computeSize(SWT.DEFAULT, SWT.DEFAULT).y});
	}

	void updateLibrary(){
		Account acc = (Account) SecurityUtils.getSubject().getPrincipal();
		//drop library leafs
		for(TreeItem t: libraryNode.getItems()){
			t.dispose();
		}
		libraryNode.removeAll();
		LibraryUtils.BuidTree(tools, libraryNode,  acc.id);
	}
	
	
	private QuestionTree createQuestionnaireTree(final Composite parent) {
		QuestionTree questionnaire = new QuestionTree(parent, SWT.BORDER);
		questionnaire.setItems(root);
		droppable(questionnaire.tree);

		questionnaire.tree.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				Tree t = (Tree) e.getSource();
				TreeItem[] tis = t.getSelection();
				if (tis.length > 0) {
					html.scrollTo(tis[0].getData().toString());
				}
			}
			
			@Override
			public void mouseDoubleClick(MouseEvent e) {
				Tree t = (Tree) e.getSource();
				TreeItem[] tis = t.getSelection();
				if (tis.length > 0) {
					Question q = SessionResource.QuestionDB(fileName).get(new UIDInfo(tis[0].getData().toString()));
					editQuestion(q);
				}
			}
		});
		return questionnaire;
	}

	void editQuestion(final Question q) {
		int protectedBy = q.getProtectedBy();
		QuestionEditorDialog qed = new QuestionEditorDialog(this.parent.getShell(), q.clone(), this.fileName, readonly);
		int status = qed.open();
		if(readonly) return;
		if (status == Dialog.OK) {
			if(maySaveChange(q)){
				if(protectedBy >= 0 && qed.question.isQuestionProtected()) qed.question.protect(protectedBy);
				SessionResource.QuestionDB(fileName).update(qed.question);
				root = SessionResource.QuestionDB(fileName).questionnaireInfo();
				update(SessionResource.QuestionDB(fileName).getTreeNode(root, q.uid));
				updateQuestionnairePreview();
			}
			else {
				MessageDialog.openError(this.parent.getShell(), "Not allowed", "You have not the rights to change the question!");
			}
		}
	}

	void update(TreeNode node) {
		questionnaire.setItems(root);
		questionnaire.updateItem(node);
		updateQuestionnairePreview();
	}

	void updateQuestionnairePreview() {
		StringBuffer result = new StringBuffer();
		InputStream file = EditorPage.class.getResourceAsStream("/de/eoda/qtafi/surveymanager/rap/pages/main.xslt");
		
		SessionResource.QuestionDB(this.fileName).transform(result, "/", file, "cid", RWT.getRequest().getParameter("cid"));
		html.setHtml(result.toString());
	}

	void draggable(Control c, final String eventContext) {
		DragSource ds = new DragSource(c, DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_LINK);
		ds.setTransfer(types);
		ds.addDragListener(new DragSourceListener() {
			@Override
			public void dragStart(DragSourceEvent event) {
				event.doit = true;
			}

			@Override
			public void dragSetData(DragSourceEvent event) {
				event.data = eventContext;
			}

			@Override
			public void dragFinished(DragSourceEvent event) {
			}
		});
	}

	void droppable(Tree c) {
		DropTarget target = new DropTarget(c, DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_LINK);
		target.setTransfer(types);
		target.addDropListener(new TreeDropListener(SessionResource.QuestionDB(this.fileName), c));
	}

	private class TreeDropListener extends DropTargetAdapter {
		
		private final Tree tree;
		QuestionDB questiondb;

		public TreeDropListener(QuestionDB q, Tree tree) {
			this.tree = tree;
			this.questiondb = q;
		}

		@Override
		public void dragEnter(DropTargetEvent event) {
			event.feedback = DND.FEEDBACK_EXPAND | DND.FEEDBACK_SELECT | DND.FEEDBACK_SCROLL;
		}

		@Override
		public void dragOver(DropTargetEvent event) {
			event.feedback = DND.FEEDBACK_EXPAND | DND.FEEDBACK_SCROLL;
			if (event.item != null) {
				TreeItem item = (TreeItem) event.item;
				Point pt = event.display.map(null, tree, event.x, event.y);
				Rectangle bounds = item.getBounds();
				if (pt.y < bounds.y + bounds.height / 3) {
					event.feedback |= DND.FEEDBACK_INSERT_BEFORE;
				} else if (pt.y > bounds.y + 2 * bounds.height / 3) {
					event.feedback |= DND.FEEDBACK_INSERT_AFTER;
				} else {
					event.feedback |= DND.FEEDBACK_SELECT;
				}
			}
		}

		@Override
		public void drop(DropTargetEvent event) {
			if(readonly) return;
			System.out.println("drop " + event + event.data);
			if (event.data == null || event.item == null) {
				event.detail = DND.DROP_NONE;
			} 
			else if (event.data.toString().startsWith("newQuestion")) {
				Object targetObject = event.item.getData();

				Question targetpage = questiondb.get(new UIDInfo(event.item.getData().toString()));
				if(maySaveChange(targetpage)){
					if(targetpage.type.equals("Page") || targetpage.type.equals("Head") || targetpage.type.equals("Foot")){
						String[] newq = event.data.toString().split("_");
						//System.out.println("target " + targetObject);
						Question q = questiondb.append(new UIDInfo(targetObject.toString()), newq[1]);
						root = questiondb.questionnaireInfo();
						TreeNode n = questiondb.getTreeNode(root, targetObject.toString());
						update(n);
						editQuestion(q);
					}
					else {
						//System.out.println("insertBefore");
						String[] newq = event.data.toString().split("_");
						//System.out.println("target " + targetObject);
						Question q = questiondb.insertBefore(new UIDInfo(targetObject.toString()), newq[1]);
						root = questiondb.questionnaireInfo();
						TreeNode n = questiondb.getTreeNode(root, targetObject.toString());
						update(n);
						editQuestion(q);
					}
				}
				else {
					MessageDialog.openError(parent.getShell(), Messages.get().Editor_Not_allowed, Messages.get().Editor_Not_allowed);
				}
			}
			else if(event.data.toString().startsWith("importQuestion_")){
				System.out.println("importquestion ");
				Object targetObject = event.item.getData();
				Question targetpage = questiondb.get(new UIDInfo(event.item.getData().toString()));
				String[] newq = event.data.toString().split("_");
				try{
					int libid = Integer.parseInt(newq[1]);
					LibQuestion libq = LibraryUtils.GetQuestion(libid);
					if(libq != null){
						if(maySaveChange(targetpage)){
							Question q = questiondb.append(new UIDInfo(targetObject.toString()), libq.question.type);
							q.set(libq.question);
							questiondb.update(q);
							root = questiondb.questionnaireInfo();
							TreeNode n = questiondb.getTreeNode(root, targetObject.toString());
							update(n);
							editQuestion(q);
						}
						else {
							MessageDialog.openError(parent.getShell(), Messages.get().Editor_Not_allowed, Messages.get().Editor_Not_allowed);
						}
					}
				}
				catch(NumberFormatException ex){ }
			}
			else if (event.data.equals("remove")) {
				System.out.println("remove question");
				Object targetObject = event.item.getData();
				System.out.println("target " + targetObject);
			}
			else {
				TreeNode oldparent = questiondb.getTreeNode(root, event.data.toString()).parent;
				TreeNode draggedNode = questiondb.getTreeNode(root, event.data.toString());
				TreeNode targetNode = questiondb.getTreeNode(root, event.item.getData().toString());

				if (isValidDrop(draggedNode, targetNode)) {
					Question draggedObject = questiondb.get(new UIDInfo(event.data.toString()));
					Question targetObject = questiondb.get(new UIDInfo(event.item.getData().toString()));

					if (draggedNode instanceof PageNode) {
						//System.out.println("Dragging page!");
						//System.out.println(targetNode);
						//System.out.println(targetNode.getLabel());
						
						questiondb.movePageBefore(new UIDInfo(targetObject.uid), new UIDInfo(draggedObject.uid));
						root = questiondb.questionnaireInfo();
						TreeNode newparent = questiondb.getTreeNode(root, targetObject.uid);
						update(newparent);
						update(oldparent);
						update(root.children.get(root.children.size() - 1));
					} 
					else {
						if (targetNode instanceof PageNode) {
							System.out.println("Move page");
							questiondb.moveInsideAsFirst(new UIDInfo(targetObject.uid), draggedObject);
							root = questiondb.questionnaireInfo();
							TreeNode newparent = questiondb.getTreeNode(root, targetObject.uid);
							update(newparent);
							update(oldparent);
							update(root.children.get(root.children.size() - 1));
						} 
						else if (targetNode instanceof QuestionNode) {
							System.out.println("Move question");
							questiondb.moveBehind(new UIDInfo(targetObject.uid), draggedObject);
							root = questiondb.questionnaireInfo();
							TreeNode newparent = questiondb.getTreeNode(root, targetObject.uid).parent;
							update(newparent);
							update(oldparent);
							update(root.children.get(root.children.size() - 1));
						}
					}
				}
			}
		}

		private boolean isValidDrop(TreeNode draggedObject, TreeNode targetObject) {
			boolean result = false;
			if (draggedObject != null && targetObject != null) {
				result = true;
				TreeNode current = targetObject;
				while (current != null && result) {
					result = current != draggedObject;
					current = current.parent;
				}
			}
			return result;
		}
	}

	private String startUploadReceiver() {
		final DiskFileUploadReceiver receiver = new DiskFileUploadReceiver();
		FileUploadHandler uploadHandler = new FileUploadHandler(receiver);
		uploadHandler.addUploadListener(new FileUploadListener() {
			public void uploadProgress(FileUploadEvent event) {
			}

			public void uploadFailed(FileUploadEvent event) {
				System.err.println("failed");
			}

			public void uploadFinished(FileUploadEvent event) {
				for(FileDetails file: event.getFileDetails()) {
					try {
						for(File target : receiver.getTargetFiles()) {
							if(target.getName().equals(file.getFileName())) {
								File newfile = Resources.ProjectResource(project.name, EditorPage.this.fileName);
								
								FileOutputStream output = new FileOutputStream(newfile);
								IOUtils.copy(new FileInputStream(target.getAbsolutePath()), output);
								output.flush();
								output.close();

								Display.getDefault().asyncExec(new Runnable() {
									public void run() {
										root = SessionResource.QuestionDB(EditorPage.this.fileName).questionnaireInfo();
										questionnaire.setItems(root);
										updateQuestionnairePreview();
									}
								});
							}
						}
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
			}
		});
		return uploadHandler.getUploadUrl();
	}
	
	
	
	/**
	 * protection strategy:
	 * 
	 * 1. not protected allow
	 * 2. if owner allow
	 * 3. 
	 * 
	 * @param q
	 * @return
	 */
	public final boolean maySaveChange(Question q){
		if(q == null) return false;
		if(! q.isQuestionProtected()){
			return true;
		}
		
		Account account = (Account) SecurityUtils.getSubject().getPrincipal();
		if(q.getProtectedBy() == account.id){
			return true;
		}
		
		int ownerid = q.getProtectedBy();
		//get role of owner and compare with current role
		int ownerrole = DbHelpers.AccountRoleById(ownerid, project.id).id;
		int currentrole = project.asRoleid;
		System.out.println("ownerrole " + ownerrole + " vs currentrole " + currentrole );
		if(ownerrole > currentrole) return true;
		
		/*
		if(SessionResource.Project().asRole.equals(Role.AS_ADMIN)){
			System.out.println("may save because role admin");
			return true;
		}
		*/
		
		
		//System.out.println("may not save!");
		return false;
	}

}
