/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.pages;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.rap.rwt.RWT;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import de.eoda.qtafi.surveymanager.lang.Messages;
import de.tinu.qtafi4.model.Filter;
import de.tinu.qtafi4.model.Question;
import de.tinu.qtafi4.model.Value;
import de.tinu.qtafi4.model.Variable;

public class FilterDialog extends Dialog {
	
	private static final long serialVersionUID = 1L;
	
	final Question question;
	final Filter filter;
	

	public FilterDialog(Shell parent, final Question q, final Filter filter) {
		super(parent);
		this.question = q;
		this.filter = filter;
		this.filter.ref = this.question.uid;
		this.filter.label = Messages.get().Dialog_Filter_Title;
	}
	
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setText(Messages.get().Dialog_Filter_Title);
	}
	
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		container.setLayout(new GridLayout(1, true));
		
		Label text = new Label(container, SWT.NONE);
		text.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		text.setData(RWT.MARKUP_ENABLED, true);
		text.setText(question.getCleanedText());
		
		ScrolledComposite scrolled = new ScrolledComposite(container, SWT.V_SCROLL);
		scrolled.setLayoutData(new GridData(GridData.FILL_BOTH));
		Composite content = new Composite(scrolled, SWT.NONE);
		content.setLayout(new GridLayout(1, true));
		if(question.type.equals("MD")){
			final String val = question.getMDValue();
			for(final Variable var: question.variables){
				Button button = new Button(content, SWT.RADIO);
				button.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
				button.setText(var.getCleanedLabel());
				if(filter.variable != null && filter.value != null){
					if(filter.variable.equals(var.name) && filter.value.equals(val)){
						button.setSelection(true);
					}
				}
				button.addSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(SelectionEvent e) {
						filter.value = val;
						filter.variable = var.name;
						filter.label = question.id + ": " + var.getCleanedLabel();
					}
				});
			}
		}
		if(question.type.equals("CV")){
			final String var = question.getCVVariable();
			for(final Value val: question.values){
				Button button = new Button(content, SWT.RADIO);
				button.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
				button.setText(val.getCleanedLabel());
				if(filter.variable != null && filter.value != null){
					if(filter.variable.equals(var) && filter.value.equals(val.key)){
						button.setSelection(true);
					}
				}
				button.addSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(SelectionEvent e) {
						filter.value = val.key;
						filter.variable = var;
						filter.label = question.id + ": " + val.getCleanedLabel();
					}
				});
			}
		}
		else {
			for(final Variable var: question.variables){
				for(final Value val : question.values){
					Button button = new Button(content, SWT.RADIO);
					button.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
					button.setText(var.getCleanedLabel() + " - " + val.getCleanedLabel());
					if(filter.variable != null && filter.value != null){
						if(filter.variable.equals(var.name) && filter.value.equals(val.key)){
							button.setSelection(true);
						}
					}
					
					button.addSelectionListener(new SelectionAdapter() {
						@Override
						public void widgetSelected(SelectionEvent e) {
							filter.value = val.key;
							filter.variable = var.name;
							filter.label = question.id + ": " + var.getCleanedLabel() + " - " + val.getCleanedLabel();
						}
					});
				}
			}
		}
		
		scrolled.setExpandHorizontal(true);
		scrolled.setExpandVertical(true);
		scrolled.setContent(content);
		Point pp = content.computeSize(SWT.DEFAULT, SWT.DEFAULT);
		pp.y += 100;
		scrolled.setMinSize(pp);
		
		return container;
	}
	
	@Override
	protected void okPressed() {
		super.okPressed();
	}
	
	public Filter getFilter(){
		return this.filter;
	}
	
	@Override
	protected Point getInitialSize() {
		return new Point(450, 500);
	}
}
