/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.pages;

import java.util.List;

import org.apache.shiro.SecurityUtils;
import org.apache.shiro.subject.Subject;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.rap.rwt.RWT;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import de.eoda.qtafi.surveymanager.lang.Messages;
import de.eoda.qtafi.surveymanager.auth.Account;
import de.eoda.qtafi.surveymanager.model.Breadcrumb;
import de.eoda.qtafi.surveymanager.model.Project;
import de.eoda.qtafi.surveymanager.rap.UIPage;
import de.eoda.qtafi.surveymanager.rap.UIPageContribution;
import de.eoda.qtafi.surveymanager.rap.UIPages;
import de.eoda.qtafi.surveymanager.util.DbHelpers;


public class HomeContribution implements UIPageContribution {

	private static final long serialVersionUID = 1L;
	
	@Override
	public String getId() {
		return "home";
	}

	@Override
	public String getTitle() {
		return Messages.get().Home_Title;
	}

	static class ProjectComposite extends Composite {

		private static final long serialVersionUID = 1L;

		public ProjectComposite(final Project project, Composite parent) {
			super(parent, SWT.BORDER);
			setLayout(new GridLayout(2, true));
			
			Label openproject = new Label(this, SWT.NONE);
			openproject.setData(RWT.MARKUP_ENABLED, true);
			FontData fd = openproject.getFont().getFontData()[0];
			fd.setHeight(22);
			openproject.setFont(new Font(Display.getCurrent(), fd));
			openproject.setText("<a href=\"/admin/manager#project|"+ project.name+"\">" + Messages.get().Project_Title_Prefix + " " + project.name + "</a>");
			
			GridData linkdata = new GridData();
			linkdata.verticalSpan = 3;
			linkdata.grabExcessHorizontalSpace = true;
			openproject.setLayoutData(linkdata);
			
			{
				Label status = new Label(this, SWT.NONE);
				
				String statustext = Messages.get().Home_Online;
				//if(project.isFinished()) statustext = "terminated";
				//if(project.isSetup()) statustext = "setup"; 
				if(! project.isOnline()) statustext = Messages.get().Home_Offline; 
				
				status.setText(Messages.get().Account_Status + ": " + statustext);
				GridData statusdata = new GridData();
				statusdata.grabExcessHorizontalSpace = true;
				statusdata.horizontalAlignment = GridData.END;
				status.setLayoutData(statusdata);
			}
			
			{
				final Label responses = new Label(this, SWT.NONE);
				responses.setText(Messages.get().Project_Responses + ": --" );
				GridData responsesdata = new GridData();
				responsesdata.grabExcessHorizontalSpace = true;
				responsesdata.horizontalAlignment = GridData.END;
				responses.setLayoutData(responsesdata);
				final Display display = Display.getCurrent();
				
				//UISession uiSession = RWT.getUISession(display);
				//isplay.getCurrent().asyncExec(new Runnable() {
				//	public void run() {
						int count = DbHelpers.ResponseCount(project.name);
				    	responses.setText(Messages.get().Project_Responses + ": " + count);
				  //  }
				//});  
			}
			
			{
				final Label role = new Label(this, SWT.NONE);
				role.setText(Messages.get().Home_AsRole + ": " + project.asRole);
				GridData roledata = new GridData();
				roledata.grabExcessHorizontalSpace = true;
				roledata.horizontalAlignment = GridData.END;
				role.setLayoutData(roledata);
			}
		}
	}
	
	
	@Override
	public UIPage createPage() {
		return new UIPage() {
			
			String projectfilter = "";
			
			Shell shell;
			Composite projectscontainer;
			ScrolledComposite scrolled;
			
			
			@Override
			public Breadcrumb[] getCrumbs() {
				return new Breadcrumb[]{ };
			}
			
			@Override
			public void dispose() {}
			
			@Override
			public void createControl(Composite parent, String args) {
				this.shell = parent.getShell();
				
				parent.setLayout(new GridLayout(1, true));
				
				Composite filter = new Composite(parent, SWT.NONE);
				filter.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
				filter.setLayout(new GridLayout(2, false));
				
				Label projectFilterLabel = new Label(filter, SWT.NONE);
				projectFilterLabel.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0348-filter.png"));
				
				final Text projectFilterInput = new Text(filter, SWT.BORDER);
				projectFilterInput.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
				projectFilterInput.addModifyListener(new ModifyListener() {
					@Override
					public void modifyText(ModifyEvent me) {
						projectfilter = projectFilterInput.getText();
						loadProjects();
						scrolled.setContent(projectscontainer);
					    scrolled.setMinSize(projectscontainer.computeSize(SWT.DEFAULT, SWT.DEFAULT));
					    scrolled.setExpandHorizontal(true);
					    scrolled.setExpandVertical(true);
					}
				});
				
				scrolled = new ScrolledComposite(parent, SWT.V_SCROLL);
				scrolled.setLayoutData(new GridData(GridData.FILL_BOTH));
				projectscontainer = new Composite(scrolled, SWT.NONE);
				projectscontainer.setLayout(new GridLayout(1, true));
				
				loadProjects();
			    
			    scrolled.setContent(projectscontainer);
			    scrolled.setMinSize(projectscontainer.computeSize(SWT.DEFAULT, SWT.DEFAULT));
			    scrolled.setExpandHorizontal(true);
			    scrolled.setExpandVertical(true);
			}
			
			void loadProjects() {
				for (Control c : projectscontainer.getChildren()) {
					c.dispose();
				}

				final Subject subject = SecurityUtils.getSubject();
				
				List<Project> projects = DbHelpers.ProjectsForAccount((Account) subject.getPrincipal());
				
				for (final Project project: projects) {
					if(projectfilter.length() > 0){
						if(! project.name.toLowerCase().contains(projectfilter.toLowerCase()) ) continue;
					}
					ProjectComposite pc = new ProjectComposite(project, projectscontainer);
					GridData layout = new GridData();
					layout.grabExcessHorizontalSpace = true;
					layout.horizontalAlignment = GridData.FILL;
					layout.verticalAlignment = GridData.FILL;
					layout.minimumHeight = 100;
					layout.heightHint = 100;
					pc.setLayoutData(layout);
				}


				if (subject.hasRole(Account.ADMINSTRATOR_ROLE)) {
					Button create = new Button(projectscontainer, SWT.PUSH);
					GridData createlayout = new GridData();
					createlayout.horizontalAlignment = GridData.END;
					createlayout.heightHint = 50;
					createlayout.widthHint = 200;
					create.setLayoutData(createlayout);
					create.setText(Messages.get().Account_Button_CreateProject);
					create.addSelectionListener(new SelectionAdapter() {
						@Override
						public void widgetSelected(SelectionEvent e) {
							CreateProjectDialog projectDialog = new CreateProjectDialog(
									shell);
							int res = projectDialog.open();
							if (res == Dialog.OK) {
								Project project = projectDialog.getProject();
								DbHelpers.ProjectCreateAndAssign(project,(Account) subject.getPrincipal());
								loadProjects();
							}
						}
					});
				}
				
				scrolled.setMinSize(projectscontainer.computeSize(SWT.DEFAULT, SWT.DEFAULT));
			}
			
		};
		
	}

	
	
	
	@Override
	public String getIdentifier() {
		return null;
	}
	

}
