/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.pages;

import java.io.OutputStream;
import java.util.List;

import org.eclipse.rap.rwt.RWT;
import org.eclipse.rap.rwt.client.service.JavaScriptExecutor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

import de.eoda.qtafi.surveymanager.lang.Messages;
import de.eoda.qtafi.surveymanager.model.Breadcrumb;
import de.eoda.qtafi.surveymanager.model.Project;
import de.eoda.qtafi.surveymanager.model.Role;
import de.eoda.qtafi.surveymanager.rap.UIPage;
import de.eoda.qtafi.surveymanager.rap.UIPageContribution;
import de.eoda.qtafi.surveymanager.rap.UIPages;
import de.eoda.qtafi.surveymanager.util.DataDownloadService;
import de.eoda.qtafi.surveymanager.util.DbColumn;
import de.eoda.qtafi.surveymanager.util.DbHelpers;
import de.eoda.qtafi.surveymanager.util.DbRow;
import de.eoda.qtafi.surveymanager.util.SessionResource;
import de.eoda.qtafi.surveymanager.util.DataDownloadService.DownloadCallback;

public class LogPageContribution  implements UIPageContribution {

	private static final long serialVersionUID = 1L;

	@Override
	public String getId() {
		return "log";
	}

	@Override
	public String getTitle() {
		return Messages.get().Log_Log;
	}

	@Override
	public String getIdentifier() {
		return "log";
	}

	@Override
	public UIPage createPage() {
		return new UIPage() {
			
			private static final long serialVersionUID = 1L;
			Project project;
			Table tableview;
			int count = -1;
			
			@Override
			public Breadcrumb[] getCrumbs() {
				return new Breadcrumb[]{ 
						new Breadcrumb(Messages.get().Project_Title_Prefix + " " + project.name , "project|" + project.name),
						new Breadcrumb(Messages.get().Log_LogData, "log") };
			}
			
			@Override
			public void dispose() { }
			
			@Override
			public void createControl(final Composite parent, String args) {
				project = SessionResource.Project();
				count = DbHelpers.LogCount(project.name);
				parent.setLayout(new GridLayout(2, false));
				
				{
					Composite toolbar = new Composite(parent, SWT.BORDER);
					GridData toolbardata = new GridData();
					toolbardata.widthHint = 200;
					
					toolbardata.grabExcessVerticalSpace = true;
					toolbardata.verticalAlignment = GridData.FILL;
					toolbar.setLayoutData(toolbardata);
					toolbar.setLayout(new GridLayout(1, true));
					
					Button export = new Button(toolbar, SWT.PUSH);
					export.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0097-download.png"));
					export.setText(Messages.get().Log_Export);
					export.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
					export.addSelectionListener(new SelectionAdapter() {
						@Override
						public void widgetSelected(SelectionEvent e) {
							StringBuilder url = new StringBuilder();
							DataDownloadService.getInstance().addDownload("logdata" + RWT.getRequest().getSession().getId(), new DownloadCallback() {
								
								@Override
								public String filename() {
									return "logdata.csv";
								}
								
								@Override
								public void service(OutputStream os) {
									DbHelpers.LogExport(project.name, os);
								}
							});
							url.append(RWT.getServiceManager().getServiceHandlerUrl(DataDownloadService.DOWNLOAD_SERVICE));
							JavaScriptExecutor executor = RWT.getClient().getService(JavaScriptExecutor.class);
							if(executor != null) {
								url.append('&').append(DataDownloadService.DATA_ID).append('=').append("logdata" + RWT.getRequest().getSession().getId());
								executor.execute("window.location.href='" + url.toString() + "'");
							}
						}
					});
					
					Button delete = new Button(toolbar, SWT.PUSH);
					delete.setText(Messages.get().Log_Delete);
					if(! project.asRole.equals(Role.AS_ADMIN)){
						delete.setEnabled(false);
					}
					delete.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0270-cancel-circle.png"));
					delete.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
					delete.addSelectionListener(new SelectionAdapter() {
						@Override
						public void widgetSelected(SelectionEvent e) {
							if(! project.asRole.equals(Role.AS_ADMIN)) return;
							DbHelpers.LogDropTable(project.name);
							if(count < 0) tableview.setItemCount(0);
							else tableview.setItemCount(DbHelpers.LogCount(project.name));
						}
					});
					
				}
				
				ScrolledComposite scrolled = new ScrolledComposite(parent, SWT.H_SCROLL | SWT.V_SCROLL);
				GridData tablelayout = new GridData();
				tablelayout.grabExcessHorizontalSpace = true;
				tablelayout.grabExcessVerticalSpace = true;
				tablelayout.horizontalAlignment = GridData.FILL;
				tablelayout.verticalAlignment = GridData.FILL;
				scrolled.setLayoutData(tablelayout);
				scrolled.setExpandHorizontal(true);
				scrolled.setExpandVertical(true);
				
				tableview = new Table(scrolled, SWT.VIRTUAL | SWT.BORDER);
				tableview.setHeaderVisible(true);
				tableview.setLinesVisible(true);
				tableview.setData(RWT.MARKUP_ENABLED, true);
				scrolled.setMinSize(tableview.computeSize(SWT.DEFAULT, SWT.DEFAULT));
				if(count < 0) tableview.setItemCount(0);
				else tableview.setItemCount(DbHelpers.LogCount(project.name));
				tableview.addListener (SWT.SetData, new Listener () {
					public void handleEvent (Event event) {
						TableItem item = (TableItem) event.item;
				        int index = tableview.indexOf (item);
				        DbRow row = DbHelpers.LogRow(SessionResource.Project().name, index+1);
				        if(row != null) item.setText(row.content);
					}
				}); 
				
				scrolled.setContent(tableview);
				List<DbColumn> cols = DbHelpers.LogColumns(project.name);
				for(final DbColumn col: cols){
					final TableColumn column = new TableColumn(tableview, SWT.NONE);
					column.setText(col.name);
					column.setWidth(100);
					
					column.addSelectionListener(new SelectionAdapter() {
						@Override
						public void widgetSelected(SelectionEvent e) {
							VariableFrequencyDialog dialog = new VariableFrequencyDialog(parent.getShell(), 
									project.name, col.name, VariableFrequencyDialog.TYPE_LOG_DATA);
							dialog.open();
						}
					});
				}
			}
		};
	}

}
