/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.pages;

import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authc.IncorrectCredentialsException;
import org.apache.shiro.authc.UnknownAccountException;
import org.apache.shiro.authc.UsernamePasswordToken;
import org.apache.shiro.subject.Subject;
import org.eclipse.rap.rwt.RWT;
import org.eclipse.rap.rwt.client.service.JavaScriptExecutor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Text;
import de.eoda.qtafi.surveymanager.lang.Messages;

import de.eoda.qtafi.surveymanager.model.Breadcrumb;
import de.eoda.qtafi.surveymanager.rap.UIPage;
import de.eoda.qtafi.surveymanager.rap.UIPageContribution;
import de.eoda.qtafi.surveymanager.rap.UIPages;


public class LoginContribution implements UIPageContribution {

	private static final long serialVersionUID = 1L;
	private boolean rememberMe = false;
	
	
	@Override
	public String getId() {
		return "login";
	}

	@Override
	public String getTitle() {
		return Messages.get().Login_Title;
	}

	@Override
	public UIPage createPage() {
		return new UIPage() {
			
			@Override
			public Breadcrumb[] getCrumbs() {
				return new Breadcrumb[]{ new Breadcrumb(Messages.get().Login_Title, "login")};
			}
			
			@Override
			public void dispose() {}
			
			@Override
			public void createControl(final Composite parent, String args) {
				
				parent.setLayout(UIPages.createMainLayout(1));
				parent.setLayoutData(new GridData(SWT.CENTER, SWT.TOP, true, false));
				final Composite composite = new Composite(parent, SWT.BORDER);
				composite.setData(RWT.CUSTOM_VARIANT, "bg-light");
				composite.setLayoutData(UIPages.createFillData());
			    composite.setLayout(UIPages.createGridLayout(3, false, true, true));
				composite.setSize(400, composite.getSize().y);
			    
				GridData labelGrid = new GridData(SWT.CENTER, SWT.TOP, false, false);
				labelGrid.widthHint = 200;
				
				Label nameLbl = new Label(composite, SWT.NONE);
				nameLbl.setText(Messages.get().Login_Name);
				nameLbl.setBackground(composite.getBackground());
				nameLbl.setLayoutData(labelGrid);
				
			    final Text name = new Text(composite, SWT.SINGLE | SWT.BORDER);
			    
			    GridData gridData = new GridData(SWT.CENTER, SWT.TOP, false, false);
			    gridData.horizontalSpan = 2;
			    gridData.minimumWidth = 400;
			    gridData.widthHint = 300;
			    name.setLayoutData(gridData);
			    name.setSize(400, name.getSize().y);
			    
				Label passLbl = new Label(composite, SWT.NONE);
				passLbl.setText(Messages.get().Login_Password);
				passLbl.setBackground(composite.getBackground());
				passLbl.setLayoutData(labelGrid);
				
				final Text password = new Text(composite, SWT.PASSWORD | SWT.BORDER);
				
				GridData pgridData = new GridData(SWT.CENTER, SWT.TOP, false, false);
			    pgridData.horizontalSpan = 2;
			    pgridData.minimumWidth = 400;
			    pgridData.widthHint = 300;
			    password.setLayoutData(pgridData);
			    password.setSize(400, name.getSize().y);
			    
			    password.addListener(SWT.DefaultSelection, new Listener(){

					@Override
					public void handleEvent(Event arg0) {
						login(name.getText(), password.getText());
					}
			    });
				
			    final Button btnRemember = new Button(composite, SWT.CHECK);
			    btnRemember.setBackground(composite.getBackground());
			    btnRemember.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1));
			    btnRemember.setText(Messages.get().Login_Remember);
			    btnRemember.addSelectionListener(new SelectionAdapter() {
			    	@Override
			    	public void widgetSelected(SelectionEvent e) {
			    		rememberMe = btnRemember.getSelection();
			    	}
				});
			    
				Button submit = new Button(composite, SWT.PUSH);
				submit.setText(Messages.get().Login_Button_Submit);
				submit.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 2, 1));
				submit.addListener(SWT.Selection, new Listener() {
					public void handleEvent(Event e) {
						switch (e.type) {
						case SWT.Selection:
							if (!login(name.getText(), password.getText())) {
								MessageBox mb = new MessageBox(parent.getShell(), SWT.ICON_ERROR);
								mb.setMessage(Messages.get().Login_Message_Invalid);
								mb.setText(Messages.get().Login_Message_Invalid_Title);
								mb.open();
							}
							break;
						}
					}
				});
			}
		};
	}
	
	private boolean login(String name, String password){
		Subject subject = SecurityUtils.getSubject();
		//if(subject.isAuthenticated()){
			//subject.logout();
			//return;
		//}
		
		try {
			UsernamePasswordToken upt = new UsernamePasswordToken(name, password);
			//upt.setRememberMe(rememberMe);
			subject.login(upt);
		}
		catch(UnknownAccountException ex){
			System.out.println("unknown account");
			return false;
		}
		catch(IncorrectCredentialsException ex){
			System.out.println("incorrect credentials");
			return false;
			
		}
		
		if(subject.isAuthenticated()) {
			JavaScriptExecutor executor = RWT.getClient().getService(JavaScriptExecutor.class);
			if (executor != null) {
				executor.execute("window.location.href = '/admin/manager#home'");
			}
		}
		return true;
	}

	@Override
	public String getIdentifier() {
		return null;
	}

}
