/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.pages;


import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.rap.rwt.RWT;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;

import de.eoda.qtafi.surveymanager.lang.Messages;
import de.eoda.qtafi.surveymanager.model.ProjectInfo;
import de.eoda.qtafi.surveymanager.util.DbHelpers;

public class ProjectInfoDialog extends Dialog {

	private static final long serialVersionUID = 1L;
	
	final ProjectInfo project;
	
	public ProjectInfoDialog(Shell parentShell, ProjectInfo project) {
		super(parentShell);
		this.project = project;
	}

	
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setText(Messages.get().ProjectInfo_Title + " " + project.name);
	}
	
	@Override
	protected Control createDialogArea(final Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		container.setLayout(new GridLayout(1, false));
		
		Label name = new Label(container, SWT.NONE);
		name.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		name.setData(RWT.MARKUP_ENABLED, true);
		name.setText("<span style=\"font-size:18pt;\">"+ project.name + "</span>");
		
		
		int status = DbHelpers.ProjectStatus(project.name);
		Label statuslabel = new Label(container, SWT.NONE);
		statuslabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		statuslabel.setData(RWT.MARKUP_ENABLED, true);
		statuslabel.setText("<span>Status: " + (status==1?Messages.get().Home_Online:Messages.get().Home_Offline) + "</span>");
		
		Label usedBy = new Label(container, SWT.NONE);
		usedBy.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		usedBy.setText(Messages.get().ProjectInfo_Administrated_By);
		
		List list = new List(container, SWT.BORDER | SWT.V_SCROLL);
		list.setLayoutData(new GridData(GridData.FILL_BOTH));
		list.setItems(DbHelpers.ProjecAccounts(project.name).toArray(new String[]{}));
		
		
		Label openProject = new Label(container, SWT.NONE);
		openProject.setData(RWT.MARKUP_ENABLED, true);
		openProject.setText("<a href=\"/admin/manager#project|"+ project.name + "\">Got into project</a>");
		openProject.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				ProjectInfoDialog.this.cancelPressed();
			}
		});
		
		return container;
	}
	
	@Override
	protected Point getInitialSize() {
		return new Point(500, 450);
	}
	
}
