/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.pages;

import java.io.File;
import java.util.Calendar;
import java.util.Date;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.shiro.SecurityUtils;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.rap.rwt.RWT;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DateTime;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.List;

import de.eoda.qtafi.surveymanager.auth.Account;
import de.eoda.qtafi.surveymanager.lang.Messages;
import de.eoda.qtafi.surveymanager.model.Breadcrumb;
import de.eoda.qtafi.surveymanager.model.Project;
import de.eoda.qtafi.surveymanager.rap.UIPage;
import de.eoda.qtafi.surveymanager.rap.composites.StockChartComposite;
import de.eoda.qtafi.surveymanager.util.DbHelpers;
import de.eoda.qtafi.surveymanager.util.DbHelpers.C3ChartData;
import de.eoda.qtafi.surveymanager.util.Resources;
import de.eoda.qtafi.surveymanager.util.SessionResource;

public class ProjectPage implements UIPage {

	private static final long serialVersionUID = 1L;
	
	String projectname;
	Project project;
	int usercount, responsecount;

	StockChartComposite responsechart;

	public ProjectPage() {
	}

	@Override
	public Breadcrumb[] getCrumbs() {
		return new Breadcrumb[]{ new Breadcrumb(Messages.get().Project_Title_Prefix + " " + projectname, "project|" + projectname) };
	}
	
	@Override
	public void dispose() {}
	
	@Override
	public void createControl(final Composite parent, String args) {
		projectname = args;
		usercount = DbHelpers.UsersCount(projectname);
		responsecount = DbHelpers.ResponseCount(projectname);
		
		Account account = (Account) SecurityUtils.getSubject().getPrincipal();
		if(! account.isProjectAccessable(projectname)){
			MessageDialog.openError(parent.getShell(), Messages.get().Project_AccessWarning_short, Messages.get().Project_AccessWarning_long);
			return;
		}
		project = DbHelpers.Project(projectname, account);
		if(project == null) return;
		
		RWT.getUISession().getHttpSession().setAttribute("project", project);
		
		parent.setLayout(new GridLayout(1, true));
		
		final ScrolledComposite scrolled = new ScrolledComposite(parent, SWT.V_SCROLL | SWT.H_SCROLL | SWT.WRAP);
		scrolled.setLayout(new GridLayout());
		
		final Composite com = new Composite(scrolled, SWT.NONE);
		com.setLayout(new GridLayout(2, true));
		
		
		if(project.isOnline()){
			createResponseChart(com, 350);
			createRuntimeGroup(com, 250);
			createResponseGroup(com, 150);
			createQuestionaireGroup(com, 150);
			createUserGroup(com, 150);
		}
		else {
			createQuestionaireGroup(com, 250);
			createUserGroup(com, 250);
			createRuntimeGroup(com, 150);
			createResponseChart(com, 320);
			createResponseGroup(com, 150);
		}
		
				
		scrolled.setContent(com);
		GridData scrolledlayout = new GridData(GridData.FILL_BOTH);
		scrolledlayout.grabExcessHorizontalSpace = true;
		scrolledlayout.grabExcessVerticalSpace = true;
		scrolled.setExpandHorizontal(true);
		scrolled.setExpandVertical(true);
		scrolled.setLayoutData(scrolledlayout);
		scrolled.addControlListener(new ControlAdapter() {
			@Override
			public void controlResized(ControlEvent e) {
				if (scrolled.getSize().x < 800) {
					com.setLayout(new GridLayout(1, true));
					com.redraw();				
					com.pack(true);
				} 
				else {
					com.setLayout(new GridLayout(2, true));
					com.redraw();				
					com.pack(true);
				}
				scrolled.setMinSize(com.computeSize(SWT.DEFAULT, SWT.DEFAULT));
				//System.out.println("ScrolledSize:" + scrolled.getSize());
			}
		});
	}
	
	private void createResponseChart(Composite parent, int height) {
		responsechart = new StockChartComposite(parent);
		//responsechart.setTitle("Response");
		C3ChartData d = DbHelpers.C3ResponseProgression(projectname, project.startdate);
		//System.out.println(d.toString());
		responsechart.setChartData(d.toString());
		GridData chartlayout = new GridData();
		chartlayout.grabExcessHorizontalSpace = true;
		chartlayout.horizontalAlignment = GridData.FILL;
		//chartlayout.horizontalSpan = 1;
		chartlayout.heightHint = height <= 0?1:(height);
		chartlayout.verticalSpan = 2;
		responsechart.setLayoutData(chartlayout);
	}

	private void createRuntimeGroup(final Composite parent, int height) {
		// Runtime
		Group runtime = new Group(parent, SWT.NONE);
		runtime.setLayout(new GridLayout(2, false));
		GridData runtimelayout = new GridData();
		runtimelayout.grabExcessHorizontalSpace = true;
		runtimelayout.horizontalAlignment = GridData.FILL;
		runtimelayout.heightHint = height <= 0?1:height;
		runtime.setLayoutData(runtimelayout);
		runtime.setText(Messages.get().Project_Runtime);

		final Label status = new Label(runtime, SWT.NONE);
		GridData statuslayout = new GridData();
		statuslayout.grabExcessHorizontalSpace = true;
		statuslayout.horizontalAlignment = GridData.FILL;
		statuslayout.horizontalSpan = 2;
		status.setLayoutData(statuslayout);
		status.setData(RWT.MARKUP_ENABLED, true);
		if(project.isOnline()) status.setText("<a href=\"" + Resources.HttpPath() + "/qtafi/projects/" + projectname + "/\" target=\"_blank\"><span style=\"font-size:26pt;\">" + Messages.get().QuestEditor_Online + "</span></a>");
		else status.setText("<a href=\"" + Resources.HttpPath() + "/qtafi/projects/" + projectname + "/\" target=\"_blank\"><span style=\"font-size:26pt;\">" + Messages.get().QuestEditor_Preview + "</span></a>");
		
		final Button deploy = new Button(runtime, SWT.NONE);
		final Button undeploy = new Button(runtime, SWT.NONE);
		
		//deploy.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		deploy.setData(RWT.MARKUP_ENABLED, true);
		deploy.setEnabled(! project.isOnline());
		deploy.setText("<span style=\"font-size:18pt;\">" + Messages.get().Project_Publish + "</span>");
		deploy.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				if(DbHelpers.ResponseCount(projectname) > 0){
					boolean res = MessageDialog.openConfirm(parent.getShell(), Messages.get().Project_DataLossWarning_short, 
							Messages.get().Project_DataLossWarning_long);
					if(! res) return;
				}
				project.deploy();
				DbHelpers.ProjectUpdate(projectname, project);
				deploy.setEnabled(! project.isOnline());
				undeploy.setEnabled(project.isOnline());
				if(project.isOnline()) status.setText("<a href=\"" + Resources.HttpPath() + "/qtafi/projects/" + projectname + "/\" target=\"_blank\"><span style=\"font-size:26pt;\">" + Messages.get().QuestEditor_Online + "</span></a>");
				else status.setText("<a href=\"" + Resources.HttpPath() + "/qtafi/projects/" + projectname + "/\" target=\"_blank\"><span style=\"font-size:26pt;\">" + Messages.get().QuestEditor_Preview + "</span></a>");
				
			}
		});
		
		//undeploy.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		undeploy.setData(RWT.MARKUP_ENABLED, true);
		undeploy.setEnabled(project.isOnline());
		undeploy.setText("<span style=\"font-size:18pt;\">" + Messages.get().Project_Unpublish + "</span>");
		undeploy.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				project.undeploy();
				DbHelpers.ProjectUpdate(projectname, project);
				deploy.setEnabled(! project.isOnline());
				undeploy.setEnabled(project.isOnline());
				if(project.isOnline()) status.setText("<a href=\"" + Resources.HttpPath() + "/qtafi/projects/" + projectname + "/\" target=\"_blank\"><span style=\"font-size:26pt;\">" + Messages.get().QuestEditor_Online + "</span></a>");
				else status.setText("<a href=\"" + Resources.HttpPath() + "/qtafi/projects/" + projectname + "/\" target=\"_blank\"><span style=\"font-size:26pt;\">" + Messages.get().QuestEditor_Preview + "</span></a>");
				
				
				//snapshot();
			}
		});
		
		/*
		final Label startlabel = new Label(runtime, SWT.NONE);
		startlabel.setText("Start");

		final DateTime startcontrol = new DateTime(runtime, SWT.BORDER | SWT.DROP_DOWN | SWT.MEDIUM);
		setDate(project.startdate, startcontrol);
		startcontrol.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				project.startdate = getDate(startcontrol);
				DbHelpers.ProjectUpdate(project.name, project);
			}
		});

		Label http_index_qml = new Label(runtime, SWT.NONE);
		GridData indexlayout = new GridData();
		indexlayout.verticalSpan = 2;
		http_index_qml.setLayoutData(indexlayout);
		http_index_qml.setData(RWT.MARKUP_ENABLED, Boolean.TRUE);
		http_index_qml.setText("<a href=\"" + Resources.HttpPath() + "/qtafi/projects/" + projectname + "/index.qml\" target=\"_blank\">Open questionnaire</a>");

		{

			final Label endlabel = new Label(runtime, SWT.NONE);
			endlabel.setText("End");

			final DateTime endcontrol = new DateTime(runtime, SWT.BORDER | SWT.DROP_DOWN | SWT.MEDIUM);
			setDate(project.enddate, endcontrol);
			endcontrol.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					project.enddate = getDate(endcontrol);
					DbHelpers.ProjectUpdate(project.name, project);
				}
			});
		}
		*/
	}
	
	synchronized void snapshot(){
		String dumfile = DbHelpers.ResponseDumpTable(project.name);
		File snap = Resources.ProjectResource(project.name, "snapshot_" + System.currentTimeMillis() + ".zip");
		String zipfile = Resources.ZIPResources(project.name, snap.getAbsolutePath());
		File dump = new File(dumfile);
		dump.delete();
	}
	
	private void createResponseGroup(Composite parent, int height) {
		Group responses = new Group(parent, SWT.NONE);
		responses.setLayout(new GridLayout(2, true));
		GridData responselayout = new GridData();
		responselayout.grabExcessHorizontalSpace = true;
		responselayout.horizontalAlignment = GridData.FILL;
		responselayout.heightHint = height <= 0?1:height;
		responses.setLayoutData(responselayout);
		responses.setText(Messages.get().Project_Responses);

		Label datapreview = new Label(responses, SWT.None);
		datapreview.setData(RWT.MARKUP_ENABLED, Boolean.TRUE);
		datapreview.setText("<a href=\"/admin/manager#data\" ><span style=\"font-size:26pt;\">" + responsecount + "</span></a> " + Messages.get().Project_Responses_lc);
		datapreview.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Label logdata = new Label(responses, SWT.None);
		logdata.setData(RWT.MARKUP_ENABLED, Boolean.TRUE);
		logdata.setText("<a href=\"/admin/manager#log\">" + Messages.get().Log_LogData + "</a>");
		logdata.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		Label reporting = new Label(responses, SWT.None);
		reporting.setData(RWT.MARKUP_ENABLED, Boolean.TRUE);
		reporting.setText("<a href=\"/admin/manager#report\" >" + Messages.get().ReportContribution_Title + "</a>");
		reporting.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
	}

	private void createUserGroup(Composite parent, int height) {
		Group users = new Group(parent, SWT.NONE);
		users.setLayout(new GridLayout(2, true));
		GridData userslayout = new GridData();
		userslayout.grabExcessHorizontalSpace = true;
		userslayout.horizontalAlignment = GridData.FILL;
		userslayout.heightHint = height <= 0?1:height;
		users.setLayoutData(userslayout);
		users.setText(Messages.get().Project_Authentication);

	
		Label usersadmin = new Label(users, SWT.None);
		usersadmin.setData(RWT.MARKUP_ENABLED, Boolean.TRUE);
		usersadmin.setLayoutData(new GridData(GridData.FILL_BOTH));
		usersadmin.setText("<a href=\"/admin/manager#users\" ><span style=\"font-size:26pt;\">" + usercount + "</span></a> " + Messages.get().Project_Users_Defined);

		Composite combowrap = new Composite(users, SWT.NONE);
		combowrap.setLayoutData(new GridData(GridData.FILL_BOTH));
		combowrap.setLayout(new GridLayout());
		final Combo authentication = new Combo(combowrap, SWT.READ_ONLY);
		authentication.setItems(new String[] { "OPEN", "PIN", "URL" });

		String auth = project.authenticationmode;
		authentication.setText(auth);
		authentication.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				project.authenticationmode = authentication.getText();
				DbHelpers.ProjectUpdate(project.name, project);
			}
		});

		if (usercount > 0) {
			Label responserate = new Label(users, SWT.None);
			responserate.setData(RWT.MARKUP_ENABLED, Boolean.TRUE);
			responserate.setLayoutData(new GridData(GridData.FILL_BOTH));
			responserate.setText("<span style=\"font-size:26pt;\">" + (int) ((DbHelpers.UsersResponseRate(SessionResource.Project().name) / (float) usercount) * 100) + "%</span> " + Messages.get().Project_Response_Rate);
		}

	}

	private void createQuestionaireGroup(final Composite com, int height) {
		// Questionnaire
		final Group questionnaire = new Group(com, SWT.NONE);
		questionnaire.setLayout(new GridLayout(2, false));
		GridData questionnairelayout = new GridData();
		questionnairelayout.grabExcessHorizontalSpace = true;
		questionnairelayout.horizontalAlignment = GridData.FILL;
		questionnairelayout.heightHint = height <= 0?1:height;
		questionnaire.setLayoutData(questionnairelayout);
		questionnaire.setText(Messages.get().Project_Questionnaire);

		final List list = new List(questionnaire, SWT.NONE);
		list.setLayoutData(new GridData(GridData.FILL_BOTH));
		list.setData(RWT.MARKUP_ENABLED, Boolean.TRUE);
		setQuestionnaires(list, projectname);

		Composite res = new Composite(questionnaire, SWT.NONE);
		res.setLayoutData(new GridData(GridData.FILL_BOTH));
		res.setLayout(new GridLayout(1, true));

		Link newqu = new Link(res, SWT.NONE);
		newqu.setData(RWT.MARKUP_ENABLED, Boolean.TRUE);
		newqu.setText("<a>" + Messages.get().Dialog_CreateQuest_NewQuest + "</a>");
		newqu.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				//if there is no default questionnaire create without asking for language
				File defaultquest = Resources.ProjectResource(projectname, "index.qml");
				if(! defaultquest.exists()){
					Resources.CreateProjectResource(projectname, "index.qml", "");
					setQuestionnaires(list, projectname);
					return;
				}
				
				CreateQuestionnaireDialog newquest = new CreateQuestionnaireDialog(com.getShell());
				int ret = newquest.open();
				if (ret == Window.OK) {
					File file = Resources.ProjectResource(projectname, newquest.getValue());
					if (file.exists()) {
						MessageDialog.openWarning(com.getShell(), Messages.get().Project_ExistsWarning_short, Messages.get().Project_ExistsWarning_long);
					} 
					else {
						Resources.CreateProjectResource(projectname, newquest.getValue(), "");
						setQuestionnaires(list, projectname);
					}
				}
			}
		});

		Link resources = new Link(res, SWT.NONE);
		resources.setData(RWT.MARKUP_ENABLED, Boolean.TRUE);
		resources.setText("<a>" + Messages.get().Project_Resources + "</a>");
		resources.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				ResourcesDialog resources = new ResourcesDialog(com.getShell(), project);
				int res = resources.open();
				setQuestionnaires(list, projectname);
			}
		});
	}

	final Pattern localePattern = Pattern.compile("index_(\\w{2,2}).qml");

	void setQuestionnaires(List list, String projectname) {
		list.removeAll();
		java.util.List<String> qmls = Resources.ListResources(projectname, "qml");
		for (final String qu : qmls) {
			if (qu.equals("index.qml")) {
				list.add("<a href=\"/admin/manager#editor|" + qu + "\" >Default questionnaire</a>");
			}
		}
		for (final String qu : qmls) {
			if (!qu.equals("index.qml")) {
				Matcher m = localePattern.matcher(qu);
				if (m.find()) {
					String lang = m.group(1);
					list.add("<a href=\"/admin/manager#editor|" + qu + "\" >" + Messages.get().Project_Questionnaire + 
							" (" + Messages.get().Project_Questionnaire_Language + ": " + lang + ")</a>");
				}
			}
		}
	}

	static void setDate(Date date, DateTime widget) {
		Calendar cal = Calendar.getInstance();
		cal.setTimeInMillis(date.getTime());
		widget.setDate(cal.get(Calendar.YEAR), cal.get(Calendar.MONTH), cal.get(Calendar.DAY_OF_MONTH));
		widget.setTime(cal.get(Calendar.HOUR_OF_DAY), cal.get(Calendar.MINUTE), cal.get(Calendar.SECOND));
	}

	static Date getDate(DateTime widget) {
		Calendar cal = Calendar.getInstance();
		Date date = new Date();
		cal.set(Calendar.YEAR, widget.getYear());
		cal.set(Calendar.MONTH, widget.getMonth());
		cal.set(Calendar.DAY_OF_MONTH, widget.getDay());

		cal.set(Calendar.HOUR_OF_DAY, widget.getHours());
		cal.set(Calendar.MINUTE, widget.getMinutes());
		cal.set(Calendar.SECOND, widget.getSeconds());

		date.setTime(cal.getTimeInMillis());
		return date;
	}

}
