/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.pages;


import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;
import java.util.Vector;

import org.apache.commons.io.FilenameUtils;
import org.apache.commons.io.IOUtils;
import org.apache.shiro.SecurityUtils;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.rap.fileupload.DiskFileUploadReceiver;
import org.eclipse.rap.fileupload.FileDetails;
import org.eclipse.rap.fileupload.FileUploadEvent;
import org.eclipse.rap.fileupload.FileUploadHandler;
import org.eclipse.rap.fileupload.FileUploadListener;
import org.eclipse.rap.rwt.service.ServerPushSession;
import org.eclipse.rap.rwt.widgets.FileUpload;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSource;
import org.eclipse.swt.dnd.DragSourceAdapter;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

import de.eoda.qtafi.surveymanager.auth.Account;
import de.eoda.qtafi.surveymanager.lang.Messages;
import de.eoda.qtafi.surveymanager.model.Project;
import de.eoda.qtafi.surveymanager.rap.UIPages;
import de.eoda.qtafi.surveymanager.rap.composites.HTMLComposite;
import de.eoda.qtafi.surveymanager.util.DbHelpers;
import de.eoda.qtafi.surveymanager.util.InlineInputParser;
import de.eoda.qtafi.surveymanager.util.SessionResource;
import de.tinu.qtafi4.db.NodeFilter;
import de.tinu.qtafi4.model.Filter;
import de.tinu.qtafi4.model.FilterRoot;
import de.tinu.qtafi4.model.IdInfo;
import de.tinu.qtafi4.model.LayoutData;
import de.tinu.qtafi4.model.Question;
import de.tinu.qtafi4.model.UIDInfo;
import de.tinu.qtafi4.model.Value;
import de.tinu.qtafi4.model.Variable;
import de.tinu.qtafi4.tree.TreeNode;

public class QuestionEditorDialog extends Dialog {

	private static final long serialVersionUID = 1L;
	
	final Question question;
	final String qmlfile;
	final boolean mayChange;
	Text id, text, number, moretext, subject, code;
	Button prtected;
	Table cattable, vartable;
	Point pt;
	HTMLComposite html;
	GridData singlelinelayout, multiplelinelayout;
	TabItem filterItem;
	//TabItem xmlItem;
	
	int column_to_edit = 0, row_to_edit = 0;
	Tree filtertree;
	
	static interface OKListener {
		void ok(Question question);
	}
	
	public static class ComboItem {
		public String id;
		public String label;
		public boolean selected = false;
		public ComboItem(){}

		public ComboItem(String id, String label){
			this.id = id;
			this.label = label;
		}
	}
	
	List<OKListener> oklistener = new ArrayList<OKListener>();
	
	Composite questionComposite;
	
	public QuestionEditorDialog(Shell parent, final Question q, final String qml, final boolean readonly) {
		super(parent);
		this.question = q;
		this.qmlfile = qml;
		if(readonly) this.mayChange = false;
		else this.mayChange = maySaveChange(question);
	}
	
	private final boolean maySaveChange(Question q){
		Project project = SessionResource.Project();
		if(! q.isQuestionProtected()){
			return true;
		}
		
		Account account = (Account) SecurityUtils.getSubject().getPrincipal();
		if(q.getProtectedBy() == account.id){
			//System.out.println("may save because owner");
			return true;
		}
		
		int ownerid = q.getProtectedBy();
		//get role of owner and compare with current role
		int ownerrole = DbHelpers.AccountRoleById(ownerid, project.id).id;
		int currentrole = project.asRoleid;
		if(ownerrole < currentrole) return true;
		/*
		if(SessionResource.Project().asRole.equals(Role.AS_ADMIN)){
			System.out.println("may save because role admin");
			return true;
		}
		*/
		return false;
	}
	
	
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setText(Messages.get().QuestEditor_Title_Prefix  + " " + question.id + " / " + Messages.get().QuestEditor_Label_Type + ": " + question.type);
	}
	
	@Override
	protected Control createButtonBar(Composite parent) {
		Control bb = super.createButtonBar(parent);
		getButton(IDialogConstants.OK_ID).setEnabled(mayChange);
		return bb;
	}
	
	@Override
	protected Control createDialogArea(Composite parentt) {
		Composite container = (Composite) super.createDialogArea(parentt);
		container.setLayout(new GridLayout(1, true));
		
		final TabFolder maintab = new TabFolder(container, SWT.NONE);
		maintab.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				System.out.println( e);
				if(e.item.equals(filterItem)){
					filterTab(filterItem, maintab);
				}
			}
		});
		GridData tablayout = new GridData();
		tablayout.horizontalAlignment = GridData.FILL;
		tablayout.verticalAlignment = GridData.FILL;
		tablayout.grabExcessVerticalSpace = true;
		tablayout.grabExcessHorizontalSpace = true;
		maintab.setLayoutData(tablayout);
		
		TabItem questionItem = new TabItem(maintab, SWT.None);
		questionItem.setText(Messages.get().QuestEditor_Question);
		
		
		final ScrolledComposite scrolled = new ScrolledComposite(maintab, SWT.V_SCROLL | SWT.H_SCROLL | SWT.WRAP);
		scrolled.setLayout(new GridLayout());
		
		
		questionComposite = new Composite(scrolled, SWT.NONE);
		GridLayout questionlayout = new GridLayout(2, false);
		questionComposite.setLayout(questionlayout);
		
		singlelinelayout = new GridData();
		singlelinelayout.widthHint = 100;
		singlelinelayout.verticalAlignment = GridData.BEGINNING;
		singlelinelayout.horizontalAlignment = GridData.END;
		singlelinelayout.heightHint = 30;
		singlelinelayout.minimumHeight = 30;
		
		multiplelinelayout = new GridData();
		multiplelinelayout.widthHint = 100;
		multiplelinelayout.verticalAlignment = GridData.BEGINNING;
		multiplelinelayout.horizontalAlignment = GridData.END;
		multiplelinelayout.heightHint = 60;
		multiplelinelayout.minimumHeight = 30;
		
		
		//System.out.println("###" + question.type);
		if(question.type.equals("Page") || question.type.equals("Head") || question.type.equals("Foot")){
			typeOutput(questionComposite);
			idInput(questionComposite);
			subjectInput(questionComposite);
		}
		else if(question.type.equals("HL")){
			typeOutput(questionComposite);
			idInput(questionComposite);
			subjectInput(questionComposite);
			numberInput(questionComposite);
			textInput(questionComposite);
			moretextInput(questionComposite);
		}
		else if(question.type.equals("CV")){
			typeOutput(questionComposite);
			idInput(questionComposite);
			subjectInput(questionComposite);
			numberInput(questionComposite);
			textInput(questionComposite);
			moretextInput(questionComposite);
			{
				Label layoutlabel = new Label(questionComposite, SWT.NONE);
				layoutlabel.setText(Messages.get().QuestEditor_Layout);
				layoutlabel.setLayoutData(singlelinelayout);
				final ComboViewer combo = new ComboViewer(questionComposite, SWT.READ_ONLY);
				combo.setContentProvider(ArrayContentProvider.getInstance());
				combo.setLabelProvider(new LabelProvider() {
			        @Override
			        public String getText(Object element) {
			            if (element instanceof ComboItem) {
			            	ComboItem current = (ComboItem) element;
			                return current.label;
			            }
			            return super.getText(element);
			        }
			    });
				
				ComboItem[] items = new ComboItem[]{ new ComboItem("radio", Messages.get().QuestEditor_Radioboxes), new ComboItem("pulldown", Messages.get().QuestEditor_Pulldown)};
				combo.setInput(items);
				combo.setSelection(new StructuredSelection(items[0]), true);
				for(ComboItem i: items){
					if(i.id.equals(question.layout.attributes.get(LayoutData.INPUT_STYLE))) {
						combo.setSelection(new StructuredSelection(i), true);
					}
				}
				
				Label varlabel = new Label(questionComposite, SWT.NONE);
				varlabel.setLayoutData(singlelinelayout);
				varlabel.setText(Messages.get().QuestEditor_Variable);
				
				final Text variable = new Text(questionComposite, SWT.BORDER | SWT.READ_ONLY);
				variable.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
				variable.setText(question.id);
				
				id.addModifyListener(new ModifyListener() {
					@Override
					public void modifyText(ModifyEvent arg0) {
						variable.setText(id.getText());
					}
				});
				
				oklistener.add(new OKListener() {
					@Override
					public void ok(Question question) {
						IStructuredSelection sel = (IStructuredSelection) combo.getSelection();
						ComboItem c = (ComboItem) sel.getFirstElement();
						if(c != null){
							question.layout.attributes.put(LayoutData.INPUT_STYLE, c.id);
						}
						question.variables.clear();
						question.variables.add(new Variable(question.id, ""));
						
					}
				});
			}
			categoriesTable(questionComposite);
		}
		else if(question.type.equals("MD")){
			typeOutput(questionComposite);
			idInput(questionComposite);
			subjectInput(questionComposite);
			numberInput(questionComposite);
			textInput(questionComposite);
			moretextInput(questionComposite);
			{
				/*
				Label layoutlabel = new Label(questionComposite, SWT.NONE);
				layoutlabel.setText(Messages.get().QuestEditor_Layout);
				layoutlabel.setLayoutData(singlelinelayout);
				
				final ComboViewer combo = new ComboViewer(questionComposite, SWT.READ_ONLY);
				combo.setContentProvider(ArrayContentProvider.getInstance());
				combo.setLabelProvider(new LabelProvider() {
			        @Override
			        public String getText(Object element) {
			            if (element instanceof ComboItem) {
			            	ComboItem current = (ComboItem) element;
			                return current.label;
			            }
			            return super.getText(element);
			        }
			    });
				
				ComboItem[] items = new ComboItem[]{ new ComboItem("checkbox", Messages.get().QuestEditor_Checkboxes), new ComboItem("pulldown", Messages.get().QuestEditor_Pulldown)};
				combo.setInput(items);
				combo.setSelection(new StructuredSelection(items[0]), true);
				for(ComboItem i: items){
					if(i.id.equals(question.layout.attributes.get(LayoutData.INPUT_STYLE))) {
						combo.setSelection(new StructuredSelection(i), true);
					}
				}
				*/
				
				/*
				oklistener.add(new OKListener() {
					@Override
					public void ok(Question question) {
						IStructuredSelection sel = (IStructuredSelection) combo.getSelection();
						ComboItem c = (ComboItem) sel.getFirstElement();
						if(c != null){
							question.layout.attributes.put(LayoutData.INPUT_STYLE, c.id);
						}
					}
				});
				*/
			}
			variablesTable(questionComposite);
		}
		else if(question.type.equals("OR")){
			typeOutput(questionComposite);
			idInput(questionComposite);
			subjectInput(questionComposite);
			numberInput(questionComposite);
			textInput(questionComposite);
			moretextInput(questionComposite);
			categoriesTable(questionComposite);
			variablesTable(questionComposite);
		}
		else if(question.type.equals("TE")){
			typeOutput(questionComposite);
			idInput(questionComposite);
			subjectInput(questionComposite);
			numberInput(questionComposite);
			textInput(questionComposite);
			moretextInput(questionComposite);
			variablesTable(questionComposite);
			{
				Label layoutlabel = new Label(questionComposite, SWT.NONE);
				layoutlabel.setText(Messages.get().QuestEditor_TextLength);
				layoutlabel.setLayoutData(singlelinelayout);
				
				final Spinner spinner = new Spinner(questionComposite, SWT.BORDER);
				spinner.setEnabled(mayChange);
				spinner.setMinimum(0);
				spinner.setMaximum(300);
				if(question.layout.attributes.get(LayoutData.INPUT_SIZE) != null){
					spinner.setSelection(Integer.parseInt(question.layout.attributes.get(LayoutData.INPUT_SIZE)));
				}
				else spinner.setSelection(20);
				spinner.setIncrement(5);
				
				Label layoutlabel2 = new Label(questionComposite, SWT.NONE);
				layoutlabel2.setText(Messages.get().QuestEditor_Lines);
				layoutlabel2.setLayoutData(singlelinelayout);
				
				final Spinner lines = new Spinner(questionComposite, SWT.BORDER);
				lines.setEnabled(mayChange);
				lines.setMinimum(1);
				lines.setMaximum(10);
				if(question.layout.attributes.get(LayoutData.INPUT_LINES) != null){
					lines.setSelection(Integer.parseInt(question.layout.attributes.get(LayoutData.INPUT_LINES)));
				}
				else lines.setSelection(1);
				lines.setIncrement(1);
				
				oklistener.add(new OKListener() {
					@Override
					public void ok(Question question) {
						question.layout.attributes.put(LayoutData.INPUT_SIZE, spinner.getText());
						question.layout.attributes.put(LayoutData.INPUT_LINES, lines.getText());
					}
				});
			}
		}
		else if(question.type.equals("ME")){
			typeOutput(questionComposite);
			idInput(questionComposite);
			subjectInput(questionComposite);
			numberInput(questionComposite);
			textInput(questionComposite);
			moretextInput(questionComposite);
			variablesTable(questionComposite);
			{
				Label layoutlabel = new Label(questionComposite, SWT.NONE);
				layoutlabel.setText(Messages.get().QuestEditor_NumberSize);
				layoutlabel.setLayoutData(singlelinelayout);
				
				final Spinner spinner = new Spinner(questionComposite, SWT.BORDER);
				spinner.setEnabled(mayChange);
				spinner.setMinimum(1);
				spinner.setMaximum(10);
				if(question.layout.attributes.get(LayoutData.INPUT_SIZE) != null){
					spinner.setSelection(Integer.parseInt(question.layout.attributes.get(LayoutData.INPUT_SIZE)));
				}
				else spinner.setSelection(2);
				spinner.setIncrement(1);
				
				oklistener.add(new OKListener() {
					@Override
					public void ok(Question question) {
						question.layout.attributes.put(LayoutData.INPUT_SIZE, spinner.getText());
					}
				});
			}
		}
		else if(question.type.equals("IMAGE")){
			typeOutput(questionComposite);
			idInput(questionComposite);
			{
				final ServerPushSession pushSession = new ServerPushSession();
				final String url = startUploadReceiver();
				
				final Label filel = new Label(questionComposite, SWT.NONE);
				filel.setText(Messages.get().QuestEditor_File);
				filel.setLayoutData(singlelinelayout);
				
				final FileUpload fup = new FileUpload(questionComposite, SWT.PUSH);
				fup.setEnabled(mayChange);
				fup.setText(Messages.get().QuestEditor_SelectFile);
				fup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
				fup.addSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(SelectionEvent e) {
						pushSession.start();
		                fup.submit(url);
					}
				});
				
				final Label previewl = new Label(questionComposite, SWT.NONE);
				previewl.setText(Messages.get().QuestEditor_Preview);
				previewl.setLayoutData(singlelinelayout);
				
				html = new HTMLComposite(questionComposite, SWT.NONE);
				html.setLayoutData(new GridData(GridData.FILL_BOTH));
				//System.out.println(question.code);
				html.setHtml(question.code);
			}
		}
		else if(question.type.equals("XHTML")){
			typeOutput(questionComposite);
			idInput(questionComposite);
			subjectInput(questionComposite);
			numberInput(questionComposite);
			textInput(questionComposite);
			moretextInput(questionComposite);
			codeInput(questionComposite);
		}
		else if(question.type.equals("RAW")){
			typeOutput(questionComposite);
			idInput(questionComposite);
			subjectInput(questionComposite);
			codeInput(questionComposite);
		}
		else if(question.type.equals("PIN")){
			typeOutput(questionComposite);
			idInput(questionComposite);
			subjectInput(questionComposite);
			textInput(questionComposite);
		}
		else if(question.type.equals("LOGIN")){
			typeOutput(questionComposite);
			idInput(questionComposite);
			subjectInput(questionComposite);
			textInput(questionComposite);
			{
				final Label loginl = new Label(questionComposite, SWT.NONE);
				loginl.setText(Messages.get().QuestEditor_ButtonLogin);
				loginl.setLayoutData(singlelinelayout);
				
				final Text logininput = new Text(questionComposite, SWT.BORDER);
				logininput.setEnabled(mayChange);
				String login = question.layout.attributes.get(LayoutData.LOGIN_BUTTON_LABEL);
				if(login != null) logininput.setText(login);
				else logininput.setText(Messages.get().Editor_AdvancedQuestions_Log_long);
				logininput.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
				oklistener.add(new OKListener() {
					@Override
					public void ok(Question question) {
						question.layout.attributes.put(LayoutData.LOGIN_BUTTON_LABEL, logininput.getText());
					}
				});
			}
		}
		else if(question.type.equals("NavigationButton")){
			typeOutput(questionComposite);
			idInput(questionComposite);
			subjectInput(questionComposite);
			
			final Label filel = new Label(questionComposite, SWT.NONE);
			filel.setText(Messages.get().QuestEditor_ButtonNext);
			filel.setLayoutData(singlelinelayout);
			
			final Text nextinput = new Text(questionComposite, SWT.BORDER);
			nextinput.setEnabled(mayChange);
			String next = question.layout.attributes.get(LayoutData.NAVIGATIONBUTTON_NEXT);
			if(next != null) nextinput.setText(next);
			else nextinput.setText(Messages.get().QuestEditor__NextPage);
			nextinput.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			
			final Label butprev = new Label(questionComposite, SWT.NONE);
			butprev.setText(Messages.get().QuestEditor_ButtonPrevious);
			butprev.setLayoutData(singlelinelayout);
			
			final Text previnput = new Text(questionComposite, SWT.BORDER);
			previnput.setEnabled(mayChange);
			String prev = question.layout.attributes.get(LayoutData.NAVIGATIONBUTTON_PREVIOUS);
			if(prev != null) previnput.setText(prev);
			else previnput.setText(Messages.get().QuestEditor__PreviousPage);
			previnput.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			oklistener.add(new OKListener() {
				@Override
				public void ok(Question question) {
					question.layout.attributes.put(LayoutData.NAVIGATIONBUTTON_NEXT, nextinput.getText());
					question.layout.attributes.put(LayoutData.NAVIGATIONBUTTON_PREVIOUS, previnput.getText());
				}
			});
		}
		else if(question.type.equals("ProgressBar")){
			typeOutput(questionComposite);
			idInput(questionComposite);
			subjectInput(questionComposite);
		}
		else if(question.type.equals("DATE")){
			typeOutput(questionComposite);
			idInput(questionComposite);
			subjectInput(questionComposite);
			textInput(questionComposite);
			variablesTable(questionComposite);
		}
		else if(question.type.equals("LANG")){
			typeOutput(questionComposite);
			idInput(questionComposite);
			subjectInput(questionComposite);
			textInput(questionComposite);
		}
		
		
		questionItem.setControl(scrolled);
		
		scrolled.setContent(questionComposite);
		GridData scrolledlayout = new GridData(GridData.FILL_BOTH);
		scrolledlayout.grabExcessHorizontalSpace = true;
		scrolledlayout.grabExcessVerticalSpace = true;
		scrolled.setExpandHorizontal(true);
		scrolled.setExpandVertical(true);
		scrolled.setLayoutData(scrolledlayout);
		
		scrolled.setMinSize(questionComposite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		
		
		filterItem = new TabItem(maintab, SWT.NONE);
		filterItem.setText(Messages.get().QuestEditor_Filter);
		/*
		xmlItem = new TabItem(maintab, SWT.NONE);
		xmlItem.setText(Messages.get().QuestEditor_XML);
		*/
		return container;
	}
	
	Composite filterComposite = null;
	
	void filterTab(TabItem filterItem, TabFolder maintab){
		//Filter tab
		if(filterComposite == null){
			filterComposite = new Composite(maintab, SWT.None);
			filterComposite.setLayout(new GridLayout(2, true));
			
			Composite filterToolbar = new Composite(filterComposite, SWT.NONE);
			GridData toolbardata = new GridData();
			toolbardata.grabExcessHorizontalSpace = true;
			toolbardata.horizontalSpan = 2;
			toolbardata.heightHint = 40;
			toolbardata.horizontalAlignment = GridData.FILL;
			filterToolbar.setLayoutData(toolbardata);
			filterToolbar.setLayout(new GridLayout(5, true));
			
			Button trash = new Button(filterToolbar, SWT.PUSH);
			trash.setText(Messages.get().Editor_Trash);
			trash.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			DropTarget trashTarget = new DropTarget(trash, DND.DROP_COPY| DND.DROP_MOVE);
			trashTarget.setTransfer(EditorPage.types);
			trashTarget.addDropListener(new DropTargetAdapter(){
				@Override
				public void drop(DropTargetEvent event) {
					//System.out.println("drop " + event.data.toString());
					TreeItem item = find(filtertree, event.data.toString());
					//System.out.println("item " + item);
					if(item != null) item.dispose();
				}
			});
			
			Button orGroup = new Button(filterToolbar, SWT.PUSH);
			orGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			orGroup.setText(Messages.get().QuestEditor_OrGroup);
			DragSource orsource = new DragSource(orGroup, DND.DROP_COPY | DND.DROP_MOVE);
			orsource.setTransfer(EditorPage.types);
			orsource.addDragListener(new DragSourceAdapter(){
				@Override
				public void dragStart(DragSourceEvent event) {
					event.doit = true;
				}
				
				@Override
				public void dragSetData(DragSourceEvent event) {
					event.data = "newGroup_OR";
				}
			});
			
			Button andGroup = new Button(filterToolbar, SWT.PUSH);
			andGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			andGroup.setText(Messages.get().QuestEditor_AndGroup);
			DragSource andsource = new DragSource(andGroup, DND.DROP_COPY | DND.DROP_MOVE);
			andsource.setTransfer(EditorPage.types);
			andsource.addDragListener(new DragSourceAdapter(){
				@Override
				public void dragStart(DragSourceEvent event) {
					event.doit = true;
				}
				
				@Override
				public void dragSetData(DragSourceEvent event) {
					event.data = "newGroup_AND";
				}
			});
			
			
			Button showif = new Button(filterToolbar, SWT.RADIO);
			showif.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			showif.setText(Messages.get().QuestEditor_ShowQuestion);
			showif.setSelection(true);
			showif.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					question.filter.type = FilterRoot.SHOW;
				}
			});
			
			Button hideif = new Button(filterToolbar, SWT.RADIO);
			hideif.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			hideif.setText(Messages.get().QuestEditor_HideQuestion);
			hideif.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					question.filter.type = FilterRoot.HIDE;
				}
			});
			
			final QuestionTree questiontree = new QuestionTree(filterComposite, SWT.BORDER);
			questiontree.setEnabled(mayChange);
			questiontree.setItems(SessionResource.QuestionDB(qmlfile).questionnaireInfo(new NodeFilter() {
				boolean before = true;
				@Override
				public boolean accept(TreeNode node) {
					if(node == null) return false;
					if(! before) return false;
					if(node.uid.equals(question.uid)){
						before = false;
						return false;
					}
					if(node.type.equals("Page")) return true;
					if(node.type.equals("CV")) return true;
					if(node.type.equals("MD")) return true;
					if(node.type.equals("OR")) return true;
					if(node.type.equals("TE")) return true;
					if(node.type.equals("ME")) return true;
					return false;
				}
			}));
			questiontree.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, true));
			
			filtertree = new Tree(filterComposite, SWT.BORDER);
			filtertree.setEnabled(mayChange);
			filtertree.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, true));
			DragSource filtertreedragsource = new DragSource(filtertree, DND.DROP_COPY | DND.DROP_MOVE);
			filtertreedragsource.setTransfer(EditorPage.types);
			filtertreedragsource.addDragListener(new DragSourceAdapter(){
				TreeItem item = null;
				@Override
				public void dragStart(DragSourceEvent event) {
					event.doit = true;
					item = filtertree.getItem(new Point(event.x, event.y));
				}
				@Override
				public void dragSetData(DragSourceEvent event) {
					System.out.println("set data " +item.getData("uid"));
					if(item != null) event.data = item.getData("uid");
				}
			});
			
			readFilterTree(question.filter, filtertree);
			
			DropTarget filterdrop = new DropTarget(filtertree, DND.DROP_COPY | DND.DROP_MOVE);
			filterdrop.setTransfer(EditorPage.types);
			filterdrop.addDropListener(new DropTargetAdapter(){
				public void dragEnter(DropTargetEvent event) {
					event.feedback = DND.FEEDBACK_SELECT;
				}
				
				public void dragOver(DropTargetEvent event) {
					event.feedback = DND.FEEDBACK_SELECT;
					if (event.item != null) {
						TreeItem item = (TreeItem) event.item;
						Point pt = event.display.map(null, filtertree, event.x, event.y);
						Rectangle bounds = item.getBounds();
						if (pt.y < bounds.y + bounds.height / 3) {
							event.feedback |= DND.FEEDBACK_INSERT_BEFORE;
						} 
						else if (pt.y > bounds.y + 2 * bounds.height / 3) {
							event.feedback |= DND.FEEDBACK_INSERT_AFTER;
						} 
						else {
							event.feedback |= DND.FEEDBACK_SELECT;
						}
					}
				}
				
				@Override
				public void drop(DropTargetEvent event) {
					if (event.data == null) {
						event.detail = DND.DROP_NONE;
						return;
					}
					
					if(event.item == null){//no node selected
						if(event.data.toString().equals("newGroup_OR")){
							TreeItem filtergroup = new TreeItem(filtertree, SWT.NONE);
							Filter group = new Filter(Filter.TYPE_GROUP_OR);
							filtergroup.setText(filterLabel(group));
							filtergroup.setData(group);
							filtergroup.setData("uid", UUID.randomUUID().toString());
							return;
						}
						if(event.data.toString().equals("newGroup_AND")){
							TreeItem filtergroup = new TreeItem(filtertree, SWT.NONE);
							Filter group = new Filter(Filter.TYPE_GROUP_AND);
							filtergroup.setText(filterLabel(group));
							filtergroup.setData(group);
							filtergroup.setData("uid", UUID.randomUUID().toString());
							return;
						}
						String uid = event.data.toString();
						Question q = SessionResource.QuestionDB(qmlfile).get(new UIDInfo(uid));
						TreeItem filteritem = new TreeItem(filtertree, SWT.NONE);
						Filter ff = new Filter();
						ff.ref = q.uid;
						ff.label = q.id + ": " + Messages.get().QuestEditor_NoneSelected;
						filteritem.setData(ff);
						filteritem.setText(filterLabel(ff));
						filteritem.setData("uid", UUID.randomUUID().toString());
						return;
					}
					else {
						TreeItem target = (TreeItem) event.item;
						System.out.println("drop target " + target);
						Filter targetFilter = (Filter) target.getData();
						if(targetFilter.type != Filter.TYPE_GROUP_AND && targetFilter.type != Filter.TYPE_GROUP_OR){
							event.detail = DND.DROP_NONE;
							return;
						}
						
						if(event.data.toString().equals("newGroup_OR")){
							TreeItem filtergroup = new TreeItem(target, SWT.NONE);
							Filter group = new Filter(Filter.TYPE_GROUP_OR);
							filtergroup.setText(filterLabel(group));
							filtergroup.setData(group);
							filtergroup.setData("uid", UUID.randomUUID().toString());
							return;
						}
						if(event.data.toString().equals("newGroup_AND")){
							TreeItem filtergroup = new TreeItem(target, SWT.NONE);
							Filter group = new Filter(Filter.TYPE_GROUP_AND);
							filtergroup.setText(filterLabel(group));
							filtergroup.setData(group);
							filtergroup.setData("uid", UUID.randomUUID().toString());
							return;
						}
						String uid = event.data.toString();
						Question q = SessionResource.QuestionDB(qmlfile).get(new UIDInfo(uid));
						TreeItem filteritem = new TreeItem(target, SWT.NONE);
					
						Filter ff = new Filter();
						ff.ref = q.uid;
						ff.label = q.id + ": " + Messages.get().QuestEditor_NoneSelected;
						
						filteritem.setData(ff);
						filteritem.setText(filterLabel(ff));
						filteritem.setData("uid", UUID.randomUUID().toString());
					}
				}
			});
			
			filtertree.addMouseListener(new MouseAdapter(){
				@Override
				public void mouseDoubleClick(MouseEvent e) {
					TreeItem[] tis = filtertree.getSelection();
					if(tis.length > 0){
						TreeItem ti = tis[0];
						Filter fi = (Filter) ti.getData();
						if(fi.type == Filter.TYPE_GROUP_AND | fi.type == Filter.TYPE_GROUP_OR) return;
						
						System.out.println(fi);
						
						Question q = SessionResource.QuestionDB(qmlfile).get(new UIDInfo(fi.ref));
						System.out.println("dialog " +q);
						FilterDialog filterDialog = new FilterDialog(QuestionEditorDialog.this.getShell(), q, fi);
						int res = filterDialog.open();
						if(res == Dialog.OK){
							ti.setText(filterDialog.getFilter().label);
							ti.setData(filterDialog.getFilter());
						}
					}
				}
			});
			filterItem.setControl(filterComposite);
			
			oklistener.add(new OKListener() {
				@Override
				public void ok(Question question) {
					question.filter.empty();
					for(TreeItem ti : filtertree.getItems()){
						Filter filter = (Filter) ti.getData();
						question.filter.add(filter);
						writeFilterRecursiv(ti);
					}
					System.out.println("filters " + question.filter);
				}
			});
		}
	}
	
	TreeItem find(final Tree tree, String uid){
		for(TreeItem i: tree.getItems()){
			if(i.getData("uid").equals(uid)) return i;
			TreeItem ti = findRecursiv(i, uid);
			if(ti != null) return ti;
		}
		return null;
	}
	
	TreeItem findRecursiv(TreeItem item, String uid){
		for(TreeItem i: item.getItems()){
			if(i.getData("uid").equals(uid)) return i;
			
			TreeItem ti = findRecursiv(i, uid);
			if(ti != null) return ti;
		}
		return null;
	}
	
	void writeFilterRecursiv(TreeItem ti){
		Filter parent = (Filter) ti.getData();
		for(TreeItem childitem: ti.getItems()){
			Filter childfilter = (Filter) childitem.getData();
			parent.children.add(childfilter);
			writeFilterRecursiv(childitem);
		}
	}
	
	void readFilterTree(FilterRoot root, Tree filtertree){
		for(Filter f: root.filters){
			TreeItem item = new TreeItem(filtertree, SWT.NONE);
			item.setText(question.id + ": "+ filterLabel(f));
			item.setData(f);
			item.setData("uid", UUID.randomUUID().toString());
			readFilterTreeRecursiv(f, item);
		}
	}
	
	void readFilterTreeRecursiv(Filter filter, TreeItem treeitem){
		for(Filter f: filter.children){
			TreeItem item = new TreeItem(treeitem, SWT.NONE);
			item.setText(question.id + ": " + filterLabel(f));
			item.setData(f);
			item.setData("uid", UUID.randomUUID().toString());
			readFilterTreeRecursiv(f, item);
		}
	}
	
	String filterLabel(Filter f){
		if(f.type == Filter.TYPE_GROUP_AND) return "AND-Group";
		if(f.type == Filter.TYPE_GROUP_OR) return "OR-Group";
		if(f.label != null) return f.label;
		return "no label";
	}
	
	
	public void okPressed(){
		question.id = id.getText();
		if(prtected.getSelection()){
			Account account = (Account) SecurityUtils.getSubject().getPrincipal();
			question.protect(account.id);
		}
		else question.unprotect();
		if(subject != null)	question.subject = subject.getText();
		if(number != null) question.number = number.getText();
		if(text != null) question.text = text.getText();
		if(moretext != null) question.moretext = moretext.getText();
		if(code != null) question.code = code.getText();
		
		if(question.type.equals("CV") || question.type.equals("OR")){
			if(cattable != null){
				Vector<Value> vs = new Vector<Value>();
				for(TableItem ti: cattable.getItems()){
					if(ti.getText(0).trim().length() > 0){
						String lab = ti.getText(1);
						String key = ti.getText(0);
						lab = InlineInputParser.parseLine(lab, question.id, key);
						vs.add(new Value(key, lab));
					}
				}
				question.setValues(vs);
			}
		}
		if(question.type.equals("MD") || question.type.equals("OR") ||
				question.type.equals("DATE") ||
				question.type.equals("TE") || question.type.equals("ME")){
			if(vartable != null){
				Vector<Variable> vs = new Vector<Variable>();
				for(TableItem ti: vartable.getItems()){
					if(ti.getText(0).trim().length() > 0){
						String lab = ti.getText(1);
						String key = ti.getText(0);
						String var = ti.getText(2);
						
						lab = InlineInputParser.parseLine(lab, question.id, key);
						
						vs.add(new Variable(var, lab));
					}
				}
				question.setVariables(vs);
			}
		}
		for(OKListener okl: oklistener){
			okl.ok(question);
		}
		
		if(idExists(question.id)){
			MessageDialog.openError(this.getShell(), Messages.get().QuestEditor_ErrorMsg_title, Messages.get().QuestEditor_Label_Id + " " + question.id + " " + Messages.get().QuestEditor_Error_Sfx);
			return;
		}
		
		if(! question.validId()){
			MessageDialog.openError(this.getShell(), Messages.get().QuestEditor_ErrorMsg_title, Messages.get().QuestEditor_ErrorMsg_InvalidId_text + " " + question.id);
			return;
		}
		
		if(! question.validVariables()){
			MessageDialog.openError(this.getShell(), Messages.get().QuestEditor_ErrorMsg_title, Messages.get().QuestEditor_ErrorMsg_InvalidVariables_text);
			return;
		}
		
		if(! question.validCategories()){
			MessageDialog.openError(this.getShell(), Messages.get().QuestEditor_ErrorMsg_title, "Invalid value detected!");
			return;
		}
		super.okPressed();
	}
		
	final boolean idExists(String id){
		Question q = SessionResource.QuestionDB(qmlfile).get(new IdInfo(id));
		if(q != null){
			if(q.uid.equals(question.uid)) return false;
			return true;
		}
		return false;
	}
	
	void categoriesTable(Composite questionComposite){
		Composite catcom = new Composite(questionComposite, SWT.NONE);
		GridData gd = new GridData();
		gd.widthHint = 90;
		gd.verticalAlignment = GridData.FILL;
		gd.grabExcessVerticalSpace = true;
		gd.heightHint = 150;
		gd.minimumHeight = 100;
		
		catcom.setLayoutData(gd);
		catcom.setLayout(new GridLayout(1,true));
		Label catlabel = new Label(catcom, SWT.NONE);
		catlabel.setText(Messages.get().QuestEditor_Categories);
		catlabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Button imp = new Button(catcom, SWT.PUSH);
		imp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		imp.setImage(UIPages.loadImage(this.getShell().getDisplay(), "resources/icons/0006-pencil.png"));
		imp.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				StringBuffer buf = new StringBuffer();
				for(TableItem ti: cattable.getItems()){
					if(ti.getText(0).trim().length() > 0) buf.append(ti.getText(0) + "\t" + ti.getText(1).trim() + "\n");
				}
				
				TextDialog td = new TextDialog(getShell(), buf.toString());
				int res = td.open();
				if(res == org.eclipse.jface.dialogs.Dialog.OK){
					for(TableItem ti: cattable.getItems()) ti.dispose();
					parseVals(cattable, td.getText());
					TableItem emptyItem = new TableItem(cattable, SWT.NONE);
				}
			}
		});
	
		
		ScrolledComposite catcontainer = new ScrolledComposite(questionComposite, SWT.V_SCROLL);
		GridData cattablelayout = new GridData();
		cattablelayout.grabExcessHorizontalSpace = true;
		cattablelayout.grabExcessVerticalSpace = true;
		cattablelayout.horizontalAlignment = GridData.FILL;
		cattablelayout.verticalAlignment = GridData.FILL;
		cattablelayout.verticalSpan = 1;
		cattablelayout.heightHint = 200;
		//catcontainer.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		catcontainer.setLayoutData(cattablelayout);
		cattable = new Table(catcontainer, SWT.BORDER);
		cattable.setEnabled(mayChange);
		cattable.setLayoutData(cattablelayout);
		cattable.setHeaderVisible(true);
		cattable.setLinesVisible(true);
		
		final TableEditor editor = new TableEditor(cattable);
		editor.horizontalAlignment = SWT.LEFT;
		editor.grabHorizontal = true;
		editor.minimumWidth = 50;
		
		cattable.addListener(SWT.MouseDown, new Listener() {
			public void handleEvent(Event event) {
				pt = new Point(event.x, event.y);
			}
		});
		
		cattable.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent event) {
				Control oldEditor = editor.getEditor();
				if (oldEditor != null) oldEditor.dispose();
				TableItem item = (TableItem)event.item;
				if (item == null) return;
				for (int i = 0; i < 2; i++) {
					Rectangle rect = item.getBounds(i);
					if (rect.contains(pt)) {
						row_to_edit = cattable.indexOf(item);
						column_to_edit = i;
					}
				}
				
				if(column_to_edit > 1 || column_to_edit < 0) return;
				
				Text newEditor = new Text(cattable, SWT.NONE);
				newEditor.setText(item.getText(column_to_edit));
				newEditor.addModifyListener(new ModifyListener() {
					@Override
					public void modifyText(ModifyEvent me) {
						Text text = (Text)editor.getEditor();
						editor.getItem().setText(column_to_edit, text.getText());
						//System.out.println(row_to_edit + " <> " + cattable.getItemCount());
						
						if((row_to_edit+1) ==  cattable.getItemCount() /*q.values.size()*/){
							TableItem emptyItem = new TableItem(cattable, SWT.NONE);
						}
					}
				});
				newEditor.selectAll();
				newEditor.setFocus();
				editor.setEditor(newEditor, item, column_to_edit);
			}
		});
		
		TableColumn keytableColumn = new TableColumn(cattable, SWT.NONE);
		keytableColumn.setText(Messages.get().QuestEditor_ColHead_Key);
		keytableColumn.setWidth(50);
		TableColumn labtableColumn = new TableColumn(cattable, SWT.NONE);
		labtableColumn.setText(Messages.get().QuestEditor_ColHead_Label);
		labtableColumn.setWidth(600);
		
		//first loading
		for(Value v: question.values){
			TableItem ti = new TableItem(cattable, SWT.NONE);
			String label = v.label;
			String key = v.key;
			//System.out.println("in: " + label);
			label = InlineInputParser.parseXML(label, question.id, key);
			//System.out.println("out: " + label);
			ti.setText(new String[]{key, label});
		}
		TableItem emptyItem = new TableItem(cattable, SWT.NONE);
		
		catcontainer.setContent(cattable);
		catcontainer.setExpandHorizontal(true);
		catcontainer.setExpandVertical(true);
	}
	
	void variablesTable(Composite questionComposite){
		Composite varcom = new Composite(questionComposite, SWT.NONE);
		GridData gd = new GridData();
		gd.widthHint = 90;
		gd.verticalAlignment = GridData.FILL;
		gd.grabExcessVerticalSpace = true;
		gd.heightHint = 150;
		gd.minimumHeight = 100;
		
		varcom.setLayoutData(gd);
		varcom.setLayout(new GridLayout(1,true));
		Label varlabel = new Label(varcom, SWT.NONE);
		varlabel.setText(Messages.get().QuestEditor_Variables);
		varlabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Button imp = new Button(varcom, SWT.PUSH);
		imp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		imp.setImage(UIPages.loadImage(getShell().getDisplay(), "resources/icons/0006-pencil.png"));
		imp.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				StringBuffer buf = new StringBuffer();
				for(TableItem ti: vartable.getItems()){
					if(ti.getText(0).trim().length() > 0) buf.append(ti.getText(0) + "\t" + ti.getText(1).trim() + "\n");
				}
				TextDialog td = new TextDialog(getShell(), buf.toString());
				int res = td.open();
				if(res == org.eclipse.jface.dialogs.Dialog.OK){
					for(TableItem ti: vartable.getItems()) ti.dispose();
					parseVars(vartable, td.getText());
					TableItem emptyItem = new TableItem(vartable, SWT.NONE);
				}
			}
		});

		ScrolledComposite varcontainer = new ScrolledComposite(questionComposite, SWT.V_SCROLL);
		varcontainer.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		vartable = new Table(varcontainer, SWT.BORDER);
		vartable.setEnabled(mayChange);
		vartable.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		vartable.setHeaderVisible(true);
		vartable.setLinesVisible(true);
		
		final TableEditor editor = new TableEditor(vartable);
		editor.horizontalAlignment = SWT.LEFT;
		editor.grabHorizontal = true;
		editor.minimumWidth = 50;
		
		vartable.addListener(SWT.MouseDown, new Listener() {
			public void handleEvent(Event event) {
				pt = new Point(event.x, event.y);
			}
		});
		
		vartable.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent event) {
				Control oldEditor = editor.getEditor();
				if (oldEditor != null) oldEditor.dispose();
				TableItem item = (TableItem)event.item;
				if (item == null) return;
				for (int i = 0; i < 2; i++) {
					Rectangle rect = item.getBounds(i);
					if (rect.contains(pt)) {
						row_to_edit = vartable.indexOf(item);
						column_to_edit = i;
					}
				}
				
				if(column_to_edit == 2) return;
				if(column_to_edit > 1 || column_to_edit < 0) return;
				
				// The control that will be the editor must be a child of the Table
				Text newEditor = new Text(vartable, SWT.NONE);
				newEditor.setText(item.getText(column_to_edit));
				newEditor.addModifyListener(new ModifyListener() {
					@Override
					public void modifyText(ModifyEvent me) {
						Text text = (Text)editor.getEditor();
						editor.getItem().setText(column_to_edit, text.getText());
						if(row_to_edit+1 == vartable.getItemCount()){
							TableItem emptyItem = new TableItem(vartable, SWT.NONE);
						}
						updataVars();
					}
				});
				newEditor.selectAll();
				newEditor.setFocus();
				editor.setEditor(newEditor, item, column_to_edit);
			}
		});
		
		TableColumn keytableColumn = new TableColumn(vartable, SWT.NONE);
		keytableColumn.setText(Messages.get().QuestEditor_ColHead_Key);
		keytableColumn.setWidth(50);
		
		TableColumn labtableColumn = new TableColumn(vartable, SWT.NONE);
		labtableColumn.setText(Messages.get().QuestEditor_ColHead_Label);
		labtableColumn.setWidth(600);
		
		TableColumn vartableColumn = new TableColumn(vartable, SWT.NONE);
		vartableColumn.setText(Messages.get().QuestEditor_Variable);
		vartableColumn.setWidth(100);
		
		id.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(ModifyEvent arg0) {
				updataVars();
			}
		});
		
		//initial
		for(Variable v: question.variables){
			TableItem ti = new TableItem(vartable, SWT.NONE);
			String label = v.label;
			String var = question.id;
			String key = "";
			if(v.name.equals(question.id)){
				key = "1";
				var = question.id;
			}
			else {
				key = v.name.replaceFirst(question.id + "_", "");
				var = v.name;
			}
			
			label = InlineInputParser.parseXML(label, question.id, key);
			ti.setText(new String[]{ key, label, var });
		}
		TableItem emptyItem = new TableItem(vartable, SWT.NONE);
		
		varcontainer.setContent(vartable);
		varcontainer.setExpandHorizontal(true);
		varcontainer.setExpandVertical(true);
	}
	
	final void updataVars(){
		String id = QuestionEditorDialog.this.id.getText();
		if(vartable.getItemCount() == 2){
			TableItem it = vartable.getItem(0);
			it.setText(2, id);
		}
		else {
			for(int i = 0; i < vartable.getItemCount(); i++){
				TableItem it = vartable.getItem(i);
				if(it.getText(0).length() > 0)	it.setText(2, id + "_" + it.getText(0));
			}
		}	
	}
	
	
	void typeOutput(Composite questionComposite){
		/*
		Label typel = new Label(questionComposite, SWT.NONE);
		typel.setText(Messages.get().QuestEditor_Label_Type);
		typel.setLayoutData(singlelinelayout);
		
		Label type = new Label(questionComposite, SWT.NONE);
		type.setText(question.type);
		type.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		*/
	}
	
	void idInput(Composite questionComposite){
		Label idl = new Label(questionComposite, SWT.NONE);
		idl.setText(Messages.get().QuestEditor_Label_Id);
		idl.setLayoutData(singlelinelayout);
	
		Composite co = new Composite(questionComposite, SWT.None);
		co.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		co.setLayout(new GridLayout(2, true));
		id = new Text(co, SWT.BORDER | SWT.SINGLE);
		id.setEnabled(mayChange);
		id.setText(question.id);
		id.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		
		prtected = new Button(co, SWT.CHECK);
		prtected.setEnabled(mayChange); 
		prtected.setText(Messages.get().QuestEditor_Protected);
		prtected.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		prtected.setSelection(question.isQuestionProtected());
	}
	
	void subjectInput(Composite questionComposite){
		Label subjectl = new Label(questionComposite, SWT.NONE);
		subjectl.setText(Messages.get().QuestEditor_Label_Subject);
		subjectl.setLayoutData(singlelinelayout);
	
		subject = new Text(questionComposite, SWT.BORDER | SWT.SINGLE);
		subject.setEnabled(mayChange);
		subject.setText(question.subject);
		subject.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
	}
	
	void codeInput(Composite questionComposite){
		Label codel = new Label(questionComposite, SWT.NONE);
		codel.setText(Messages.get().QuestEditor_Label_Code);
		codel.setLayoutData(singlelinelayout);
	
		code = new Text(questionComposite, SWT.BORDER | SWT.MULTI);
		code.setText(question.code);
		code.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
	}
	
	void textInput(Composite questionComposite){
		Label textl = new Label(questionComposite, SWT.NONE);
		textl.setText(Messages.get().QuestEditor_Label_Text);
		textl.setLayoutData(multiplelinelayout);
		
		text = new Text(questionComposite, SWT.BORDER | SWT.MULTI);
		text.setEnabled(mayChange);
		text.setText(question.text);
		text.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
	}
	
	void numberInput(Composite questionComposite){
		Label numberl = new Label(questionComposite, SWT.NONE);
		numberl.setText(Messages.get().QuestEditor_Label_Number);
		numberl.setLayoutData(singlelinelayout);
	
		number = new Text(questionComposite, SWT.BORDER | SWT.SINGLE);
		number.setEnabled(mayChange);
		number.setText(question.number);
		number.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
	}
	
	void moretextInput(Composite questionComposite){
		Label moretextl = new Label(questionComposite, SWT.NONE);
		moretextl.setText(Messages.get().QuestEditor_Label_Subline);
		moretextl.setLayoutData(multiplelinelayout);
	
		moretext = new Text(questionComposite, SWT.BORDER | SWT.MULTI);
		moretext.setEnabled(mayChange);
		moretext.setText(question.moretext);
		moretext.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
	}
	
	static String extension(String filename){
		String e = FilenameUtils.getExtension(filename).toLowerCase();
		if(e == null) return "png";
		if(e.equals("")) return "png";
		return e;
	}
	
	private String startUploadReceiver() {
		
		final DiskFileUploadReceiver receiver = new DiskFileUploadReceiver();
		FileUploadHandler uploadHandler = new FileUploadHandler(receiver);
		uploadHandler.addUploadListener(new FileUploadListener() {
			public void uploadProgress(FileUploadEvent event) {
			}

			public void uploadFailed(FileUploadEvent event) {
				System.err.println("failed");
			}

			public void uploadFinished(FileUploadEvent event) {
				for(FileDetails file: event.getFileDetails()) {
					try {
						for (File target: receiver.getTargetFiles()) {			
							if (target.getName().equals(file.getFileName())) {
								String ext = extension(target.getName());
								ByteArrayOutputStream img = new ByteArrayOutputStream();
								IOUtils.copy(new FileInputStream(target.getAbsolutePath()), img);
								img.flush();
								img.close();
								//byte[] imgbase64 = Base64.encodeBase64(img.toByteArray());
								byte[] imgbase64 = org.basex.util.Base64.encode(img.toByteArray());
								question.code = "<img src=\"data:image/"+ext+";base64,"+new String(imgbase64)+"\" />";
								Display.getDefault().asyncExec(new Runnable(){
									public void run(){
										html.setHtml(question.code);
										questionComposite.layout(true);
									}
								});
							}
						}
					} 
					catch (Exception e) {
						e.printStackTrace();
					}
				}
			}
		});
		return uploadHandler.getUploadUrl();
	}
	
	void parseVals(Table table, String input){
		String[] lines = input.split("\n");
		for(String l: lines){
			String key = "";
			String label = "";
			
			String[] key_val = l.split("\t");
			
			if(key_val.length >= 2){	
				key = key_val[0];
				label = key_val[1];
			}
			else if(key_val.length == 1) {
				key = key_val[0];
				label = "";
			}
			else continue;
			
			TableItem ti = new TableItem(table, SWT.NONE);
			ti.setText(new String[]{ key, label });
		}
	}
	
	
	/**
	 * 
	 * @param table
	 * @param input
	 */
	void parseVars(Table table, String input){
		String[] lines = input.split("\n");
		String questid = id.getText();
		for(String l: lines){
			String key = "";
			String label = "";
			String var = questid;
			
			String[] key_val = l.split("\t");
			
			if(key_val.length >= 2){	
				key = key_val[0];
				label = key_val[1];
			}
			else if(key_val.length == 1) {
				key = key_val[0];
				label = "";
			}
			else continue;
			
			if(lines.length == 1){
				var = questid;
			}
			else {
				var = questid + "_" + key;
			}
			
			TableItem ti = new TableItem(table, SWT.NONE);
			ti.setText(new String[]{ key, label, var });
		}
	}
	
	/*
	@Override
	protected Point getInitialLocation(Point initialSize) {
		Rectangle monitorBounds = Display.getCurrent().getPrimaryMonitor().getBounds();
		Rectangle dialogBounds = getShell().getBounds();
		return new Point(monitorBounds.x + (monitorBounds.width - dialogBounds.width) / 2,
				monitorBounds.y + (monitorBounds.height - dialogBounds.height) / 2);
	}
	*/
	
	@Override
	protected Point getInitialSize() {
		Rectangle monitorBounds = Display.getCurrent().getPrimaryMonitor().getBounds();
		//Rectangle dialogBounds = getShell().getBounds();
		return new Point(monitorBounds.width - 100, monitorBounds.height - 100);
	}
	
}
