/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.pages;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSource;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.DragSourceListener;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

import de.tinu.qtafi4.tree.TreeNode;

public class QuestionTree {
	
	
	final Tree tree;
	Map<String, TreeItem> treemap = new HashMap<String, TreeItem>();
	
	public QuestionTree(Composite parent, int style) {
		tree = new Tree(parent, SWT.BORDER | SWT.MULTI | SWT.FULL_SELECTION);
		tree.setLayout(new FillLayout());
		tree.setLinesVisible(true);
		
		DragSource ds = new DragSource(tree, DND.DROP_MOVE | DND.DROP_COPY);
		ds.setTransfer(EditorPage.types);
		ds.addDragListener (new DragSourceListener () {
			
			TreeItem[] item = null;
			
			@Override
			public void dragStart(DragSourceEvent event) {
				event.doit = true;
				if(tree.getSelectionCount() > 1){
					item = tree.getSelection();
				}
				else {
					item = new TreeItem[]{ tree.getItem(new Point(event.x, event.y))};
				}
			}
			
			@Override
			public void dragSetData (DragSourceEvent event) {
				if(item != null){
					if(item.length == 1) {
						if(item[0] != null){
							event.data = item[0].getData();
						}
					}
					else {
						StringBuffer buf = new StringBuffer();
						for(int i = 0; i < item.length; i++){
							buf.append(item[i].getData());
							if(i < (item.length-1)) buf.append("@@");
						}
						event.data = buf.toString();
					}
				} else {
					event.doit = false;
				}
			}
			
			@Override
			public void dragFinished(DragSourceEvent event) {
				//tree.redraw();
			}
		});
	}
	
	public void selectNode(String uid){
		tree.deselectAll();
		TreeItem i = treemap.get(uid);
		if(i != null) {
			TreeItem pi = i.getParentItem();
			if(pi != null) pi.setExpanded(true);
			i.setExpanded(true);
			tree.select(i);
		}
	}
	
	
	public void setEnabled(boolean enabled){
		tree.setEnabled(enabled);
	}
	
	public void setLayoutData(Object data){
		tree.setLayoutData(data);
	}
	
	
	public Tree getTree() {
		return this.tree;
	}
	
	
	public void addSelectionListener(SelectionListener sl){
		tree.addSelectionListener(sl);
	}
	
	public void updateItem(TreeNode node){
		System.out.println("update " + node);
		if(node == null) return;
		TreeItem item = treemap.get(node.uid);
		if(item == null) return;
		
		item.setData(node.uid);
		//item.setText(node.type + " " + node.id);
		item.setText(node.label);
		System.out.println("treeitem " + item);
		
		TreeItem[] childitems = item.getItems();
		int i = 0;
		for(TreeNode nn: node.getChildren()){
			if(i < childitems.length){
				TreeItem it = childitems[i];
				it.setData(nn.uid);
				//it.setText(nn.type + " " + nn.id);
				it.setText(nn.label);
			}
			else {
				TreeItem q = new TreeItem(item, SWT.NONE);
				treemap.put(nn.uid, q);
				q.setData(nn.uid);
				//q.setText(nn.type + " " + nn.id);
				q.setText(nn.label);
			}
			i++;
		}
		
		item.setItemCount(node.getChildren().size());
		System.out.println("setitem count " + node.getChildren().size());
		
		
		/*
		if(i < childitems.length){
			for(; i < childitems.length; i++){
				System.out.println("remove item");
				childitems[i] = null;
			}
		}
		*/
		
		tree.update();
	}
	
	public void setItems(TreeNode root){
		tree.removeAll();
		for(TreeNode n: root.getChildren()){
			TreeItem page = new TreeItem(tree, SWT.NONE);
			treemap.put(n.uid, page);
			page.setData(n.uid);
			page.setText(n.label);
			for(TreeNode nn: n.getChildren()){
				TreeItem q = new TreeItem(page, SWT.NONE);
				treemap.put(nn.uid, q);
				q.setData(nn.uid);
				//q.setText(nn.type + " " + nn.id);
				q.setText(nn.label);
			}
		}
	}
}
