/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.pages;

import java.io.IOException;
import java.io.Writer;
import java.util.List;
import java.util.Vector;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.rap.rwt.RWT;
import org.eclipse.rap.rwt.client.service.JavaScriptExecutor;
import org.eclipse.rap.rwt.service.ServiceHandler;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSource;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.DragSourceListener;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import de.eoda.qtafi.surveymanager.lang.Messages;
import de.eoda.qtafi.surveymanager.model.Breadcrumb;
import de.eoda.qtafi.surveymanager.rap.UIPage;
import de.eoda.qtafi.surveymanager.rap.UIPages;
import de.eoda.qtafi.surveymanager.rap.composites.ClickListener;
import de.eoda.qtafi.surveymanager.rap.export.POIDocument;
import de.eoda.qtafi.surveymanager.rap.export.XLSXDocument;
import de.eoda.qtafi.surveymanager.rap.report.CVReportItem;
import de.eoda.qtafi.surveymanager.rap.report.HLReportItem;
import de.eoda.qtafi.surveymanager.rap.report.ImageReportItem;
import de.eoda.qtafi.surveymanager.rap.report.MDReportItem;
import de.eoda.qtafi.surveymanager.rap.report.MEReportItem;
import de.eoda.qtafi.surveymanager.rap.report.ORReportItem;
import de.eoda.qtafi.surveymanager.rap.report.ReportItem;
import de.eoda.qtafi.surveymanager.rap.report.TEReportItem;
import de.eoda.qtafi.surveymanager.util.SessionResource;
import de.tinu.qtafi4.model.Question;
import de.tinu.qtafi4.model.UIDInfo;
import de.tinu.qtafi4.tree.TreeNode;

public class ReportPage implements UIPage {

	private static final long serialVersionUID = 1L;
	public static Transfer[] types = new Transfer[] { TextTransfer.getInstance()  };
	
	Composite result, overview, report;
	ScrolledComposite scrolled;
	SashForm sashForm;
	int height = 0;
	String previewHandler, exporthandler , xlsxexporthandler;
	
	List<ReportItem> reportelements = new Vector<ReportItem>();
	
	@Override
	public Breadcrumb[] getCrumbs() {
		String pn = SessionResource.Project().name;
		return new Breadcrumb[]{
				new Breadcrumb(Messages.get().Project_Title_Prefix + " " + pn , "project|" + SessionResource.Project().name), 
				new Breadcrumb(Messages.get().ReportContribution_Title, "report") };
	}
	
	@Override
	public void dispose() {
		previewHandler = "preview_" + RWT.getRequest().getSession().getId();
		RWT.getServiceManager().unregisterServiceHandler(previewHandler);
	}
	
	@Override
	public void createControl(final Composite parent, String args) {
		parent.setLayout(new GridLayout(2, false));
		{
			previewHandler = "preview_" + RWT.getRequest().getSession().getId();
			
			RWT.getServiceManager().unregisterServiceHandler(previewHandler);
			RWT.getServiceManager().registerServiceHandler(previewHandler, new ServiceHandler() {
				
				@Override
				public void service(HttpServletRequest request, HttpServletResponse response)
						throws IOException, ServletException {
					
					response.setCharacterEncoding("utf-8");
					Writer wr = response.getWriter();
					wr.append("<html><head>");
					wr.append("<title>Report preview</title>");
					wr.append("<meta http-equiv=\"content-type\" content=\"text/html; charset=UTF-8\"></meta>");
					wr.append("<script type=\"text/javascript\" src=\"rwt-resources/resources/jquery.js\" ></script>");
					wr.append("<script type=\"text/javascript\" src=\"rwt-resources/resources/d3.min.js\" ></script>");
					wr.append("<script type=\"text/javascript\" src=\"rwt-resources/resources/c3.min.js\" ></script>");
					wr.append("<link type=\"text/css\" href=\"rwt-resources/resources/c3.css\" rel=\"stylesheet\"></link>");
					wr.append("<link type=\"text/css\" href=\"rwt-resources/resources/css/bootstrap.css\" rel=\"stylesheet\"></link>");
					wr.append("</head><body>");
					wr.append("<div class=\"container\" style=\"width:740px;\" >");
					
					final StringBuffer result = new StringBuffer();
					for(Control c: reportelements){
						ReportItem item = (ReportItem) c;
						result.append(item.getHtml());
					}
					wr.append(result.toString());
					wr.append("</div>");
					wr.append("</body></html>");
					wr.flush();
					wr.close();
				}
			});
		}
		
		{
			exporthandler = "export_" + RWT.getRequest().getSession().getId();
			RWT.getServiceManager().unregisterServiceHandler(exporthandler);
			RWT.getServiceManager().registerServiceHandler(exporthandler, new ServiceHandler() {
				
				@Override
				public void service(HttpServletRequest request, HttpServletResponse response)
						throws IOException, ServletException {
					
					response.setContentType("application/vnd.openxmlformats-officedocument.wordprocessingml.document");
					 response.setHeader("Content-Disposition",  "attachment; filename=\"report.docx\"");
					POIDocument doc = new POIDocument();
					for(Control c: reportelements){
						ReportItem item = (ReportItem) c;
						item.toDocument(doc);
					}
					doc.write(response.getOutputStream());
				}
			});
		}
		
		{
			xlsxexporthandler = "xlsxexport_" + RWT.getRequest().getSession().getId();
			RWT.getServiceManager().unregisterServiceHandler(xlsxexporthandler);
			RWT.getServiceManager().registerServiceHandler(xlsxexporthandler, new ServiceHandler() {
				
				@Override
				public void service(HttpServletRequest request, HttpServletResponse response)
						throws IOException, ServletException {
					
					response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
					response.setHeader("Content-Disposition",  "attachment; filename=\"report.xlsx\"");
					XLSXDocument doc = new XLSXDocument();
					for(Control c: reportelements){
						ReportItem item = (ReportItem) c;
						item.toDocument(doc);
					}
					doc.write(response.getOutputStream());
				}
			});
		}
		
		
		//"CV"
		
		{
			Composite toolbar = new Composite(parent, SWT.BORDER);
			GridData toolbardata = new GridData();
			toolbardata.widthHint = 200;
			toolbardata.grabExcessVerticalSpace = true;
			toolbardata.verticalAlignment = GridData.FILL;
			toolbar.setLayoutData(toolbardata);
			toolbar.setLayout(new GridLayout(1, true));
			
			Button export = new Button(toolbar, SWT.PUSH);
			export.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0097-download.png"));
			export.setText(Messages.get().ReportPage_Export + " Word");
			export.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			export.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					JavaScriptExecutor executor = RWT.getClient().getService(JavaScriptExecutor.class);
					String url = RWT.getServiceManager().getServiceHandlerUrl(exporthandler);
					
					if(executor != null) {
						executor.execute("window.open('"+url+"');");
					}
				}
			});
			

			Button exportxlsx = new Button(toolbar, SWT.PUSH);
			exportxlsx.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0097-download.png"));
			exportxlsx.setText(Messages.get().ReportPage_Export + " Excel" );
			exportxlsx.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			exportxlsx.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					JavaScriptExecutor executor = RWT.getClient().getService(JavaScriptExecutor.class);
					String url = RWT.getServiceManager().getServiceHandlerUrl(xlsxexporthandler);
					
					if(executor != null) {
						executor.execute("window.open('"+url+"');");
					}
				}
			});
			
			
			
			
			Button printable = new Button(toolbar, SWT.PUSH);
			printable.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0085-printer.png"));
			printable.setText(Messages.get().ReportPage_Printable);
			printable.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			printable.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					JavaScriptExecutor executor = RWT.getClient().getService(JavaScriptExecutor.class);
					String url = RWT.getServiceManager().getServiceHandlerUrl(previewHandler);
					
					if(executor != null) {
						executor.execute("window.open('"+url+"');");
					}
					
					/*
					DataDownloadService.getInstance().addDownload("data" + RWT.getRequest().getSession().getId(), new DownloadCallback() {
						
						@Override
						public String filename() {
							return "printable.html";
						}
						
						@Override
						public void service(OutputStream os) {
							
						}
					});
					
					JavaScriptExecutor executor = RWT.getClient().getService(JavaScriptExecutor.class);
					if(executor != null) {
						executor.execute(
						"var printable = window.open('', 'Printable view', 'toolbar=yes, scrollbars=yes, resizable=yes, width=800, height=700');\n" +
						"$('.html-composite').each(function(i, obj) {\n" +
						"	printable.document.write($(this).html());\n" +
						"});\n");
					}
					*/
				}
			});
			
			
			//Button trash = new Button(toolbar, SWT.PUSH);
			//trash.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0173-bin.png"));
			//trash.setText(Messages.get().ReportPage_Trash);
			//trash.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			DropTarget dt = new DropTarget(toolbar, DND.DROP_COPY | DND.DROP_MOVE);
			dt.setTransfer(types);
			dt.addDropListener(new DropTargetAdapter(){
				public void dragEnter(DropTargetEvent event) {
					event.feedback = DND.FEEDBACK_SELECT;
				}
				
				public void dragOver(DropTargetEvent event) {
					event.feedback = DND.FEEDBACK_SELECT;
				}
				
				@Override
				public void drop(DropTargetEvent event) {
					if(event.data.toString().startsWith("reportdrag")) {
						final String rem = event.data.toString();
						parent.getDisplay().asyncExec(new Runnable(){
							public void run() {
								removeItem(rem);
								scrolled.layout(true, true);
								scrolled.setMinSize(result.computeSize(SWT.DEFAULT, SWT.DEFAULT));
							}
						});
					}
				}
			});
		}
		
		sashForm = new SashForm(parent, SWT.BORDER);
		
		sashForm.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, true));
		sashForm.setLayout(new GridLayout(1, true));
		QuestionTree tree = new QuestionTree(sashForm, SWT.NONE);
		final TreeNode rootnode = SessionResource.QuestionDB("index.qml").questionnaireInfo();
		tree.setItems(rootnode);
		
		scrolled = new ScrolledComposite(sashForm, SWT.H_SCROLL | SWT.V_SCROLL );
		scrolled.setLayoutData(new GridData(GridData.FILL_BOTH));
		result = new Composite(scrolled, SWT.NONE);
		{
			GridLayout grid = new GridLayout(2, false);
			grid.horizontalSpacing = 0;
			grid.marginLeft = 0;
			grid.marginTop = 0;
			grid.marginBottom = 0;
			grid.marginRight = 0;
			result.setLayout(grid);
		}
		
		overview = new Composite(result, SWT.None);
		{
			GridLayout grid = new GridLayout(1, true);
			grid.horizontalSpacing = 0;
			grid.marginLeft = 0;
			grid.marginTop = 0;
			grid.marginBottom = 0;
			grid.marginRight = 0;
			overview.setLayout(grid);
		}
		GridData overviewdata = new GridData();
		overviewdata.widthHint = 50;
		overviewdata.grabExcessVerticalSpace = true;
		overviewdata.verticalAlignment = GridData.FILL;
		overview.setLayoutData(overviewdata);
		
		report = new Composite(result, SWT.NONE);
		report.setData(RWT.CUSTOM_VARIANT, "report-composite");
		{
			GridLayout grid = new GridLayout(1, true);
			grid.horizontalSpacing = 0;
			grid.marginLeft = 0;
			grid.marginTop = 0;
			grid.marginBottom = 0;
			grid.marginRight = 0;
			report.setLayout(grid);
		}
		//report.setLayout(new GridLayout(1, true));
		GridData reportdata = new GridData();
		reportdata.grabExcessHorizontalSpace = true;
		reportdata.horizontalAlignment = GridData.FILL;
		reportdata.grabExcessVerticalSpace = true;
		reportdata.verticalAlignment = GridData.FILL;
		report.setLayoutData(reportdata);
		
		
		DropTarget target = new DropTarget(result, DND.DROP_MOVE|DND.DROP_COPY);
		target.setTransfer(types);
		target.addDropListener(new DropTargetAdapter() {
			
			public void dragEnter(DropTargetEvent event) {
				event.feedback = DND.FEEDBACK_SELECT;
			}
			
			public void dragOver(DropTargetEvent event) {
				event.feedback = DND.FEEDBACK_SELECT;
			}
			
			@Override
			public void drop(DropTargetEvent event) {
				if(event.data.toString().startsWith("reportdrag")) return;
				String uid = event.data.toString();
				//System.out.println("drop "+  uid);
				if(uid.contains("@@")){
					String[] uids = uid.split("@@");
					for(String qid: uids){
						Question q = SessionResource.QuestionDB("index.qml").get(new UIDInfo(qid));
						if(q == null) continue;
						if(q.type.equals("Page")){
							if(q.type.equals("Page")){
								TreeNode pagenode = SessionResource.QuestionDB("index.qml").getTreeNode(rootnode, qid);
								for(TreeNode n: pagenode.children){
									Question qq = SessionResource.QuestionDB("index.qml").get(new UIDInfo(n.uid));
									appendReportItem(qq);
								}
							}
						}
						else appendReportItem(q);
					}
					return;
				}
				
				Question q = SessionResource.QuestionDB("index.qml").get(new UIDInfo(uid));
				if(q == null) return;
				if(q.type.equals("Page")){
					TreeNode pagenode = SessionResource.QuestionDB("index.qml").getTreeNode(rootnode, uid);
					for(TreeNode n: pagenode.children){
						Question qq = SessionResource.QuestionDB("index.qml").get(new UIDInfo(n.uid));
						appendReportItem(qq);
					}
				}
				else {
					appendReportItem(q);
				}
			}
		});
		scrolled.setExpandHorizontal(true);
		scrolled.setExpandVertical(true);
		scrolled.setAlwaysShowScrollBars(true);
		scrolled.setContent(result);
		sashForm.setWeights(new int[]{1, 3});
		scrolled.setMinSize(result.computeSize(SWT.DEFAULT, SWT.DEFAULT));
	}
	
	public void applyToAll(ReportItem config){
		for(ReportItem item: reportelements){
			//if(item == config) continue;
			if(config.getClass().getName().equals(item.getClass().getName())){
				item.setBreak(config.breakElement);
				config.onOk();
				item.options = config.options;
				item.type = config.type;
				item.build();
				GridData layoutdata = new GridData();
				layoutdata.grabExcessHorizontalSpace = true;
				layoutdata.horizontalAlignment = GridData.FILL;
				layoutdata.heightHint = item.height;
				item.configbutton.setLayoutData(layoutdata);
			}
		}
		scrolled.layout(true, true);
		scrolled.setMinSize(result.computeSize(SWT.DEFAULT, SWT.DEFAULT));
	}
	
	void appendReportItem(Question question){
		final ReportItem item;
		
		if (question.type.equals("CV")){
			item = new CVReportItem(report, SWT.NONE, question);
		}
		else if (question.type.equals("MD")){
			item = new MDReportItem(report, SWT.NONE, question);
		}
		else if (question.type.equals("HL")){
			item = new HLReportItem(report, SWT.NONE, question);
		}
		else if (question.type.equals("OR")){
			item = new ORReportItem(report, SWT.NONE, question);
		}
		else if (question.type.equals("IMAGE")){
			item = new ImageReportItem(report, SWT.NONE, question);
		}
		else if(question.type.equals("TE")){
			item = new TEReportItem(report, SWT.NONE, question);
		}
		else if(question.type.equals("ME")){
			item = new MEReportItem(report, SWT.NONE, question);
		}
		else
			item = null;
					
		
		/*switch(question.type){
			case "CV" : {item = new CVReportItem(report, SWT.NONE, question); break; }
			case "MD" : {item = new MDReportItem(report, SWT.NONE, question); break; }
			case "HL" : {item = new HLReportItem(report, SWT.NONE, question); break; }
			case "OR" : {item = new ORReportItem(report, SWT.NONE, question); break; }
			case "IMAGE" : {item = new ImageReportItem(report, SWT.NONE, question); break; }
			case "TE" : {item = new TEReportItem(report, SWT.NONE, question); break; }
			case "ME" : {item = new MEReportItem(report, SWT.NONE, question); break; }
			case "DATE" : {item = new DATEReportItem(report, SWT.NONE, question); break; }
			default : {item = null; break; }
		}*/
		if(item == null) return;
		item.build();
		reportelements.add(item);
		
		
		final Button button = new Button(overview, SWT.PUSH);
		item.configbutton = button;
		item.addClickListener(new ClickListener() {	
			@Override
			public void singleclick(String id) {}
			
			@Override
			public void dblclick(String id) {
				ReportPropertyDialog rpd = new ReportPropertyDialog(result.getShell(), item);
				rpd.reportpage = ReportPage.this;
				int res = rpd.open();
				if(res == Dialog.OK){
					item.build();
					GridData layoutdata = new GridData();
					layoutdata.grabExcessHorizontalSpace = true;
					layoutdata.horizontalAlignment = GridData.FILL;
					layoutdata.heightHint = item.height;
					button.setLayoutData(layoutdata);
					scrolled.layout(true, true);
					scrolled.setMinSize(result.computeSize(SWT.DEFAULT, SWT.DEFAULT));
				}
			}
		});
		
		GridData layoutdata = new GridData();
		layoutdata.grabExcessHorizontalSpace = true;
		layoutdata.horizontalAlignment = GridData.FILL;
		layoutdata.heightHint = item.height-2;
		button.setLayoutData(layoutdata);
		button.setData(item);
		button.setData("uid", "reportdrag-" + question.uid);
		draggable(button, "reportdrag-" + question.uid);
		
		
		button.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent event) {
				ReportPropertyDialog rpd = new ReportPropertyDialog(result.getShell(), item);
				rpd.reportpage = ReportPage.this;
				int res = rpd.open();
				if(res == Dialog.OK){
					item.build();
					GridData layoutdata = new GridData();
					layoutdata.grabExcessHorizontalSpace = true;
					layoutdata.horizontalAlignment = GridData.FILL;
					layoutdata.heightHint = item.height;
					button.setLayoutData(layoutdata);
					scrolled.layout(true, true);
					scrolled.setMinSize(result.computeSize(SWT.DEFAULT, SWT.DEFAULT));
				}
			}
		});
		/*
		button.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseDoubleClick(MouseEvent e) {
				removeItem(button.getData("uid").toString());
				scrolled.layout(true, true);
				scrolled.setMinSize(result.computeSize(SWT.DEFAULT, SWT.DEFAULT));
			}
		});
		*/
		
		DropTarget breaktarget = new DropTarget(button, DND.DROP_COPY | DND.DROP_LINK | DND.DROP_MOVE);
		breaktarget.setTransfer(types);
		breaktarget.addDropListener(new DropTargetAdapter(){
			public void dragEnter(DropTargetEvent event) {
				event.feedback = DND.FEEDBACK_SELECT;
			}
			
			public void dragOver(DropTargetEvent event) {
				event.feedback = DND.FEEDBACK_SELECT;
			}
			@Override
			public void drop(DropTargetEvent event) {
				if(event.data.toString().startsWith("reportdrag")) return;
				String uid = event.data.toString();
				Question q = SessionResource.QuestionDB("index.qml").get(new UIDInfo(uid));
				if(q.type.equals("CV") || q.type.equals("MD")){
					item.setBreak(q);
					item.build();
					GridData layoutdata = new GridData();
					layoutdata.grabExcessHorizontalSpace = true;
					layoutdata.horizontalAlignment = GridData.FILL;
					layoutdata.heightHint = item.height;
					button.setLayoutData(layoutdata);
					
					scrolled.layout(true, true);
					scrolled.setMinSize(result.computeSize(SWT.DEFAULT, SWT.DEFAULT));
				}
			}
		});
		
		//result.layout(true, true);
		scrolled.layout(true, true);
		scrolled.setMinSize(result.computeSize(SWT.DEFAULT, SWT.DEFAULT));
	}
	
	
	void draggable(Control c, final String eventContext){
		DragSource ds = new DragSource(c, DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_LINK);
		ds.setTransfer(types);
		ds.addDragListener (new DragSourceListener () {
			@Override
			public void dragStart(DragSourceEvent event) {
				event.doit = true;
			}
			
			@Override
			public void dragSetData (DragSourceEvent event) {
				event.data = eventContext;
			}
			
			@Override
			public void dragFinished(DragSourceEvent event) {
			}
		});
	}
	
	protected void removeItem(Button button){
		Control report = (Control) button.getData();
		report.dispose();
		button.dispose();
		reportelements.remove(report);
	}
	
	protected void removeItem(String uid){
		Control toDispose = null;
		for(Control cc: overview.getChildren()){
			if(cc.getData("uid").equals(uid)){
				toDispose = cc;
			}
		}
		if(toDispose != null){
			Control xx  = (Control) toDispose.getData();
			toDispose.dispose();
			xx.dispose();
			reportelements.remove(xx);
		}
	}
}
