/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.pages;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.List;

import org.apache.commons.io.IOUtils;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.rap.fileupload.DiskFileUploadReceiver;
import org.eclipse.rap.fileupload.FileDetails;
import org.eclipse.rap.fileupload.FileUploadEvent;
import org.eclipse.rap.fileupload.FileUploadHandler;
import org.eclipse.rap.fileupload.FileUploadListener;
import org.eclipse.rap.rwt.RWT;
import org.eclipse.rap.rwt.client.service.JavaScriptExecutor;
import org.eclipse.rap.rwt.service.ServerPushSession;
import org.eclipse.rap.rwt.widgets.FileUpload;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

import de.eoda.qtafi.surveymanager.lang.Messages;
import de.eoda.qtafi.surveymanager.model.Project;
import de.eoda.qtafi.surveymanager.rap.UIPages;
import de.eoda.qtafi.surveymanager.util.DataDownloadService;
import de.eoda.qtafi.surveymanager.util.DataDownloadService.DownloadCallback;
import de.eoda.qtafi.surveymanager.util.Resources;

public class ResourcesDialog extends Dialog {

	private static final long serialVersionUID = 1L;
	final Project project;
	Table resourcestable;
	
	public ResourcesDialog(Shell parentShell, Project project) {
		super(parentShell);
		this.project = project;
	}
	
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setText(Messages.get().Dialog_Resources_Title);
	}
	
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite container =  (Composite) super.createDialogArea(parent);
		container.setLayout(new GridLayout());
		
		resourcestable = new Table(container, SWT.BORDER);
		resourcestable.setLayoutData(new GridData(GridData.FILL_BOTH));
		resourcestable.setHeaderVisible(false);
		resourcestable.setLinesVisible(false);
		
		TableColumn namecol = new TableColumn(resourcestable, SWT.NONE);
		namecol.setText(Messages.get().Dialog_Resources_Columns_Namecol);
		namecol.setWidth(300);
		
		TableColumn action = new TableColumn(resourcestable, SWT.NONE);
		action.setText(Messages.get().Dialog_Resources_Columns_Action);
		action.setWidth(50);
		
		TableColumn download = new TableColumn(resourcestable, SWT.NONE);
		download.setText(Messages.get().Dialog_Resources_Columns_Download);
		download.setWidth(50);
		
		final ServerPushSession pushSession = new ServerPushSession();
		final String url = startUploadReceiver();
		
		final FileUpload upload = new FileUpload(container, SWT.PUSH);
		upload.setText(Messages.get().Dialog_Resources_FileUpload_Title);
		upload.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				 pushSession.start();
                 upload.submit(url);
			}
		});
		
		loadResources();
		return container;
	}
	
	protected Button createButton(Composite parent, int id, String label, boolean defaultButton) {
		if (id == IDialogConstants.CANCEL_ID) return null;
	    return super.createButton(parent, id, Messages.get().Dialog_Resources_Button_Close, defaultButton);
	}
	
	private String startUploadReceiver() {
		final DiskFileUploadReceiver receiver = new DiskFileUploadReceiver();
		FileUploadHandler uploadHandler = new FileUploadHandler(receiver);
		uploadHandler.addUploadListener(new FileUploadListener() {
			public void uploadProgress(FileUploadEvent event) {
				// handle upload progress
			}

			public void uploadFailed(FileUploadEvent event) {
				System.err.println("failed");
			}

			public void uploadFinished(FileUploadEvent event) {
				for(FileDetails file: event.getFileDetails()) {
					try {
						for (File target: receiver.getTargetFiles()) {			
							if (target.getName().equals(file.getFileName())) {
								File newfile = Resources.ProjectResource(project.name, target.getName());
								
								FileOutputStream output = new FileOutputStream(newfile);
								IOUtils.copy(new FileInputStream(target.getAbsolutePath()), output);
								output.flush();
								output.close();
								
								Display.getDefault().asyncExec(new Runnable() {
								    public void run() {
								    	loadResources();
								    }
								});
								
							}
						}
					} 
					catch (Exception e) {
						e.printStackTrace();
					}
				}
			}
		});
		return uploadHandler.getUploadUrl();
	}
	
	
	void loadResources(){
		//remove resources
		for(TableItem t: resourcestable.getItems()) {
			TableEditor te = (TableEditor) t.getData();
			te.getEditor().dispose();
			te.dispose();
			t.dispose();
		}
		
		List<String> res =  Resources.ListResources(project.name);
		for(final String r: res){
			TableItem ti = new TableItem(resourcestable, SWT.NONE);
			ti.setText(r);
			{
				TableEditor te = new TableEditor(resourcestable);
				te.grabHorizontal = true;
				te.grabVertical = true;
				
				ti.setData(te);
				
				Button bb = new Button(resourcestable, SWT.PUSH);
				bb.setImage(UIPages.loadImage(this.getParentShell().getDisplay() , "resources/icons/0173-bin.png"));
				bb.computeSize(SWT.DEFAULT, resourcestable.getItemHeight() - 3);
				
				bb.addSelectionListener(new SelectionAdapter(){
					@Override
					public void widgetSelected(SelectionEvent e) {
						Resources.DeleteProjectResource(project.name, r);
						Display.getDefault().asyncExec(new Runnable() {
						    public void run() {
						    	loadResources();
						    }
						});
					}
				});
				
				te.setEditor(bb, ti, 1);
			}
			{
				TableEditor te = new TableEditor(resourcestable);
				te.grabHorizontal = true;
				te.grabVertical = true;
				
				Button downloadb = new Button(resourcestable, SWT.PUSH);
				downloadb.setImage(UIPages.loadImage(this.getParentShell().getDisplay() , "resources/icons/0097-download.png"));
				downloadb.computeSize(SWT.DEFAULT, resourcestable.getItemHeight() - 3);
				te.setEditor(downloadb, ti, 2);
				downloadb.addSelectionListener(new SelectionAdapter(){
					@Override
					public void widgetSelected(SelectionEvent e) {
						DataDownloadService.getInstance().addDownload("resource"+RWT.getRequest().getSession().getId(), new DownloadCallback() {
							
							@Override
							public String filename() {
								return r;
							}
							
							@Override
							public void service(OutputStream os) {
								File f = Resources.ProjectResource(project.name, r);
								try {
									IOUtils.copy(new FileInputStream(f), os);
								} catch (FileNotFoundException e) {
									e.printStackTrace();
								} catch (IOException e) {
									e.printStackTrace();
								}
							}
						});
						StringBuilder url = new StringBuilder();
						url.append(RWT.getServiceManager().getServiceHandlerUrl(DataDownloadService.DOWNLOAD_SERVICE));
						JavaScriptExecutor executor = RWT.getClient().getService(JavaScriptExecutor.class);
						if(executor != null) {
							url.append('&').append(DataDownloadService.DATA_ID).append('=').append("resource" + RWT.getRequest().getSession().getId());
							executor.execute("window.location.href='" + url.toString() + "'");
						}
					}
				});
			}
		}
	}
	
	
	@Override
	protected Point getInitialSize() {
		return new Point(650, 600);
	}
	
}
