/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.pages;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;

import org.apache.shiro.SecurityUtils;
import org.apache.shiro.subject.Subject;
import org.eclipse.rap.fileupload.DiskFileUploadReceiver;
import org.eclipse.rap.fileupload.FileDetails;
import org.eclipse.rap.fileupload.FileUploadEvent;
import org.eclipse.rap.fileupload.FileUploadHandler;
import org.eclipse.rap.fileupload.FileUploadListener;
import org.eclipse.rap.rwt.RWT;
import org.eclipse.rap.rwt.client.service.JavaScriptExecutor;
import org.eclipse.rap.rwt.service.ServerPushSession;
import org.eclipse.rap.rwt.widgets.FileUpload;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;

import de.eoda.qtafi.surveymanager.auth.Account;
import de.eoda.qtafi.surveymanager.lang.Messages;
import de.eoda.qtafi.surveymanager.model.Breadcrumb;
import de.eoda.qtafi.surveymanager.model.Project;
import de.eoda.qtafi.surveymanager.rap.UIPage;
import de.eoda.qtafi.surveymanager.rap.UIPages;
import de.eoda.qtafi.surveymanager.util.DataDownloadService;
import de.eoda.qtafi.surveymanager.util.DataDownloadService.DownloadCallback;
import de.eoda.qtafi.surveymanager.util.DbHelpers;
import de.eoda.qtafi.surveymanager.util.Resources;
import de.eoda.qtafi.surveymanager.util.SessionResource;
import de.eoda.qtafi.surveymanager.util.User;

/**
 * 
 * 
 * 
 * 
 * 
 * @author martin
 *
 */
public class UsersPage implements UIPage {

	private static final long serialVersionUID = 1L;	
	
	Table tableview;
	Point pt;
	int column_to_edit = 0, row_to_edit = 0;
	boolean filtered = false;
	
	@Override
	public Breadcrumb[] getCrumbs() {
		String pn = SessionResource.Project().name;
		return new Breadcrumb[]{
				new Breadcrumb(Messages.get().Project_Title_Prefix + " " + pn , "project|" + SessionResource.Project().name), 
				new Breadcrumb(Messages.get().UsersPage_Users, "user")};
	}
	
	@Override
	public void dispose() {	}
	
	@Override
	public void createControl(Composite parent, String args) {
		parent.setLayout(new GridLayout(2, false));
		DbHelpers.UsersEnsureTableExists(SessionResource.Project().name);
		final boolean exists = DbHelpers.UsersResponseExists(SessionResource.Project().name);
		Subject subject = SecurityUtils.getSubject();
		final Project project = DbHelpers.Project(SessionResource.Project().name, (Account) subject.getPrincipal());
		
		final Button filter;
		{
			Composite toolbar = new Composite(parent, SWT.BORDER);
			GridData toolbardata = new GridData();
			toolbardata.widthHint = 250;
			toolbardata.grabExcessVerticalSpace = true;
			toolbardata.verticalAlignment = GridData.FILL;
			toolbar.setLayoutData(toolbardata);
			toolbar.setLayout(new GridLayout(1, true));
			
			Button export = new Button(toolbar, SWT.PUSH);
			export.setText(Messages.get().UsersPage_ExportPins);
			export.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0097-download.png"));
			export.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			export.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					DataDownloadService.getInstance().addDownload("users"+RWT.getRequest().getSession().getId(), new DownloadCallback() {
						
						@Override
						public String filename() {
							return "users.csv";
						}
						
						@Override
						public void service(OutputStream os) {
							DbHelpers.UsersExport(SessionResource.Project().name, os, exists, project.authenticationmode.equals("URL"), filtered);
						}
					});
					
					StringBuilder url = new StringBuilder();
					url.append(RWT.getServiceManager().getServiceHandlerUrl(DataDownloadService.DOWNLOAD_SERVICE));
					JavaScriptExecutor executor = RWT.getClient().getService(JavaScriptExecutor.class);
					if(executor != null) {
						url.append('&').append(DataDownloadService.DATA_ID).append('=').append("users" + RWT.getRequest().getSession().getId());
						executor.execute("window.location.href='" + url.toString() + "'");
					}
				}
			});
			
			final ServerPushSession pushSession = new ServerPushSession();
			
			final FileUpload importb = new FileUpload(toolbar, SWT.PUSH);
			importb.setText(Messages.get().UsersPage_ImportPins);
			importb.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0098-upload.png"));
			importb.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			importb.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					pushSession.start();
					importb.submit(startUploadReceiver());
				}
			});
			
			
			Button create = new Button(toolbar, SWT.PUSH);
			create.setText(Messages.get().UsersPage_CreatePins);
			create.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			create.addSelectionListener(new SelectionAdapter() {
				
				@Override
				public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
					DbHelpers.UsersCreate(SessionResource.Project().name, 100);
					tableview.setItemCount(DbHelpers.UsersCount(SessionResource.Project().name));
				}
			});
			
			Button delete = new Button(toolbar, SWT.PUSH);
			delete.setText(Messages.get().UsersPage_Button_Delete);
			delete.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0270-cancel-circle.png"));
			delete.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			delete.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					DbHelpers.UsersRemoveAll(SessionResource.Project().name);
					tableview.setItemCount(DbHelpers.UsersCount(SessionResource.Project().name));
				}
			});
			
			
			filter = new Button(toolbar, SWT.TOGGLE);
			filter.setText(Messages.get().UsersPage_Filter);
			filter.setImage(UIPages.loadImage(parent.getDisplay(), "resources/icons/0348-filter.png"));
			filter.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			filter.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					filtered = filter.getSelection();
					if(filter.getSelection()){
						tableview.setItemCount(0);
						tableview.setItemCount(DbHelpers.UsersCount(SessionResource.Project().name, filtered));
					}
					else {
						tableview.setItemCount(0);
						tableview.setItemCount(DbHelpers.UsersCount(SessionResource.Project().name));
					}
				}
			});
		}
		
		
		ScrolledComposite scrolled = new ScrolledComposite(parent, SWT.H_SCROLL | SWT.V_SCROLL);
		GridData tablelayout = new GridData();
		tablelayout.grabExcessHorizontalSpace = true;
		tablelayout.grabExcessVerticalSpace = true;
		tablelayout.horizontalAlignment = GridData.FILL;
		tablelayout.verticalAlignment = GridData.FILL;
		scrolled.setLayoutData(tablelayout);
		scrolled.setExpandHorizontal(true);
		scrolled.setExpandVertical(true);
		
		
	
		tableview = new Table(scrolled, SWT.VIRTUAL | SWT.BORDER);
		tableview.setHeaderVisible(true);
		tableview.setLinesVisible(true);
		tableview.setData(RWT.MARKUP_ENABLED, true);
		scrolled.setMinSize(tableview.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		tableview.setItemCount(DbHelpers.UsersCount(SessionResource.Project().name));
		
		
		final TableEditor editor = new TableEditor(tableview);
		editor.horizontalAlignment = SWT.LEFT;
		editor.grabHorizontal = true;
		editor.minimumWidth = 50;
		
		tableview.addListener(SWT.MouseDown, new Listener() {
			public void handleEvent(Event event) {
				pt = new Point(event.x, event.y);
			}
		});
		
		tableview.addListener (SWT.SetData, new Listener () {
			public void handleEvent (Event event) {
				TableItem item = (TableItem) event.item;
		        int index = tableview.indexOf(item);
		        User user = DbHelpers.Users(SessionResource.Project().name, index, exists, filter.getSelection());
		        //System.out.println(index + " " + user.n);
		        item.setData(user);
		        if(user != null){
		        	String pin = user.pin;
		        	String url = "";
		        	if(project.authenticationmode.equals("URL")){
		        		String u = Resources.HttpPath() + "/qtafi/projects/" + project.name + "/index.qml?" + user.pin;
		        		url = "<a href=\"" + u + "\" target=\"_blank\">" + u + "</a>";
					}
		        	
		        	item.setText(new String[]{Integer.toString(user.n), pin, user.info, Boolean.toString(user.answered), url});
		        }
			}
		});
		
		tableview.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent event) {
				Control oldEditor = editor.getEditor();
				if (oldEditor != null) oldEditor.dispose();
				final TableItem item = (TableItem)event.item;

				if (item == null) return;
				for (int i = 0; i < 3; i++) {
					Rectangle rect = item.getBounds(i);
					if (rect.contains(pt)) {
						//row_to_edit = tableview.indexOf(item);
						row_to_edit = Integer.parseInt(item.getText(0));
						//System.out.println("user " + row_to_edit);
						column_to_edit = i;
					}
				}
				
				if(row_to_edit < 0 || column_to_edit < 0) return;
				if(column_to_edit != 1 && column_to_edit != 2) return;
				
				// The control that will be the editor must be a child of the Table
				Text newEditor = new Text(tableview, SWT.NONE);
				newEditor.setText(item.getText(column_to_edit));
				newEditor.addModifyListener(new ModifyListener() {
					@Override
					public void modifyText(ModifyEvent me) {
						Text text = (Text)editor.getEditor();
						editor.getItem().setText(column_to_edit, text.getText());
						
						String pin = editor.getItem().getText(1);
						String info = editor.getItem().getText(2);
						
						DbHelpers.UserUpdate(SessionResource.Project().name, new User(row_to_edit, pin, info));
						
						User user = DbHelpers.Users(SessionResource.Project().name, row_to_edit, exists, filter.getSelection());
						String url = "";
			        	if(project.authenticationmode.equals("URL")){
			        		String u = Resources.HttpPath() + "/qtafi/projects/" + project.name + "/index.qml?" + user.pin;
			        		url = "<a href=\"" + u + "\" target=\"_blank\">" + u + "</a>";
						}
			        	item.setText(new String[]{Integer.toString(user.n), pin, user.info, Boolean.toString(user.answered), url});
					}
				});
				newEditor.selectAll();
				newEditor.setFocus();
				editor.setEditor(newEditor, item, column_to_edit);
			}
		});
		
		scrolled.setContent(tableview);
		
		{
			final TableColumn column = new TableColumn(tableview, SWT.NONE);
			column.setText("N");
			column.setWidth(50);
		}
		
		{
			final TableColumn column = new TableColumn(tableview, SWT.NONE);
			column.setText(Messages.get().UsersPage_Col_Pin);
			column.setWidth(200);
		}
		{
			final TableColumn column = new TableColumn(tableview, SWT.NONE);
			column.setText(Messages.get().UsersPage_Col_Info);
			column.setWidth(200);
		}
		{
			final TableColumn column = new TableColumn(tableview, SWT.NONE);
			column.setText(Messages.get().UsersPage_Col_Answered);
			column.setWidth(100);
		}
		
		{
			final TableColumn column = new TableColumn(tableview, SWT.NONE);
			column.setText("URL");
			column.setWidth(300);
		}
	}
	

	private String startUploadReceiver() {
		final DiskFileUploadReceiver receiver = new DiskFileUploadReceiver();
		FileUploadHandler uploadHandler = new FileUploadHandler(receiver);
		uploadHandler.addUploadListener(new FileUploadListener() {
			public void uploadProgress(FileUploadEvent event) {
			}

			public void uploadFailed(FileUploadEvent event) {
				System.err.println("failed");
			}

			public void uploadFinished(FileUploadEvent event) {
				String pin;
				for(FileDetails file: event.getFileDetails()) {
					try {
						for(File target: receiver.getTargetFiles()) {
							if(target.getName().equals(file.getFileName())) {
						
								BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(target.getAbsolutePath())));
								String line = null;
								int linenum = 0;
								while((line = br.readLine()) != null ){
									line = line.replaceAll("\"", "");
									String[] cells = line.split("[\t;,]");
									if(cells.length > 0){
										if(linenum == 0 && cells[0].trim().toLowerCase().contains("pin")) continue;
										pin = cells[0].trim();
										if(DbHelpers.isValidPin(pin)){
											User us = new User(-1, pin, false);
											DbHelpers.UsersAppend(SessionResource.Project().name, us);
										}
									}
								}
								br.close();
								
								Display.getDefault().asyncExec(new Runnable() {
									public void run() {
										tableview.setItemCount(DbHelpers.UsersCount(SessionResource.Project().name));
									}
								});
							}
						}
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
			}
		});
		return uploadHandler.getUploadUrl();
	}
	
	
}
