/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.pages;

import java.text.DecimalFormat;
import java.text.NumberFormat;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.rap.rwt.RWT;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

import de.eoda.qtafi.surveymanager.lang.Messages;
import de.eoda.qtafi.surveymanager.util.DbHelpers;
import de.eoda.qtafi.surveymanager.util.DbHelpers.FrequencyData;



public class VariableFrequencyDialog extends Dialog {

	private static final long serialVersionUID = 1L;
	
	public static final int TYPE_RESPONSE_DATA = 1;
	public static final int TYPE_LOG_DATA = 2;
	
	final String variable;
	final int type;
	final String projectname;
	
	DecimalFormat format = new DecimalFormat("##.0");
	
	public VariableFrequencyDialog(Shell parent, String projectname, String variable, int type) {
		super(parent);
		this.type = type;
		this.variable = variable;
		this.projectname = projectname;
	}

	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setText( Messages.get().Dialog_Frequency_TitlePrefix + " " + variable);
	}
	
	
	protected Control createDialogArea(final Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		container.setLayout(new GridLayout(1, true));
		
		Label label = new Label(container, SWT.NONE);
		label.setData(RWT.MARKUP_ENABLED, true);
		label.setText("<span style=\"font-size:16pt;\">Frequency of " + variable + "</span>");
		label.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		ScrolledComposite scrolled = new ScrolledComposite(container, SWT.H_SCROLL | SWT.V_SCROLL);
		scrolled.setLayoutData(new GridData(GridData.FILL_BOTH));
		Table frequencytable = new Table(scrolled, SWT.BORDER);
		frequencytable.setHeaderVisible(true);
		frequencytable.setLinesVisible(true);
		
		final TableColumn column1 = new TableColumn(frequencytable, SWT.NONE);
		column1.setText(Messages.get().Dialog_Frequency_Category);
		column1.setWidth(200);
		
		final TableColumn column2 = new TableColumn(frequencytable, SWT.NONE);
		column2.setText(Messages.get().Dialog_Frequency_Count);
		column2.setAlignment(SWT.RIGHT);
		column2.setWidth(100);
		
		final TableColumn column3 = new TableColumn(frequencytable, SWT.NONE);
		column3.setText(Messages.get().Dialog_Frequency_Percentage);
		column3.setAlignment(SWT.RIGHT);
		column3.setWidth(100);
		
		FrequencyData frequ = null;
		if(type == TYPE_RESPONSE_DATA) frequ = DbHelpers.ResponseFrequency(projectname, variable);
		else frequ = DbHelpers.LogFrequency(projectname, variable);
		
		for(String cat: frequ.frequ.keySet()){
			TableItem row = new TableItem(frequencytable, SWT.NONE);
			row.setText(0, cat);
			row.setText(1, Integer.toString(frequ.frequ.get(cat)));
			row.setText(2, format.format((frequ.frequ.get(cat) / (double)frequ.total) * 100 ));
		}
	
		frequencytable.setSortColumn(column1);
		
		scrolled.setContent(frequencytable);
		scrolled.setMinSize(frequencytable.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		scrolled.setExpandHorizontal(true);
		scrolled.setExpandVertical(true);
		
		return container;
	}
	
	protected Button createButton(Composite parent, int id, String label, boolean defaultButton) {
		if (id == IDialogConstants.CANCEL_ID) return null;
	    return super.createButton(parent, id, Messages.get().Dialog_Frequency_Close, defaultButton);
	}
	
	@Override
	protected Point getInitialSize() {
		return new Point(520, 700);
	}
}
