<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:qtafi="urn:qtafi">
	<xsl:output method="xml" version="1.0" indent="yes" omit-xml-declaration="yes" encoding="UTF-8" doctype-public="-//W3C//DTD XHTML 1.0 Strict//EN http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"/>

	<xsl:decimal-format decimal-separator="," grouping-separator="." />
	
	<xsl:template match="code" />
	<xsl:param name="locale" select="'en'" />
	<xsl:param name="cid" />

<xsl:template match="/">
	<div style="background-color:#909090;" id="rootform">
	<!-- 
	<xsl:if test="/qtafi/codebook/Head">
		<xsl:call-template name="Header"/>
	</xsl:if>
	-->
	<xsl:apply-templates />
	<!-- 
	<xsl:if test="/qtafi/codebook/Foot">
		<xsl:call-template name="Footer" />		
	</xsl:if>
	-->
	</div>
</xsl:template>

<xsl:template name="infobar">
	<div class="row">
		<span class="badge"><xsl:value-of select="@id" /></span>
		<xsl:if test="filter">
			<span class="glyphicon glyphicon-filter" >&#160;</span>
		</xsl:if>
		<xsl:if test="@protectedby">
			<span class="glyphicon glyphicon-lock" >&#160;</span>
		</xsl:if>
	</div>
	<xsl:if test="not(filter)">
		<div class="row" style="margin-top:10px; margin-bottom:5px;">
			<span class="label label-info" style="font-weight:bold;">Ask all</span>
		</div>
	</xsl:if>
	<xsl:for-each select="filter/show_if | filter/use_if">
		<div class="row" style="margin-top:10px; margin-bottom:5px;">
			<span class="label label-info" style="font-weight:bold;">Ask if
				<xsl:value-of select="@variable" />
				&#160;
				<xsl:choose>
					<xsl:when test="@operator"><xsl:value-of select="@operator" /></xsl:when>
					<xsl:otherwise><xsl:value-of select="'is'" /></xsl:otherwise>
				</xsl:choose>
				&#160;
				<xsl:value-of select="@value" />
			</span>
		</div>
	</xsl:for-each>
	<xsl:for-each select="filter/hide_if | filter/ignore_if">
		<div class="row" style="margin-top:10px; margin-bottom:5px;">
			<span class="label label-info" style="font-weight:bold;">Don't Ask if
			<xsl:value-of select="@variable" />
			&#160;
			<xsl:choose>
				<xsl:when test="@operator"><xsl:value-of select="@operator" /></xsl:when>
				<xsl:otherwise><xsl:value-of select="' is'" /></xsl:otherwise>
			</xsl:choose>
			&#160;
			<xsl:value-of select="@value" />
		</span>
		</div>
	</xsl:for-each>
</xsl:template>

<!-- 
<xsl:template match="qtafi:TE">
	<input type="text" size="50" />
</xsl:template>

<xsl:template match="qtafi:ME">
	<input type="text" size="5" />
</xsl:template>
-->
<xsl:template match="CHOOSE">
	<xsl:message>locale <xsl:value-of select="$locale"/></xsl:message> 
	<xsl:apply-templates select="*[@lang=$locale]" />
</xsl:template>

<xsl:template match="Page">
	<div style="page-break-after: always; padding:15px; background-color:#ffffff; box-shadow: 0px 0px 0px 3px #606060; -moz-box-shadow: 0px 0px 0px 3px #606060;">
		<xsl:attribute name="id">
			<xsl:value-of select="@uid" />
		</xsl:attribute>
		
		<div class="form">
			<xsl:attribute name="id">
				<!-- count(preceding-sibling::Employee) -->
				<xsl:value-of select="'page_'" />
				<xsl:value-of select="count(preceding-sibling::Page)" />
			</xsl:attribute>
			
			<div  class="text-center"><span class="label label-default" >Page <xsl:value-of select="(count(preceding-sibling::Page) + 1)" /></span></div>
			
			<xsl:for-each select="children/*">
				<div class="container">
					<xsl:attribute name="id">
						<xsl:value-of select="@id" />
					</xsl:attribute>
					<xsl:apply-templates select="."/>
				</div>
			</xsl:for-each>
		</div>
		<xsl:if test="code">
			<xsl:copy-of select="code/node()" />
		</xsl:if>
	</div>
</xsl:template>

<xsl:template match="MODULE">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>

	<xsl:for-each select="children">
		<xsl:for-each select="*">
			<div>
				<xsl:attribute name="id">
					<xsl:value-of select="@id" />
				</xsl:attribute>
				<xsl:apply-templates select="."/>
			</div>
		</xsl:for-each>
	</xsl:for-each>
</xsl:template>

<xsl:template name="Header">
	<div class="form" style="border-bottom:solid black 1px;" id="head">
	<xsl:for-each select="/qtafi/codebook/Head/children/*">	
		<div class="container page_header1">
			<xsl:attribute name="id">
				<xsl:value-of select="@id" />
			</xsl:attribute>
			<xsl:apply-templates select="."/>
		</div>
	</xsl:for-each>
	</div>
</xsl:template>

<xsl:template name="Footer">
	<div class="form" style="border-bottom:solid black 1px;" id="foot">
	<xsl:for-each select="/qtafi/codebook/Foot/children/*">
		<xsl:if test="not(invisible)">
			<div class="container page_footer1">
				<xsl:attribute name="id">
					<xsl:value-of select="@id" />
				</xsl:attribute>
				<xsl:apply-templates select="."/>
			</div>
		</xsl:if>
	</xsl:for-each>
	</div>
</xsl:template>

<xsl:template match="Foot|Head" />

<xsl:template match="NavigationButton">
	<xsl:variable name="defbackwardlabel">
		<xsl:choose>
			<xsl:when test="$locale = 'en'">
				 <xsl:value-of select="'Previous page'" />
			</xsl:when>
			<xsl:when test="$locale = 'de'">
				<xsl:value-of select="'Zurück'" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="'Previous page'" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>
	
	<xsl:variable name="defforwardlabel">
		<xsl:choose>
			<xsl:when test="$locale = 'en'">
				<xsl:value-of select="'Next page'" />
			</xsl:when>
			<xsl:when test="$locale = 'de'">
				<xsl:value-of select="'Weiter'" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="'Next page'" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>
	
	<xsl:variable name="depth">
		<xsl:if test="not(@level)">
			<xsl:call-template name="depth" />
		</xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
		<xsl:variable name="backwardlabel">
			<xsl:if test="not(@backwardlabel)"><xsl:value-of select="$defbackwardlabel"/></xsl:if>
			<xsl:if test="@backwardlabel"><xsl:value-of select="@backwardlabel"/></xsl:if>
		</xsl:variable>
		<xsl:variable name="forwardlabel">
			<xsl:if test="not(@forwardlabel)"><xsl:value-of select="$defforwardlabel"/></xsl:if>
			<xsl:if test="@forwardlabel"><xsl:value-of select="@forwardlabel"/></xsl:if>
		</xsl:variable>
		<div class="row navigationbutton">
			<div class="col-xs-2" style="text-align:right;">>&#160;
				<xsl:if test="$forwardlabel != 'null'">
					<qtafi:USE_IF variable="__page" runat="server" value="eval(__maxpage)" operator="IS_NOT">
						<input type="button" class="btn btn-primary NavigationButton_button" onClick="proceed(1);" value="{$forwardlabel}" />
					</qtafi:USE_IF>
				</xsl:if>&#160;
			</div>
			&#160;
			<div class="col-xs-8">
				<xsl:if test="children">
					<xsl:call-template name="child-elements" />					
				</xsl:if>
			</div>
			&#160;
			<div class="col-xs-2  pull-right">
				<xsl:if test="$backwardlabel != 'null'">
					<qtafi:USE_IF variable="__page" value="2" runat="server" operator="IS_NOT">
						<input type="button" class="btn btn-primary NavigationButton_button" onClick="proceed(-1);" value="{$backwardlabel}" />
					</qtafi:USE_IF>
					&#160;
				</xsl:if>
			</div>
		</div>
</xsl:template>

<xsl:template match="ProgressBar">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<xsl:if test="not(//PIN) and not(//qtafi:PIN)">
		<xsl:variable name="percent" select="32" />
		<div class="row">
			<div class="col-md-offset-3 col-xs-6">
				<div class="progress">
					<div class="progress-bar progress-bar-success" role="progressbar" aria-valuenow="{$percent}" aria-valuemin="0" aria-valuemax="100" style="width: {$percent}%">
						<span class="sr-only">40% Complete (success)</span>
					</div>
				</div>
			</div>
		</div>
	</xsl:if>
	<xsl:if test="code">
		<xsl:copy-of select="code/node()" />
	</xsl:if>
</xsl:template>

<xsl:template match="HL">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)">
			<xsl:call-template name="depth" />
		</xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<div class="question{$depth} hl{$depth}" id="TA_{$quid}" style="page-break-inside: avoid;">
		<xsl:attribute name="uid">
			<xsl:value-of select="@uid"/>
		</xsl:attribute>
		<xsl:call-template name="infobar" />
		<xsl:call-template name="question-head" />
	</div>
	<xsl:for-each select="children">
		<xsl:call-template name="child-elements" />
	</xsl:for-each>
	
	<xsl:if test="code">
		<xsl:copy-of select="code/node()" />
	</xsl:if>
</xsl:template>

<xsl:template match="DATE">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<div class="question{$depth} date{$depth} dateinput" id="TA_{$quid}" style="page-break-inside: avoid;">
		<xsl:attribute name="uid">
			<xsl:value-of select="@uid"/>
		</xsl:attribute>
		<xsl:call-template name="infobar" />
		<xsl:call-template name="question-head" />
	
		<xsl:for-each select="variable">
			<div class="row">
				<xsl:copy-of select="node()" />&#160;
				<input type="text" value="2005/12"></input>
			</div>
		</xsl:for-each>
	</div>
	<script type="text/javascript">
	$('#rootform .dateinput input').datepicker({
		minViewMode: 1,
		format: "yyyy/mm",
		immediateUpdates: true,
		defaultViewDate: { year: 1977, month: 04 }
	});
	</script>
</xsl:template>


<xsl:template match="LABEL">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)">
			<xsl:call-template name="depth" />
		</xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	
	<div class="question{$depth} label{$depth}" id="TA_{$quid}">
		<xsl:copy-of select="text/node()" />
	</div>
	<xsl:for-each select="children">
		<xsl:call-template name="child-elements" />
	</xsl:for-each>
	<xsl:if test="code">
		<xsl:copy-of select="code/node()" />
	</xsl:if>
</xsl:template>

<xsl:template match="LOGIN">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<div class="question{$depth} pin{$depth}" id="TA_{$quid}" style="page-break-inside: avoid;">
		<xsl:attribute name="uid">
			<xsl:value-of select="@uid"/>
		</xsl:attribute>
		<xsl:call-template name="infobar" />
		<div class="row text-center">
			<button class="btn btn-default"><xsl:value-of select="layout/@loginButtonLabel" /></button>
		</div>
	</div>
</xsl:template>

<xsl:template match="CV">
	<xsl:choose>
		<xsl:when test="layout/@inputType = 'Pulldown'">
			<xsl:call-template name="CV-pulldown" />
		</xsl:when>
		<xsl:otherwise>
			<xsl:call-template name="CV-radio" />
		</xsl:otherwise>
	</xsl:choose>
	
	<xsl:if test="code">
		<xsl:copy-of select="code/node()" />
	</xsl:if>
</xsl:template>

<xsl:template name="elementPath">
   <xsl:for-each select="(ancestor-or-self::*)">/*[<xsl:value-of
     select="1+count(preceding-sibling::*)"/>]</xsl:for-each>
</xsl:template>

<xsl:template name="CV-pulldown">
</xsl:template>

<xsl:template name="CV-radio">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<div class="question{$depth} cv{$depth}" id="TA_{$quid}" style="page-break-inside: avoid;">
		<xsl:attribute name="uid">
			<xsl:value-of select="@uid"/>
		</xsl:attribute>
		<xsl:call-template name="infobar" />
		<xsl:call-template name="question-head" />
		<xsl:variable name="variable" select="variable/@name" />
		<xsl:for-each select="value[not(@missing) or @missing='false']|children">
			<xsl:variable name="odd_even" select="(position() mod 2) + 1"/>
			<xsl:if test="name() = 'children'">
				<div class="row">
					<div class="col-xs-12">
						<xsl:call-template name="child-elements" />
					</div>
				</div>
			</xsl:if>	
			<xsl:if test="(not(@missing) or @missing='false') and name()='value'">
				<div class="row bodyrow">
					
					<div class="col-xs-11">
						<xsl:attribute name="id">
							<xsl:text>value_</xsl:text><xsl:value-of select="$variable" /><xsl:text>_</xsl:text><xsl:value-of select="@key" />
						</xsl:attribute>
						<xsl:call-template name="label-for">
							<xsl:with-param name="for">
								<xsl:value-of select="$variable" />
								<xsl:value-of select="'-'" />
								<xsl:value-of select="@key" />
							</xsl:with-param>
						</xsl:call-template>
					</div>
					<div class="col-xs-1">
						<input type="radio">
							<xsl:attribute name="name">
								<xsl:value-of select="$variable" />
							</xsl:attribute>
							<xsl:attribute name="value">
								<xsl:value-of select="@key" />
							</xsl:attribute>
						</input>
						<xsl:value-of select="'&#160;'" />
					</div>
				</div>
			</xsl:if>
		</xsl:for-each>
	</div>
</xsl:template>

<!-- 
 if contains <qtafi:xx dont use label for
-->
<xsl:template name="label-for">
	<xsl:param name="for" />
	<xsl:choose>
		<xsl:when test="qtafi:TE or qtafi:ME or qtafi:TL or qtafi:CV or qtafi:MD or qtafi:OR">
			<label>
				<!-- <xsl:apply-templates select="."/> -->
				<xsl:copy-of select="node()" />
			</label>
		</xsl:when>
		<xsl:otherwise>
			<label>
				<xsl:attribute name="for">
					<xsl:value-of select="$for" />
				</xsl:attribute>
				<!-- <xsl:apply-templates select="."/> -->
				<xsl:copy-of select="node()" />
			</label>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template match="MD" name="MD_element">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	
	<div class="question{$depth} md{$depth}" id="TA_{$quid}" style="page-break-inside: avoid;">
		<xsl:attribute name="uid">
			<xsl:value-of select="@uid"/>
		</xsl:attribute>
		<xsl:call-template name="infobar" />
		<xsl:call-template name="question-head" />
		<xsl:for-each select="variable|children">
			<xsl:variable name="odd" select="(position() mod 2) + 1"/>
			<xsl:if test="name() = 'children' and count(./*) &gt; 0">
				<div class="row">
					<div class="col-xs-12">
						<xsl:call-template name="child-elements" />
					</div>
				</div>
			</xsl:if>
			<xsl:if test="not(@subitem) and name() = 'variable'">
				<div class="row bodyrow">
					<div class="col-xs-11" >
						<xsl:attribute name="id">
							<xsl:text>variable_</xsl:text><xsl:value-of select="@name" />
						</xsl:attribute>
						<xsl:call-template name="label-for">
							<xsl:with-param name="for">
								<xsl:value-of select="@name" />
								<xsl:value-of select="'-1'" />
							</xsl:with-param>
						</xsl:call-template>
					</div>
					<div class="col-xs-1">
						<input type="checkbox">
							<xsl:attribute name="name">
								<xsl:value-of select="@name" />
							</xsl:attribute>
							<xsl:attribute name="value">
								<xsl:value-of select="1" />
							</xsl:attribute>
						</input>
					</div>
				</div>
			</xsl:if>
		</xsl:for-each>
	</div>
	<xsl:if test="code">
		<xsl:copy-of select="code/node()" />
	</xsl:if>
</xsl:template>

<xsl:template match="OR">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	
	<div class="question{$depth} or{$depth}" id="TA_{$quid}" style="page-break-inside: avoid;">
		<xsl:attribute name="uid">
			<xsl:value-of select="@uid"/>
		</xsl:attribute>
		<xsl:call-template name="infobar" />
		<xsl:call-template name="question-head" />
		<xsl:variable name="value-count"><xsl:value-of select="count(value[not(@missing) or @missing='false'])" /></xsl:variable>
		<!-- value labels -->
		<div class="row">
			<div>
				<xsl:attribute name="class">
					<xsl:value-of select="'col-xs-'" />
					<xsl:value-of select="(12 - $value-count)" />
					<xsl:value-of select="' '" />
				</xsl:attribute>
				<label>
					<xsl:value-of select="'&#160;'" />
				</label>
			</div>
			
			<xsl:for-each select="value[not(@missing) or @missing='false']">
				<div class="col-xs-1 OR_head{$depth}" index="{position()}" style="text-align:center;">
					<xsl:attribute name="id">
						<xsl:text>value_</xsl:text><xsl:value-of select="../variable[1]/@name"/><xsl:text>_</xsl:text><xsl:value-of select="@key" />
					</xsl:attribute>
					<xsl:if test="../width">
						<xsl:attribute name="width">
							<xsl:value-of select="../width" />
						</xsl:attribute>
					</xsl:if>
					<xsl:copy-of select="node()" />
					<xsl:value-of select="'&#160;'" />
				</div>
			</xsl:for-each>
		</div>
		<!-- keys -->
		<div class="row">
			<div>
				<xsl:attribute name="class">
					<xsl:value-of select="'col-xs-'" />
					<xsl:value-of select="(12 - $value-count)" />
					<xsl:value-of select="' '" />
				</xsl:attribute>
				<label>
			
					<xsl:value-of select="'&#160;'" />
				</label>
			</div>
		
			<xsl:for-each select="value[not(@missing) or @missing='false']">
				<div class="col-xs-1 center-block OR_value{$depth}" index="{position()}" style="text-align:center;">
					<xsl:if test="not(@hidekey) or @hidekey='false'">
						<xsl:value-of select="@key" />
					</xsl:if>
					<xsl:value-of select="'&#160;'" />
				</div>
			</xsl:for-each>
			
		</div>
		<xsl:for-each select="variable|children">
			<xsl:variable name="label" select="@label" />
			<xsl:variable name="variable" select="@name" />
			<xsl:if test="name() = 'children' and count(./*) &gt; 0">
				<div class="row">
					<xsl:call-template name="child-elements" />
				</div>
			</xsl:if>
			<xsl:if test="name() = 'variable'">
				<div class="row bodyrow">
					<div>
						<xsl:attribute name="class">
							<xsl:value-of select="'col-xs-'" />
							<xsl:value-of select="(12 - $value-count)" />
							<xsl:value-of select="' '" />
						</xsl:attribute>
						<xsl:attribute name="id">
							<xsl:text>variable_</xsl:text><xsl:value-of select="$variable" />
						</xsl:attribute>
						<label>
							<xsl:copy-of select="node()" />
							<xsl:value-of select="'&#160;'" />
						</label>
					</div>
					<xsl:for-each select="../value[not(@missing) or @missing='false']">
						<div class="col-xs-1" index="{position()}" style="text-align:center;">
							<input type="radio">
								<xsl:attribute name="name">
									<xsl:value-of select="$variable" />
								</xsl:attribute>
								<xsl:attribute name="value">
									<xsl:value-of select="@key" />
								</xsl:attribute>
							</input>
							<xsl:value-of select="'&#160;'" />
						</div>
					</xsl:for-each>
				</div>
			</xsl:if>
		</xsl:for-each>
	</div>
	<xsl:if test="code">
		<xsl:copy-of select="code/node()" />
	</xsl:if>
</xsl:template>

<xsl:template match="ME">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<div class="question{$depth} me{$depth}" id="TA_{$quid}" style="page-break-inside: avoid;">
		<xsl:attribute name="uid">
			<xsl:value-of select="@uid"/>
		</xsl:attribute>
		<xsl:call-template name="infobar" />
		<xsl:call-template name="question-head" />
		<xsl:for-each select="variable|children">
			<xsl:variable name="odd_even" select="(position() mod 2) + 1"/>
			<xsl:if test="name() = 'children'">
				<div class="row">
					<xsl:call-template name="child-elements" />
				</div>
			</xsl:if>
			<xsl:if test="name() = 'variable'">
				<div class="row bodyrow">
					<input type="number" >
						<xsl:attribute name="variable">
							<xsl:value-of select="@name" />
						</xsl:attribute>
						<xsl:attribute name="max-length">
							<xsl:value-of select="../layout/@inputSize" />
						</xsl:attribute>
					</input>
					&#160;
					<xsl:call-template name="label-for">
						<xsl:with-param name="for">
							<xsl:value-of select="@name" />
						</xsl:with-param>
					</xsl:call-template>
				</div>
			</xsl:if>
		</xsl:for-each>
	</div>
	<xsl:if test="code">
		<xsl:copy-of select="code/node()" />
	</xsl:if>
</xsl:template>

<xsl:template match="TE" name="TE_element">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<div class="question{$depth} te{$depth}" id="TA_{$quid}" style="page-break-inside: avoid;">
		<xsl:attribute name="uid">
			<xsl:value-of select="@uid"/>
		</xsl:attribute>
		<xsl:call-template name="infobar" />
		<xsl:call-template name="question-head" />
		<xsl:choose>
			<xsl:when test="layout/@inputLines = '1'">
				<xsl:for-each select="variable">
					<div class="row bodyrow">
						<xsl:call-template name="label-for">
							<xsl:with-param name="for">
								<xsl:value-of select="@name" />
							</xsl:with-param>
						</xsl:call-template>
						&#160;
						<input type="text">
							<xsl:attribute name="variable">
								<xsl:value-of select="@name" />
							</xsl:attribute>
							<xsl:attribute name="size">
								<xsl:value-of select="../layout/@inputSize" />
							</xsl:attribute>
						</input>
					</div>
				</xsl:for-each>
			</xsl:when>
			<xsl:otherwise>
				<xsl:for-each select="variable">
					<div class="row bodyrow">
						<xsl:call-template name="label-for">
							<xsl:with-param name="for">
								<xsl:value-of select="@name" />
							</xsl:with-param>
						</xsl:call-template>
						&#160;
						<textarea type="text">
							<xsl:attribute name="variable">
								<xsl:value-of select="@name" />
							</xsl:attribute>
							<xsl:attribute name="width">
								<xsl:value-of select="../layout/@inputSize" />
							</xsl:attribute>
							<xsl:attribute name="height">
								<xsl:value-of select="../layout/@inputLines" />
							</xsl:attribute>
							&#160;
						</textarea>
					</div>
				</xsl:for-each>
			</xsl:otherwise>
		</xsl:choose>
	</div>
	<xsl:if test="code">
		<xsl:copy-of select="code/node()" />
	</xsl:if>
</xsl:template>


<xsl:template match="RE|CONTAINER">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	
	<div class="question{$depth} container{$depth}" id="TA_{$quid}" style="page-break-inside: avoid;">
		<xsl:attribute name="uid">
			<xsl:value-of select="@uid"/>
		</xsl:attribute>
		
		<xsl:call-template name="infobar" />
		<xsl:call-template name="question-head" />
		<xsl:for-each select="children">
			<div class="row">
				<xsl:call-template name="child-elements" />
			</div>
		</xsl:for-each>
	</div>
	<xsl:if test="code">
		<xsl:copy-of select="code/node()" />
	</xsl:if>

</xsl:template>

<xsl:template name="child-elements">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="(@level + 1)"/></xsl:if>
	</xsl:variable>
	<div>
		<xsl:if test="@layout = 'horizontal'">
			<table width="100%">
				<tr>
					<xsl:for-each select="*">
						<xsl:if test="not(invisible)">
							<td class="item{$depth}">
								<div>
									<xsl:attribute name="id">
										<xsl:value-of select="@id" />
									</xsl:attribute>
									<xsl:apply-templates select="."/>
								</div>
							</td>
						</xsl:if>
					</xsl:for-each>
				</tr>
			</table>
		</xsl:if>
		<xsl:if test="not(layout = 'horizontal')">
			<div class="row item{$depth}">
				<xsl:for-each select="*">
					<xsl:if test="not(invisible)">
						<div>
							<xsl:attribute name="id">
								<xsl:value-of select="@id" />
							</xsl:attribute>
							<xsl:apply-templates select="."/>
						</div>
					</xsl:if>
				</xsl:for-each>
			</div>
		</xsl:if>
	</div>
</xsl:template>

<xsl:template match="XHTML">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	
	<div class="question{$depth} xhtml{$depth}" id="TA_{$quid}" style="page-break-inside: avoid;">
		<xsl:attribute name="uid">
			<xsl:value-of select="@uid"/>
		</xsl:attribute>
		<xsl:call-template name="infobar" />
		<xsl:call-template name="question-head" />
		<xsl:if test="code">
			<div class="XHTML_code{$depth}">
				<xsl:copy-of select="code/node()" />
			</div>
		</xsl:if>
		<xsl:for-each select="children">
			<div class="row">
				<xsl:call-template name="child-elements" />
			</div>
		</xsl:for-each>
	</div>
</xsl:template>

<xsl:template match="RAW">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<div class="question{$depth} raw{$depth}" id="TA_{$quid}" style="page-break-inside: avoid;">
		<xsl:attribute name="uid">
			<xsl:value-of select="@uid"/>
		</xsl:attribute>
		<xsl:call-template name="infobar" />
		<xsl:if test="code">
			<xsl:copy-of select="code/node()" />
		</xsl:if>
	</div>
</xsl:template>

<xsl:template match="IMAGE">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	
	<div class="question{$depth} image{$depth}" id="TA_{$quid}" style="page-break-inside: avoid;">
		<xsl:attribute name="uid">
			<xsl:value-of select="@uid"/>
		</xsl:attribute>
		<xsl:call-template name="infobar" />
		<xsl:if test="code">
			<div class="row">
				<xsl:copy-of select="code/node()" />
			</div>
		</xsl:if>
	</div>
</xsl:template>


<xsl:template match="TL">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<div class="question{$depth} tl{$depth}" id="TA_{$quid}" style="page-break-inside: avoid;">
	
		<xsl:call-template name="infobar" />
		<xsl:call-template name="question-head" />
		<xsl:for-each select="variable|children">
			<xsl:if test="name() = 'children'">
				<div class="row">
					<xsl:call-template name="child-elements" />
				</div>
			</xsl:if>
			<xsl:if test="name() = 'variable'">
				<div class="row bodyrow">
					<div>
						<qtafi:TL>
							<xsl:attribute name="variable">
								<xsl:value-of select="@name" />
							</xsl:attribute>
							<xsl:if test="../width">
								<xsl:attribute name="width">
									<xsl:value-of select="../width" />
								</xsl:attribute>
							</xsl:if>
							<xsl:if test="../height">
								<xsl:attribute name="height">
									<xsl:value-of select="../height" />
								</xsl:attribute>
							</xsl:if>
						</qtafi:TL>
						&#160;&#160;
						<xsl:call-template name="label-for">
							<xsl:with-param name="for">
								<xsl:value-of select="@name" />
							</xsl:with-param>
						</xsl:call-template>
					</div>
				</div>
			</xsl:if>
		</xsl:for-each>
	</div>
	<xsl:if test="code">
		<xsl:copy-of select="code/node()" />
	</xsl:if>

</xsl:template>

<xsl:template match="COLUMNCONTAINER">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	
	<xsl:variable name="columncount" select="count(children/CV|children/OR/value[not(@missing) or @missing !='true']|children/MD|children/TE|children/ME|children/LABEL)" />
	<xsl:variable name="rowcount"><xsl:call-template name="row-count" /></xsl:variable>
	<!--<xsl:message>Rowcount=<xsl:value-of select="$rowcount" /></xsl:message>
	<xsl:message>Colcount=<xsl:value-of select="$columncount" /></xsl:message> -->
	<div class="question{$depth} cc{$depth}" id="TA_{$quid}" style="page-break-inside: avoid;">
		<xsl:call-template name="infobar" />
		<xsl:call-template name="question-head" />
		<div class="row">
			<xsl:for-each select="children/*">
				<div index="{position()}">
					<xsl:choose>
						<xsl:when test="name() = 'OR'">
							<xsl:attribute name="class">
								<xsl:value-of select="'col-xs-'" />
								<xsl:value-of select="count(value[@missing != 'true' or not(@missing)])" />
								<xsl:value-of select="' '" />
								<xsl:value-of select="'CC_columnlabel'+ $depth" />
							</xsl:attribute>
						</xsl:when>
						<xsl:when test="name() = 'LABEL' and position()!=last()">
							<xsl:attribute name="class">
								<xsl:value-of select="'col-xs-2 '" />
								<xsl:value-of select="'CC_columnlabel'" />
								<xsl:value-of select="$depth" />
							</xsl:attribute>
						</xsl:when>
						<xsl:when test="name() = 'LABEL' and position()=last()">
							<xsl:attribute name="class">
								<xsl:value-of select="'CC_columnlabel'" />
								<xsl:value-of select="$depth" />
							</xsl:attribute>
						</xsl:when>
						<xsl:when test="name() = 'TE'">
							<xsl:attribute name="class">
								<xsl:value-of select="'col-xs-3 '" />
								<xsl:value-of select="'CC_columnlabel'" />
								<xsl:value-of select="$depth" />
							</xsl:attribute>
						</xsl:when>
						<xsl:when test="name() = 'ME'">
							<xsl:attribute name="class">
								<xsl:value-of select="'col-xs-2 '" />
								<xsl:value-of select="'CC_columnlabel'" />
								<xsl:value-of select="$depth" />
							</xsl:attribute>
						</xsl:when>
						<xsl:otherwise>
							<xsl:attribute name="class">
								<xsl:value-of select="'col-xs-1 '" />
								<xsl:value-of select="'CC_columnlabel'" />
								<xsl:value-of select="$depth" />
							</xsl:attribute>
						</xsl:otherwise>
					</xsl:choose>
					<xsl:copy-of select="text/node()" />&#160;
				</div>
			</xsl:for-each>
		</div>
		<xsl:call-template name="ORHEAD" />
		<xsl:call-template name="COLUMN-ROW">
			<xsl:with-param name="rownum" select="1" />
			<xsl:with-param name="rowcount" select="$rowcount" />
			<xsl:with-param name="columncount" select="$columncount" />
		</xsl:call-template>
	</div>
	<xsl:if test="code">
		<xsl:copy-of select="code/node()" />
	</xsl:if>

</xsl:template>

<xsl:template name="row-count">
	<xsl:if test="name(children/*[1]) = 'CV'">
		<xsl:value-of select="count(children/CV[1]/value[@missing != 'true' or not(@missing)])" />
	</xsl:if>
	<xsl:if test="name(children/*[1]) = 'LABEL'">
		<xsl:value-of select="count(children/LABEL[1]/label)" />
	</xsl:if>
	<xsl:if test="name(children/*[1]) != 'CV' and name(children/*[1]) != 'LABEL'">
		<xsl:value-of select="count(children/*[1]/variable)" />
	</xsl:if>
</xsl:template>

<xsl:template name="COLUMN-ROW">
	<xsl:param name="rownum" />
	<xsl:param name="rowcount" />
	<xsl:param name="columncount" />
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="labelid" select="generate-id(children/LABEL/label[$rownum])" />
	<xsl:variable name="odd_even" select="($rownum mod 2) + 1"/>
	
	<xsl:for-each select="children/LABEL/*[name()='label' or name()='children']">
		<xsl:variable name="dd" select="generate-id()"/>
		<xsl:if test="$dd = $labelid and name(preceding-sibling::*[1]) = 'children'">
			<div class="row">
				<div class="col-xs-{$columncount}">
					<xsl:copy-of select="preceding-sibling::*[1]/LABEL/text/node()" />
				</div>
			</div>
		</xsl:if>
	</xsl:for-each>
	<div class="row bodyrow">
		<xsl:for-each select="children/*">
			<xsl:choose>
				<xsl:when test="name() = 'LABEL' and position()=last()">
					<div>
						<label>
							<xsl:copy-of select="label[$rownum]/node()" />
						</label>
					</div>
				</xsl:when>
				<xsl:when test="name() = 'LABEL' and position()!=last()">
					<div class="col-xs-2">
						<label>
							<xsl:copy-of select="label[$rownum]/node()" />
						</label>
					</div>
				</xsl:when>
				<xsl:when test="name() = 'CV'">
					<div class="col-xs-1">
						<xsl:if test="value[$rownum]">
							<qtafi:CV>
								<xsl:attribute name="variable"><xsl:value-of select="variable/@name" /></xsl:attribute>
								<xsl:attribute name="value"><xsl:value-of select="value[$rownum]/@key" /></xsl:attribute>
							</qtafi:CV>
						</xsl:if>
					</div>
				</xsl:when>
				<xsl:when test="name() = 'MD'">
					<div class="col-xs-1">
						<xsl:if test="variable[$rownum]">
							<qtafi:MD>
								<xsl:attribute name="variable"><xsl:value-of select="variable[$rownum]/@name" /></xsl:attribute>
								<xsl:attribute name="value"><xsl:value-of select="value[1]/@key" /></xsl:attribute>
							</qtafi:MD>
						</xsl:if>
					</div>
				</xsl:when>
				<xsl:when test="name() = 'OR'">
					<xsl:variable name="var" select="variable[$rownum]/@name" />
					<xsl:for-each select="value[(not(@missing) or @missing='false')]">
						<div class="col-xs-1" style="text-align:center;">
							<xsl:if test="../variable[$rownum]">
								<qtafi:OR>
									<xsl:attribute name="variable"><xsl:value-of select="$var" /></xsl:attribute>
									<xsl:attribute name="value"><xsl:value-of select="@key" /></xsl:attribute>
								</qtafi:OR>
							</xsl:if>
						</div>
					</xsl:for-each>
				</xsl:when>
				<xsl:when test="name() = 'TE'">
					<div class="col-xs-3">
						<xsl:if test="variable[$rownum]">
							<qtafi:TE>
								<xsl:attribute name="variable"><xsl:value-of select="variable[$rownum]/@name" /></xsl:attribute>
								<xsl:attribute name="size"><xsl:value-of select="size" /></xsl:attribute>
							</qtafi:TE>
						</xsl:if>
					</div>
				</xsl:when>
				<xsl:when test="name() = 'ME'">
					<div class="col-xs-2">
						<xsl:if test="variable[$rownum]">
							<qtafi:ME>
								<xsl:attribute name="variable"><xsl:value-of select="variable[$rownum]/@name" /></xsl:attribute>
								<xsl:attribute name="size"><xsl:value-of select="size" /></xsl:attribute>
							</qtafi:ME>
						</xsl:if>
					</div>
				</xsl:when>
			</xsl:choose>
		</xsl:for-each>
	</div>
	<xsl:if test="$rownum = $rowcount">
		<xsl:for-each select="children/LABEL/*[name()='label' or name()='children']">
			<xsl:variable name="dd" select="generate-id()"/>
			<xsl:if test="$dd = $labelid and name(following-sibling::*[1]) = 'children'">
				<div class="row">
					<div class="col-xs-2">
						<xsl:copy-of select="following-sibling::*[1]/LABEL/text/node()" />
					</div>
				</div>
			</xsl:if>
		</xsl:for-each>
	</xsl:if>
	<xsl:if test="$rownum &lt; $rowcount">
		<xsl:call-template name="COLUMN-ROW">
			<xsl:with-param name="rownum" select="$rownum + 1" />
			<xsl:with-param name="rowcount" select="$rowcount" />
			<xsl:with-param name="columncount" select="$columncount" />
		</xsl:call-template>
	</xsl:if>
</xsl:template>

<xsl:template name="ORHEAD">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:if test="children/OR">
		<div class="row">
			<xsl:for-each select="children/*">
				<xsl:choose>
					<xsl:when test="name() = 'OR'">
						<xsl:for-each select="value[@missing != 'true' or not(@missing)]">
							<div style="text-align:center;" class="col-xs-1 OR_head{($depth + 1)}"><xsl:copy-of select="node()" />&#160;</div>	
						</xsl:for-each>
					</xsl:when>
					<xsl:when test="name() = 'LABEL' and position()=last()">
						<div>&#160;</div>
					</xsl:when>
					<xsl:when test="name() = 'LABEL' and position()!=last()">
						<div class="col-xs-2">&#160;</div>
					</xsl:when>
					<xsl:when test="name() = 'TE' and position()!=last()">
						<div class="col-xs-3">&#160;</div>
					</xsl:when>
					<xsl:when test="name() = 'ME' and position()!=last()">
						<div class="col-xs-2">&#160;</div>
					</xsl:when>
					<xsl:otherwise>
						<div class="col-xs-1">&#160;</div>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:for-each>
		</div>
		<div class="row">
			<xsl:for-each select="children/*">
				<xsl:choose>
					<xsl:when test="name() = 'OR'">
						<xsl:for-each select="value[@missing != 'true' or not(@missing)]">
							<div style="text-align:center;" class="col-xs-1 OR_value{($depth + 1)}">
								<xsl:if test="not(@hidekey) or @hidekey='false'">
									<xsl:value-of select="@key" />
								</xsl:if>
								&#160;
							</div>	
						</xsl:for-each>
					</xsl:when>
					<xsl:when test="name() = 'LABEL' and position()=last()">
						<div>&#160;</div>
					</xsl:when>
					<xsl:when test="name() = 'LABEL' and position()!=last()">
						<div class="col-xs-2">&#160;</div>
					</xsl:when>
					<xsl:when test="name() = 'TE' and position()!=last()">
						<div class="col-xs-3">&#160;</div>
					</xsl:when>
					<xsl:when test="name() = 'ME' and position()!=last()">
						<div class="col-xs-2">&#160;</div>
					</xsl:when>
					<xsl:otherwise>
						<div class="col-xs-1">&#160;</div>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:for-each>
		</div>
	</xsl:if>
</xsl:template>

<xsl:template name="question-head">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:if test="text/* or string-length(normalize-space(text)) &gt; 0">
		<div class="row question_head1">
			<xsl:attribute name="id">
				<xsl:value-of select="@uid" />
			</xsl:attribute>
			<xsl:choose>
				<xsl:when test="number/* or string-length(normalize-space(number)) &gt; 0">
					<div class="col-xs-11 question_text{$depth}" id="text_{@id}">
						<h1><xsl:copy-of select="text/node()"/></h1>
					</div>
					<div class="col-xs-1 question_nr{$depth}" id="nr_{@id}">
						<h2><xsl:copy-of select="number/node()"/></h2>
					</div>
				</xsl:when>
				<xsl:otherwise>
					<div class="question_text{$depth}" id="text_{@id}">
						<h1><xsl:copy-of select="text/node()"/></h1>
					</div>
				</xsl:otherwise>
			</xsl:choose>
		</div>
	</xsl:if>
	<xsl:if test="moretext/* or string-length(normalize-space(moretext)) &gt; 0">
		<div class="row">
			<xsl:attribute name="id">
				<xsl:value-of select="@uid" />
			</xsl:attribute>
			<div class="question_more{$depth}" id="moretext_{@id}">
				<h3><xsl:copy-of select="moretext/node()"/></h3>
			</div>
		</div>
	</xsl:if>
</xsl:template>

<xsl:template match="Setup">
</xsl:template>

<xsl:template match="Finish">
</xsl:template>

<xsl:template name="depth">
	<xsl:value-of select="count(ancestor::*[name() != 'MODULE' and name() != 'children']) - 2"/>
</xsl:template>


</xsl:stylesheet>