/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.report;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.apache.commons.dbcp2.BasicDataSource;
import org.eclipse.rap.rwt.RWT;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import de.eoda.qtafi.surveymanager.lang.Messages;
import de.eoda.qtafi.surveymanager.model.Project;
import de.eoda.qtafi.surveymanager.rap.export.Report;
import de.eoda.qtafi.surveymanager.rap.export.TabularReportData;
import de.eoda.qtafi.surveymanager.rap.export.TabularReportData.Cell;
import de.tinu.qtafi4.model.Question;
import de.tinu.qtafi4.model.Value;
import de.tinu.qtafi4.model.Variable;

public class DATEReportItem extends ReportItem {

	private static final long serialVersionUID = 1L;
	
	Text titleInput;
	Button countbutton, percentbutton;
	ReportData data;
	boolean removeBreak = false;
	
	public DATEReportItem(Composite parent, int style, Question q) {
		super(parent, style, q);
		data = new ReportData();
		this.options = Report.OPTION_COUNT;
	}

	public void createDialog(Composite parent){
		Label titlelabel = new Label(parent, SWT.NONE);
		titlelabel.setText(Messages.get().ReportPage_Title);
		titlelabel.setLayoutData(new GridData(200, 22));
		
		titleInput = new Text(parent, SWT.BORDER);
		titleInput.setEditable(false);
		titleInput.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		titleInput.setText(this.title);
		
		Label showlabel = new Label(parent, SWT.NONE);
		showlabel.setText(Messages.get().ReportPage_Show);
		GridData gd = new GridData();
		gd.verticalSpan = 2;
		showlabel.setLayoutData(gd);
		
		percentbutton = new Button(parent, SWT.RADIO);
		percentbutton.setText(Messages.get().ReportPage_Percent);
		percentbutton.setSelection((options & Report.OPTION_PERCENT) == Report.OPTION_PERCENT);
		
		countbutton = new Button(parent, SWT.RADIO);
		countbutton.setText(Messages.get().ReportPage_Count);
		countbutton.setSelection((options & Report.OPTION_COUNT) == Report.OPTION_COUNT);
		
		Label brelabel = new Label(parent, SWT.NONE);
		brelabel.setText(Messages.get().ReportPage_Break);
		brelabel.setLayoutData(new GridData(200, 22));
		
		final Text breaklist = new Text(parent, SWT.BORDER | SWT.READ_ONLY);
		breaklist.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		if(hasBreak()) breaklist.setText(breakElement.toString());
		else breaklist.setText("--");
		
		Label empty = new Label(parent, SWT.NONE);
		empty.setText("");
		
		Button trash = new Button(parent, SWT.PUSH);
		trash.setText(Messages.get().ReportPage_RemoveBreak);
		trash.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				removeBreak = true;
				breaklist.setText("--");
			}
		});
		
		/*
		Label filterlabel = new Label(parent, SWT.NONE);
		filterlabel.setText("Filter");
		filterlabel.setLayoutData(new GridData(200, 22));
		
		List filterlist = new List(parent, SWT.BORDER | SWT.V_SCROLL);
		filterlist.setLayoutData(new GridData(GridData.FILL_BOTH));
		*/
	}
	
	@Override
	public void onOk() {
		this.type =  ReportItem.TYPE_FREQUENCY;
		this.title = titleInput.getText();
		this.options = countbutton.getSelection() ? Report.OPTION_COUNT: Report.OPTION_PERCENT;
		if(removeBreak){
			breakElement = null;
			removeBreak = false;
		}
		super.onOk();
	}
	
	@Override
	public void toDocument(Report doc) {
		if(hasBreak()){
			String bs = breakElement.subject;
			if((options & Report.OPTION_COUNT) == Report.OPTION_COUNT) {
				doc.appendHead(this, Messages.get().ReportPage_BreakBy + " " + bs, "count");
			}
			else if((options & Report.OPTION_PERCENT) == Report.OPTION_PERCENT){
				doc.appendHead(this, Messages.get().ReportPage_BreakBy + " " + bs, "percent");
			}
			if(breakElement.type.equals("CV")){
				for(Variable v: q.variables){
					if(! v.getCleanedLabel().isEmpty()) doc.appendSubHead(v.getCleanedLabel());
					doc.appendTable(this.tableXcv(v), options);
				}
			}
			else if(breakElement.type.equals("MD")){
				for(Variable v: q.variables){
					if(! v.getCleanedLabel().isEmpty()) doc.appendSubHead(v.getCleanedLabel());
					doc.appendTable(this.tableXmd(v), options);
				}
			}
		}
		else {
			if((options & Report.OPTION_COUNT) == Report.OPTION_COUNT) {
				doc.appendHead(this, "count");
			}
			else if((options & Report.OPTION_PERCENT) == Report.OPTION_PERCENT){
				doc.appendHead(this, "percent");
			}
			for(Variable v: q.variables){
				if(! v.getCleanedLabel().isEmpty()) doc.appendSubHead(v.getCleanedLabel());
				doc.appendTable(this.table(v), options);
			}
		}
		//doc.appendFoot(q.text);
	}
	
	TabularReportData table(Variable v){
		TabularReportData table = new TabularReportData(options);
		Project project = (Project) RWT.getUISession().getHttpSession().getAttribute("project");
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		Connection con = null;
		int overall = data.get(v.name, "*");
		try { 
			con = ds.getConnection();
			StringBuffer buf = new StringBuffer();
			buf.append("SELECT count(__n) as count, `" +v.name +"` as `" + v.name + "` \n");
			buf.append("FROM `" + project.name + "` \n");
			buf.append("WHERE NOT `" + v.name + "` IS NULL AND CHAR_LENGTH(`" + v.name + "`) > 0 \n");
			buf.append("GROUP BY `" + v.name + "`");
			System.out.println(buf.toString());
			PreparedStatement pst = con.prepareStatement(buf.toString());
			ResultSet res = pst.executeQuery();
			while(res.next()){
				double d = res.getDouble("count");
				table.addRow(res.getString(v.name), new Cell((int)d, (d / (double)overall) * 100.));
			}
			res.close();
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return table;
	}
	
	TabularReportData tableXcv(Variable v){
		TabularReportData table = new TabularReportData(options);
		Project project = (Project) RWT.getUISession().getHttpSession().getAttribute("project");
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		Connection con = null;
		Question cv = breakElement;
		for(Value bv: cv.values){
			table.addColumnName(bv.getCleanedLabel());
		}
		try { 
			con = ds.getConnection();
			StringBuffer buf = new StringBuffer();
			buf.append("SELECT count(__n) as count, `" +v.name +"` as " + v.name + " \n");
			buf.append("FROM `" + project.name + "` \n");
			buf.append("WHERE NOT `" + v.name + "` IS NULL AND CHAR_LENGTH(`" + v.name + "`) > 0 \n");
			buf.append("GROUP BY `" + v.name + "`");
			System.out.println(buf.toString());
			PreparedStatement pst = con.prepareStatement(buf.toString());
			ResultSet res =  pst.executeQuery();
			while(res.next()){
				String key = res.getString(v.name);
				double cc = res.getDouble("count");
				AddressTuple rt = new AddressTuple(q.variables.get(0).name, key);
				Cell[] cells = new Cell[cv.values.size()];
				int cn = 0;
				for(Value bv: cv.values){
					double breakoverall = data.get(new Address(new AddressTuple(v.name, "*"), new AddressTuple(cv.variables.get(0).name, bv.key)));
					int c = data.get(new Address(rt, new AddressTuple(cv.variables.get(0).name, bv.key)));
					
					cells[cn] = new Cell(c, (c / (double)breakoverall) * 100.);
					cn++;
				}
				table.addRow(res.getString(v.name), cells);
			}
			res.close();
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return table;
	}
	
	TabularReportData tableXmd(Variable v){
		TabularReportData table = new TabularReportData(options);
		Project project = (Project) RWT.getUISession().getHttpSession().getAttribute("project");
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		int overall = data.get(v.name, "*");
		Connection con = null;
		Question md = breakElement;
		for(Variable bv: md.variables){
			table.addColumnName(bv.getCleanedLabel());
		}
		try { 
			con = ds.getConnection();
			StringBuffer buf = new StringBuffer();
			buf.append("SELECT count(__n) as count, `" +v.name +"` as " + v.name + " \n");
			buf.append("FROM `" + project.name + "` \n");
			buf.append("WHERE NOT `" + v.name + "` IS NULL AND CHAR_LENGTH(`" + v.name + "`) > 0 \n");
			buf.append("GROUP BY `" + v.name + "`");
			System.out.println(buf.toString());
			PreparedStatement pst = con.prepareStatement(buf.toString());
			ResultSet res =  pst.executeQuery();
			while(res.next()){
				String key = res.getString(v.name);
				double cc = res.getDouble("count");
				AddressTuple rt = new AddressTuple(v.name, key);
				Cell[] cells = new Cell[md.variables.size()];
				int cn = 0;
				for(Variable bv: md.variables){
					double breakoverall = data.get(new Address(new AddressTuple(v.name, "*"), new AddressTuple(bv.name, "1")));
					int c = data.get(new Address(rt, new AddressTuple(bv.name, "1")));
					cells[cn] = new Cell(c, (c / (double)breakoverall) * 100.);					
					cn++;
				}
				table.addRow(res.getString(v.name), cells);
			}
			res.close();
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return table;
	}
}
