/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.report;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import de.eoda.qtafi.surveymanager.lang.Messages;
import de.eoda.qtafi.surveymanager.rap.export.ChartData;
import de.eoda.qtafi.surveymanager.rap.export.Report;
import de.eoda.qtafi.surveymanager.rap.export.TabularReportData;
import de.eoda.qtafi.surveymanager.rap.export.TabularReportData.Cell;
import de.tinu.qtafi4.model.Question;
import de.tinu.qtafi4.model.Value;
import de.tinu.qtafi4.model.Variable;

public class MDReportItem extends ReportItem {

	private static final long serialVersionUID = 1L;
	
	Button chartbutton, totalbutton, countbutton, percentbutton;
	Text titleInput;
	boolean removeBreak = false;
	ReportData data;
	
	public MDReportItem(Composite parent, int style, Question q) {
		super(parent, style, q);
		data = new ReportData();
	}

	public void createDialog(Composite parent){
		Label titlelabel = new Label(parent, SWT.NONE);
		titlelabel.setText(Messages.get().ReportPage_Title);
		titlelabel.setLayoutData(new GridData(200, 22));
		
		titleInput = new Text(parent, SWT.BORDER);
		titleInput.setEditable(false);
		titleInput.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		titleInput.setText(this.title);
		
		Label chartlabel = new Label(parent, SWT.NONE);
		chartlabel.setText(Messages.get().ReportPage_Layout);
		chartlabel.setLayoutData(new GridData(200, 22));
		
		chartbutton = new Button(parent, SWT.CHECK);
		chartbutton.setText(Messages.get().ReportPage_Chart);
		chartbutton.setSelection(type == ReportItem.TYPE_CHART);
		
		Label totallabel = new Label(parent, SWT.NONE);
		totallabel.setText(Messages.get().ReportPage_Totals);
		totallabel.setLayoutData(new GridData(200, 22));
		
		totalbutton = new Button(parent, SWT.CHECK);
		totalbutton.setText(Messages.get().ReportPage_ShowTotals);
		totalbutton.setSelection((options & Report.OPTION_TOTALS) == Report.OPTION_TOTALS);
		
		Label showlabel = new Label(parent, SWT.NONE);
		showlabel.setText(Messages.get().ReportPage_Show);
		GridData gd = new GridData();
		gd.verticalSpan = 2;
		showlabel.setLayoutData(gd);
		
		percentbutton = new Button(parent, SWT.RADIO);
		percentbutton.setText(Messages.get().ReportPage_Percent);
		percentbutton.setSelection((options & Report.OPTION_PERCENT) == Report.OPTION_PERCENT);
		
		countbutton = new Button(parent, SWT.RADIO);
		countbutton.setText(Messages.get().ReportPage_Count);
		countbutton.setSelection((options & Report.OPTION_COUNT) == Report.OPTION_COUNT);
		
		Label brelabel = new Label(parent, SWT.NONE);
		brelabel.setText(Messages.get().ReportPage_Break);
		brelabel.setLayoutData(new GridData(200, 22));
		
		final Text breaklist = new Text(parent, SWT.BORDER | SWT.READ_ONLY);
		breaklist.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		if(hasBreak()) breaklist.setText(breakElement.toString());
		else breaklist.setText("--");
		
		Label empty = new Label(parent, SWT.NONE);
		empty.setText("");
		
		Button trash = new Button(parent, SWT.PUSH);
		trash.setText(Messages.get().ReportPage_RemoveBreak);
		trash.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				removeBreak = true;
				breaklist.setText("--");
			}
		});
		/*
		Label filterlabel = new Label(parent, SWT.NONE);
		filterlabel.setText("Filter");
		filterlabel.setLayoutData(new GridData(200, 22));
		
		List filterlist = new List(parent, SWT.BORDER | SWT.V_SCROLL);
		filterlist.setLayoutData(new GridData(GridData.FILL_BOTH));
		*/
	}
	
	@Override
	public void onOk() {
		this.type = chartbutton.getSelection() ? ReportItem.TYPE_CHART : ReportItem.TYPE_FREQUENCY;
		this.title = titleInput.getText();
		this.options = 0;
		if(this.countbutton.getSelection()) this.options = this.options | Report.OPTION_COUNT;
		else if(this.percentbutton.getSelection()) this.options = this.options | Report.OPTION_PERCENT;
		if(this.totalbutton.getSelection()) this.options = this.options | Report.OPTION_TOTALS; 
		if(removeBreak){
			this.breakElement = null;
			removeBreak = false;
		}
		super.onOk();
	}
	
	@Override
	public void toDocument(Report doc) {
		if(type == TYPE_CHART) {
			if(hasBreak()){
				String bs = breakElement.text;
				doc.appendHead(this, Messages.get().ReportPage_BreakBy + " " + bs, typeLabel(options));
				
				if(breakElement.type.equals("CV")){
					doc.appendChart(new ChartData(this.tableXcv()), options);
				}
				else if(breakElement.type.equals("MD")){
					doc.appendChart(new ChartData(this.tableXmd()), options);
				}
			}
			else {
				doc.appendHead(this, typeLabel(options));
				doc.appendChart(new ChartData(this.table()), options);
			}
		}
		else {
			if(hasBreak()){
				String bs = breakElement.text;
				doc.appendHead(this, Messages.get().ReportPage_BreakBy + " " + bs, typeLabel(options));
				
				if((options & Report.OPTION_PERCENT) == Report.OPTION_PERCENT){
					//doc.appendSubHead("Percent");
					if(breakElement.type.equals("CV")){
						doc.appendTable(this.tableXcv(), options);
					}
					else if(breakElement.type.equals("MD")){
						doc.appendTable(this.tableXmd(), options);
					}
				}
				if((options & Report.OPTION_COUNT) == Report.OPTION_COUNT) {
					//doc.appendSubHead("Count");
					if(breakElement.type.equals("CV")){
						doc.appendTable(this.tableXcv(), options);
					}
					else if(breakElement.type.equals("MD")){
						doc.appendTable(this.tableXmd(), options);
					}
				}
			}
			else {
				doc.appendHead(this, typeLabel(options));
				doc.appendTable(this.table(), options);
			}
		}
		doc.appendMissingInfo(missingData(), options);
		//doc.appendFoot(q.text);
	}
	
	TabularReportData table(){
		TabularReportData table = new TabularReportData(options);
		AddressTuple[] varTuples = new AddressTuple[q.variables.size()];
		int i = 0;
		for(Variable v: q.variables){
			varTuples[i++] = new AddressTuple(v.name, "*");
		}
		
		int overall = data.get(new Address(varTuples));
		//System.out.println("MD overall " + overall);
		for(Variable v: q.variables){
			int c = data.get(v.name, "1");
			table.addRow(v.getCleanedLabel(), new Cell( c, (c / (double)overall) * 100.));
		}
		{
			table.addTotalRow(Messages.get().ReportPage_Totals, new Cell((int)sum(), (sum() / (double)overall) * 100.));
		}
		return table;
	}
	
	TabularReportData tableXcv(){
		TabularReportData table = new TabularReportData(options);
		Question cv = breakElement;
		for(Value bv: cv.values){
			table.addColumnName(bv.getCleanedLabel());
		}
		
		for(Variable v: q.variables){
			AddressTuple rt = new AddressTuple(v.name, "1");
			Cell[] cells = new Cell[cv.values.size()];
			int cn = 0;
			for(Value bv: cv.values){
				double breakoverall = data.get(new Address(new AddressTuple(cv.variables.get(0).name, bv.key)));
				int c = data.get(new Address(rt, new AddressTuple(cv.variables.get(0).name, bv.key)));
				cells[cn] = new Cell(c, (c / (double)breakoverall) * 100.);
				cn++;
			}
			table.addRow(v.getCleanedLabel(), cells);
		}
		{	
			int cn = 0;
			Cell[] cells = new Cell[cv.values.size()];
			for(Value bv: cv.values){
				double breakoverall = data.get(new Address(new AddressTuple(cv.variables.get(0).name, bv.key)));
				double sum = sum(new AddressTuple(cv.variables.get(0).name, bv.key));
				cells[cn] = new Cell((int)sum, (sum / breakoverall) * 100.);
				cn++;
			}
			table.addTotalRow(Messages.get().ReportPage_Total, cells);
		}
		return table;
	}
	
	TabularReportData tableXmd(){
		TabularReportData table = new TabularReportData(options);
		Question md = breakElement;
		for(Variable bv: md.variables){
			table.addColumnName(bv.getCleanedLabel());
		}
		
		for(Variable v : q.variables){
			AddressTuple rt = new AddressTuple(v.name, "1");
			int cn = 0;
			Cell[] cells = new Cell[md.variables.size()];
			for(Variable bv: md.variables){
				double breakoverall = data.get(new Address(new AddressTuple(bv.name, "1")));
				int c = data.get(new Address(rt, new AddressTuple(bv.name, "1")));
				cells[cn] = new Cell(c, (c / (double)breakoverall) * 100.);
				cn++;
			}
			table.addRow(v.getCleanedLabel(), cells);
		}
		{
			int cn = 0;
			Cell[] cells = new Cell[md.variables.size()];
			for(Variable bv: md.variables){
				double breakoverall = data.get(new Address(new AddressTuple(bv.name, "1")));
				double sum = sum(new AddressTuple(bv.name, "1"));
				cells[cn] = new Cell( (int)sum, (sum / breakoverall) * 100.);
				cn++;
			}
			table.addTotalRow(Messages.get().ReportPage_Total, cells);
		}
		return table;
	}
	
	MissingData missingData(){
		MissingData md = new MissingData();
		AddressTuple[] varTuples = new AddressTuple[q.variables.size()];
		int i = 0;
		for(Variable v: q.variables){
			varTuples[i++] = new AddressTuple(v.name, "*");
		}
		md.responses = data.get(new Address(varTuples));
		md.missing = data.get() - md.responses;
		md.type = "MC";
		return md;
	}
	
	double sum(){
		double sum = 0;
		for(Variable v : q.variables){
			int c = data.get(v.name, "1");
			sum += c;
		}
		return sum;
	}
	
	double sum(AddressTuple breakdata){
		double sum = 0;
		for(Variable v : q.variables){
			AddressTuple rt = new AddressTuple(v.name, "1");
			int c = data.get(new Address(rt, breakdata));
			sum += c;
		}
		return sum;
	}
}
