/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.report;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.apache.commons.dbcp2.BasicDataSource;
import org.eclipse.rap.rwt.RWT;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import de.eoda.qtafi.surveymanager.lang.Messages;
import de.eoda.qtafi.surveymanager.model.Project;
import de.eoda.qtafi.surveymanager.rap.export.Report;
import de.eoda.qtafi.surveymanager.rap.export.TabularReportData;
import de.eoda.qtafi.surveymanager.rap.export.TabularReportData.Cell;
import de.tinu.qtafi4.model.Question;
import de.tinu.qtafi4.model.Value;
import de.tinu.qtafi4.model.Variable;

public class MEReportItem extends ReportItem {

	private static final long serialVersionUID = 1L;
	
	Text titleInput;
	//Button countbutton, avgbutton;
	boolean removeBreak = false;
	ReportData data;
	
	public MEReportItem(Composite parent, int style, Question q) {
		super(parent, style, q);
		data = new ReportData();
		this.options = Report.OPTION_MEAN;
	}
	
	public void createDialog(Composite parent){
		Label titlelabel = new Label(parent, SWT.NONE);
		titlelabel.setText(Messages.get().ReportPage_Title);
		titlelabel.setLayoutData(new GridData(200, 22));
		
		titleInput = new Text(parent, SWT.BORDER);
		titleInput.setEditable(false);
		titleInput.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		titleInput.setText(this.title);
		/*
		Label showlabel = new Label(parent, SWT.NONE);
		showlabel.setText("Show");
		GridData gd = new GridData();
		gd.verticalSpan = 2;
		showlabel.setLayoutData(gd);
		
		avgbutton = new Button(parent, SWT.RADIO);
		avgbutton.setText("Mean");
		avgbutton.setSelection((options & Report.OPTION_MEAN) == Report.OPTION_MEAN);
		
		countbutton = new Button(parent, SWT.RADIO);
		countbutton.setText("Count");
		countbutton.setSelection((options & Report.OPTION_COUNT) == Report.OPTION_COUNT);
		*/
		Label brelabel = new Label(parent, SWT.NONE);
		brelabel.setText(Messages.get().ReportPage_Break);
		brelabel.setLayoutData(new GridData(200, 22));
		
		final Text breaklist = new Text(parent, SWT.BORDER | SWT.READ_ONLY);
		breaklist.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		if(hasBreak()) breaklist.setText(breakElement.toString());
		else breaklist.setText("--");
		
		Label empty = new Label(parent, SWT.NONE);
		empty.setText("");
		
		Button trash = new Button(parent, SWT.PUSH);
		trash.setText(Messages.get().ReportPage_RemoveBreak);
		trash.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				removeBreak = true;
				breaklist.setText("--");
			}
		});
		
		/*
		Label filterlabel = new Label(parent, SWT.NONE);
		filterlabel.setText("Filter");
		filterlabel.setLayoutData(new GridData(200, 22));
		
		List filterlist = new List(parent, SWT.BORDER | SWT.V_SCROLL);
		filterlist.setLayoutData(new GridData(GridData.FILL_BOTH));
		*/
	}
	
	@Override
	public void onOk() {
		this.type = ReportItem.TYPE_FREQUENCY;
		this.title = titleInput.getText();
		//this.options = countbutton.getSelection() ? Report.OPTION_COUNT: Report.OPTION_MEAN;
		this.options = Report.OPTION_MEAN;
		if(removeBreak){
			breakElement = null;
			removeBreak = false;
		}
		super.onOk();
	}
	
	@Override
	public void toDocument(Report doc) {
		if(hasBreak()){
			String bs = breakElement.text;
			doc.appendHead(this, Messages.get().ReportPage_BreakBy + " " + bs, typeLabel(options));
			if(breakElement.type.equals("CV")){
				for(Variable v: q.variables){
					if(! v.getCleanedLabel().isEmpty()) doc.appendSubHead(v.getCleanedLabel());
					doc.appendTable(this.tableMeanXcv(v), options);
				}
			}
			else if(breakElement.type.equals("MD")){
				for(Variable v: q.variables){
					if(! v.getCleanedLabel().isEmpty()) doc.appendSubHead(v.getCleanedLabel());
					doc.appendTable(this.tableMeanXmd(v), options);
				}
			}
		}
		else {
			doc.appendHead(this, typeLabel(options));
			for(Variable v: q.variables){
				if(! v.getCleanedLabel().isEmpty()) doc.appendSubHead(v.getCleanedLabel());
				doc.appendTable(this.tableMean(v), options);
			}
		}
		doc.appendMissingInfo(missingData(), options);
		//doc.appendFoot(q.text);
	}
	
	TabularReportData tableMean(Variable v){
		TabularReportData table = new TabularReportData(options);
		Project project = (Project) RWT.getUISession().getHttpSession().getAttribute("project");
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		Connection con = null;
		try { 
			con = ds.getConnection();
			StringBuffer buf = new StringBuffer();
			buf.append("SELECT count(*) as count, avg(`" +v.name +"`) as `" + v.name + "` \n");
			buf.append("FROM `" + project.name + "` \n");
			buf.append("WHERE NOT `" + v.name + "` IS NULL AND CHAR_LENGTH(`" + v.name + "`) > 0 AND `"+v.name+ "`!= '-9'\n");
			System.out.println(buf.toString());
			PreparedStatement pst = con.prepareStatement(buf.toString());
			ResultSet res =  pst.executeQuery();
			while(res.next()){
				table.addRow(Messages.get().ReportPage_Mean, new Cell(res.getInt("count"), res.getDouble(v.name)));
			}
			res.close();
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return table;
	}
	
	TabularReportData tableMeanXcv(Variable v){
		TabularReportData table = new TabularReportData(options);
		Question cv = breakElement;
		for(Value bv: cv.values){
			table.addColumnName(bv.getCleanedLabel());
		}
		
		Cell[] cells = new Cell[cv.values.size()];
		int cn = 0;
		for(Value bv: cv.values){
			double av = data.avg(v.name, new Address(new AddressTuple(cv.variables.get(0).name, bv.key)));
			cells[cn] = new Cell(0, av);
			cn++;
		}
		table.addRow(Messages.get().ReportPage_Mean, cells);
		return table;
	}
	
	TabularReportData tableMeanXmd(Variable v){
		TabularReportData table = new TabularReportData(options);
		Question md = breakElement;
		for(Variable bv: md.variables){
			table.addColumnName(bv.getCleanedLabel());
		}
		
		Cell[] cells = new Cell[md.variables.size()];
		int cn = 0;
		for(Variable bv: md.variables){
			double c = data.avg(v.name, new Address(new AddressTuple(bv.name, "1")));
			cells[cn] = new Cell(0, c);
			cn++;
		}
		table.addRow(Messages.get().ReportPage_Mean, cells);
		return table;
	}
	
	MissingData missingData(){
		MissingData md = new MissingData();
		AddressTuple[] varTuples = new AddressTuple[q.variables.size()];
		int i = 0;
		for(Variable v: q.variables){
			varTuples[i++] = new AddressTuple(v.name, "*");
		}
		md.responses = data.get(new Address(varTuples));
		md.missing = data.get() - md.responses;
		md.type = "ME";
		return md;
	}
}
