/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.report;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import de.eoda.qtafi.surveymanager.lang.Messages;
import de.eoda.qtafi.surveymanager.rap.export.ChartData;
import de.eoda.qtafi.surveymanager.rap.export.Report;
import de.eoda.qtafi.surveymanager.rap.export.TabularReportData;
import de.eoda.qtafi.surveymanager.rap.export.TabularReportData.Cell;
import de.tinu.qtafi4.model.Question;
import de.tinu.qtafi4.model.Value;
import de.tinu.qtafi4.model.Variable;

public class ORReportItem extends ReportItem {

	private static final long serialVersionUID = 1L;
	
	Button chartbutton, countbutton, percentbutton, meanbutton;
	Text titleInput;
	boolean removeBreak = false;
	ReportData data;
	
	public ORReportItem(Composite parent, int style, Question q) {
		super(parent, style, q);
		data = new ReportData();
		this.options = Report.OPTION_MEAN;
	}

	public void createDialog(Composite parent){
		Label titlelabel = new Label(parent, SWT.NONE);
		titlelabel.setText(Messages.get().ReportPage_Title);
		titlelabel.setLayoutData(new GridData(200, 22));
		
		titleInput = new Text(parent, SWT.BORDER);
		titleInput.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		titleInput.setText(this.title);
		
		Label chartlabel = new Label(parent, SWT.NONE);
		chartlabel.setText(Messages.get().ReportPage_Layout);
		chartlabel.setLayoutData(new GridData(200, 22));
		
		chartbutton = new Button(parent, SWT.CHECK);
		chartbutton.setText(Messages.get().ReportPage_Chart);
		chartbutton.setSelection(type == ReportItem.TYPE_CHART);
		
		Label showlabel = new Label(parent, SWT.NONE);
		showlabel.setText("Show");
		GridData gd = new GridData();
		gd.verticalSpan = 3;
		showlabel.setLayoutData(gd);
		
		meanbutton = new Button(parent, SWT.RADIO);
		meanbutton.setText(Messages.get().ReportPage_Mean);
		meanbutton.setSelection((options & Report.OPTION_MEAN) == Report.OPTION_MEAN);
		
		percentbutton = new Button(parent, SWT.RADIO);
		percentbutton.setText(Messages.get().ReportPage_Percent);
		percentbutton.setSelection((options & Report.OPTION_PERCENT) == Report.OPTION_PERCENT);
		
		countbutton = new Button(parent, SWT.RADIO);
		countbutton.setText(Messages.get().ReportPage_Count);
		countbutton.setSelection((options & Report.OPTION_COUNT) == Report.OPTION_COUNT);
		
		Label brelabel = new Label(parent, SWT.NONE);
		brelabel.setText(Messages.get().ReportPage_Break);
		brelabel.setLayoutData(new GridData(200, 22));
		
		final Text breaklist = new Text(parent, SWT.BORDER | SWT.READ_ONLY);
		breaklist.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		if(hasBreak()) breaklist.setText(breakElement.toString());
		else breaklist.setText("--");
		
		Label empty = new Label(parent, SWT.NONE);
		empty.setText("");
		
		Button trash = new Button(parent, SWT.PUSH);
		trash.setText(Messages.get().ReportPage_RemoveBreak);
		trash.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				breaklist.setText("--");
				removeBreak = true;
			}
		});
		/*
		Label filterlabel = new Label(parent, SWT.NONE);
		filterlabel.setText("Filter");
		filterlabel.setLayoutData(new GridData(200, 22));
		
		List filterlist = new List(parent, SWT.BORDER | SWT.V_SCROLL);
		filterlist.setLayoutData(new GridData(GridData.FILL_BOTH));
		*/
	}
	
	@Override
	public void onOk() {
		this.type = chartbutton.getSelection() ? ReportItem.TYPE_CHART : ReportItem.TYPE_FREQUENCY;
		this.title = titleInput.getText();
		this.options = 0;
		if(this.countbutton.getSelection()) this.options = this.options | Report.OPTION_COUNT;
		else if(this.percentbutton.getSelection()) this.options = this.options | Report.OPTION_PERCENT;
		else if(this.meanbutton.getSelection()) this.options = this.options | Report.OPTION_MEAN;
	
		
		if(removeBreak){
			this.breakElement = null;
			removeBreak = false;
		}
		super.onOk();
	}
	
	@Override
	public void toDocument(Report doc) {
		if(type == TYPE_CHART) {
			if(hasBreak()){
				String bs = breakElement.text;
				doc.appendHead(this, Messages.get().ReportPage_BreakBy + " " + bs, typeLabel(options));
				if((options & Report.OPTION_PERCENT) == Report.OPTION_PERCENT){
					if(breakElement.type.equals("CV")){
						for(Variable v: q.variables){
							doc.appendSubHead(v.getCleanedLabel());
							doc.appendChart(new ChartData(this.tableXcv(v)), options);
						}
					}
					else if(breakElement.type.equals("MD")){
						for(Variable v: q.variables){
							doc.appendSubHead(v.getCleanedLabel());
							doc.appendChart(new ChartData(this.tableXmd(v)), options);
						}
					}
				}
				if((options & Report.OPTION_COUNT) == Report.OPTION_COUNT) {
					if(breakElement.type.equals("CV")){
						for(Variable v: q.variables){
							doc.appendSubHead(v.getCleanedLabel());
							doc.appendChart(new ChartData(this.tableXcv(v)), options);
						}
					}
					else if(breakElement.type.equals("MD")){
						for(Variable v: q.variables){
							doc.appendSubHead(v.getCleanedLabel());
							doc.appendChart(new ChartData(this.tableXmd(v)), options);
						}
					}
				}
				if((options & Report.OPTION_MEAN) == Report.OPTION_MEAN){
					if(breakElement.type.equals("CV")){
						ChartData cd = new ChartData(this.tableMeanXcv());
						cd.setScale(minValue(), maxValue());
						doc.appendChart(cd, options);
					}
					else if(breakElement.type.equals("MD")){
						ChartData cd = new ChartData(this.tableMeanXmd());
						cd.setScale(minValue(), maxValue());
						doc.appendChart(cd, options);
					}
				}
			}
			else {
				doc.appendHead(this);
				if((options & Report.OPTION_PERCENT) == Report.OPTION_PERCENT){
					for(Variable v: q.variables){
						doc.appendSubHead(v.getCleanedLabel());
						doc.appendChart(new ChartData(this.table(v)), options);
					}
				}
				if((options & Report.OPTION_COUNT) == Report.OPTION_COUNT){
					for(Variable v: q.variables){
						doc.appendSubHead(v.getCleanedLabel());
						doc.appendChart(new ChartData(this.table(v)), options);
					}
				}
				if((options & Report.OPTION_MEAN) == Report.OPTION_MEAN){
					ChartData cd = new ChartData(this.tableMean());
					cd.setScale(minValue(), maxValue());
					doc.appendChart(cd, options);
				}
			}
		}
		else {
			if(hasBreak()){
				String bs = breakElement.text;
				doc.appendHead(this, Messages.get().ReportPage_BreakBy + " " + bs, typeLabel(options));
				if((options & Report.OPTION_PERCENT) == Report.OPTION_PERCENT) {
					if(breakElement.type.equals("CV")){
						for(Variable v: q.variables){
							doc.appendSubHead(v.getCleanedLabel());
							doc.appendTable(this.tableXcv(v), options);
						}
					}
					else if(breakElement.type.equals("MD")){
						for(Variable v: q.variables){
							doc.appendSubHead(v.getCleanedLabel());
							doc.appendTable(this.tableXmd(v), options);
						}
					}
				}
				if((options & Report.OPTION_COUNT) == Report.OPTION_COUNT){
					if(breakElement.type.equals("CV")){
						for(Variable v: q.variables){
							doc.appendSubHead(v.getCleanedLabel());
							doc.appendTable(this.tableXcv(v), options);
						}
					}
					else if(breakElement.type.equals("MD")){
						for(Variable v: q.variables){
							doc.appendSubHead(v.getCleanedLabel());
							doc.appendTable(this.tableXmd(v), options);
						}
					}
				}
				if((options & Report.OPTION_MEAN) == Report.OPTION_MEAN) {
					//doc.appendSubHead("mean");
					if(breakElement.type.equals("CV")){
						doc.appendTable(this.tableMeanXcv(), options);
					}
					else if(breakElement.type.equals("MD")){
						doc.appendTable(this.tableMeanXmd(), options);
					}
				}
			}
			else {
				doc.appendHead(this, typeLabel(options));
				if((options & Report.OPTION_PERCENT) == Report.OPTION_PERCENT){
					for(Variable v: q.variables){
						doc.appendSubHead(v.getCleanedLabel());
						doc.appendTable(this.table(v), options);
					}
				}
				if((options & Report.OPTION_COUNT) == Report.OPTION_COUNT){
					for(Variable v: q.variables){
						doc.appendSubHead(v.getCleanedLabel());
						doc.appendTable(this.table(v), options);
					}
				}
				if((options & Report.OPTION_MEAN) == Report.OPTION_MEAN) {
					//doc.appendSubHead("mean");
					doc.appendTable(this.tableMean(), options);
				}
			}
		}
		
		doc.appendMissingInfo(missingData(), options);
		//doc.appendFoot(q.text);
	}
	
	TabularReportData tableMean() {
		TabularReportData table = new TabularReportData(options);
		int overall = data.get();
		for(Variable v: q.variables){
			int c = data.get(v.name, "*");
			table.addRow(v.getCleanedLabel(), new Cell(c, avg(v.name)) );
		}
		return table;
	}
	
	TabularReportData table(Variable var) {
		TabularReportData table = new TabularReportData(options);
		int overall = data.get(var.name, "*");
		for(Value v: q.values){
			int c = data.get(var.name, v.key);
			table.addRow(v.key + " "+ v.getCleanedLabel(), new Cell(c, ((c / (double)overall) * 100.)));
		}
		return table;
	}
	
	TabularReportData tableMeanXcv() {
		TabularReportData table = new TabularReportData(options);
		Question cv = breakElement;
	
		for(Value bv: cv.values){
			table.addColumnName(bv.getCleanedLabel());
		}
	
		for(Variable v: q.variables){
			Cell[] cells = new Cell[cv.values.size()];
			int cn = 0;
			for(Value bv: cv.values){
				//System.out.println(v.name + ", " + cv.variables.get(0).name + ", " + bv.key);
				cells[cn] = new Cell( data.get(new Address(new AddressTuple(v.name, "*"))), avg(v.name, cv.variables.get(0).name, bv.key));
				cn++;
			}
			
			table.addRow(v.getCleanedLabel(), cells);
		}
		return table;
	}
	
	TabularReportData tableXcv(Variable var) {
		TabularReportData table = new TabularReportData(options);
		int overall = data.get(var.name, "*");
		Question cv = breakElement;
		for(Value bv: cv.values){
			table.addColumnName(bv.getCleanedLabel());
		}
		
		for(Value v: q.values){
			Cell[] cells = new Cell[cv.values.size()];
			int cn = 0;
			for(Value bv: cv.values){
				int c = data.get(new Address(new AddressTuple(var.name, v.key), new AddressTuple(cv.variables.get(0).name, bv.key)));
				double breakoverall = data.get(new Address(new AddressTuple(var.name, "*"), new AddressTuple(cv.variables.get(0).name, bv.key)));
				if(breakoverall == 0) cells[cn] = new Cell(c, Double.NaN);
				else cells[cn] = new Cell(c, (c / breakoverall) * 100.);
				cn++;
			}
			table.addRow(v.key + " "+ v.getCleanedLabel(), cells);
		}
		
		return table;
	}
	
	
	TabularReportData tableMeanXmd() {
		TabularReportData table = new TabularReportData(options);
		Question md = breakElement;
		for(Variable bv: md.variables){
			table.addColumnName(bv.getCleanedLabel());
		}
		
		for(Variable v: q.variables){
			int cn = 0;
			Cell[] cells = new Cell[md.variables.size()];
			for(Variable bv: md.variables){
				cells[cn] = new Cell( data.get(new Address(new AddressTuple(v.name, "*"), new AddressTuple(bv.name, "1"))),  avg(v.name, bv.name, "1"));
				cn++;
			}
			table.addRow(v.getCleanedLabel(), cells);
		}
		return table;
	}
	
	TabularReportData tableXmd(Variable var) {
		TabularReportData table = new TabularReportData(options);
		Question md = breakElement;
		for(Variable bv: md.variables){
			table.addColumnName(bv.getCleanedLabel());
		}
		
		for(Value v: q.values){
			int cn = 0;
			Cell[] cells = new Cell[md.variables.size()];
			for(Variable bv: md.variables){
				int c = data.get(new Address(new AddressTuple(var.name, v.key), new AddressTuple(bv.name, "1")));
				double breakoverall = data.get( new Address(new AddressTuple(var.name, "*"),new AddressTuple(bv.name, "1")));
				
				if(breakoverall == 0) cells[cn] = new Cell(c, Double.NaN);
				else cells[cn] = new Cell(c,  (c / breakoverall) * 100.);
				cn++;
			}
			table.addRow(v.key + " " + v.getCleanedLabel(), cells);
		}
		return table;
	}
	
	MissingData missingData(){
		MissingData or = new MissingData();
		AddressTuple[] varTuples = new AddressTuple[q.variables.size()];
		int i = 0;
		for(Variable v: q.variables){
			varTuples[i++] = new AddressTuple(v.name, "*");
		}
		or.responses = data.get(new Address(varTuples));
		or.missing = data.get() - or.responses;
		or.type = "OR";
		return or;
	}
	
	Double avg(String var){
		int count = 0;
		double sum = 0;
		for(Value v: q.values){
			try{
				if(Integer.parseInt(v.key) < 0) continue;
			}
			catch(NumberFormatException ex){ }
			int c = data.get(var, v.key);
			double vk = Double.parseDouble(v.key);
			sum += (vk * c);
			count += c;
		}
		if(count == 0) return Double.NaN;
		return sum/count;
	}
	
	Double avg(String var, String breakvar, String breakkey){
		AddressTuple t1 = new AddressTuple(breakvar, breakkey);
		int count = 0;
		double sum = 0;
		for(Value v: q.values){
			try {
				if(Integer.parseInt(v.key) < 0) continue;
			}
			catch(NumberFormatException ex){ }
			int c = data.get(new Address(new AddressTuple[]{t1, new AddressTuple(var, v.key) }) );
			double vk = Double.parseDouble(v.key);
			sum += (vk * c);
			count += c;
		}
		if(count == 0) return Double.NaN;
		return sum/count;
	}
	
	int maxValue(){
		int max = Integer.MIN_VALUE;
		for(Value v: q.values){
			try{
				int k = Integer.parseInt(v.key);
				if(k < 0) continue;
				if(k > max) max = k;
			}
			catch(NumberFormatException ex){}
		}
		return max;
	}
	
	int minValue(){
		int min = Integer.MAX_VALUE;
		for(Value v: q.values){
			try {
				int k = Integer.parseInt(v.key);
				if(k < 0) continue;
				if(k < min) min = k;
			}
			catch(NumberFormatException ex){}
		}
		return min;
	}
}
