/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.report;

import gnu.trove.map.hash.THashMap;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.apache.commons.dbcp2.BasicDataSource;
import org.eclipse.rap.rwt.RWT;

import de.eoda.qtafi.surveymanager.model.Project;

public class ReportData {
	
	private final List<String> requests = new ArrayList<String>();
	private final THashMap<String, Integer> data = new THashMap<String, Integer>();
	Connection con = null;
	PreparedStatement pst;
	
	void fetch(Address address){
		//System.out.println("fetch " + address);
		Project project = (Project) RWT.getUISession().getHttpSession().getAttribute("project");
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		try { 
			con = ds.getConnection();
			
			List<String> group = new Vector<String>();
			List<String> filter = new Vector<String>();
			List<String> selects = new Vector<String>();
		
			for(int i = 0; i < address.parts.length; i++){
				if(address.parts[i].val.equals("*")){
					filter.add("(`" + address.parts[i].var + "`!='-9' AND NOT `" + address.parts[i].var + "` IS NULL)");
				}
				else if(! address.parts[i].val.equals("*")){
					selects.add("`" + address.parts[i].var + "` as `" + address.parts[i].var + "`");
					group.add("`" + address.parts[i].var + "`");
					//filter.add("`" + address.parts[i].var + "`='" + address.parts[i].val + "'");
				}
				else {
					
				}
			}
			StringBuffer buf = new StringBuffer();
			buf.append("SELECT count(__n) as count ");
			if(selects.size() > 0){
				 buf.append(", ");
				 for(Iterator<String> it = selects.iterator(); it.hasNext(); ){
					buf.append(it.next());
					if(it.hasNext()) buf.append(", ");
				}
			}
			
			buf.append(" ");
			buf.append("FROM `" + project.name + "`");
			buf.append(" ");
			if(filter.size() > 0){
				buf.append("WHERE ");
				for(Iterator<String> it = filter.iterator(); it.hasNext(); ){
					buf.append( it.next()  );
					if(it.hasNext()) buf.append(" OR ");
				}
			}
			buf.append(" ");
			if(group.size() > 0){
				buf.append("GROUP BY ");
				for(Iterator<String> it = group.iterator(); it.hasNext(); ){
					buf.append( it.next() );
					if(it.hasNext()) buf.append(",");
				}
			}
			//
			//System.out.println(buf.toString());
			pst = con.prepareStatement(buf.toString());
		
			ResultSet res = pst.executeQuery();
			ResultSetMetaData meta = res.getMetaData();
			
			List<String> cols = new Vector<String>();
			for(int c = 1; c <= meta.getColumnCount(); c++){
				String n = meta.getColumnName(c);
				if(n.equals("count")) continue;
				cols.add(n);
			}
			
			boolean rs = false;
			while(res.next()){
				rs = true;
				int count = res.getInt("count");
				if(cols.size() > 0){
					int i = 0;
					Address x = address.clone();
					for(AddressTuple s: address.parts){
						if(cols.contains(s.var)){
							String val = res.getString(s.var);
							if(val == null) val = "-9";
							if(val.trim().length() == 0) val = "-9";
							//System.out.println(x.parts[i] + " -> " + s.var + ":" + val );
							x.parts[i] = new AddressTuple(s.var, val);
						}
						i++;
					}
					add(x, count);
				}
				else {
					set(address, count);
				}
			}
			if(! rs) set(address, 0);
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
	
	
	void set(Address address, int count){
		//System.out.println("set: " + address.getRepresentation() + " -> " + count);
		data.put(address.getRepresentation(), count);
		requests.add(address.getRequestRepresentation());
	}
	
	void add(Address address, int count){
		Integer c = data.get(address.getRepresentation());
		if(c != null){
			//System.out.println("add: " + address.getRepresentation() + " -> " + (count + c.intValue()));
			data.put(address.getRepresentation(), (count + c.intValue()));
		}
		else{
			//System.out.println("set: " + address.getRepresentation() + " -> " + (count));
			data.put(address.getRepresentation(), count);
			requests.add(address.getRequestRepresentation());
		}
	}
	
	public int get(String var, String val){
		return get(new Address(new AddressTuple(var, val)));
	}
	
	public int get(){
		return get(new Address());
	}
	
	public int get(Address address){
		if(! requests.contains(address.getRequestRepresentation())){
			fetch(address);
		}
		
		Integer c = data.get(address.getRepresentation());
		/*
		if(c == null){
			fetch(address);
			c = data.get(address.getRepresentation());
		}
		*/
		
		//System.out.println("get: " + address.getRepresentation() + " -> " + c);
		if(c == null) return 0;
		return c.intValue();
	}
	
	public double avg(String var){
		//System.out.println("avg ");
		Project project = (Project) RWT.getUISession().getHttpSession().getAttribute("project");
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		double average = 0.;
		try { 
			con = ds.getConnection();
			List<String> group = new Vector<String>();
			List<String> filter = new Vector<String>();
			List<String> selects = new Vector<String>();
			selects.add("avg(`" + var + "`) as `" + var + "`");
			
			StringBuffer buf = new StringBuffer();
			buf.append("SELECT count(__n) as count ");
			if(selects.size() > 0){
				 buf.append(", ");
				 for(Iterator<String> it = selects.iterator(); it.hasNext(); ){
					buf.append(it.next());
					if(it.hasNext()) buf.append(", ");
				}
			}
			
			buf.append(" ");
			buf.append("FROM `" + project.name + "`");
			buf.append(" ");
			if(filter.size() > 0){
				buf.append("WHERE ");
				for(Iterator<String> it = filter.iterator(); it.hasNext(); ){
					buf.append( it.next()  );
					if(it.hasNext()) buf.append(" AND ");
				}
			}
			buf.append(" ");
			if(group.size() > 0){
				buf.append("GROUP BY ");
				for(Iterator<String> it = group.iterator(); it.hasNext(); ){
					buf.append( it.next() );
					if(it.hasNext()) buf.append(",");
				}
			}
			//System.out.println(buf.toString());
			pst = con.prepareStatement(buf.toString());
		
			ResultSet res = pst.executeQuery();
			ResultSetMetaData meta = res.getMetaData();
			
			List<String> cols = new Vector<String>();
			for(int c = 1; c <= meta.getColumnCount(); c++){
				String n = meta.getColumnName(c);
				if(n.equals("count")) continue;
				cols.add(n);
			}
			
			if(res.next()){
				int count = res.getInt("count");
				average = res.getDouble(var);
			}
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return average;
	}
	
	public double avg(String var, Address address){
		//System.out.println("avg ");
		Project project = (Project) RWT.getUISession().getHttpSession().getAttribute("project");
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		double average = 0.;
		try { 
			con = ds.getConnection();
			List<String> group = new Vector<String>();
			List<String> filter = new Vector<String>();
			List<String> selects = new Vector<String>();
			selects.add("avg(`" + var + "`) as `" + var + "`");
			
			for(int i = 0; i < address.parts.length; i++){
				filter.add("`" + address.parts[i].var + "`='" + address.parts[i].val + "'");
			}
			
			StringBuffer buf = new StringBuffer();
			buf.append("SELECT count(__n) as count ");
			if(selects.size() > 0){
				 buf.append(", ");
				 for(Iterator<String> it = selects.iterator(); it.hasNext(); ){
					buf.append(it.next());
					if(it.hasNext()) buf.append(", ");
				}
			}
			
			buf.append(" ");
			buf.append("FROM `" + project.name + "`");
			buf.append(" ");
			if(filter.size() > 0){
				buf.append("WHERE ");
				buf.append("`" + var + "`!='-9' ");
				if(filter.size() > 0) buf.append(" AND ");
				for(Iterator<String> it = filter.iterator(); it.hasNext(); ){
					buf.append( it.next()  );
					if(it.hasNext()) buf.append(" AND ");
				}
			}
			buf.append(" ");
			if(group.size() > 0){
				buf.append("GROUP BY ");
				for(Iterator<String> it = group.iterator(); it.hasNext(); ){
					buf.append( it.next() );
					if(it.hasNext()) buf.append(",");
				}
			}
			//System.out.println(buf.toString());
			pst = con.prepareStatement(buf.toString());
		
			ResultSet res = pst.executeQuery();
			ResultSetMetaData meta = res.getMetaData();
			
			List<String> cols = new Vector<String>();
			for(int c = 1; c <= meta.getColumnCount(); c++){
				String n = meta.getColumnName(c);
				if(n.equals("count")) continue;
				cols.add(n);
			}
			
			if(res.next()){
				int count = res.getInt("count");
				average = res.getDouble(var);
			}
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return average;
	}
	
}
