/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.rap.report;

import java.text.DecimalFormat;

import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

import de.eoda.qtafi.surveymanager.lang.Messages;
import de.eoda.qtafi.surveymanager.rap.composites.HTMLComposite;
import de.eoda.qtafi.surveymanager.rap.export.Report;
import de.eoda.qtafi.surveymanager.rap.export.HtmlDocument;
import de.tinu.qtafi4.model.Question;

public abstract class ReportItem extends HTMLComposite {
	
	private static final long serialVersionUID = 1L;
	final static DecimalFormat format = new DecimalFormat("#0.0");
	
	public static final int TYPE_FREQUENCY = 1;
	public static final int TYPE_CHART = 2;
	
	protected static final int TABLE_ROW_ITEM_HEIGHT = 30;
	protected static final int CHART_ROW_ITEM_HEIGHT = 28;
	protected static final int CHART_OFFSET_HEIGHT = 120;
	protected static final int CHART_LEFT_LEGEND_WIDTH = 250;
	
	public int type = TYPE_FREQUENCY;
	public int options = Report.OPTION_PERCENT;
	
	public final Question q;
	public Button configbutton;
	public Question breakElement = null;
	public String title;
	public int height;
	
	public ReportItem(Composite parent, int style, Question q) {
		super(parent, style);
		this.q = q;
		this.title = q.text;
	}
	
	public void setBreak(Question question){
		breakElement = question;
	}
	
	public boolean hasBreak(){
		return breakElement != null;
	}
	
	public String typeLabel(int opt){
		if((options & Report.OPTION_PERCENT) == Report.OPTION_PERCENT) return Messages.get().ReportPage_Percent;
		if((options & Report.OPTION_COUNT) == Report.OPTION_COUNT) return Messages.get().ReportPage_Count;
		if((options & Report.OPTION_MEAN) == Report.OPTION_MEAN) return Messages.get().ReportPage_Mean;
		return "";
	}
	
	public void createDialog(Composite parent){
	}
	
	public void onOk(){
	}
	
	public void onCancel(){
	}
	
	public void build(){
		HtmlDocument html = new HtmlDocument();
		this.toDocument(html);
		GridData layoutdata = new GridData();
		layoutdata.grabExcessHorizontalSpace = true;
		layoutdata.horizontalAlignment = GridData.FILL;
		layoutdata.heightHint = html.getHeight();
		this.setLayoutData(layoutdata);
		this.height = html.getHeight();
		//this.setHtml("<div style=\"height: " + html.getHeight() + ";overflow:auto;page-break-inside: avoid;\" >"+ html.getHtml() + "</div>");
		this.setHtml("<div style=\"min-height: " + html.getHeight() + ";overflow:auto;page-break-inside: avoid;\" >"+ html.getHtml() + "</div>");
	}
	
	public abstract void toDocument(Report doc);
}
