/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.util;

import java.io.IOException;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.eclipse.rap.rwt.service.ServiceHandler;

public class DataDownloadService implements ServiceHandler {

	public static final String DOWNLOAD_SERVICE = "downloadDataService";
	public static final String DATA_ID = "dataid";
	
	public static interface DownloadCallback {
		String filename();
		void service(OutputStream os);
	}
	
	
	private static DataDownloadService instance;
	
	private Map<String, DownloadCallback> downloads = new HashMap<String, DownloadCallback>();
	
	public static DataDownloadService getInstance(){
		if(instance == null) instance = new DataDownloadService();
		return instance;
	}
	
	private  DataDownloadService() {
	}
	
	public void addDownload(String id, DownloadCallback dd){
		downloads.put(id, dd);
	}
	
	@Override
	public void service(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
		String dataid = request.getParameter(DATA_ID);
		DownloadCallback dc = downloads.get(dataid);
		if(dc != null){
			downloads.remove(dataid);
		    response.setContentType("application/octet-stream");
		    response.setContentLength(-1);
		    //response.addHeader("Transfer-Encoding", "chunked");
		    response.setHeader("Content-Disposition",  "attachment; filename=\""+dc.filename()+ "\"");
		    OutputStream os = response.getOutputStream();
		    dc.service(os);
		    os.flush();
		    os.close();
		}
	}
}
