/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.util;

import java.io.File;
import java.io.OutputStream;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.sql.Types;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import java.util.Vector;

import org.apache.commons.dbcp2.BasicDataSource;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authc.credential.PasswordService;
import org.apache.shiro.subject.Subject;
import org.eclipse.rap.rwt.RWT;

import de.eoda.qtafi.surveymanager.auth.Account;
import de.eoda.qtafi.surveymanager.model.Project;
import de.eoda.qtafi.surveymanager.model.ProjectInfo;
import de.eoda.qtafi.surveymanager.model.Role;
import de.tinu.qtafi4.db.QuestionDB;
import de.tinu.qtafi4.model.Variable;

public class DbHelpers {

	static Connection con = null;
	static PreparedStatement pst;
	
	
	public static List<Account> AccountsAll(){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		List<Account> accounts = new Vector<Account>();
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT * FROM `__accounts`");
			ResultSet res = pst.executeQuery();
			while(res.next()){
				Account acc = new Account(res.getInt("id"), res.getString("name"));
				acc.setAdministrator(res.getInt("administrator") == 1);
				accounts.add(acc);
			}
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				con.close();
			}
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return accounts;
	}
	
	public static void AccountSetPassword(Account account, String password){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		PasswordService passservice = ((de.eoda.qtafi.surveymanager.auth.SecurityManager)SecurityUtils.getSecurityManager()).passwordService;
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("UPDATE `__accounts` SET password=? WHERE id=?");
			pst.setString(1, passservice.encryptPassword(password));
			pst.setInt(2, account.id);
			pst.execute();
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				con.close();
			}
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
	
	public static void AccountUpdate(Account account){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("UPDATE `__accounts` SET administrator=? WHERE id=?");
			pst.setInt(1, account.isAdministrator() ? 1: 0);
			pst.setInt(2, account.id);
			pst.execute();
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				con.close();
			}
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
	
	
	public static boolean AccountNameExists(String account){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		boolean exists = false;
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT count(*) FROM `__accounts` WHERE name=?");
			pst.setString(1, account);
			ResultSet res = pst.executeQuery();
			if(res.next()){
				if(res.getInt(1) == 1) exists = true; 
			}
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				con.close();
			}
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return exists;
	}
	
	public static Account AccountCreate(String name, String password, boolean administrator){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		PasswordService passservice = ((de.eoda.qtafi.surveymanager.auth.SecurityManager)SecurityUtils.getSecurityManager()).passwordService;
		Account newaccount = null;
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("INSERT INTO `__accounts` (name, password, administrator) VALUES (?, ?, ?)", Statement.RETURN_GENERATED_KEYS);
			pst.setString(1, name);
			pst.setString(2, passservice.encryptPassword(password));
			pst.setInt(3, administrator ? 1: 0);
			pst.execute();
			/*
			ResultSet keys = pst.getGeneratedKeys();
			if(keys.next()){
				int id = keys.getInt(1); 
				newaccount = new Account(id, name);
				PreparedStatement pst2 = con.prepareStatement("INSERT INTO `__accountroles` (roleid, accountid) VALUES (1, ?)");
				pst2.setInt(1, id);
				pst2.execute();
				pst2.close();
			}
			*/
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				con.close();
			}
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return newaccount;
	}
	
	public static Role AccountRoleById(int accountid, int projectid){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		Role role = new Role(0, "");
		try { 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT asign.roleid as roleid, role.role as role FROM `__role` as role, `__projectassignment` asign WHERE role.id=asign.roleid AND asign.accountid=? AND asign.projectid=?");
			pst.setInt(1, accountid);
			pst.setInt(2, projectid);
			ResultSet res = pst.executeQuery();
			if(res.next()){
				role = new Role(res.getInt("roleid"), res.getString("role"));
			}
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				con.close();
			}
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return role;
	}
	
	
	
	public static void AssignProject(Account account, ProjectInfo project, Role role){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		try { 
			con = ds.getConnection();
			pst = con.prepareStatement("INSERT INTO `__projectassignment` (accountid, projectid, roleid) VALUES (?, ?, ?) ");
			pst.setInt(1, account.id);
			pst.setInt(2, project.id);
			pst.setInt(3, role.id);
			pst.execute();
			pst.close();
			
			//make accessable if account is loggedin
			Subject subject = SecurityUtils.getSubject();
			Account current = (Account) subject.getPrincipal();
			if(account.id == current.id){
				current.addAccessibleProject(project.name);
			}
			
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				con.close();
			}
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
	
	public static void UnassignProject(Account account, Project project){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("DELETE FROM `__projectassignment` WHERE accountid=? AND projectid=? ");
			pst.setInt(1, account.id);
			pst.setInt(2, project.id);
			pst.execute();
			pst.close();
			
			//make not accessable if account is logged in
			Subject subject = SecurityUtils.getSubject();
			Account current = (Account) subject.getPrincipal();
			if(account.id == current.id){
				current.removeAccessibleProject(project.name);
			}
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				con.close();
			}
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
	
	
	public static List<ProjectInfo> ProjectsAll(){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		List<ProjectInfo> projects = new Vector<ProjectInfo>();
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT * FROM `__projects`");
			ResultSet res = pst.executeQuery();
			while(res.next()){
				String name = res.getString("name"); 
				ProjectInfo pr = new ProjectInfo(res.getInt("id"), name);
				projects.add(pr);
			}
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				con.close();
			}
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return projects;
	}
	
	
	public static void ProjectUpdate(String projectname, Project project){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("UPDATE `__projects` SET start=?, end=?, authentication=?, log=?, status=? WHERE name=?");
			pst.setTimestamp(1, new Timestamp(project.startdate.getTime()));
			pst.setTimestamp(2, new Timestamp(project.enddate.getTime()));
			pst.setString(3, project.authenticationmode.toUpperCase());
			pst.setString(4, project.logmode);
			pst.setInt(5, project.status);
			pst.setString(6, project.name);
			pst.execute();
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}	
	}
	

	public static boolean ProjectExists(String projectname){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		boolean exist = true;
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT count(*) FROM `__projects` WHERE name=?");
			pst.setString(1, projectname.toLowerCase());
			ResultSet res = pst.executeQuery();
			if(res.next()){
				if(res.getInt(1) == 0) exist = false;
			}
			else exist = false;
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return exist;
	}
	
	public static int ProjectStatus(String projectname){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		int status = 0;
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT status as status FROM `__projects` WHERE name=?");
			pst.setString(1, projectname.toLowerCase());
			ResultSet res = pst.executeQuery();
			if(res.next()){
				status = res.getInt("status"); 
			}
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return status;
	}
	
	public static List<String> ProjecAccounts(String projectname){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		Vector<String> names = new Vector<String>();
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT acc.name as name, assign.accountid FROM `__projects` pr,  `__projectassignment` as assign , `__accounts` as acc WHERE acc.id=assign.accountid AND assign.projectid=pr.id AND pr.name=?");
			pst.setString(1, projectname.toLowerCase());
			ResultSet res = pst.executeQuery();
			while(res.next()){
				names.add(res.getString("name")); 
			}
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return names;
	}
	
	
	public static Project Project(String projectname, Account account){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		Project project = null;
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT pr.id as id, pr.name as name, pr.status as status, pr.start as start, pr.end as end, pr.authentication as authentication, pr.log as log, role.role as role, role.id as roleid "+
			"FROM `__projects` as pr, `__projectassignment` as assign, `__role` as role WHERE role.id=assign.roleid AND pr.id=assign.projectid AND pr.name=? AND assign.accountid=?");
			pst.setString(1, projectname.toLowerCase());
			pst.setInt(2, account.id);
			ResultSet res = pst.executeQuery();
			if(res.next()){
				project = new Project(res.getInt("id"), res.getString("name"));
				{
					long start = res.getTimestamp("start").getTime();
					project.startdate = new java.util.Date(start);
				}
				{
					long end = res.getTimestamp("end").getTime();
					project.enddate = new java.util.Date(end);
				}
				{
					String auth = res.getString("authentication");
					if(auth != null) project.authenticationmode = auth.toUpperCase();
					else project.authenticationmode = "PIN";
				}
				{
					String l = res.getString("log");
					if(l != null) project.logmode = l;
					else project.logmode = "off";
				}
				String role = res.getString("role");
				project.asRoleid = res.getInt("roleid");
				project.asRole = role;
				
				project.status = res.getInt("status"); 
			}
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return project;
	}
	
	public static void ProjectCreate(Project project){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("INSERT INTO `__projects` (name, start, end, authentication, log) VALUES (?, ?, ?, ?, ?)", Statement.RETURN_GENERATED_KEYS);
			pst.setString(1, project.name.trim().toLowerCase());
			pst.setTimestamp(2, new Timestamp(project.startdate.getTime()));
			pst.setTimestamp(3, new Timestamp(project.enddate.getTime()));
			pst.setString(4, project.authenticationmode.toUpperCase());
			pst.setString(5, project.logmode);
			
			pst.execute();
			ResultSet keys = pst.getGeneratedKeys();
			keys.next(); 
			int res = keys.getInt(1);
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
	
	public static void ProjectCreateAndAssign(Project project, Account account){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("INSERT INTO `__projects` (name, start, end, authentication, log) VALUES (?, ?, ?, ?, ?)", Statement.RETURN_GENERATED_KEYS);
			pst.setString(1, project.name.trim().toLowerCase());
			pst.setTimestamp(2, new Timestamp(project.startdate.getTime()));
			pst.setTimestamp(3, new Timestamp(project.enddate.getTime()));
			pst.setString(4, project.authenticationmode.toUpperCase());
			pst.setString(5, project.logmode);
			
			pst.execute();
			ResultSet keys = pst.getGeneratedKeys();
			keys.next(); 
			int res = keys.getInt(1);
			pst.close();
			
			pst = con.prepareStatement("INSERT INTO `__projectassignment` (accountid, projectid, roleid) VALUES (?, ?, ?)");
			pst.setInt(1, account.id);
			pst.setInt(2, res);
			pst.setInt(3, 1); //Role admin
			pst.execute();
			pst.close();
			
			//make accessable
			account.addAccessibleProject(project.name);
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
	
	public static void ProjectRemove(ProjectInfo project){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		try{
			con = ds.getConnection();
			pst = con.prepareStatement("DELETE FROM `__projectassignment` WHERE projectid=?");
			pst.setInt(1, project.id);
			pst.execute();
			pst.close();
			
			pst = con.prepareStatement("DELETE FROM `__projects` WHERE id=?");
			pst.setInt(1, project.id);
			pst.execute();
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
	
	public static List<Project> ProjectsForAccount(Account account){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		List<Project> ps = new Vector<>();
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT p.status as status, p.id as id, p.name as project, p.start as start, p.end as end, p.authentication as authentication, p.log as log, rol.role as role, rol.id as roleid "+
			"FROM `__projectassignment` as asign, `__projects` as p, `__role` as rol WHERE rol.id=asign.roleid AND asign.projectid=p.id AND asign.accountid=?");
			pst.setInt(1, account.id);
			ResultSet res = pst.executeQuery();
			while(res.next()){
				Project project = new Project(res.getInt("id"), res.getString("project"));
				{
					long start = res.getTimestamp("start").getTime();
					project.startdate = new java.util.Date(start);
				}
				{
					long end = res.getTimestamp("end").getTime();
					project.enddate = new java.util.Date(end);
				}
				
				String auth = res.getString("authentication");
				if(auth != null) project.authenticationmode = auth.toUpperCase();
				else project.authenticationmode = "PIN";
				
				String l = res.getString("log");
				if(l != null) project.logmode = l;
				else project.logmode = "off";
				
				project.status = res.getInt("status");
				project.asRole = res.getString("role");
				project.asRoleid = res.getInt("roleid");
				ps.add(project);
			}
			pst.close();
		}
		catch(Exception ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return ps;
	}
	
	/*
	 * ################### RESPONSE METHODS
	 */
	
	public static int ResponseCount(String projectname){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		int count = -1;
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT count(*) FROM `INFORMATION_SCHEMA`.`COLUMNS` WHERE `TABLE_SCHEMA`=? AND `TABLE_NAME`=?");
			pst.setString(1, "qtafi");
			pst.setString(2, projectname);
			ResultSet res = pst.executeQuery();
			if(res.next()){
				if(res.getInt(1) > 0){
					pst = con.prepareStatement("SELECT count(*) FROM `" + projectname+"`");
					ResultSet res2 = pst.executeQuery();
					if(res2.next()) count = res2.getInt(1);
					pst.close();
				}
			}
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return count;
	}
	
	public static class C3ChartData {
		StringBuffer x = new StringBuffer();
		StringBuffer data = new StringBuffer();
		
		public String toString(){
			StringBuffer buf = new StringBuffer();
			buf.append("{\n");
			buf.append("\"x\": \"x\",\n");
			buf.append("\"columns\": [\n");
			
			buf.append("[\"x\", ");
			if(x.length() > 2) buf.append(x);
			else buf.append(new Date().getTime());
			buf.append("],\n");
			
			buf.append("[\"Count\", ");
			if(data.length() > 2) buf.append(data);
			else buf.append("0");
			buf.append("]\n");
			
			buf.append("],\n");
			
			buf.append("\"type\": \"line\"\n");
			buf.append("}\n");
			buf.append("");
			return buf.toString();
		}
	}
	
	public static Calendar cal = Calendar.getInstance();
	
	/**
	 * 
	 * @param projectname
	 * @param buf
	 * @param start
	 */
	public static C3ChartData C3ResponseProgression(String projectname, final Date start){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		C3ChartData data = new C3ChartData();
		/*
		if(start != null){
			data.x.append(start.get);
			data.data.append(0);
		}
		*/
		
		try {
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT count(*) FROM `INFORMATION_SCHEMA`.`COLUMNS` WHERE `TABLE_SCHEMA`=? AND `TABLE_NAME`=?");
			pst.setString(1, "qtafi");
			pst.setString(2, projectname);
			ResultSet res = pst.executeQuery();
			if(res.next()){
				if(res.getInt(1) > 0){
					pst.close();
					//pst = con.prepareStatement("SELECT  __inited FROM `" + projectname + "`");
					pst = con.prepareStatement("SELECT ROUND(__inited,-6) AS inited,"+
							"Date(ROUND(__inited, -6)) as histdate," +
							"COUNT(*) AS count FROM `" + projectname + "` GROUP BY inited ORDER BY inited");
					
					ResultSet res2 = pst.executeQuery();
					int c = 0;
					while(res2.next()){
						if(res2.isFirst()){
							cal.setTimeInMillis(res2.getTimestamp("histdate").getTime());
							cal.add(Calendar.DAY_OF_YEAR, -1);
							data.x.append(cal.getTimeInMillis()+ ",");
							data.data.append("0,");
						}
					
						data.x.append(res2.getTimestamp("histdate").getTime());
						c += res2.getInt("count");
						
						data.data.append(c);
						if(! res2.isLast()){
							data.x.append(",");
							data.data.append(",");
						}
					}
					
					res2.close();
					pst.close();
				}
			}
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return data;
	}
	
	public static void HCResponseProgression(String projectname, StringBuffer buf, final Date start){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		buf.append("[");
		if(start != null){
			buf.append("["+start.getTime() + ",0]");
		}
		try {
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT count(*) FROM `INFORMATION_SCHEMA`.`COLUMNS` WHERE `TABLE_SCHEMA`=? AND `TABLE_NAME`=?");
			pst.setString(1, "qtafi");
			pst.setString(2, projectname);
			ResultSet res = pst.executeQuery();
			if(res.next()){
				if(res.getInt(1) > 0){
					pst.close();
					pst = con.prepareStatement("SELECT  __inited FROM `" + projectname + "`");
					ResultSet res2 = pst.executeQuery();
					int c = 1;
					while(res2.next()){
						if(res2.isFirst()) buf.append(",");
						buf.append("["+res2.getTimestamp("__inited").getTime() + ","+c+"]");
						if(! res2.isLast()) buf.append(",");
						c++;
					}
					
					res2.close();
					pst.close();
				}
			}
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		buf.append("]");
	}
	
	
	public static List<DbColumn> ResponseColumns(String projectname, boolean order){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		List<DbColumn> columns = new Vector<DbColumn>();
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT `COLUMN_TYPE`, `COLUMN_NAME` FROM `INFORMATION_SCHEMA`.`COLUMNS` WHERE `TABLE_SCHEMA`=? AND `TABLE_NAME`=?");
			pst.setString(1, "qtafi");
			pst.setString(2, projectname);
			ResultSet res = pst.executeQuery();
			if(order){
				QuestionDB questiondb = SessionResource.QuestionDB("index.qml");
				List<Variable> varorder = questiondb.getAllVariables();
				List<String> dbvariables = new Vector<String>();
				while(res.next()) {
					String col = res.getString("COLUMN_NAME");
					dbvariables.add(col);
				}
				Vector<Variable> torem = new Vector<Variable>();
				for(Variable v: varorder){
					if(dbvariables.contains(v.name)){
						dbvariables.remove(v.name);
					}
					else {
						System.out.println("variable " + v.name + " not in database");
						torem.add(v);
					}
				}
				
				for(Variable rem: torem){
					varorder.remove(rem);
				}
				
				for(String mis: dbvariables){
					varorder.add(new Variable(mis, ""));
				}
				
				for(Variable v: varorder){
					columns.add(new DbColumn(v.name, v.label));
				}
			}
			else {
				while(res.next()) {
					String col = res.getString("COLUMN_NAME");
					columns.add(new DbColumn(col));
				}
			}
			pst.close();
			
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return columns;
	}
	
//	private static final boolean varlistContains(List<Variable> list, String n){
//		for(Variable v: list) if(v.name.equals(n)) return true;
//		return false;
//	}
	
	public static DbRow ResponseRow(String projectname, List<DbColumn> columns, int index){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		DbRow r = null;
		try{ 
			con = ds.getConnection();
			StringBuffer buf = new StringBuffer();
			buf.append("SELECT ");
			for(Iterator<DbColumn> it = columns.iterator(); it.hasNext(); ){
				buf.append("`" + it.next().name + "`");
				if(it.hasNext()) buf.append(", ");
			}
			buf.append(" FROM `" + projectname + "` WHERE __n=?");
			//pst = con.prepareStatement("SELECT * FROM `" + projectname + "` WHERE __n=?");
			pst = con.prepareStatement(buf.toString());
			pst.setInt(1, index);
			ResultSet res = pst.executeQuery();
			ResultSetMetaData rsmd = res.getMetaData();
			String cell;
			if(res.next()) {
				r = new DbRow(rsmd.getColumnCount());
				for(int i = 0; i < rsmd.getColumnCount(); i++){
					cell = res.getString(i+1);
					if(cell == null) r.content[i] = "-9";
					else if(cell.trim().equals("")) r.content[i] = "-9";
					else r.content[i] = cell.replaceAll("&", "+");
				}
			}
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return r;
	}
	
	final static byte[] MISSING = new byte[]{'-','9'};
	final static byte[] SEPARATOR = new byte[]{';'};
	final static byte[] NEWLINE = new byte[]{'\n'};
	final static byte[] QUOTE = new byte[]{'"'};
	
	public static void ResponseExport(String projectname, OutputStream os, boolean order){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		try {
			if(! order){
				con = ds.getConnection();
				pst = con.prepareStatement("SELECT * FROM `" + projectname + "`");
				ResultSet res = pst.executeQuery();
				ResultSetMetaData rsmd = res.getMetaData();
				
				int c = rsmd.getColumnCount();
				byte[] cell;
				//Header
				for(int i = 0; i < rsmd.getColumnCount(); i++){
					os.write(QUOTE);
					cell = (rsmd.getColumnName(i+1)).getBytes();
					os.write(cell);
					os.write(QUOTE);
					if(i < c-1) os.write(SEPARATOR);
				}
				os.write(NEWLINE);
				String cd = "";
				while(res.next()) {
					for(int i = 0; i < c; i++){
						cd = res.getString(i+1);
						int type = rsmd.getColumnType(i+1);
						if(type == Types.INTEGER){
							if(cd == null) os.write(MISSING);
							else if(cd.trim().equals("")) os.write(MISSING);
							else os.write(cd.getBytes());
						}
						else {
							os.write(QUOTE);
							if(cd == null) os.write(MISSING);
							else if(cd.trim().equals("")) os.write(MISSING);
							else {
								cd = cd.replaceAll("\"", "'");
								os.write(cd.getBytes());
							}
							os.write(QUOTE);
						}
						if(i < c-1) os.write(SEPARATOR);
					}
					os.write(NEWLINE);
				}
				pst.close();
			}
			else {
				List<DbColumn> cols = ResponseColumns(projectname, true);
				StringBuffer buf = new StringBuffer();
				buf.append("SELECT ");
				for(Iterator<DbColumn> it = cols.iterator(); it.hasNext(); ){
					buf.append("`" + it.next().name + "`");
					if(it.hasNext()) buf.append(", ");
				}
				buf.append(" FROM `" + projectname + "`");
				
				con = ds.getConnection();
				pst = con.prepareStatement(buf.toString());
				ResultSet res = pst.executeQuery();
				ResultSetMetaData rsmd = res.getMetaData();
				int c = rsmd.getColumnCount();
				for(int i = 0; i < rsmd.getColumnCount(); i++){
					os.write(QUOTE);
					os.write(rsmd.getColumnName(i+1).getBytes());
					os.write(QUOTE);
					if(i < c-1) os.write(SEPARATOR);
				}
				os.write(NEWLINE);
				String cd = "";
				while(res.next()) {
					for(int i = 0; i < c; i++){
						cd = res.getString(i+1);
						int type = rsmd.getColumnType(i+1);
						if(type == Types.INTEGER){
							if(cd == null) os.write(MISSING);
							else if(cd.trim().equals("")) os.write(MISSING);
							else os.write(cd.getBytes());
						}
						else {
							os.write(QUOTE);
							if(cd == null) os.write(MISSING);
							else if(cd.trim().equals("")) os.write(MISSING);
							else {
								cd = cd.replaceAll("\"", "'");
								os.write(cd.getBytes());
							}
							os.write(QUOTE);
						}
						if(i < c-1) os.write(SEPARATOR);
					}
					os.write(NEWLINE);
				}
				pst.close();
			}
		}
		catch(Exception ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
	
	public static String ResponseDumpTable(String projectname){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		int rowcount = -1;
		File dumpfile = null;
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT count(*) FROM `INFORMATION_SCHEMA`.`COLUMNS` WHERE `TABLE_SCHEMA`=? AND `TABLE_NAME`=?");
			pst.setString(1, ds.getDefaultCatalog());
			pst.setString(2, projectname);
			ResultSet res = pst.executeQuery();
			if(res.next()){
				if(res.getInt(1) > 0){
					dumpfile = Resources.ProjectResource(projectname, "dump_" + System.currentTimeMillis() + ".sql");
					MySQLDump dump = new MySQLDump(ds, dumpfile);
					rowcount = dump.dump(projectname);
				}
			}
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		if(dumpfile == null) return null;
		return dumpfile.getAbsolutePath();
	}
	
	public static void ResponseDropTable(String projectname){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("DROP TABLE IF EXISTS `" + projectname + "`");
			pst.execute();
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
	
	public static class FrequencyData {
		
		public HashMap<String, Integer> frequ = new HashMap<String, Integer>();
		public int total;
		public int totaladjusted;
		
		void add(int count, String cat){
			if(cat == null) cat = "-9";
			else if(cat.trim().equals("")) cat = "-9";
			if(frequ.get(cat) == null){
				frequ.put(cat, count);
			}
			else frequ.put(cat, frequ.get(cat) + count);
			
			if(! cat.equals("-9")) totaladjusted += count;
			total += count;
		}
	}
	
	public static FrequencyData ResponseFrequency(String projectname, String column){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		FrequencyData fr = new FrequencyData();
		try { 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT count(__n) as count, `"+ column +"` as cat FROM `" + projectname + "` GROUP BY `"+ column +"`");
			ResultSet res =  pst.executeQuery();
			int c;
			String cat;
			while(res.next()){
				c = res.getInt("count");
				cat = res.getString("cat");
				fr.add(c, cat);
			}
			res.close();
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return fr;
	}
	
	
	/*
	 * #################### LOG METHODS
	 */
	
	public static int LogCount(String projectname){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		int count = -1;
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT count(*) FROM `INFORMATION_SCHEMA`.`COLUMNS` WHERE `TABLE_SCHEMA`=? AND `TABLE_NAME`=?");
			pst.setString(1, "qtafi");
			pst.setString(2, projectname + ".log");
			ResultSet res = pst.executeQuery();
			if(res.next()){
				if(res.getInt(1) > 0){
					pst = con.prepareStatement("SELECT count(*) FROM `" + projectname+".log`");
					ResultSet res2 = pst.executeQuery();
					if(res2.next()) count = res2.getInt(1);
					pst.close();
				}
			}
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return count;
	}
	
	
	public static List<DbColumn> LogColumns(String projectname){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		List<DbColumn> columns = new Vector<DbColumn>();
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT `COLUMN_TYPE`, `COLUMN_NAME` FROM `INFORMATION_SCHEMA`.`COLUMNS` WHERE `TABLE_SCHEMA`=? AND `TABLE_NAME`=?");
			pst.setString(1, "qtafi");
			pst.setString(2, projectname + ".log");
			ResultSet res = pst.executeQuery();
			int c = 1;
			while(res.next()) {
				columns.add(new DbColumn(res.getString("COLUMN_NAME")));
				c++;
			}
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return columns;
	}
	
	public static DbRow LogRow(String projectname, int index){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		DbRow r = null;
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT * FROM `" + projectname + ".log` WHERE logid=?");
			pst.setInt(1, index);
			ResultSet res = pst.executeQuery();
			ResultSetMetaData rsmd = res.getMetaData();
			if(res.next()) {
				r = new DbRow(rsmd.getColumnCount());
				for(int i = 0; i < rsmd.getColumnCount(); i++){
					r.content[i] = res.getString(i+1);
				}
			}
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return r;
	}
	
	public static void LogExport(String projectname, OutputStream os){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT * FROM `" + projectname + ".log`");
			ResultSet res = pst.executeQuery();
			ResultSetMetaData rsmd = res.getMetaData();
			int c = rsmd.getColumnCount();
			for(int i = 0; i < rsmd.getColumnCount(); i++){
				os.write(QUOTE);
				os.write(rsmd.getColumnName(i+1).getBytes());
				os.write(QUOTE);
				if(i < c-1) os.write(SEPARATOR);
			}
			os.write(NEWLINE);
			String cd = "";
			while(res.next()) {
				for(int i = 0; i < c; i++){
					cd = res.getString(i+1);
					os.write(QUOTE);
					if(cd != null) os.write(cd.getBytes());
					else os.write("-9".getBytes());
					os.write(QUOTE);
					if(i < c-1) os.write(SEPARATOR);
				}
				os.write(NEWLINE);
			}
			pst.close();
		}
		catch(Exception ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
	
	public static void LogDropTable(String projectname){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("DROP TABLE IF EXISTS `" + projectname + ".log`");
			pst.execute();
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
	
	public static FrequencyData LogFrequency(String projectname, String column){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		FrequencyData fr = new FrequencyData();
		try { 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT count(DISTINCT logid) as count, `"+ column +"` as cat FROM `" + projectname + ".log` GROUP BY `"+ column +"`");
			ResultSet res =  pst.executeQuery();
			int c;
			String cat;
			while(res.next()){
				c = res.getInt("count");
				cat = res.getString("cat");
				fr.add(c, cat);
			}
			res.close();
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return fr;
	}
	
	/**
	 * Count of users
	 * 
	 * 
	 * @param projectname
	 * @return
	 */
	public static int UsersCount(String projectname){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		int count = 0;
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT count(*) FROM `INFORMATION_SCHEMA`.`COLUMNS` WHERE `TABLE_SCHEMA`=? AND `TABLE_NAME`=?");
			pst.setString(1, "qtafi");
			pst.setString(2, projectname + ".users");
			ResultSet res = pst.executeQuery();
			if(res.next()){
				if(res.getInt(1) > 0){
					pst = con.prepareStatement("SELECT count(*) FROM `" + projectname + ".users`");
					ResultSet res2 = pst.executeQuery();
					if(res2.next()) count = res2.getInt(1);
					pst.close();
				}
			}
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return count;
	}
	
	public static int UsersCount(String projectname, boolean filtered){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		int count = 0;
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT count(*) FROM `INFORMATION_SCHEMA`.`COLUMNS` WHERE `TABLE_SCHEMA`=? AND `TABLE_NAME`=?");
			pst.setString(1, "qtafi");
			pst.setString(2, projectname + ".users");
			ResultSet res = pst.executeQuery();
			if(res.next()){
				if(res.getInt(1) > 0){
					pst = con.prepareStatement("SELECT count(*) FROM `" + projectname + ".users` as us LEFT JOIN `"+projectname+"` as res ON us.pin = res.__pin WHERE res.__pin IS NULL");
					ResultSet res2 = pst.executeQuery();
					if(res2.next()) count = res2.getInt(1);
					pst.close();
				}
			}
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return count;
	}
	
	
	/**
	 * SELECT count(*) FROM `INFORMATION_SCHEMA`.`COLUMNS` WHERE `TABLE_SCHEMA`=qtafi AND `TABLE_NAME`=
	 */
	public static int UsersResponseRate(String projectname){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		int count = 0;
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT count(DISTINCT TABLE_NAME) FROM `INFORMATION_SCHEMA`.`COLUMNS` WHERE `TABLE_SCHEMA`=? AND `TABLE_NAME` in (?,?)");
			pst.setString(1, "qtafi");
			pst.setString(2, projectname + ".users");
			pst.setString(3, projectname);
			
			ResultSet res = pst.executeQuery();
			if(res.next()){
				if(res.getInt(1) == 2){
					pst = con.prepareStatement("SELECT count(res.__logins) as count FROM `" + projectname + ".users` as us LEFT JOIN `"+projectname+"` as res ON us.pin = res.__pin WHERE res.__logins>0");
					ResultSet res2 = pst.executeQuery();
					if(res2.next()) count = res2.getInt(1);
					pst.close();
				}
			}
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return count;
	}
	
	
	public static boolean UsersResponseExists(String projectname){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		boolean exists = false;
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT count(DISTINCT TABLE_NAME) FROM `INFORMATION_SCHEMA`.`COLUMNS` WHERE `TABLE_SCHEMA`=? AND `TABLE_NAME`=?");
			pst.setString(1, "qtafi");
			pst.setString(2, projectname);
			ResultSet res = pst.executeQuery();
			if(res.next()){
				if(res.getInt(1) == 1) exists = true;
			}
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return exists;
	}
	
	
	/**
	 * 
	 * SELECT us.n as n, us.pin as pin, res.__logins as logins  FROM `" + projectname + ".users` as us, `"+projectname+"` as res WHERE us.pin = res.__pin AND us.n=? 
	 * 
	 * 
	 * SELECT us.n as n, us.pin as pin, test.__logins as logins  FROM `test.users` as us
	 * LEFT JOIN test ON us.pin = test.__pin
	 * 
	 * 
	 * 
	 * @param projectname
	 * @param row
	 * @return
	 */
	public static User Users(String projectname, int row, boolean responseExist, boolean filtered){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		User user = null;
		try{ 
			con = ds.getConnection();
			if(responseExist){
				if(filtered){
					pst = con.prepareStatement("SELECT us.n as n, us.pin as pin, us.info as info, res.__logins as logins  FROM `" + projectname + ".users` as us LEFT JOIN `"+projectname+"` as res ON us.pin = res.__pin WHERE res.__pin IS NULL LIMIT ?,1");
					pst.setInt(1, row);
					ResultSet res = pst.executeQuery();
					if(res.next()){
						int lgins = res.getInt("logins");
						user = new User(res.getInt("n"), res.getString("pin"), res.getString("info"), lgins > 0);
					}
					res.close();
				}
				else {
					pst = con.prepareStatement("SELECT us.n as n, us.pin as pin, us.info as info, res.__logins as logins  FROM `" + projectname + ".users` as us LEFT JOIN `"+projectname+"` as res ON us.pin = res.__pin LIMIT ?,1");
					pst.setInt(1, row);
					ResultSet res = pst.executeQuery();
					if(res.next()){
						int lgins = res.getInt("logins");
						user = new User(res.getInt("n"), res.getString("pin"), res.getString("info"), lgins > 0);
					}
					res.close();
				}
			}
			else {
				pst = con.prepareStatement("SELECT n, pin, info FROM `" + projectname + ".users` WHERE n=? ");
				pst.setInt(1, row);
				ResultSet res = pst.executeQuery();
				if(res.next()){
					int lgins = 0;
					user = new User(res.getInt("n"), res.getString("pin"), res.getString("info"), lgins > 0);
				}
				res.close();
			}
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return user;
	}
	
	public static void UserUpdate(String projectname, User user){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("UPDATE `"+projectname+".users` SET pin=?, info=? WHERE n=? ");
			pst.setString(1, user.pin);
			pst.setString(2, user.info);
			pst.setInt(3, user.n);
		
			pst.execute();
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}	
	}
	
	public static void UsersDropTable(String projectname){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("DROP TABLE IF EXISTS `" + projectname + ".users`");
			pst.execute();
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
	
	
	public static int UsersDumpTable(String projectname){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		int rowcount = -1;
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT count(*) FROM `INFORMATION_SCHEMA`.`COLUMNS` WHERE `TABLE_SCHEMA`=? AND `TABLE_NAME`=?");
			pst.setString(1, ds.getDefaultCatalog());
			pst.setString(2, projectname);
			ResultSet res = pst.executeQuery();
			if(res.next()){
				if(res.getInt(1) > 0){
					MySQLDump dump = new MySQLDump(ds, Resources.ProjectResource(projectname, "users_" + System.currentTimeMillis() + ".sql"));
					rowcount = dump.dump(projectname + ".users");
				}
			}	
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return rowcount;
	}
	
	public static int UsersEnsureTableExists(String projectname){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		try{ 
			con = ds.getConnection();
			Statement st = con.createStatement();
			st.execute("CREATE TABLE IF NOT EXISTS `" + projectname + ".users` (n INT PRIMARY KEY, pin VARCHAR(100), info VARCHAR(200), UNIQUE(pin) )" );
			st.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return 1;
	}
	
	final static String alphabet = "123456789abcdefghklmnpqrstuvwxyzABCDEFGHKLMNPQRSTUVWXYZ";
	final static String positivchars = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ,.:~=|-_+*#!$%";
	final static Random r = new Random();
	
	
	public static boolean isValidPin(String pin){
		if(pin.trim().length() < 1) return false;
		
		for(int i = 0; i < pin.length(); i++){
			char c = pin.charAt(i);
			if(positivchars.indexOf(c) < 0){
				return false;
			}
		}
		return true;
	}
	
	
	private static char randomChar(){
	    return alphabet.charAt(r.nextInt(alphabet.length()));
	}
	
	private static String createRandomPin(int len){
		char[] pin = new char[len];
		for(int i = 0; i < len; i++){
			pin[i] = randomChar();
		}
		return new String(pin);
	}
	
	public static int UsersCreate(String projectname, int count){
		UsersEnsureTableExists(projectname);
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		int start = 0;
		try{ 
			con = ds.getConnection();
			Statement st1 = con.createStatement();
			ResultSet res1 = st1.executeQuery("SELECT max(n) FROM `" + projectname + ".users`");
			if(res1.next()){
				if(res1.getString(1) != null){
					start = res1.getInt(1) + 1;
				}
			}
			st1.close();
			//Statement st = con.createStatement();
			for(int i = 0; i < count; i++){
				String pin = createRandomPin(6);
				//System.out.println("insert pin " + (start +i)+ "->" + pin);
				while(! insertPin(con, projectname, (start +i), pin));
				//st.execute("INSERT INTO `" + projectname + ".users` (n, pin) VALUES ("+(start + i)+", \""+pin+"\")");
			}
			//st.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return 1;
	}
	
	private static boolean insertPin(Connection conn, String projectname, int num, String pin){
		try{
			Statement st = conn.createStatement();
			st.execute("INSERT INTO `" + projectname + ".users` (n, pin) VALUES ("+num+", \""+pin+"\")");
			st.close();
			return true;
		}
		catch(SQLException ex){
			System.out.println("double pin " + pin);
			return false;
		}
	}
	
	
	public static int UsersRemoveAll(String projectname){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT count(*) FROM `INFORMATION_SCHEMA`.`COLUMNS` WHERE `TABLE_SCHEMA`=? AND `TABLE_NAME`=?");
			pst.setString(1, "qtafi");
			pst.setString(2, projectname + ".users");
			ResultSet res = pst.executeQuery();
			if(res.next()){
				if(res.getInt(1) > 0){
					Statement st = con.createStatement();
					st.execute("DELETE FROM `" + projectname + ".users`");
					st.close();
				}
			}
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return 1;
	}
	
	public static void UsersExport(String projectname, OutputStream out, boolean responseExist, boolean appendurls, boolean filtered){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		
		try {
			if(appendurls){
				out.write(QUOTE);
				out.write("PIN".getBytes());
				out.write(QUOTE);
				out.write(SEPARATOR);
				out.write(QUOTE);
				out.write("ANSWERED".getBytes());
				out.write(QUOTE);
				out.write(SEPARATOR);
				out.write(QUOTE);
				out.write("URL".getBytes());
				out.write(QUOTE);
				out.write(NEWLINE);
				//out.write("\"PIN\"ANSWERED\tURL\n".getBytes());
			}
			else {
				out.write(QUOTE);
				out.write("PIN".getBytes());
				out.write(QUOTE);
				out.write(SEPARATOR);
				out.write(QUOTE);
				out.write("ANSWERED".getBytes());
				out.write(QUOTE);
				out.write(NEWLINE);
				//out.write("PIN\tANSWERED\n".getBytes());
			}
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT count(*) FROM `INFORMATION_SCHEMA`.`COLUMNS` WHERE `TABLE_SCHEMA`=? AND `TABLE_NAME`=?");
			pst.setString(1, "qtafi");
			pst.setString(2, projectname + ".users");
			ResultSet res = pst.executeQuery();
			if(res.next()){
				if(res.getInt(1) > 0){
					if(responseExist){
						if(filtered){
							pst = con.prepareStatement("SELECT us.pin as pin, res.__logins as logins  FROM `" + projectname + ".users` as us LEFT JOIN `"+projectname+"` as res ON us.pin = res.__pin WHERE res.__pin IS NULL");
						}
						else {
							pst = con.prepareStatement("SELECT us.pin as pin, res.__logins as logins  FROM `" + projectname + ".users` as us LEFT JOIN `"+projectname+"` as res ON us.pin = res.__pin");
						}
						ResultSet res2 = pst.executeQuery();
						while(res2.next()){
							out.write(QUOTE);
							out.write(res2.getBytes(1));
							out.write(QUOTE);
							out.write(SEPARATOR);
							out.write(QUOTE);
							out.write((res2.getInt(2) > 0? "true".getBytes(): "false".getBytes()));
							out.write(QUOTE);
							
							if(appendurls){
								out.write(SEPARATOR);
								out.write(QUOTE);
								out.write((Resources.HttpPath() + "/qtafi/projects/" + projectname + "/index.qml?" + res2.getString(1)).getBytes());
								out.write(QUOTE);
							}
							if(! res2.isLast()){
								out.write(NEWLINE);
							}
						}
						res2.close();
						pst.close();
					}
					else {
						pst = con.prepareStatement("SELECT pin FROM `" + projectname + ".users`");
						ResultSet res2 = pst.executeQuery();
						while(res2.next()){
							out.write(QUOTE);
							out.write(res2.getBytes(1));
							out.write(QUOTE);
							out.write(SEPARATOR);
							out.write(QUOTE);
							out.write("false".getBytes());
							out.write(QUOTE);
							if(appendurls){
								out.write(SEPARATOR);
								out.write(QUOTE);
								out.write((Resources.HttpPath() + "/qtafi/projects/" + projectname + "/index.qml?" + res2.getString(1)).getBytes());
								out.write(QUOTE);
							}
							if(! res2.isLast()) out.write(NEWLINE);
						}
						res2.close();
						pst.close();
					}
				}
			}
		}
		catch(Exception ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
	
	public static void UsersAppend(String projectname, User user){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		int start = 0;
		try {
			con = ds.getConnection();
			Statement st1 = con.createStatement();
			ResultSet res1 = st1.executeQuery("SELECT max(n) FROM `" + projectname + ".users`");
			if(res1.next()){
				if(res1.getString(1) != null){
					start = res1.getInt(1) + 1;
				}
			}
			st1.close();
			
			if(user.pin != null){
				PreparedStatement st = con.prepareStatement("INSERT INTO `" + projectname + ".users` (n, pin, info) VALUES (?, ?, ?)");
				st.setInt(1, start);
				st.setString(2, user.pin);
				st.setString(3, user.info == null? "": user.info);
				st.execute();
				st.close();
			}
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
	
	
	public static List<Role> RoleAll(){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		List<Role> roles = new Vector<Role>();
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT * FROM `__role`");
			ResultSet res = pst.executeQuery();
			while(res.next()){
				Role r = new Role(res.getInt("id"), res.getString("role"));
				roles.add(r);
			}
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				con.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return roles;
	}
	
}
