/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.util;

import java.util.Arrays;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class InlineInputParser {
	
	
	public static String parseLine(String line, String id, String key){
		String r = parseTxt(line, id, key);
		r = parseNr(r, id, key);
		r = parsePreview(r, id, key);
		return r;
	}
	
	public static String parseXML(String line, String id, String key){
		String p = parseXMLPP(line);
		p = parseXMLPreview(p);
		return p;
	}
	
	static String parseTxt(final String txtinput, String id, String key) {
		final Pattern text = Pattern.compile("\\[([_]{1,10})\\]");
		final Matcher textMatcher = text.matcher(txtinput);
		StringBuffer buf = new StringBuffer();

		int c = 1;
		while (textMatcher.find()) {
			for (int i = 1; i <= textMatcher.groupCount(); i++) {
				String grs = textMatcher.group(i);
				int st = textMatcher.start(i);
				int e = textMatcher.end(i);
				textMatcher.appendReplacement(buf,
						"<qtafi:TE xmlns:qtafi=\"urn:qtafi\" managed=\"true\" variable=\"" + id + "_" + key + (c > 1 ? "_" + c : "") + "_TE\" size=\"" + ((e - st) * 5) + "\"/>");
			}
			c++;
		}
		textMatcher.appendTail(buf);
		return buf.toString();
	}

	static String parseNr(final String nrinput, String id, String key) {
		final Pattern text = Pattern
				.compile("\\[([_\\.]+)([\\.]){1,1}([_]*)\\]");
		final Matcher textMatcher = text.matcher(nrinput);
		StringBuffer buf = new StringBuffer();
		int c = 1;
		while (textMatcher.find()) {
			if (textMatcher.groupCount() == 3) {
				int prefix = textMatcher.end(1) - textMatcher.start(1);
				int postfix = textMatcher.end(3) - textMatcher.start(3);
				textMatcher.appendReplacement(buf,
						"<qtafi:ME xmlns:qtafi=\"urn:qtafi\" managed=\"true\" variable=\"" + id + "_" + key + (c > 1 ? "_" + c : "") + "_ME\" size=\"" + prefix + "\" />");
				// System.out.println("nr len " + prefix + "." + postfix);
			}
			c++;
		}
		textMatcher.appendTail(buf);
		return buf.toString();
	}

	static String parsePreview(final String txtinput, String id, String key) {
		final Pattern text = Pattern.compile("\\[([\\w ]{1,})\\]");
		final Matcher textMatcher = text.matcher(txtinput);
		StringBuffer buf = new StringBuffer();
		int c = 1;
		while (textMatcher.find()) {
			for (int i = 1; i <= textMatcher.groupCount(); i++) {
				String grs = textMatcher.group(i);
				int st = textMatcher.start(i);
				int e = textMatcher.end(i);
				textMatcher.appendReplacement(buf, "<span class=\"onlypaper\">" + grs + "</span>");
			}
			c++;
		}
		textMatcher.appendTail(buf);
		return buf.toString();
	}
	
	static String parseXMLPreview(final String xmlinput) {
		final Pattern tagPattern = Pattern.compile("<span([ ]{1,})class=\"onlypaper\">([\\w ]{2,})<\\/span>");
		final Matcher tagMatcher = tagPattern.matcher(xmlinput);
		StringBuffer buf = new StringBuffer();
		while (tagMatcher.find()) {
			//final String tag1 = tagMatcher.group(1);
			//System.out.println(tag1);
			//final String tag2 = tagMatcher.group(2);
			tagMatcher.appendReplacement(buf, "["+tagMatcher.group(2)+"]");
		}
		tagMatcher.appendTail(buf);
		return buf.toString();
	}
	
	
	static String parseXMLPP(final String xmlinput) {
		final Pattern tagPattern = Pattern.compile("<qtafi:([\\S]+)(\\s+)([^/>]+)/>");
		final Matcher tagMatcher = tagPattern.matcher(xmlinput);
		final Pattern attributesPattern = Pattern.compile("(\\S+)=[\"]?((?:.(?![\"]?\\s+(?:\\S+)=|[>\"]))+.)[\"]?");
		StringBuffer buf = new StringBuffer();
		//System.out.println("parseXML " + xmlinput);
		while (tagMatcher.find()) {
			//System.out.println("find " + tagMatcher.groupCount());
			if (tagMatcher.groupCount() == 3) {
				final String tag = tagMatcher.group(1);
				final String attrs = tagMatcher.group(3);
				final Matcher attributesMatcher = attributesPattern.matcher(attrs);
				//System.out.println("tag: " + tag);
				boolean isManaged = false;
				int len = 0;
				while (attributesMatcher.find()) {
					if (attributesMatcher.groupCount() == 2) {
						final String name = attributesMatcher.group(1);
						final String val = attributesMatcher.group(2);
						if (name.equals("managed")){
							if (val.equalsIgnoreCase("true")) isManaged = true;
						} 
						else if (name.equals("size")) {
							len = Integer.parseInt(val.replaceAll("\"", ""));
						}
						//System.out.println("attribute: " + name + " -> " + val);
					}
				}
				if (isManaged) {
					if (tag.equals("TE")) {
						char[] xxx = new char[(int) (len / 5)];
						Arrays.fill(xxx, '_');
						tagMatcher.appendReplacement(buf, "[" + new String(xxx)+ "]");
					} 
					else if (tag.equals("ME")) {
						char[] xxx = new char[(int) (len)];
						Arrays.fill(xxx, '_');
						tagMatcher.appendReplacement(buf, "[" + new String(xxx)+ ".]");
					} 
					else {
						tagMatcher.appendReplacement(buf, tagMatcher.group());
					}
				} 
				else{
					tagMatcher.appendReplacement(buf, tagMatcher.group());
				}
			}
		}
		tagMatcher.appendTail(buf);
		return buf.toString();
	}

	
	public static void main(String... args){
		//String in = "sdfsdf sdf s [Please got to question B23]";
		//System.out.println(in);
		//String out = parsePreview(in, "id1", "key2");
		//System.out.println(out);
		String out = parseXMLPreview("sdfsfsfsd sd sdf sd ");
		System.out.println(out);
	}
	
	
}
