/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.util;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.apache.commons.dbcp2.BasicDataSource;
import org.eclipse.rap.rwt.RWT;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

import de.eoda.qtafi.surveymanager.model.LibQuestion;
import de.tinu.qtafi4.db.QuestionDBImpl;
import de.tinu.qtafi4.model.Question;

public class LibraryUtils {
	
	public static final int PUBLIC = 1;
	public static final int PRIVATE = 0;

	static Connection con = null;
	static PreparedStatement pst;
	
	public static void BuidTree(final Tree librarytree, TreeItem parent,  int accountid){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT DISTINCT * FROM `__library` WHERE accountid=? OR public=1");
			pst.setInt(1, accountid);
			ResultSet res = pst.executeQuery();
			while(res.next()){
				String subject = res.getString("subject");
				String quid = res.getString("quid");
				TreeItem item = new TreeItem(parent, SWT.NONE);
				item.setText(typeLabel(res.getString("type")) + " - " + quid + " - " + subject);
				item.setData("importQuestion_" + res.getInt("id"));
				item.setData("lid" , res.getInt("id"));
			}
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				con.close();
			}
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
	
	
	static String typeLabel(String el){
		switch(el){
		case "CV": return "SC";
		case "MD": return "MC";
		case "OR": return "OR";
		}
		return el;
	}
	
	public static LibQuestion GetQuestion(int id){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		LibQuestion question = null;
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("SELECT * FROM `__library` WHERE id=?");
			pst.setInt(1, id);
			ResultSet res = pst.executeQuery();
			if(res.next()){
				int vis = res.getInt("public");
				question = new LibQuestion(QuestionDBImpl.deserialize(res.getString("qml"), res.getString("type")), vis);
			}
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				con.close();
			}
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return question;
	}
	
	
	public static void UpdateQuestion(int id, Question question, int accountid, int publicity){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("UPDATE `__library` SET quid=?, subject=?, public=?, qml=? WHERE id=?");
			pst.setString(1, question.id);
			pst.setString(2, question.subject);
			pst.setInt(3, publicity);
			pst.setString(4, QuestionDBImpl.serialize(question));
			pst.setInt(5, id);
			pst.execute();
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				con.close();
			}
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
	
	public static void InsertQuestion(Question question, int accountid, int publicity){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("INSERT INTO `__library` (quid, subject, accountid, public, type, qml) VALUES (?, ?, ?, ?, ?, ?)");
			pst.setString(1, question.id);
			pst.setString(2, question.subject);
			pst.setInt(3, accountid);
			pst.setInt(4, publicity);
			
			pst.setString(5, question.type);
			String qml = QuestionDBImpl.serialize(question);
	
			pst.setString(6, qml);
			pst.execute();
			
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				con.close();
			}
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
	
	public static void DeleteQuestion(int id){
		BasicDataSource ds = (BasicDataSource) RWT.getApplicationContext().getAttribute("datasource");
		try{ 
			con = ds.getConnection();
			pst = con.prepareStatement("DELETE FROM `__library` WHERE id=?");
			pst.setInt(1, id);
			pst.execute();
			pst.close();
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				con.close();
			}
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
	
}
