/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.util;

import java.io.BufferedWriter;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;

import javax.sql.DataSource;

/**
 * http://www.bosit.be/blog/2008/03/29/dumping-a-mysql-database-from-java/
 * 
 * 
 * @author maks
 *
 */
public class MySQLDump {

    private static String version = "0.1";
    final DataSource datasource;
    final File dumpfile;
    
    Connection conn;
    String databaseProductVersion = null;
    //private String mysqlVersion = null;

	public MySQLDump(DataSource datasource, File dumpfile) {
		this.datasource = datasource;
		this.dumpfile = dumpfile;
	}

	public synchronized int dump(String database){
		int rows = 0;
		try {
			this.conn = datasource.getConnection();
			DatabaseMetaData databaseMetaData = conn.getMetaData();
			databaseProductVersion = databaseMetaData.getDatabaseProductVersion();
			BufferedWriter out = new BufferedWriter(new FileWriter(dumpfile));
			this.header(out, database);
			this.dumpCreateTable(out, database);
			rows = this.dumpTable(out, database);
			out.close();
		} 
		catch (SQLException se) {
			se.printStackTrace();
		} 
		catch (IOException e) {
			e.printStackTrace();
		} 
		finally {
			try {
				conn.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return rows;
	}
	
    void header(BufferedWriter out, String database) throws IOException {
        out.write("-- MySQL Dump " + version + "\n--\n");
        out.write("-- Database: " + database + "\n");
        out.write("-- ------------------------------------------------------\n");
        out.write("-- Server Version: " + databaseProductVersion + "\n--");
    }
    
	void dumpCreateDatabase(BufferedWriter out, String database) throws SQLException, IOException {
		String createDatabase = null;
		Statement s = conn.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_READ_ONLY);
		s.executeQuery("SHOW CREATE DATABASE `" + database + "`");
		ResultSet rs = s.getResultSet();
		while (rs.next()) {
			createDatabase = rs.getString("Create Database") + ";";
		}
		out.write(createDatabase);
	}

    void dumpCreateTable(BufferedWriter out, String table) throws SQLException, IOException {
        String createTable = null;
        out.write("\n\n--\n-- Table structure for table `" + table + "`\n--\n\n");
        Statement s = conn.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_READ_ONLY);
        s.executeQuery("SHOW CREATE TABLE `" + table + "`");
        ResultSet rs = s.getResultSet ();
        while(rs.next ()){
            createTable = rs.getString("Create Table") + ";";
        }
        out.write(createTable);
    }

	int dumpTable(BufferedWriter out, String table) throws SQLException, IOException {
		Statement s = conn.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_READ_ONLY);
		out.write("\n\n--\n-- Dumping data for table `" + table + "`\n--\n\n");
		s.executeQuery("SELECT * FROM `" + table + "`");
		ResultSet rs = s.getResultSet();
		ResultSetMetaData rsMetaData = rs.getMetaData();
		int columnCount = rsMetaData.getColumnCount();
		StringBuffer prefix = new StringBuffer();
		prefix.append("INSERT INTO `" + table + "` (");
		for (int i = 1; i <= columnCount; i++) {
			if (i == columnCount) {
				prefix.append("`" + rsMetaData.getColumnName(i) + "`) VALUES(");
			} 
			else {
				prefix.append("`" + rsMetaData.getColumnName(i) + "`,");
			}
		}
		StringBuffer postfix = new StringBuffer();
		int count = 0;
		while (rs.next()) {
			postfix = new StringBuffer();
			for (int i = 1; i <= columnCount; i++) {
				if (i == columnCount) {
					// System.err.println(rs.getMetaData().getColumnClassName(i));
					postfix.append("'" + rs.getString(i) + "');\n");
				} 
				else {
					// System.err.println(rs.getMetaData().getColumnTypeName(i));
					if (rs.getMetaData().getColumnTypeName(i).equalsIgnoreCase("LONGBLOB")) {
						try {
							postfix.append("'"+ escapeString(rs.getBytes(i)).toString() + "',");
						} 
						catch (Exception e) {
							postfix.append("NULL,");
						}
					} 
					else {
						try {
							postfix.append("'"+ rs.getString(i).replaceAll("\n", "\\\\n").replaceAll("'", "\\\\'") + "',");
						}
						catch (Exception e) {
							postfix.append("NULL,");
						}
					}
				}
			}
			out.write(prefix.toString() + postfix.toString() + "\n");
			++count;
		}
		rs.close();
		s.close();

		return count;
	}

    /**
    * Escape string ready for insert via mysql client
    *
    * @param  bIn       String to be escaped passed in as byte array
    * @return bOut      MySQL compatible insert ready ByteArrayOutputStream
    */
    final ByteArrayOutputStream escapeString(byte[] bIn){
        int numBytes = bIn.length;
        ByteArrayOutputStream bOut = new ByteArrayOutputStream(numBytes+ 2);
        for (int i = 0; i < numBytes; ++i) {
            byte b = bIn[i];

            switch (b) {
            case 0: /* Must be escaped for 'mysql' */
                    bOut.write('\\');
                    bOut.write('0');
                    break;

            case '\n': /* Must be escaped for logs */
                    bOut.write('\\');
                    bOut.write('n');
                    break;

            case '\r':
                    bOut.write('\\');
                    bOut.write('r');
                    break;

            case '\\':
                    bOut.write('\\');
                    bOut.write('\\');

                    break;

            case '\'':
                    bOut.write('\\');
                    bOut.write('\'');

                    break;

            case '"': /* Better safe than sorry */
                    bOut.write('\\');
                    bOut.write('"');
                    break;

            case '\032': /* This gives problems on Win32 */
                    bOut.write('\\');
                    bOut.write('Z');
                    break;

            default:
                    bOut.write(b);
            }
        }
        return bOut;
    }

}
