/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.eoda.qtafi.surveymanager.util;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.List;
import java.util.Properties;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.eclipse.rap.rwt.RWT;

public class Resources {

	private static String projectspath = null;
	private static String archivpath = null;
	
	private static String Rootpath(){
		if(projectspath == null){
			Properties pp = new Properties();
			try {
				pp.load(new FileInputStream(ResolveFile("/WEB-INF/projects.properties")));
				projectspath = pp.getProperty("rootpath");
				if(projectspath == null) throw new IllegalArgumentException("Projectspath may not be null!");
			} 
			catch (Exception e) {
				e.printStackTrace();
			}
		}
		return projectspath;
	}
	
	public static String Archivpath(){
		if(archivpath == null){
			Properties pp = new Properties();
			try {
				pp.load(new FileInputStream(ResolveFile("/WEB-INF/projects.properties")));
				archivpath = pp.getProperty("archiv");
				if(archivpath == null) throw new IllegalArgumentException("Archivpath may not be null!");
			} 
			catch (Exception e) {
				e.printStackTrace();
			}
		}
		return archivpath;
	}
	
	private static void EnsureProjectDirectory(String project){
		File ff = new File(Rootpath() + "/" + project + "/");
		if(ff.isDirectory()) return;
		ff.mkdirs();
	}
	
	public static String HttpPath(){
		int port = RWT.getRequest().getServerPort();
		if(port == 443){
			return "https://" +RWT.getRequest().getServerName();
		}
		if(port == 80){
			return "http://" +RWT.getRequest().getServerName();
		}
		return "http://" +RWT.getRequest().getServerName() +":"+ RWT.getRequest().getServerPort();
	}
	
	public static File ResolveFile(String name){
		return new File(ContextListener.servletContext.getRealPath(name));
	}
	
	public static File ProjectResource(String project, String resource){
		EnsureProjectDirectory(project);
		return new File(Rootpath() + "/" + project + "/" + resource);	
	}
	
	public static File CreateProjectResource(String project, String resource, String content){
		EnsureProjectDirectory(project);
		File ff = new File(Rootpath() + "/" + project + "/" + resource);
		try{
			if(! ff.exists()) ff.createNewFile();
			if(content != null){
				BufferedWriter fw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(ff)));
				fw.write(content);
				fw.flush();
				fw.close();
			}
		}
		catch(IOException ex){
			ex.printStackTrace();
		}
		return ff;
	}
	
	public static File DeleteProjectResource(String project, String resource){
		EnsureProjectDirectory(project);
		File ff = new File(Rootpath() + "/" + project + "/" + resource);
		if(ff.exists()) ff.delete();
		return ff;
	}
	
	public static List<String> ListResources(String project){
		EnsureProjectDirectory(project);
		File projectdir = new File(Rootpath() + "/" + project);
		Vector<String> files = new Vector<String>();
		if(projectdir.isDirectory()){
			for(File f: projectdir.listFiles()){
				if(f.isFile()) files.add(f.getName());
			}
		}
		return files;
	}
	
	public static List<String> ListResources(String project, final String extension){
		EnsureProjectDirectory(project);
		File projectdir = new File(Rootpath() + "/" + project);
		Vector<String> files = new Vector<String>();
		if(projectdir.isDirectory()){
			for(File f: projectdir.listFiles(new FilenameFilter() {
				@Override
				public boolean accept(File dir, String name) {
					if(name.toLowerCase().endsWith(extension.toLowerCase())) return true;
					return false;
				}
			})){
				if(f.isFile()) files.add(f.getName());
			}
		}
		return files;
	}

	public synchronized static String ZIPResources(String project, String archiv){
		//File file = ProjectResource(project, archiv);
		File file = new File(archiv);
		zipFolder(new File(Rootpath() + "/" + project), file);
		if(file.exists()) return file.getAbsolutePath();
		return null;
	}
	
	public static void Remove(String project){
		File file = new File(Rootpath() + "/" + project);
		try {
			FileUtils.deleteDirectory(file);
		} 
		catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public static void zipFolder(final File folder, final File zipFile) {
		try {
			if(! folder.exists()) return;
			zipFolder(folder, new FileOutputStream(zipFile), zipFile);
		} 
		catch (IOException e) {
			e.printStackTrace();
		}
	}

	static void zipFolder(final File folder, final OutputStream outputStream, File without) throws IOException {
		try (ZipOutputStream zipOutputStream = new ZipOutputStream(outputStream)) {
			processFolder(folder, zipOutputStream, folder.getPath().length() + 1, without);
		}
	}

	static void processFolder(final File folder, final ZipOutputStream zipOutputStream, final int prefixLength, final File without) throws IOException {
		for (final File file : folder.listFiles()) {
			if (file.isFile()) {
				if(file.getAbsolutePath().equals(without.getAbsolutePath())) continue;
				final ZipEntry zipEntry = new ZipEntry(file.getPath().substring(prefixLength));
				zipOutputStream.putNextEntry(zipEntry);
				try (FileInputStream inputStream = new FileInputStream(file)) {
					IOUtils.copy(inputStream, zipOutputStream);
				}
				zipOutputStream.closeEntry();
			}
			else if (file.isDirectory()) {
				processFolder(file, zipOutputStream, prefixLength, without);
			}
		}
	}
}
