<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
	xmlns:xalan="http://xml.apache.org/xalan"
    xmlns:str="xalan://java.lang.String" 
    xmlns:qtafi="urn:qtafi">
    
	<xsl:output method="xml" version="1.0" indent="yes" omit-xml-declaration="yes" encoding="UTF-8" doctype-public="-//W3C//DTD XHTML 1.0 Strict//EN http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"/>

	<xsl:decimal-format decimal-separator="," grouping-separator="." />
	<xsl:param name="action-path" select="/"/>
	
	<xsl:param name="max-page" />
	<xsl:param name="page"/>
	<!-- Authentication -->
	<xsl:param name="mode"/>
	
	<xsl:param name="async"/>
	<xsl:param name="locale" select="de"/>
	
	<xsl:template match="text()|code" />

<xsl:template match="/">
	<xsl:apply-templates />
</xsl:template>

<xsl:template match="CHOOSE">
	<xsl:message>locale <xsl:value-of select="$locale"/></xsl:message> 
	<xsl:apply-templates select="*[@lang=$locale]" />
</xsl:template>

<xsl:template match="Page">
	<xsl:attribute name="id">
		<xsl:value-of select="@id" />
	</xsl:attribute>
	<form method="post" id="rootform" action="{$action-path}" name="qtafi2" accept-charset="UTF-8" onSubmit="return onFormSubmit();">
		<qtafi:HIDDEN variable="__page"/>
		<qtafi:HIDDEN variable="__requestpage" />
		<qtafi:HIDDEN variable="__maxpage" />
		<qtafi:HIDDEN variable="__timestamp" />
		<xsl:if test="/qtafi/codebook/Head">
			<xsl:call-template name="Header"/>
		</xsl:if>
		<xsl:for-each select="children/*">
			<div class="container">
				<xsl:attribute name="id">
					<xsl:value-of select="@id" />
				</xsl:attribute>
				<xsl:apply-templates select="."/>
			</div>
		</xsl:for-each>
		<xsl:if test="/qtafi/codebook/Foot">
			<xsl:call-template name="Footer" />		
		</xsl:if>
	</form>
	<xsl:if test="code">
		<xsl:copy-of select="code/node()" />
	</xsl:if>
</xsl:template>

<xsl:template match="MODULE">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<xsl:call-template name="dynamic-filter">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
	<xsl:for-each select="children">
		<xsl:for-each select="*">
			<div>
				<xsl:attribute name="id">
					<xsl:value-of select="@id" />
				</xsl:attribute>
				<xsl:apply-templates select="."/>
			</div>
		</xsl:for-each>
	</xsl:for-each>
	<xsl:call-template name="dynamic-filter-init">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
</xsl:template>

<xsl:template name="Header">
	<xsl:for-each select="/qtafi/codebook/Head/children/*">	
		<div class="container page_header1">
			<xsl:attribute name="id">
				<xsl:value-of select="@id" />
			</xsl:attribute>
			<xsl:apply-templates select="."/>
		</div>
	</xsl:for-each>
</xsl:template>

<xsl:template name="Footer" >
	<xsl:for-each select="/qtafi/codebook/Foot/children/*">
		<xsl:if test="not(invisible)">
			<div class="container page_footer1">
				<xsl:attribute name="id">
					<xsl:value-of select="@id" />
				</xsl:attribute>
				<xsl:apply-templates select="."/>
			</div>
		</xsl:if>
	</xsl:for-each>
</xsl:template>

<xsl:template match="Foot|Head" />

<xsl:template match="NavigationButton">
	<xsl:variable name="defbackwardlabel">
		<xsl:choose>
			<xsl:when test="$locale = 'en'">
				 <xsl:value-of select="'Previous page'" />
			</xsl:when>
			<xsl:when test="$locale = 'de'">
				<xsl:value-of select="'Zurück'" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="'Previous page'" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>
	
	<xsl:variable name="defforwardlabel">
		<xsl:choose>
			<xsl:when test="$locale = 'en'">
				<xsl:value-of select="'Next page'" />
			</xsl:when>
			<xsl:when test="$locale = 'de'">
				<xsl:value-of select="'Weiter'" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="'Next page'" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>
	
	<xsl:variable name="depth">
		<xsl:if test="not(@level)">
			<xsl:call-template name="depth" />
		</xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<xsl:if test="not(//LOGIN) and not(//qtafi:PIN)">
		<xsl:variable name="backwardlabel">
			<xsl:if test="not(layout/@navigationButtonPrevious)"><xsl:value-of select="$defbackwardlabel"/></xsl:if>
			<xsl:if test="layout/@navigationButtonPrevious"><xsl:value-of select="layout/@navigationButtonPrevious"/></xsl:if>
		</xsl:variable>
		<xsl:variable name="forwardlabel">
			<xsl:if test="not(layout/@navigationButtonNext)"><xsl:value-of select="$defforwardlabel"/></xsl:if>
			<xsl:if test="layout/@navigationButtonNext"><xsl:value-of select="layout/@navigationButtonNext"/></xsl:if>
		</xsl:variable>
		<div class="row navigationbutton">
			<div class="col-sm-2">
				<xsl:if test="$backwardlabel != 'null'">
					<qtafi:USE_IF variable="__page" value="2" runat="server" operator="IS_NOT">
						<input type="button" class="btn btn-primary NavigationButton_button" onClick="proceed(-1);" value="{$backwardlabel}" />
					</qtafi:USE_IF>
				</xsl:if>&#160;
			</div>
			<div class="col-sm-8">&#160;
				<xsl:if test="children">
					<xsl:call-template name="child-elements" />					
				</xsl:if>
			</div>
			<div class="col-sm-2 pull-right" style="text-align:right;">&#160;
				<xsl:if test="$forwardlabel != 'null'">
					<qtafi:USE_IF variable="__page" runat="server" value="eval(__maxpage)" operator="IS_NOT">
						<input type="button" class="btn btn-primary NavigationButton_button" onClick="proceed(1);" value="{$forwardlabel}" />
					</qtafi:USE_IF>
				</xsl:if>
			</div>
		</div>
	</xsl:if>
</xsl:template>

<xsl:template match="ProgressBar">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<xsl:if test="not(//PIN) and not(//qtafi:PIN)">
		<xsl:variable name="percent" select="((($page - 1) div (($max-page - 1)*1.0) ) * 100)" />
		<div class="row">
			<div class="col-md-offset-3 col-sm-6">
				<div class="progress">
					<div class="progress-bar progress-bar-success" role="progressbar" aria-valuenow="{$percent}" aria-valuemin="0" aria-valuemax="100" style="width: {$percent}%">
						<span class="sr-only">40% Complete (success)</span>
					</div>
				</div>
			</div>
		</div>
	</xsl:if>
	<xsl:if test="code">
		<xsl:copy-of select="code/node()" />
	</xsl:if>
</xsl:template>

<xsl:template match="HL">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)">
			<xsl:call-template name="depth" />
		</xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<xsl:call-template name="dynamic-filter">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
	<div class="question{$depth} hl{$depth}" id="TA_{$quid}">
		<xsl:call-template name="question-head" />
	</div>
	<xsl:for-each select="children">
		<xsl:call-template name="child-elements" />
	</xsl:for-each>
	<xsl:if test="code">
		<xsl:copy-of select="code/node()" />
	</xsl:if>
	<xsl:call-template name="dynamic-filter-init">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
</xsl:template>

<xsl:template match="LANG">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)">
			<xsl:call-template name="depth" />
		</xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<div class="question{$depth} lang{$depth}" id="TA_{$quid}" style="page-break-inside: avoid;">
		<xsl:attribute name="uid">
			<xsl:value-of select="@uid"/>
		</xsl:attribute>
		<div><xsl:copy-of select="text/node()"/><qtafi:LANG /></div>
	</div>
	
	<xsl:if test="code">
		<xsl:copy-of select="code/node()" />
	</xsl:if>
</xsl:template>


<xsl:template match="LABEL">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)">
			<xsl:call-template name="depth" />
		</xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<xsl:call-template name="dynamic-filter">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
	<div class="question{$depth} label{$depth}" id="TA_{$quid}">
		<xsl:copy-of select="text/node()" />
	</div>
	<xsl:for-each select="children">
		<xsl:call-template name="child-elements" />
	</xsl:for-each>
	<xsl:if test="code">
		<xsl:copy-of select="code/node()" />
	</xsl:if>
	<xsl:call-template name="dynamic-filter-init">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
</xsl:template>


<xsl:template match="DATE">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<div class="question{$depth} date{$depth} dateinput" id="TA_{$quid}">
		<xsl:attribute name="uid">
			<xsl:value-of select="@uid"/>
		</xsl:attribute>
		<xsl:call-template name="question-head" />
		<xsl:for-each select="variable">
			<div class="row">
				<xsl:copy-of select="node()" />&#160;
				<qtafi:TE>
					<xsl:attribute name="variable" >
						<xsl:value-of select="@name"></xsl:value-of>
					</xsl:attribute>
				</qtafi:TE>
			</div>
		</xsl:for-each>
	</div>
	<script type="text/javascript">
	$('#rootform .dateinput input').datepicker({
		minViewMode: 1,
		format: "yyyy/mm",
		immediateUpdates: true,
		defaultViewDate: { year: 1977, month: 04 }
	});
	</script>
</xsl:template>

<xsl:template match="LOGIN">
	<xsl:message>mode <xsl:value-of select="$mode" /></xsl:message>
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<xsl:call-template name="dynamic-filter">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
	<div class="question{$depth} login{$depth}" id="TA_{$quid}">
		<xsl:if test="text">
			<div class="row">
				<xsl:copy-of select="text/node()" />
				<xsl:value-of select="'&#160;'" />
			</div>
		</xsl:if>
		<div class="row text-center">
			<xsl:choose>
				<xsl:when test="$mode = 'PIN' or $mode='URL'">
					<qtafi:ERROR>
						<xsl:if test="layout/@invalidCredentials">
							<MESSAGE code="1">
								<div class="row">
									<div class="alert alert-danger"><xsl:value-of select="layout/@invalidCredentials" /></div>
								</div>
							</MESSAGE>
						</xsl:if>
						<xsl:if test="not(layout/@invalidCredentials)">
							<MESSAGE code="1">
								<div class="row">
									<div class="alert alert-danger">PIN invalid</div>
								</div>
							</MESSAGE>
						</xsl:if>
					</qtafi:ERROR>
					<qtafi:PIN class="input-form-control" />
					<button class="btn btn-default" onclick="login();"><xsl:value-of select="layout/@loginButtonLabel" /></button>
				</xsl:when>
				<xsl:otherwise>
					<button class="btn btn-default" onClick="login();"><xsl:value-of select="layout/@loginButtonLabel" /></button>
				</xsl:otherwise>
			</xsl:choose>
		</div>
	</div>
</xsl:template>

<xsl:template match="PIN">
	<xsl:variable name="loginabel">
		<xsl:choose>
			<xsl:when test="$locale = 'en'">
				<xsl:value-of select="'Login'" />
			</xsl:when>
			<xsl:when test="$locale = 'de'">
				<xsl:value-of select="'Anmelden'" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="'Login'" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<xsl:call-template name="dynamic-filter">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
	<div class="question{$depth} pin{$depth}" id="TA_{$quid}">
		<xsl:if test="text">
			<div class="row">
				<h2><xsl:copy-of select="text/node()" /></h2>
				<xsl:value-of select="'&#160;'" />
			</div>
		</xsl:if>
		<xsl:if test="moretext">
			<div class="row">
				<span id="moretext_{@id}"><xsl:copy-of select="moretext/node()" /></span>
				<xsl:value-of select="'&#160;'" />
			</div>
		</xsl:if>
		<xsl:if test="$mode != '3'">
			<div class="row">
				<qtafi:ERROR>
					<xsl:if test="errormessage">
						<MESSAGE code="1"><div class="alert alert-danger"><xsl:value-of select="errormessage" /></div></MESSAGE>
					</xsl:if>
					<xsl:if test="not(errormessage)">
						<MESSAGE code="1"><div class="alert alert-danger">PIN invalid</div></MESSAGE>
					</xsl:if>
				</qtafi:ERROR>
				<qtafi:PIN class="input-form-control" />
				<input class="btn btn-primary PIN_button" id="buttonlabel_{@id}" type="button" onclick="login();">
					<xsl:attribute name="value">
						<xsl:if test="@buttonlabel"><xsl:value-of select="@buttonlabel"/></xsl:if>
						<xsl:if test="not(@buttonlabel)"><xsl:value-of select="$loginabel"/></xsl:if>
					</xsl:attribute>
				</input>
			</div>
		</xsl:if>
	</div>
	<xsl:for-each select="children">
		<xsl:call-template name="child-elements" />
	</xsl:for-each>
	<xsl:if test="code">
		<xsl:copy-of select="code/node()" />
	</xsl:if>
	<xsl:call-template name="dynamic-filter-init">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
</xsl:template>


<xsl:template match="CV">
	<xsl:choose>
		<xsl:when test="layout/@inputStyle = 'pulldown'">
			<xsl:call-template name="CV-pulldown" />
		</xsl:when>
		<xsl:otherwise>
			<xsl:call-template name="CV-radio" />
		</xsl:otherwise>
	</xsl:choose>
	<xsl:if test="code">
		<xsl:copy-of select="code/node()" />
	</xsl:if>
</xsl:template>


<xsl:template name="CV-pulldown">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<xsl:call-template name="dynamic-filter">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
	<div class="question{$depth} cv{$depth}" id="TA_{$quid}">
		<xsl:call-template name="question-head" />
		<xsl:variable name="variable" select="variable/@name" />
		<div class="row">
			<div>
				<qtafi:CV variable="{$variable}" type="pulldown">
					<xsl:attribute name="size">
						<xsl:if test="../layout/@pulldownSize"><xsl:value-of select="../layout/@pulldownSize" /></xsl:if>
						<xsl:if test="not(size)"><xsl:value-of select="'1'" /></xsl:if>
					</xsl:attribute>
					<xsl:for-each select="value[@missing = 'true']">
						<option>
							<xsl:attribute name="value">
								<xsl:value-of select="@key" />
							</xsl:attribute>
							<xsl:attribute name="label">
								<xsl:value-of select="." />
							</xsl:attribute>
						</option>
					</xsl:for-each>
					<xsl:if test="not(children)">
						<xsl:for-each select="value[not(@missing)]">
							<option>
								<xsl:attribute name="value">
									<xsl:value-of select="@key" />
								</xsl:attribute>
								<xsl:attribute name="label">
									<xsl:value-of select="." />
								</xsl:attribute>
							</option>
						</xsl:for-each>
					</xsl:if>
					<xsl:if test="children">
						<xsl:for-each select="value[not(@missing)]|children">
							<xsl:if test="name() ='children'">
								<optgroup>
									<xsl:attribute name="label">
										<xsl:value-of select="LABEL/text" />
									</xsl:attribute>
									<xsl:variable name="startn" select="position() +1" />
									<xsl:variable name="endtemp">
										<xsl:call-template name="next-CV-label">
											<xsl:with-param name="currentnum" select="$startn" />
										</xsl:call-template>
									</xsl:variable>
									<xsl:variable name="endn" select="substring-before($endtemp, '|')" />
									<!--
									<xsl:message>Start: <xsl:value-of select="$startn" /></xsl:message>
									<xsl:message>End: <xsl:value-of select="$endn" /></xsl:message> 
									-->
									<xsl:for-each select="../value[not(@missing)]|../children">
										<xsl:if test="position() &gt;= $startn">
											<xsl:choose>
											<xsl:when test="string-length($endn) &lt; 1">
												<option>
													<xsl:attribute name="value">
														<xsl:value-of select="@key" />
													</xsl:attribute>
													<xsl:attribute name="label">
														<xsl:value-of select="." />
													</xsl:attribute>
												</option>
											</xsl:when>
											<xsl:otherwise>
												<xsl:if test="position() &lt; $endn">
													<option>
														<xsl:attribute name="value">
															<xsl:value-of select="@key" />
														</xsl:attribute>
														<xsl:attribute name="label">
															<xsl:value-of select="." />
														</xsl:attribute>
													</option>
												</xsl:if>
											</xsl:otherwise>
											</xsl:choose>
										</xsl:if>
									</xsl:for-each>
								</optgroup>
							</xsl:if>
						</xsl:for-each>
					</xsl:if>
				</qtafi:CV>
				&#160; &#160;<!-- a space -->
				<label>
					<xsl:attribute name="for">
						<xsl:value-of select="$variable" />
					</xsl:attribute>
					<xsl:copy-of select="variable/node()" />
				</label>
			</div>
		</div>
	</div>
	<xsl:call-template name="dynamic-filter-init">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
</xsl:template>

<xsl:template name="next-CV-label">
	<xsl:param name="currentnum" />
	<xsl:for-each select="../value[not(@missing)]|../children">
		<xsl:if test="position() &gt; $currentnum">
			<xsl:if test="name() = 'children'">
				<xsl:value-of select="position()" /><xsl:text>|</xsl:text>
			</xsl:if>
		</xsl:if>
	</xsl:for-each>
</xsl:template>

<xsl:template name="CV-radio">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<xsl:call-template name="dynamic-filter">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
	<div class="question{$depth} cv{$depth}" id="TA_{$quid}">
		<xsl:call-template name="question-head" />
		<xsl:variable name="variable" select="variable/@name" />
		<xsl:for-each select="value[not(@missing) or @missing='false']|children">
			<xsl:variable name="odd_even" select="(position() mod 2) + 1"/>
			<xsl:if test="name() = 'children'">
				<div class="row">
					<div class="col-sm-12">
						<xsl:call-template name="child-elements" />
					</div>
				</div>
			</xsl:if>	
			<xsl:if test="(not(@missing) or @missing='false') and name()='value'">
				<div class="row bodyrow">
					<div class="col-sm-1">
						<qtafi:CV>
							<xsl:attribute name="variable">
								<xsl:value-of select="$variable" />
							</xsl:attribute>
							<xsl:attribute name="value">
								<xsl:value-of select="@key" />
							</xsl:attribute>
						</qtafi:CV>
						<xsl:value-of select="'&#160;'" />
					</div>
					<div class="col-sm-11">
						<xsl:attribute name="id">
							<xsl:text>value_</xsl:text><xsl:value-of select="$variable" /><xsl:text>_</xsl:text><xsl:value-of select="@key" />
						</xsl:attribute>
						<xsl:call-template name="label-for">
							<xsl:with-param name="for">
								<xsl:value-of select="$variable" />
								<xsl:value-of select="'-'" />
								<xsl:value-of select="@key" />
							</xsl:with-param>
						</xsl:call-template>
					</div>
				</div>
			</xsl:if>
		</xsl:for-each>
	</div>
	<xsl:call-template name="dynamic-filter-init">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
</xsl:template>

<!-- 
 if contains <qtafi:xx dont use label for
-->
<xsl:template name="label-for">
	<xsl:param name="for" />
	<xsl:choose>
		<xsl:when test="qtafi:TE or qtafi:ME or qtafi:TL or qtafi:CV or qtafi:MD or qtafi:OR">
			<!-- 
			<label onClick="$('#{$for}').trigger('click');">
				<xsl:copy-of select="node()" />
			</label>
			 -->
			 <label>
				<xsl:copy-of select="node()" />
			</label>
		</xsl:when>
		<xsl:otherwise>
			<label>
				<xsl:attribute name="for">
					<xsl:value-of select="$for" />
				</xsl:attribute>
				<xsl:copy-of select="node()" />
			</label>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template match="MD" name="MD_element">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<xsl:call-template name="dynamic-filter">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
	<div class="question{$depth} md{$depth}" id="TA_{$quid}">
		<xsl:call-template name="question-head" />
		<xsl:for-each select="variable|children">
			<xsl:variable name="odd" select="(position() mod 2) + 1"/>
			<xsl:if test="name() = 'children' and count(./*) &gt; 0">
				<div class="row">
					<div class="col-sm-12">
						<xsl:call-template name="child-elements" />
					</div>
				</div>
			</xsl:if>
			<xsl:if test="not(@subitem) and name() = 'variable'">
				<div class="row bodyrow">
					<div class="col-sm-1">
						<qtafi:MD>
							<xsl:attribute name="variable">
								<xsl:value-of select="@name" />
							</xsl:attribute>
							<xsl:attribute name="value">
								<xsl:value-of select="1" />
							</xsl:attribute>
						</qtafi:MD>
					</div>
					<div class="col-sm-11" >
						<xsl:attribute name="id">
							<xsl:text>variable_</xsl:text><xsl:value-of select="@name" />
						</xsl:attribute>
						<xsl:call-template name="label-for">
							<xsl:with-param name="for">
								<xsl:value-of select="@name" />
								<xsl:value-of select="'-1'" />
							</xsl:with-param>
						</xsl:call-template>
					</div>
				</div>
			</xsl:if>
		</xsl:for-each>
	</div>
	<xsl:if test="code">
		<xsl:copy-of select="code/node()" />
	</xsl:if>
	<xsl:call-template name="dynamic-filter-init">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
</xsl:template>

<xsl:template match="OR">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<xsl:call-template name="dynamic-filter">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
	<div class="question{$depth} or{$depth}" id="TA_{$quid}">
		<xsl:call-template name="question-head" />
		<xsl:variable name="value-count"><xsl:value-of select="count(value[not(@missing) or @missing='false'])" /></xsl:variable>
		<!-- value labels -->
		<div class="row">
			<xsl:for-each select="value[not(@missing) or @missing='false']">
				<div class="col-sm-1 OR_head{$depth}" index="{position()}" style="text-align:center;">
					<xsl:attribute name="id">
						<xsl:text>value_</xsl:text><xsl:value-of select="../variable[1]/@name"/><xsl:text>_</xsl:text><xsl:value-of select="@key" />
					</xsl:attribute>
					<xsl:if test="../width">
						<xsl:attribute name="width">
							<xsl:value-of select="../width" />
						</xsl:attribute>
					</xsl:if>
					<xsl:copy-of select="node()" />
					<xsl:value-of select="'&#160;'" />
				</div>
			</xsl:for-each>
			<xsl:value-of select="'&#160;'" />
		</div>
		<!-- keys -->
		<div class="row">
			<xsl:for-each select="value[not(@missing) or @missing='false']">
				<div class="col-sm-1 center-block OR_value{$depth}" index="{position()}" style="text-align:center;">
					<xsl:if test="not(@hidekey) or @hidekey='false'">
						<xsl:value-of select="@key" />
					</xsl:if>
					<xsl:value-of select="'&#160;'" />
				</div>
			</xsl:for-each>
			<xsl:value-of select="'&#160;'" />
		</div>
		<xsl:for-each select="variable|children">
			<xsl:variable name="label" select="@label" />
			<xsl:variable name="variable" select="@name" />
			<xsl:if test="name() = 'children' and count(./*) &gt; 0">
				<div class="row">
					<xsl:call-template name="child-elements" />
				</div>
			</xsl:if>
			<xsl:if test="name() = 'variable'">
				<div class="row bodyrow">
					<xsl:for-each select="../value[not(@missing) or @missing='false']">
						<div class="col-sm-1" index="{position()}" style="text-align:center;">
							<qtafi:OR>
								<xsl:attribute name="variable">
									<xsl:value-of select="$variable" />
								</xsl:attribute>
								<xsl:attribute name="value">
									<xsl:value-of select="@key" />
								</xsl:attribute>
							</qtafi:OR>
							<xsl:value-of select="'&#160;'" />
						</div>
					</xsl:for-each>
					<div>
						<xsl:attribute name="class">
							<xsl:value-of select="'col-sm-'" />
							<xsl:value-of select="(12 - $value-count)" />
							<xsl:value-of select="' '" />
						</xsl:attribute>
						<xsl:attribute name="id">
							<xsl:text>variable_</xsl:text><xsl:value-of select="$variable" />
						</xsl:attribute>
						<label>
							<xsl:copy-of select="node()" />
							<xsl:value-of select="'&#160;'" />
						</label>
					</div>
				</div>
			</xsl:if>
		</xsl:for-each>
	</div>
	<xsl:if test="code">
		<xsl:copy-of select="code/node()" />
	</xsl:if>
	<xsl:call-template name="dynamic-filter-init">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
</xsl:template>

<xsl:template match="ME">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<xsl:call-template name="dynamic-filter">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
	<div class="question{$depth} me{$depth}" id="TA_{$quid}">
		<xsl:call-template name="question-head" />
		<xsl:for-each select="variable|children">
			<xsl:variable name="odd_even" select="(position() mod 2) + 1"/>
			<xsl:if test="name() = 'children'">
				<div class="row">
					<xsl:call-template name="child-elements" />
				</div>
			</xsl:if>
			<xsl:if test="name() = 'variable'">
				<div class="row bodyrow">
					<qtafi:ME type="text" >
						<xsl:attribute name="variable">
							<xsl:value-of select="@name" />
						</xsl:attribute>
						<xsl:attribute name="size">
							<xsl:value-of select="../layout/@inputSize" />
						</xsl:attribute>
					</qtafi:ME>&#160;
					<xsl:call-template name="label-for">
						<xsl:with-param name="for">
							<xsl:value-of select="@name" />
						</xsl:with-param>
					</xsl:call-template>
				</div>
			</xsl:if>
		</xsl:for-each>
	</div>
	<xsl:if test="code">
		<xsl:copy-of select="code/node()" />
	</xsl:if>
	<xsl:call-template name="dynamic-filter-init">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
</xsl:template>

<xsl:template match="TE" name="TE_element">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<xsl:call-template name="dynamic-filter">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
	
	<div class="question{$depth} te{$depth}" id="TA_{$quid}">
		<xsl:call-template name="question-head" />
		<xsl:choose>
			<xsl:when test="layout/@inputLines = '1'">
				<xsl:for-each select="variable">
					<div class="row bodyrow">
						<qtafi:TE type="text">
							<xsl:attribute name="variable">
								<xsl:value-of select="@name" />
							</xsl:attribute>
							<xsl:attribute name="size">
								<xsl:value-of select="../layout/@inputSize" />
							</xsl:attribute>
						</qtafi:TE>&#160;
						<xsl:call-template name="label-for">
							<xsl:with-param name="for">
								<xsl:value-of select="@name" />
							</xsl:with-param>
						</xsl:call-template>
					</div>
				</xsl:for-each>
			</xsl:when>
			<xsl:otherwise>
				<xsl:for-each select="variable">
					<div class="row bodyrow">
						<qtafi:TL type="text">
							<xsl:attribute name="variable">
								<xsl:value-of select="@name" />
							</xsl:attribute>
							<xsl:attribute name="width">
								<xsl:value-of select="../layout/@inputSize" />
							</xsl:attribute>
							<xsl:attribute name="height">
								<xsl:value-of select="../layout/@inputLines" />
							</xsl:attribute>
						</qtafi:TL>&#160;
						<xsl:call-template name="label-for">
							<xsl:with-param name="for">
								<xsl:value-of select="@name" />
							</xsl:with-param>
						</xsl:call-template>
					</div>
				</xsl:for-each>
			</xsl:otherwise>
		</xsl:choose>
	</div>
	<xsl:if test="code">
		<xsl:copy-of select="code/node()" />
	</xsl:if>
	<xsl:call-template name="dynamic-filter-init">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
</xsl:template>

<xsl:template match="RE|CONTAINER">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<xsl:call-template name="dynamic-filter">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
	<div class="question{$depth} container{$depth}" id="TA_{$quid}">
		<xsl:call-template name="question-head" />
		<xsl:for-each select="children">
			<div class="row">
				<xsl:call-template name="child-elements" />
			</div>
		</xsl:for-each>
	</div>
	<xsl:if test="code">
		<xsl:copy-of select="code/node()" />
	</xsl:if>
	<xsl:call-template name="dynamic-filter-init">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
</xsl:template>

<xsl:template name="child-elements">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="(@level + 1)"/></xsl:if>
	</xsl:variable>
	<div>
		<xsl:if test="@layout = 'horizontal'">
			<table width="100%">
				<tr>
					<xsl:for-each select="*">
						<xsl:if test="not(invisible)">
							<td class="item{$depth}">
								<div>
									<xsl:attribute name="id">
										<xsl:value-of select="@id" />
									</xsl:attribute>
									<xsl:apply-templates select="."/>
								</div>
							</td>
						</xsl:if>
					</xsl:for-each>
				</tr>
			</table>
		</xsl:if>
		<xsl:if test="not(layout = 'horizontal')">
			<div class="row item{$depth}">
				<xsl:for-each select="*">
					<xsl:if test="not(invisible)">
						<div>&#160;
							<xsl:attribute name="id">
								<xsl:value-of select="@id" />
							</xsl:attribute>
							<xsl:apply-templates select="."/>
						</div>
					</xsl:if>
				</xsl:for-each>
			</div>
		</xsl:if>
	</div>
</xsl:template>

<xsl:template match="XHTML">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<xsl:call-template name="dynamic-filter">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
	<div class="question{$depth} xhtml{$depth}" id="TA_{$quid}">
		<xsl:call-template name="question-head" />
		<xsl:if test="code">
			<div class="XHTML_code{$depth}">
				<xsl:copy-of select="code/node()" />
			</div>
		</xsl:if>
		<xsl:for-each select="children">
			<div class="row">
				<xsl:call-template name="child-elements" />
			</div>
		</xsl:for-each>
	</div>
	<xsl:call-template name="dynamic-filter-init">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
</xsl:template>

<xsl:template match="RAW">
	<xsl:if test="code">
		<xsl:copy-of select="code/node()" />
	</xsl:if>
</xsl:template>

<xsl:template match="IMAGE">
	<xsl:if test="code">
		<div class="row">
			<xsl:copy-of select="code/node()" />
		</div>
	</xsl:if>
</xsl:template>

<xsl:template match="TL">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<xsl:call-template name="dynamic-filter">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
	<div class="question{$depth} tl{$depth}" id="TA_{$quid}">
		<xsl:call-template name="question-head" />
		<xsl:for-each select="variable|children">
			<xsl:if test="name() = 'children'">
				<div class="row">
					<xsl:call-template name="child-elements" />
				</div>
			</xsl:if>
			<xsl:if test="name() = 'variable'">
				<div class="row bodyrow">
					<div>
						<qtafi:TL>
							<xsl:attribute name="variable">
								<xsl:value-of select="@name" />
							</xsl:attribute>
							<xsl:if test="../width">
								<xsl:attribute name="width">
									<xsl:value-of select="../width" />
								</xsl:attribute>
							</xsl:if>
							<xsl:if test="../height">
								<xsl:attribute name="height">
									<xsl:value-of select="../height" />
								</xsl:attribute>
							</xsl:if>
						</qtafi:TL>
						&#160;&#160;
						<xsl:call-template name="label-for">
							<xsl:with-param name="for">
								<xsl:value-of select="@name" />
							</xsl:with-param>
						</xsl:call-template>
					</div>
				</div>
			</xsl:if>
		</xsl:for-each>
	</div>
	<xsl:if test="code">
		<xsl:copy-of select="code/node()" />
	</xsl:if>
	<xsl:call-template name="dynamic-filter-init">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
</xsl:template>

<xsl:template match="COLUMNCONTAINER">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="quid" select="generate-id()"/>
	<xsl:call-template name="dynamic-filter">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
	<xsl:variable name="columncount" select="count(children/CV|children/OR/value[not(@missing) or @missing !='true']|children/MD|children/TE|children/ME|children/LABEL)" />
	<xsl:variable name="rowcount"><xsl:call-template name="row-count" /></xsl:variable>
	<!--<xsl:message>Rowcount=<xsl:value-of select="$rowcount" /></xsl:message>
	<xsl:message>Colcount=<xsl:value-of select="$columncount" /></xsl:message> -->
	<div class="question{$depth} cc{$depth}" id="TA_{$quid}">
		<xsl:call-template name="question-head" />
		<div class="row">
			<xsl:for-each select="children/*">
				<div index="{position()}">
					<xsl:choose>
						<xsl:when test="name() = 'OR'">
							<xsl:attribute name="class">
								<xsl:value-of select="'col-sm-'" />
								<xsl:value-of select="count(value[@missing != 'true' or not(@missing)])" />
								<xsl:value-of select="' '" />
								<xsl:value-of select="'CC_columnlabel'+ $depth" />
							</xsl:attribute>
						</xsl:when>
						<xsl:when test="name() = 'LABEL' and position()!=last()">
							<xsl:attribute name="class">
								<xsl:value-of select="'col-sm-2 '" />
								<xsl:value-of select="'CC_columnlabel'" />
								<xsl:value-of select="$depth" />
							</xsl:attribute>
						</xsl:when>
						<xsl:when test="name() = 'LABEL' and position()=last()">
							<xsl:attribute name="class">
								<xsl:value-of select="'CC_columnlabel'" />
								<xsl:value-of select="$depth" />
							</xsl:attribute>
						</xsl:when>
						<xsl:when test="name() = 'TE'">
							<xsl:attribute name="class">
								<xsl:value-of select="'col-sm-3 '" />
								<xsl:value-of select="'CC_columnlabel'" />
								<xsl:value-of select="$depth" />
							</xsl:attribute>
						</xsl:when>
						<xsl:when test="name() = 'ME'">
							<xsl:attribute name="class">
								<xsl:value-of select="'col-sm-2 '" />
								<xsl:value-of select="'CC_columnlabel'" />
								<xsl:value-of select="$depth" />
							</xsl:attribute>
						</xsl:when>
						<xsl:otherwise>
							<xsl:attribute name="class">
								<xsl:value-of select="'col-sm-1 '" />
								<xsl:value-of select="'CC_columnlabel'" />
								<xsl:value-of select="$depth" />
							</xsl:attribute>
						</xsl:otherwise>
					</xsl:choose>
					<xsl:copy-of select="text/node()" />&#160;
				</div>
			</xsl:for-each>
		</div>
		<xsl:call-template name="ORHEAD" />
		<xsl:call-template name="COLUMN-ROW">
			<xsl:with-param name="rownum" select="1" />
			<xsl:with-param name="rowcount" select="$rowcount" />
			<xsl:with-param name="columncount" select="$columncount" />
		</xsl:call-template>
	</div>
	<xsl:if test="code">
		<xsl:copy-of select="code/node()" />
	</xsl:if>
	<xsl:call-template name="dynamic-filter-init">
		<xsl:with-param name="quid" select="$quid" />
	</xsl:call-template>
</xsl:template>

<xsl:template name="row-count">
	<xsl:if test="name(children/*[1]) = 'CV'">
		<xsl:value-of select="count(children/CV[1]/value[@missing != 'true' or not(@missing)])" />
	</xsl:if>
	<xsl:if test="name(children/*[1]) = 'LABEL'">
		<xsl:value-of select="count(children/LABEL[1]/label)" />
	</xsl:if>
	<xsl:if test="name(children/*[1]) != 'CV' and name(children/*[1]) != 'LABEL'">
		<xsl:value-of select="count(children/*[1]/variable)" />
	</xsl:if>
</xsl:template>

<xsl:template name="COLUMN-ROW">
	<xsl:param name="rownum" />
	<xsl:param name="rowcount" />
	<xsl:param name="columncount" />
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:variable name="labelid" select="generate-id(children/LABEL/label[$rownum])" />
	<xsl:variable name="odd_even" select="($rownum mod 2) + 1"/>
	
	<xsl:for-each select="children/LABEL/*[name()='label' or name()='children']">
		<xsl:variable name="dd" select="generate-id()"/>
		<xsl:if test="$dd = $labelid and name(preceding-sibling::*[1]) = 'children'">
			<div class="row">
				<div class="col-sm-{$columncount}">
					<xsl:copy-of select="preceding-sibling::*[1]/LABEL/text/node()" />
				</div>
			</div>
		</xsl:if>
	</xsl:for-each>
	<div class="row bodyrow">
		<xsl:for-each select="children/*">
			<xsl:choose>
				<xsl:when test="name() = 'LABEL' and position()=last()">
					<div>
						<label>
							<xsl:copy-of select="label[$rownum]/node()" />
						</label>
					</div>
				</xsl:when>
				<xsl:when test="name() = 'LABEL' and position()!=last()">
					<div class="col-sm-2">
						<label>
							<xsl:copy-of select="label[$rownum]/node()" />
						</label>
					</div>
				</xsl:when>
				<xsl:when test="name() = 'CV'">
					<div class="col-sm-1">
						<xsl:if test="value[$rownum]">
							<qtafi:CV>
								<xsl:attribute name="variable"><xsl:value-of select="variable/@name" /></xsl:attribute>
								<xsl:attribute name="value"><xsl:value-of select="value[$rownum]/@key" /></xsl:attribute>
							</qtafi:CV>
						</xsl:if>
					</div>
				</xsl:when>
				<xsl:when test="name() = 'MD'">
					<div class="col-sm-1">
						<xsl:if test="variable[$rownum]">
							<qtafi:MD>
								<xsl:attribute name="variable"><xsl:value-of select="variable[$rownum]/@name" /></xsl:attribute>
								<xsl:attribute name="value"><xsl:value-of select="value[1]/@key" /></xsl:attribute>
							</qtafi:MD>
						</xsl:if>
					</div>
				</xsl:when>
				<xsl:when test="name() = 'OR'">
					<xsl:variable name="var" select="variable[$rownum]/@name" />
					<xsl:for-each select="value[(not(@missing) or @missing='false')]">
						<div class="col-sm-1" style="text-align:center;">
							<xsl:if test="../variable[$rownum]">
								<qtafi:OR>
									<xsl:attribute name="variable"><xsl:value-of select="$var" /></xsl:attribute>
									<xsl:attribute name="value"><xsl:value-of select="@key" /></xsl:attribute>
								</qtafi:OR>
							</xsl:if>
						</div>
					</xsl:for-each>
				</xsl:when>
				<xsl:when test="name() = 'TE'">
					<div class="col-sm-3">
						<xsl:if test="variable[$rownum]">
							<qtafi:TE>
								<xsl:attribute name="variable"><xsl:value-of select="variable[$rownum]/@name" /></xsl:attribute>
								<xsl:attribute name="size"><xsl:value-of select="size" /></xsl:attribute>
							</qtafi:TE>
						</xsl:if>
					</div>
				</xsl:when>
				<xsl:when test="name() = 'ME'">
					<div class="col-sm-2">
						<xsl:if test="variable[$rownum]">
							<qtafi:ME>
								<xsl:attribute name="variable"><xsl:value-of select="variable[$rownum]/@name" /></xsl:attribute>
								<xsl:attribute name="size"><xsl:value-of select="size" /></xsl:attribute>
							</qtafi:ME>
						</xsl:if>
					</div>
				</xsl:when>
			</xsl:choose>
		</xsl:for-each>
	</div>
	<xsl:if test="$rownum = $rowcount">
		<xsl:for-each select="children/LABEL/*[name()='label' or name()='children']">
			<xsl:variable name="dd" select="generate-id()"/>
			<xsl:if test="$dd = $labelid and name(following-sibling::*[1]) = 'children'">
				<div class="row">
					<div class="col-sm-2">
						<xsl:copy-of select="following-sibling::*[1]/LABEL/text/node()" />
					</div>
				</div>
			</xsl:if>
		</xsl:for-each>
	</xsl:if>
	<xsl:if test="$rownum &lt; $rowcount">
		<xsl:call-template name="COLUMN-ROW">
			<xsl:with-param name="rownum" select="$rownum + 1" />
			<xsl:with-param name="rowcount" select="$rowcount" />
			<xsl:with-param name="columncount" select="$columncount" />
		</xsl:call-template>
	</xsl:if>
</xsl:template>

<xsl:template name="ORHEAD">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:if test="children/OR">
		<div class="row">
			<xsl:for-each select="children/*">
				<xsl:choose>
					<xsl:when test="name() = 'OR'">
						<xsl:for-each select="value[@missing != 'true' or not(@missing)]">
							<div style="text-align:center;" class="col-sm-1 OR_head{($depth + 1)}"><xsl:copy-of select="node()" />&#160;</div>	
						</xsl:for-each>
					</xsl:when>
					<xsl:when test="name() = 'LABEL' and position()=last()">
						<div>&#160;</div>
					</xsl:when>
					<xsl:when test="name() = 'LABEL' and position()!=last()">
						<div class="col-sm-2">&#160;</div>
					</xsl:when>
					<xsl:when test="name() = 'TE' and position()!=last()">
						<div class="col-sm-3">&#160;</div>
					</xsl:when>
					<xsl:when test="name() = 'ME' and position()!=last()">
						<div class="col-sm-2">&#160;</div>
					</xsl:when>
					<xsl:otherwise>
						<div class="col-sm-1">&#160;</div>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:for-each>
		</div>
		<div class="row">
			<xsl:for-each select="children/*">
				<xsl:choose>
					<xsl:when test="name() = 'OR'">
						<xsl:for-each select="value[@missing != 'true' or not(@missing)]">
							<div style="text-align:center;" class="col-sm-1 OR_value{($depth + 1)}">
								<xsl:if test="not(@hidekey) or @hidekey='false'">
									<xsl:value-of select="@key" />
								</xsl:if>
								&#160;
							</div>	
						</xsl:for-each>
					</xsl:when>
					<xsl:when test="name() = 'LABEL' and position()=last()">
						<div>&#160;</div>
					</xsl:when>
					<xsl:when test="name() = 'LABEL' and position()!=last()">
						<div class="col-sm-2">&#160;</div>
					</xsl:when>
					<xsl:when test="name() = 'TE' and position()!=last()">
						<div class="col-sm-3">&#160;</div>
					</xsl:when>
					<xsl:when test="name() = 'ME' and position()!=last()">
						<div class="col-sm-2">&#160;</div>
					</xsl:when>
					<xsl:otherwise>
						<div class="col-sm-1">&#160;</div>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:for-each>
		</div>
	</xsl:if>
</xsl:template>

<xsl:template name="question-head">
	<xsl:variable name="depth">
		<xsl:if test="not(@level)"><xsl:call-template name="depth" /></xsl:if>
		<xsl:if test="@level"><xsl:value-of select="@level"/></xsl:if>
	</xsl:variable>
	<xsl:if test="text/* or string-length(normalize-space(text)) &gt; 0">
		<div class="row question_head1">
			<xsl:choose>
				<xsl:when test="number/* or string-length(normalize-space(number)) &gt; 0">
					<div class="col-sm-1 question_nr{$depth}" id="nr_{@id}">
						<h2><xsl:copy-of select="number/node()"/></h2>
					</div>
					<div class="col-sm-11 question_text{$depth}" id="text_{@id}">
						<h1><xsl:copy-of select="text/node()"/></h1>
					</div>
				</xsl:when>
				<xsl:otherwise>
					<div class="question_text{$depth}" id="text_{@id}">
						<h1><xsl:copy-of select="text/node()"/></h1>
					</div>
				</xsl:otherwise>
			</xsl:choose>
		</div>
	</xsl:if>
	<xsl:if test="moretext/* or string-length(normalize-space(moretext)) &gt; 0">
		<div class="row question_subhead1">
			<div class="question_more{$depth}" id="moretext_{@id}">
				<h3><xsl:copy-of select="moretext/node()"/></h3>
			</div>
		</div>
	</xsl:if>
</xsl:template>

<xsl:template match="Setup">
</xsl:template>

<xsl:template match="Finish">
</xsl:template>

<xsl:template name="depth">
	<xsl:value-of select="count(ancestor::*[name() != 'MODULE' and name() != 'children']) - 2"/>
</xsl:template>

<xsl:template name="filter-rec">
	<xsl:param name="quid" />
	<xsl:for-each select="show_if[@runat='client'] | hide_if[@runat='client'] | use_if[@runat='client'] | ignore_if[@runat='client'] | show_group | hide_group | use_group | ignore_group">
		{
			quid:'<xsl:value-of select="$quid" />',
			type:'<xsl:value-of select="name()" />',
			variable:'<xsl:value-of select="@variable" />',
			value:'<xsl:value-of select="@value" />',
			operator:'<xsl:value-of select="@operator" />',
			condition:'<xsl:value-of select="@condition" />',
			children:[
			<xsl:call-template name="filter-rec">
				<xsl:with-param name="quid" select="$quid"/>
			</xsl:call-template>]
		}
		<xsl:if test="position() != last()">
			<xsl:value-of select="','" />
		</xsl:if>
	</xsl:for-each>
</xsl:template>

<xsl:template name="dynamic-filter">
	<xsl:param name="quid" />
	<script type="text/javascript">
	<xsl:for-each select="filter">
	filter('<xsl:value-of select="$quid"/>', [
		<xsl:call-template name="filter-rec">
			<xsl:with-param name="quid" select="$quid"/>
		</xsl:call-template>
	]);
	</xsl:for-each>
	<xsl:value-of select="'&#160;'" />
	</script>
	
	<xsl:for-each select="filter/importfiltervariable">
		<qtafi:HIDDEN>
			<xsl:attribute name="variable">
				<xsl:value-of select="@name" />
			</xsl:attribute>
		</qtafi:HIDDEN>
	</xsl:for-each>
</xsl:template>

<xsl:template name="dynamic-filter-init">
	<xsl:param name="quid" />
	<script type="text/javascript">
		initFilter('<xsl:value-of select="$quid" />');
	</script>
</xsl:template>

</xsl:stylesheet>