/*******************************************************************************
 * Questionnaire runtime
 *
 * Copyright (C) 2015 martin.guist@eoda.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.tinu.qtafi4;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FilenameFilter;
import java.io.InputStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
/**
 * 
 * @author maks
 *
 */
public class FolderProjectConfiguration implements ProjectConfiguration {

	private final String name; 
	private AuthenticationType authentication = AuthenticationType.PIN;
	private Runtime runtime = Runtime.SETUP;
	private LogMode logmode = LogMode.OFF;
	private String rootpath;
	
	public FolderProjectConfiguration(String n){
		System.out.println("project name " + n);
		name = n;
	}
	
	public String getProjectName(){
		return name;
	}
	
	public void setRootPath(String root){
		rootpath = root;
	}
	
	public void init(){
		try {
			DocumentBuilderFactory builderfactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = builderfactory.newDocumentBuilder();
			Document doc = builder.parse(new InputSource(getProjectResource(".config")));
			NodeList authlist = doc.getDocumentElement().getElementsByTagName("authentication");
			if(authlist != null){
				Node auth = authlist.item(0);
				if(auth != null){
					switch(auth.getTextContent().toLowerCase()){
						case "no": authentication = AuthenticationType.NO; break;
						case "pin": authentication = AuthenticationType.PIN; break;
						case "open": authentication = AuthenticationType.OPEN; break;
						case "cookie": authentication = AuthenticationType.COOKIE; break;
						case "url": authentication = AuthenticationType.URL; break;
						case "urlbase64": authentication = AuthenticationType.URLBASE64; break;
					}
				}
			}
			
			DateFormat dateandhourreading = new SimpleDateFormat("dd.MM.yyyy kk:mm");
			DateFormat datereading = new SimpleDateFormat("dd.MM.yyyy");
			
			Date now = new Date();
			
			Date startDate = new Date();
			startDate.setTime(now.getTime()-1);
			NodeList startlist = doc.getDocumentElement().getElementsByTagName("start");
			if(startlist != null){
				Node startn = startlist.item(0);
				if(startn != null){
					String startString = startn.getTextContent();
					if(startString.length() > 10) startDate = dateandhourreading.parse(startString);
					else startDate = datereading.parse(startString);
				}
			}

			Date endDate = new Date();
			endDate.setTime(now.getTime()+1);
			NodeList endlist = doc.getDocumentElement().getElementsByTagName("end");
			if(endlist != null){
				Node endn = endlist.item(0);
				if(endn != null){
					String endString = endn.getTextContent();
					if(endString.length() > 10) endDate = dateandhourreading.parse(endString);
					else endDate = datereading.parse(endString);
				}
			}
			
			if(startDate.after(now)) runtime = Runtime.SETUP;
			else if(endDate.before(now)) runtime = Runtime.TERMINATED;
			else runtime = Runtime.ONLINE;
			
			NodeList loglist = doc.getDocumentElement().getElementsByTagName("log");
			if(loglist != null){
				Node logn = loglist.item(0);
				if(logn != null){
					switch(logn.getTextContent().toLowerCase()){
						case "off": logmode = LogMode.OFF; break;
						case "verbose": logmode = LogMode.VERBOSE; break;
					}
				}
			}
		}
		catch(Exception ex){
			ex.printStackTrace();
		}
	}
	
	
	@Override
	public boolean projectResourceExists(String file) {
		return new File(rootpath + "/" + name + "/" + file).exists();
	}
	
	
	@Override
	public List<String> listQuestionnaires() {
		File[] fs =  new File(rootpath + "/" + name + "/").listFiles(new FilenameFilter() {
			@Override
			public boolean accept(File dir, String name) {
				if(name.toLowerCase().endsWith(".qml")) return true;
				return false;
			}
		});
		List<String> files = new ArrayList<String>();
		for(File f: fs) files.add(f.getName());
		
		return files;
	}
	
	@Override
	public InputStream getProjectResource(String file) {
		try {
			//System.out.println("get resource " + rootpath + " / " + name+ " / " + file);
			return new FileInputStream(new File(rootpath + "/" + name + "/" + file));
		} 
		catch (FileNotFoundException e) {
			System.err.println("resource " + file + " not found!");
		}
		return null;
	}
	
	public InputStream getConfigurationResource(String file){
		try {
			return new FileInputStream(new File(rootpath + "/" + name + "/" + file));
		} 
		catch (FileNotFoundException e) {
			System.err.println("file " + file + " not found!");
		}
		return null;
	}
	
	public AuthenticationType getAuthentication(){
		return authentication;
	}
	
	public Runtime getRuntime(){
		return runtime;
	}
	
	public LogMode getLogMode(){
		return logmode;
	}
	
	@Override
	public String toString() {
		return name;
	}
}
