/*******************************************************************************
 * Questionnaire runtime
 *
 * Copyright (C) 2015 martin.guist@eoda.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.tinu.qtafi4;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;

public class QMLFilter implements Filter {
	
	@Autowired
	ServletContext context;
	
	private QMLProcessor processor = new QMLProcessor();
	private ProjectConfigurationFactory projectConfigurationFactory;
	private DataSource datasource;
	
	
	public void setProjectConfigurationFactory(ProjectConfigurationFactory projectConfigurationFactory) {
		this.projectConfigurationFactory = projectConfigurationFactory;
	}
	
	public void setDataSource(DataSource ds) {
		this.datasource = ds;
	}
	
	@Override
	public void init(FilterConfig config) throws ServletException {
		processor.setDataSource(datasource);
		processor.init(config);
	}
	
	@Override
	public void doFilter(ServletRequest request, ServletResponse response,
			FilterChain chain) throws IOException, ServletException {
		if (request instanceof HttpServletRequest) {
			HttpSession session = ((HttpServletRequest) request).getSession();
			//System.out.println("#########################" + session.getId());
			try {
				request.setCharacterEncoding("utf-8");
				QMLRequest qmlrequest = new QMLRequest((HttpServletRequest)request, this.projectConfigurationFactory);
				if(qmlrequest.isAsynchron()){
					System.out.println("async");
					processor.doAsynchron(qmlrequest, (HttpServletResponse) response);
				}
				else processor.doHttp(qmlrequest, (HttpServletResponse) response);
			}
			catch (Exception e) {
				e.printStackTrace();
			}
		}
		else {
			 chain.doFilter(request, response);
		}
	}
	
	@Override
	public void destroy() {
	}
	
}
