/*******************************************************************************
 * Questionnaire runtime
 *
 * Copyright (C) 2015 martin.guist@eoda.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.tinu.qtafi4;

import java.io.FileNotFoundException;
import java.util.Enumeration;
import java.util.Locale;
import java.util.UUID;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.codec.binary.Base64;

import de.tinu.qtafi4.persistence.ColumnRegistry;
import de.tinu.qtafi4.qml.QMLDocument;

public class QMLRequest  {
	
	public static String sessionLangAttribute = "javax.servlet.jsp.jstl.fmt.locale.session";
	
	private final ProjectConfigurationFactory configurationFactory;
	private final HttpServletRequest request;
	public final String rootpath;
	//private final String previewpath;
	
	private String sessionId = "";
	private ProjectConfiguration project;
	private String requestfile;
	private ColumnRegistry columnregistry;
	private QMLDocument document;
	private SurveyError error = SurveyError.OK;
	private Locale locale = Locale.GERMAN;

	
	
	/**
	 * Session Tracking
	 * 
	 * 1. Cookie
	 * 
	 * 2. URL
	 * URL Pattern
	 * http-root-path/project/file.qml
	 * 
	 * Session Pattern
	 * http-root-path/project/file.<session>.qml
	 * 
	 * 
	 * 
	 * @param request
	 * @throws FileNotFoundException
	 */
	public QMLRequest(HttpServletRequest request, ProjectConfigurationFactory factory) throws FileNotFoundException {
		this.request = request;
		this.configurationFactory = factory;
		rootpath = request.getServletContext().getInitParameter("http-root-path").toLowerCase();
		//previewpath = request.getServletContext().getInitParameter("http-preview-path").toLowerCase();
		
		///qtafi/projects/test/index.qml
		String uri = request.getRequestURI();
		System.out.println("uri " + uri);
		
		
		String relativepath = uri.replaceFirst(rootpath, "");
		String projectname = relativepath;
		if(relativepath.startsWith("/")) projectname = projectname.substring(1);
		
		projectname = projectname.substring(0, projectname.indexOf("/"));
		//System.out.println("projectname " +  projectname);
		
		String projecturl = relativepath.replaceFirst(projectname+"/", "");
		//System.out.println("projecturl " +  projecturl);
		
		int pos = projecturl.lastIndexOf(".qml");
		setLocale();
		project = configurationFactory.getProjectConfiguration(projectname);
		
		document = new QMLDocument();
		if(pos < 0){
			requestfile = "/index";
			String localerequestfile = requestfile + "_" + locale.getLanguage().toLowerCase() + ".qml";
			if(project.projectResourceExists(localerequestfile)){
				document.parse(project.getProjectResource(localerequestfile));
			}
			else if(project.projectResourceExists(requestfile+".qml")){
				document.parse(project.getProjectResource(requestfile+".qml"));
			}
			else error = SurveyError.FILE_NOT_FOUND;
		}
		else {
			requestfile = projecturl.substring(0, pos);
			int start = requestfile.indexOf(".");
			if(start > 0){
				String base64id = requestfile.substring(start+1);
				decodeSession(base64id);
				//System.out.println("session "+ sessionId + " <- " + base64id);
				requestfile = requestfile.substring(0,start);
			}
			String localerequestfile = requestfile + "_" + locale.getLanguage().toLowerCase() + ".qml";
			if(project.projectResourceExists(localerequestfile)){
				System.out.println("locale " + localerequestfile);
				document.parse(project.getProjectResource(localerequestfile));
			}
			else if(project.projectResourceExists(requestfile+".qml")){
				System.out.println("default " + requestfile);
				document.parse(project.getProjectResource(requestfile+".qml"));
			}
			else error = SurveyError.FILE_NOT_FOUND;
		}

		columnregistry = new ColumnRegistry();
		System.out.println("recode "+ recodeUrl());
	}
	
	public String getUrl(){
		return rootpath + "/" + project.getProjectName() + requestfile;
	}
	
	
	public String encodeSession(){
		byte[] encoded = Base64.encodeBase64(sessionId.getBytes());
		return new String(encoded);
	}
	
	private void decodeSession(String in){
		byte[] decoded = Base64.decodeBase64(in);
		sessionId = new String(decoded);
	}
	
	public String recodeUrl(){
		String base64id = encodeSession();
		//System.out.println("session "+ sessionId + " -> " + base64id);
		return rootpath + "/" + project.getProjectName() + requestfile + "." + base64id + ".qml";
	}
	
	private void setLocale(){
		if(request.getParameter("lang") != null){
			locale = Locale.forLanguageTag(request.getParameter("lang"));
			System.out.println("Locale from paramenter " + locale);
			request.getSession().setAttribute(sessionLangAttribute, locale);
		}
		else if(request.getSession().getAttribute(sessionLangAttribute) != null){
			locale = (Locale) request.getSession().getAttribute(sessionLangAttribute);
			System.out.println("Locale from session " + locale);
		}
		else {
			locale = request.getLocale();
			System.out.println("Locale from request " + locale);
		}
	}
	
	public void setLocale(Locale locale){
		this.locale = locale;
	}
	
	public Locale getLocale(){
		return locale;
	}
	
	public String getLanguage(){
		return locale.getLanguage();
	}
	
	public QMLDocument getQMLDocument(){
		return document;
	}
	
	public ColumnRegistry getColumnRegistry(){
		return columnregistry;
	}
	
	
	public HttpServletRequest getHttpRequest(){
		return request;
	}
	
	public void newSessionId(){
		sessionId = UUID.randomUUID().toString();
	}
	
	public String getSessionId(){
		if(sessionId.length() < 2) sessionId = UUID.randomUUID().toString();
		return sessionId;
	}
	
	public ProjectConfiguration getProjectConfiguration(){
		return project;
	}
	
	public void setError(SurveyError er){
		this.error = er;
	}
	
	public SurveyError getError(){
		return error;
	}
	
	public boolean isPostRequest(){
		return request.getMethod().equalsIgnoreCase("post");
	}
	
	public boolean isAsynchron(){
		return request.getParameter("__async") != null? request.getParameter("__async").equals("true")? true:false:false;
	}
	
	public String getRequestfile() {
		return requestfile;
	}
	
	public String toString(){
		StringBuffer buf = new StringBuffer();
		for(Enumeration<String> en = request.getAttributeNames(); en.hasMoreElements(); ){
			String name = en.nextElement();
			buf.append(name + "=" + request.getAttribute(name) + ";");
		}
		return buf.toString();
	}
	
}
