/*******************************************************************************
 * Questionnaire runtime
 *
 * Copyright (C) 2015 martin.guist@eoda.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.tinu.qtafi4;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.io.IOUtils;

public class ResourceFilter implements Filter {
	
	private String rootpath = "";
	private ProjectConfigurationFactory projectConfigurationFactory;
	
	@Override
	public void init(FilterConfig filter) throws ServletException {
		rootpath = filter.getServletContext().getInitParameter("http-root-path");
	}
	
	public void setProjectConfigurationFactory(ProjectConfigurationFactory projectConfigurationFactory) {
		this.projectConfigurationFactory = projectConfigurationFactory;
	}
	
	
	@Override
	public void destroy() {
	}

	@Override
	public void doFilter(ServletRequest srequest, ServletResponse sresponse,
			FilterChain filter) throws IOException, ServletException {
		
		HttpServletRequest request = (HttpServletRequest) srequest;
		HttpServletResponse response = (HttpServletResponse) sresponse;
		
		//String path = request.getPathInfo();
		String uri = request.getRequestURI().toLowerCase();
		
		String relativepath = uri.replaceFirst(rootpath, "");
		String projectname = relativepath;
		if(relativepath.startsWith("/")) projectname = projectname.substring(1);
		
		int slash =  projectname.indexOf("/");
		if(slash < 0) {
			filter.doFilter(srequest, sresponse);
			return;
		}
		projectname = projectname.substring(0, slash);
		//System.out.println(projectname);
		
		String requestfile = relativepath.replaceFirst(projectname+"/", "");
		if(requestfile.equals("/")){
			response.sendRedirect(rootpath + "/"+ projectname + "/index.qml");
			return;
		}
		
		InputStream input = this.projectConfigurationFactory.getProjectConfiguration(projectname).getProjectResource(requestfile);
		if(input != null){
			OutputStream output = response.getOutputStream();
			IOUtils.copy(input, output);
			response.flushBuffer();
		}
		else {
			filter.doFilter(srequest, sresponse);
		}
		/*
		File ff = new File(projectspath + "/" + projectname + "/" + requestfile);
		if(ff.exists()){
			copy(new FileInputStream(ff), response.getOutputStream());
			response.flushBuffer();
		}
		else filter.doFilter(srequest, sresponse);
		*/
	}
	/*
	public static void copy(InputStream input, OutputStream output) throws IOException {
		copy(input, output, 2048);
	}
	
	public static void copy(InputStream input, OutputStream output, int bufferSize) throws IOException {
		byte[] buf = new byte[bufferSize];
		int bytesRead = input.read(buf);
		while (bytesRead != -1) {
			output.write(buf, 0, bytesRead);
			bytesRead = input.read(buf);
		}
		output.flush();
	}
	*/
}
