/*******************************************************************************
 * Questionnaire runtime
 *
 * Copyright (C) 2015 martin.guist@eoda.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.tinu.qtafi4.auth;


import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Arrays;
import java.util.HashMap;

import javax.servlet.http.HttpSession;
import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.events.XMLEvent;

import org.apache.commons.codec.binary.Base64;








import de.tinu.qtafi4.AuthenticationType;
import de.tinu.qtafi4.ProjectConfiguration;
import de.tinu.qtafi4.QMLRequest;
import de.tinu.qtafi4.Runtime;
import de.tinu.qtafi4.data.AnonymUser;
import de.tinu.qtafi4.data.PreviewUser;
import de.tinu.qtafi4.data.SurveyUser;
import de.tinu.qtafi4.persistence.SQLCommunicator;

/**
 * 
 * 
 * @author maks
 *
 */
public class AuthenticationManager {
	
	private static final String USER_XML = "user.xml";
	
	private final SQLCommunicator communicator;
	
	public AuthenticationManager(SQLCommunicator communicator){
		this.communicator = communicator;
	}

	public Authentication newAuthentication(QMLRequest request){
		request.newSessionId();
		Authentication auth = new Authentication(request.getSessionId());
		auth.setAuthenticated(false);
		auth.setPrincipal(new AnonymUser());
		return auth;
	}
	
	public Authentication getAuthentication(QMLRequest request){
		if(request.getProjectConfiguration().getRuntime() == Runtime.SETUP){
			Authentication auth = new Authentication(request.getSessionId());
			HttpSession sess = request.getHttpRequest().getSession();
			Object id = sess.getAttribute("__id");
			if(id != null){
				if(id.equals(request.getSessionId())) {
					auth.setAuthenticated(true);
					auth.setPrincipal(new PreviewUser(auth.getId()));
					return auth;
				}
			}
			auth.setPrincipal(new AnonymUser());
			return auth;
		}
		Authentication auth = new Authentication(request.getSessionId());
		if(communicator.existsID(auth, request.getProjectConfiguration().getProjectName())){
			auth.setAuthenticated(true);
			loadUser(auth);
			return auth;
		}
		auth.setPrincipal(new AnonymUser());
		return auth;
	}
	
	private void loadUser(Authentication auth){
		SurveyUser principal = new SurveyUser(auth.getId());
		auth.setPrincipal(principal);
	}
	
	public void login(Authentication auth, AuthenticationToken token) throws AuthenticationError {
		if(configuredLogin(auth, token)){
			auth.setAuthenticated(true);
			//loadUser(auth);
			SurveyUser principal = new SurveyUser(auth.getId());
			auth.setPrincipal(principal);
			
			
			if(token.getQMLRequest().getProjectConfiguration().getRuntime() == Runtime.SETUP){
				HttpSession sess = token.getQMLRequest().getHttpRequest().getSession();
				System.out.println("login  " + sess.getAttribute("__id"));
				
				sess.setAttribute("__id", auth.getId());
				principal.setRequestPage(2);
			}
			
			
			return;
		}
		
		auth.setPrincipal(new AnonymUser());
		throw new AuthenticationError(AuthenticationError.INVALID_CREDENTIALS);
	}
	
	private synchronized boolean configuredLogin(Authentication auth, AuthenticationToken token){
		if(token.getQMLRequest().getProjectConfiguration().getRuntime() == Runtime.SETUP){
			if(token.getQMLRequest().getProjectConfiguration().getAuthentication() == AuthenticationType.NO){
				return false;
			}
			//OPEN
			if(token.getQMLRequest().getProjectConfiguration().getAuthentication() == AuthenticationType.OPEN){
				return true;
			}
			if(token.getQMLRequest().getProjectConfiguration().getAuthentication() == AuthenticationType.PIN
					|| token.getQMLRequest().getProjectConfiguration().getAuthentication() == AuthenticationType.URL){
				//1. look in database 
				String pass = token.getCredentials();
				if(pass == null) return false;
				System.out.println("PIN " + pass );
				
				if(communicator.existsCredentials(pass, token.getProjectName())){
					System.out.println("PIN exists " + pass );
					auth.setCredentials(token.getCredentials());
					
					return true;
				}
				else {//2. look in file
					if(validatePin(pass, token.getQMLRequest().getProjectConfiguration(), auth)){
						auth.setCredentials(token.getCredentials());
						return true;
					}
					return false;
				}
			}
			return false;
		}
		
		System.out.println("configured login " + token.getQMLRequest().getProjectConfiguration().getAuthentication());
		//NO AUTHENTICATION
		if(token.getQMLRequest().getProjectConfiguration().getAuthentication() == AuthenticationType.NO){
			return false;
		}
		//OPEN
		if(token.getQMLRequest().getProjectConfiguration().getAuthentication() == AuthenticationType.OPEN){
			communicator.appendID(auth, token.getProjectName());
			return true;
		}
		//COOKIE String pin, ProjectConfiguration config, Authentication auth
		if(token.getQMLRequest().getProjectConfiguration().getAuthentication() == AuthenticationType.COOKIE){
			if(communicator.existsID(auth, token.getProjectName())){
					//
			}
			else {
				communicator.appendID(auth, token.getProjectName());
			}
			return true;
		}
		//PIN field
		if(token.getQMLRequest().getProjectConfiguration().getAuthentication() == AuthenticationType.PIN){
			//1. look in database 
			String pass = token.getCredentials();
			if(pass == null) return false;
			System.out.println("PIN " + pass );
			
			if(communicator.existsCredentials(pass, token.getProjectName())){
				System.out.println("PIN exists " + pass );
				auth.setCredentials(token.getCredentials());
				communicator.updateID(auth, token.getProjectName());
				return true;
			}
			else {//2. look in file
				if(validatePin(pass, token.getQMLRequest().getProjectConfiguration(), auth)){
					auth.setCredentials(token.getCredentials());
					communicator.appendID(auth, token.getProjectName());
					return true;
				}
				return false;
			}
		}
		
		//URL
		if(token.getQMLRequest().getProjectConfiguration().getAuthentication() == AuthenticationType.URL){
			String query  = token.getQMLRequest().getHttpRequest().getQueryString();
			System.out.println("URL query " + query);
			try {
				query = URLDecoder.decode(query, "UTF-8");
			} 
			catch (UnsupportedEncodingException e) {}
			
			System.out.println(query);
			//1. look in database
			if(communicator.existsCredentials(query, token.getProjectName())){
				System.out.println("URL existsCredentials " + query);
				auth.setCredentials(query);
				communicator.updateID(auth, token.getProjectName());
				return true;
			}
			if(validatePin(query, token.getQMLRequest().getProjectConfiguration(), auth)){
				auth.setCredentials(query);
				communicator.appendID(auth, token.getProjectName());
				return true;
			}
		}
		//URL hex
		if(token.getQMLRequest().getProjectConfiguration().getAuthentication() == AuthenticationType.URLBASE64){
			String query  = token.getQMLRequest().getHttpRequest().getQueryString();
			//System.out.println(">"+ query + "<");
			String xx = new String(Base64.decodeBase64(query.getBytes()));
			try {
				xx = URLDecoder.decode(xx, "UTF-8");
			} catch (UnsupportedEncodingException e) {}
			//String xx = new String(b);//DatatypeConverter.parseBase64Binary(query);//(query.getBytes());
			//System.out.println(">"+xx + "<");
			
			String pin ="";
			String[] sp = xx.split("&");
			for(String p : sp){
				String[] kv = p.split("=");
				if(kv.length == 2){
					String key = kv[0];
					String val = kv[1];
					if(key.equals("__pin")) pin = val;
					else auth.putProperty(kv[0], kv[1]);
				}
			}
			//user.setPin(query);
			//1. look in database
			if(communicator.existsCredentials(pin, token.getProjectName())){
				auth.setCredentials(pin);
				communicator.updateID(auth, token.getProjectName());
				return true;
			}
			if(validatePin(query, token.getQMLRequest().getProjectConfiguration(), auth)){
				auth.setCredentials(pin);
				communicator.appendID(auth, token.getProjectName());
				return true;
			}
		}
		return false;
	}
	
	public void logout(){
	}
	
	protected boolean validatePin(String pin, ProjectConfiguration config, Authentication auth){
		return validateSQLPin(pin, config, auth);
	}
	
	protected boolean validateSQLPin(String pin, ProjectConfiguration config, Authentication auth){
		Connection conn = null;
		boolean valid = false;
		try{
			conn = communicator.datasource.getConnection();
			PreparedStatement pst = conn.prepareStatement("SELECT count(*) FROM `INFORMATION_SCHEMA`.`COLUMNS` WHERE `TABLE_SCHEMA`=? AND `TABLE_NAME`=?");
			pst.setString(1, "qtafi");
			pst.setString(2, config.getProjectName() + ".users");
			ResultSet res = pst.executeQuery();
			if(res.next()){
				if(res.getInt(1) > 0){
					pst.close();
					pst = conn.prepareStatement("SELECT count(n) as c FROM `" + config.getProjectName() + ".users` WHERE pin=?");
					pst.setString(1, pin);
					ResultSet res2 = pst.executeQuery();
					if(res2.next()){
						if(res2.getInt(1) == 1) valid = true;
					}
					pst.close();
				}
			}
		}
		catch(SQLException ex){
			ex.printStackTrace();
		}
		finally{
			try {
				conn.close();
			} 
			catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return valid;
	}
	
	
	protected boolean validateXMLPin(String pin, ProjectConfiguration config, Authentication auth){
		char[] pass = pin.toCharArray();
		try {
			XMLInputFactory inputf = XMLInputFactory.newInstance();
			XMLEventReader xmlreader = inputf.createXMLEventReader(config.getConfigurationResource(USER_XML));
			boolean insideuser = false;
			boolean valid = false;
			HashMap<String,String> userdata = new HashMap<String,String>();
			for(; xmlreader.hasNext();){
				XMLEvent xe = xmlreader.nextEvent();
				if(xe.getEventType() == XMLStreamConstants.START_ELEMENT){
					if(xe.asStartElement().getName().getLocalPart().equals("user")){
						insideuser = true;
					}
					else if(insideuser){
						if(xe.asStartElement().getName().getLocalPart().equals("pin")){
							if(Arrays.equals(pass, xmlreader.getElementText().trim().toCharArray())){
								auth.setCredentials(pin);
								valid = true;
							}
						}
						else {
							userdata.put(xe.asStartElement().getName().getLocalPart(), xmlreader.getElementText());
						}
					}
				}
				else if(xe.getEventType() == XMLStreamConstants.END_ELEMENT){
					if(xe.asEndElement().getName().getLocalPart().equals("user")){
						insideuser = false;
						if(valid){
							for(String key : userdata.keySet()){
								auth.putProperty(key, userdata.get(key));
							}
							return true;
						}
					}
				}
			}
		}
		catch(javax.xml.stream.XMLStreamException ex){
			ex.printStackTrace();
		}
		return false;
	}
}
