/*******************************************************************************
 * Questionnaire runtime
 *
 * Copyright (C) 2015 martin.guist@eoda.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.tinu.qtafi4.data;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import de.tinu.qtafi4.QMLRequest;
/**
 * 
 * 
 * 
 * @author maks
 *
 */
public class SurveyUser implements Serializable, User {
	
	private static final long serialVersionUID = 1L;
	
	public static final String PAGE = "__page";
	public static final String REQUESTPAGE = "__requestpage";
	public static final String MAXPAGE = "__maxpage";
	
	public final String id;
	
	protected HashMap<String,Object> parameter = new HashMap<String, Object>();
	protected int requestpage = 0;
	
	public SurveyUser(String id){
		this.id = id;
	}
	
	public String getId(){
		return id;
	}
	
	public void setRequestPage(int i){
		requestpage = i;
	}
	
	@Override
	public int getRequestPage() {
		if(requestpage < 0) {
			if(getIntValue("__lastaccessedpage") > 0){
				return getIntValue("__lastaccessedpage");
			}
			else return 2;
		}
		return requestpage;
	}
	
	public Set<String> getNames(){
		return parameter.keySet();
	}
	
	public Object getValue(String var){
		if(parameter.containsKey(var)) return parameter.get(var);
		return "-9";
	}
	
	public void setValue(String key, Object ob){
		parameter.put(key, ob);
	}
	
	public void write(QMLRequest request){
		if(request.getHttpRequest().getParameter(REQUESTPAGE) != null){
			setRequestPage(Integer.parseInt(request.getHttpRequest().getParameter(REQUESTPAGE).toString()));
		}
		else setRequestPage(-9);
		
		if(request.getHttpRequest().getParameter(PAGE) != null){
			setValue(PAGE, Integer.parseInt(request.getHttpRequest().getParameter(PAGE).toString()));
		}
		else setValue(PAGE, -9);
		
		if(request.getLocale() != null){
			setValue("__language", request.getLocale().getLanguage());
		}
		else setValue("__language", -9);
		
		for(String key: request.getHttpRequest().getParameterMap().keySet()){
			String[] array = request.getHttpRequest().getParameterMap().get(key);
			String v = array[0];
			//System.out.println("put " + key + " " + v);
			parameter.put(key, v);
		}
	}
	
	public int getIntValue(String var) {
		if(parameter.containsKey(var)){
			try{
				return Integer.parseInt(parameter.get(var).toString());
			}
			finally { }
		}
		return -9;
	}
	
	
	public Map<String, Object> toMap() {
		return parameter;
	}
	
	@Override
	public String toString() {
		return "SurveyUser " + id;
	}
}
