/*******************************************************************************
 * Questionnaire runtime
 *
 * Copyright (C) 2015 martin.guist@eoda.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.tinu.qtafi4.log;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import javax.sql.DataSource;

import de.tinu.qtafi4.QMLRequest;

public class SQLLog {
	
	private final DataSource datasource;
	private Connection connection = null;
	
	public SQLLog(DataSource datasource) {
		this.datasource = datasource;
	}
	
	private String getTable(String pr){
		return pr + ".log" ;
	}

	private String getIPAddress(QMLRequest request){
		return request.getHttpRequest().getRemoteAddr();
	}
	
	private String stringifyRequest(QMLRequest request){
		return request.toString();
	}
	
	private int getPage(QMLRequest request){
		String xx = request.getHttpRequest().getParameter("__requestpage");
		if(xx == null) return -9;
		try{
			return Integer.parseInt(xx);
		}
		catch(Exception e) { 
			return -9; 
		}
	}
	
	public void logIncident(QMLRequest request, IncidentType incident, Phase phase, String projectname) {
		PreparedStatement pstmt = null;
		try {
			connection = datasource.getConnection();
			pstmt = connection.prepareStatement("insert into `" + getTable(projectname) + "` " +
					"(sessionid, originated, request, path, ip, incidenttype, phase, requestpage, qmlfile) " +
					"values (?, CURRENT_TIMESTAMP, ?, ?, ?, ?, ?, ?, ?)");
			
			//sessionid
			pstmt.setString(1, request.getSessionId());
			
			//request
			pstmt.setString(2, request.toString());
			
			//path
			pstmt.setString(3, request.getHttpRequest().getRequestURI());
			
			//ip
			pstmt.setString(4, getIPAddress(request));
			
			//incidenttype
			pstmt.setShort(5, incident.getValue());
			
			//phase
			pstmt.setShort(6, phase.getValue());
			
			//requestpage
			pstmt.setInt(7, getPage(request));
			
			//qmlfile
			pstmt.setString(8, request.getRequestfile());
			
			pstmt.executeUpdate();
			pstmt.close();

		}
		catch(java.sql.SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				connection.close();
			} 
			catch (SQLException ex){
				ex.printStackTrace();
			}
		}
	}

	public void logRequest(QMLRequest request, String projectname) {
		PreparedStatement pstmt = null;
		try {
			connection = datasource.getConnection();
			pstmt = connection.prepareStatement("insert into `" + getTable(projectname) + "` "+
					"(sessionid, originated, request, path, ip, incidenttype, requestpage, qmlfile) values (?, CURRENT_TIMESTAMP, ?, ?, ?, ?, ?, ?)");
			
			//sessionid
			pstmt.setString(1, request.getSessionId());
			
			//request
			//pstmt.setString(2, request.toString());
			pstmt.setString(2, "");
			
			//path
			pstmt.setString(3, request.getHttpRequest().getRequestURI());
			
			//ip
			pstmt.setString(4, getIPAddress(request));
			
			//incidenttype
			pstmt.setShort(5, IncidentType.INFO.getValue());
			
			//requestpage
			pstmt.setInt(6, getPage(request));
			
			//qmlfile
			pstmt.setString(7, request.getRequestfile());
			
			pstmt.executeUpdate();
			pstmt.close();
		}
		catch(java.sql.SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				connection.close();
			} 
			catch (SQLException ex){ 
				ex.printStackTrace(); 
			}
		}
	}

	public void logVerbose(QMLRequest request, String message, Phase phase, String projectname) {
		PreparedStatement pstmt = null;
		try{
			connection = datasource.getConnection();
			pstmt = connection.prepareStatement("insert into `" + getTable(projectname) + "` "+
					"(sessionid, originated, request, path, ip, incidenttype, message, qmlfile) values (?, CURRENT_TIMESTAMP, ?, ?, ?, ?, ?, ?)");
			
			//sessionid
			pstmt.setString(1, request.getSessionId());
			
			//request
			pstmt.setString(2, request.toString());
			
			//path
			pstmt.setString(3, request.getHttpRequest().getRequestURI());
			
			//ip
			pstmt.setString(4, getIPAddress(request));
			
			//incidenttype
			pstmt.setShort(5, IncidentType.INFO.getValue());
			
			//message
			pstmt.setString(6, message);
			
			//qmlfile
			pstmt.setString(7, request.getRequestfile());
			
			pstmt.executeUpdate();
			pstmt.close();
		}
		catch(java.sql.SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				connection.close();
			} 
			catch (SQLException ex){ 
				ex.printStackTrace(); 
			}
		}
	}

	public int getIncidentCount(QMLRequest request, IncidentType incident, String projectname) {
		PreparedStatement pstmt = null;
		int count = 0;
		try{
			connection = datasource.getConnection();
			pstmt = connection.prepareStatement("select count(*) from `" + getTable(projectname) + "` where incidenttype=? AND ip=?");
			pstmt.setShort(1, incident.getValue());
			pstmt.setString(2, getIPAddress(request));
			ResultSet rs = pstmt.executeQuery();
			if(rs.next()){
				count = rs.getInt(1);
			}
			rs.close();
			pstmt.close();
		}
		catch(java.sql.SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				connection.close();
			}
			catch (SQLException ex){ 
				ex.printStackTrace();
			}
		}
		return count;
	}
	
//	@Override
//	public IncidentResult getIncidentResult(QMLRequest request, String projectname) {
//		return null;
//	}
	

	public void init(String project){
		Vector<String> fields = new Vector<String>();
		Statement st = null;
		fields.add("logid INT PRIMARY KEY NOT NULL AUTO_INCREMENT");
		fields.add("sessionid CHAR(100)");
		fields.add("originated DATETIME");
		fields.add("request TEXT");
		fields.add("query TEXT");
		fields.add("path VARCHAR(200)");
		fields.add("ip CHAR(20)" );
		fields.add("incidenttype INT");
		fields.add("requestpage INT");
		fields.add("qmlfile VARCHAR(100)");
		
		fields.add("phase INT");
		fields.add("message TEXT");
		try {
			connection = datasource.getConnection();
			st = connection.createStatement();
			st.executeUpdate("create table if not exists `" + getTable(project) + "` ("+joinStatements(fields, ",")+") CHARACTER SET utf8");
			st.close();
		}
		catch(java.sql.SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {connection.close();} 
			catch (SQLException ex){ 
				ex.printStackTrace(); 
			}
		}
	}
	
	private static String joinStatements(Vector<String> v, String delimiter){
		StringBuffer b = new StringBuffer();
		int len = v.size();
		for(int i = 0; i < len; i++){
			b.append(v.get(i));
			if(i < len-1) b.append(delimiter);
		}
		return b.toString();
	}
}
