/*******************************************************************************
 * Questionnaire runtime
 *
 * Copyright (C) 2015 martin.guist@eoda.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.tinu.qtafi4.persistence;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import javax.sql.DataSource;

import de.tinu.qtafi4.auth.Authentication;
import de.tinu.qtafi4.data.User;

public class SQLCommunicator {
	
	public static final String SYSTEM_PREFIX = "__";
	public static final String DB_TYPE_1 = "VARCHAR(2)";
	public static final String DB_TYPE_2 = "VARCHAR(20)";
	public static final String DB_TYPE_3 = "TEXT";
	
	public static final int DB_TYPE_LEN_1 = 2;
	public static final int DB_TYPE_LEN_2 = 20;
	
	private Connection connection = null;
	public final DataSource datasource;
	
	public SQLCommunicator(DataSource datasource){
		this.datasource = datasource;
	}
	
	private String getTable(String pr){
		return pr;
	}
	
	public boolean existsCredentials(String credentials, String projectname){
		if(credentials == null) return false;
		if(credentials.equals("-9")) return false;
		if(credentials.length() < 1) return false;
		boolean res = false;
		PreparedStatement pstmt = null;
		try {
			connection = datasource.getConnection();
			pstmt = connection.prepareStatement("select __pin from `" + getTable(projectname) + "` where __pin=?");
			pstmt.setString(1, credentials);
			ResultSet rs = pstmt.executeQuery();
			if(rs.next()){
				res = true;
			}
			rs.close();
			pstmt.close();
		}
		catch(java.sql.SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				connection.close();
			}
			catch (SQLException ex){ }
		}
		return res;
	}
	
	public boolean existsID(Authentication data, String projectname){
		boolean res = false;
		PreparedStatement pstmt = null;
		try{
			connection = datasource.getConnection();
			pstmt = connection.prepareStatement("select * from `" + getTable(projectname) + "` where __id=?");
			pstmt.setString(1, data.getId().toString());
			ResultSet rs = pstmt.executeQuery();
			if(rs.next()){
				res = true;
			}
			rs.close();
			pstmt.close();
		}
		catch(java.sql.SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				connection.close();
			} 
			catch (SQLException ex){ }
		}
		return res;
	}
	
	public void appendID(Authentication data, String projectname) {
		PreparedStatement pstmt = null;
		try {
			connection = datasource.getConnection();
			pstmt = connection.prepareStatement("insert into `" + getTable(projectname) + "` (__id, __pin, __inited, __lastchange, __maxaccessedpage, __logins, __lastaccessedpage, __updatecount) values (?, ?, CURRENT_TIMESTAMP, CURRENT_TIMESTAMP, -1, 1, -1, 1)");
			pstmt.setString(1, data.getId().toString());
			pstmt.setString(2, data.getCredentials());
			pstmt.executeUpdate();
			pstmt.close();
		}
		catch(java.sql.SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				connection.close();
			} 
			catch (SQLException ex){ }
		}
	}
	
	public void updateID(Authentication data, String projectname) {
		String pin = data.getCredentials();
		if(pin == null) System.err.println("PIN null");
		if(pin.equals("-9")) System.err.println("PIN -0");
		if(pin.length() < 1) System.err.println("PIN too short");
		System.out.println("Update id " + data.getId());
		PreparedStatement pstmt = null;
		try {
			connection = datasource.getConnection();
			pstmt = connection.prepareStatement("update `" + getTable(projectname) + "` " +
					"set __id=?, __logins=__logins+1, __lastchange=CURRENT_TIMESTAMP where __pin=?");
			pstmt.setString(1, data.getId());
			pstmt.setString(2, pin);
			pstmt.executeUpdate();
			pstmt.close();
		}
		catch(java.sql.SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				connection.close();
			} 
			catch (SQLException ex){ }
		}
	}
	
	public void writeMeta(Authentication data, String projectname) {
	}
	
	public void registerColumns(ColumnRegistry reg, String table){
		Statement st = null;
		try {
			connection = datasource.getConnection();
			Vector<String> existingfields = new Vector<String>();
			st = connection.createStatement();
			ResultSet rs = st.executeQuery("show columns from `" + getTable(table) + "`");
			while(rs.next()) existingfields.add(rs.getString("Field"));
			rs.close();
			Vector<String> varstoadd = new Vector<String>();
			for(String f: reg.getColumns()){
				if(f.startsWith(SYSTEM_PREFIX)) continue;
				if(! existingfields.contains(f) && ! varstoadd.contains(f)){
					if(f.trim().length() > 0) varstoadd.add("`" + f + "` " + DB_TYPE_1);
				}
			}
			if(varstoadd.size() > 0){
				st.executeUpdate("alter table `" + getTable(table) + "` add (" + joinStatements(varstoadd, ",") + ")");
			}
			st.close();
		}
		catch(java.sql.SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				connection.close(); 
			} 
			catch (SQLException ex){ }
		}
	}
	
	public void read(User data, String projectname){
		PreparedStatement pstmt = null;
		try {
			connection = datasource.getConnection();
			pstmt = connection.prepareStatement("select * from `" + getTable(projectname) + "` where __id=?");
			pstmt.setString(1, data.getId());
			ResultSet rs = pstmt.executeQuery();
			ResultSetMetaData rsmd = rs.getMetaData();
			int cols = rsmd.getColumnCount();
			if(rs.next()){
				for(int i = 1; i <= cols; i++){
					String name = rsmd.getColumnName(i);
					int type = rsmd.getColumnType(i);
					if(type != Types.BLOB){
						String s = rs.getString(i);
						if(s != null) data.setValue(name, s);
						else data.setValue(name, "-9");
					}
				}
			}
			rs.close();
			pstmt.close();
		}
		catch(java.sql.SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				connection.close();
			}
			catch (SQLException ex){ }
		}
	}
	
	public void write(User user, String project){
		Vector<String> dbvars = modifyColumns(user, project);
		Vector<String> updatevars = new Vector<String>();
		Vector<String> updatevals = new Vector<String>();
		Set<String> keys = user.toMap().keySet();
		for(String ke: keys){
			if(! dbvars.contains(ke)) continue;
			String val = user.getValue(ke).toString();
			
			if(! isValidColumn(ke)) continue;
			if(! ke.startsWith(SYSTEM_PREFIX) && val != null){
				if(val.trim().length() == 0){
					updatevars.add(ke);
					updatevals.add("-9");
				}
				else{
					updatevars.add(ke);
					updatevals.add(val);
				}
			}
		}
		PreparedStatement pstmt = null;
		try {
			connection = datasource.getConnection();
			updatevars.add("__language");
			updatevals.add(user.getValue("__language").toString());
			
			if(updatevars.size()>0){
				int page = user.getIntValue("__page");
				if(page > 0){
					updatevars.add("__lastaccessedpage");
					updatevals.add(Integer.toString(page));
				}
				pstmt = connection.prepareStatement("update `" + getTable(project) + "` set " + joinIntoPreparedStatement(updatevars) +
						", __lastchange=CURRENT_TIMESTAMP, __updatecount=__updatecount+1 ,__maxaccessedpage=IF(__maxaccessedpage<"+page+","+page+",__maxaccessedpage) where __id=?");
				for(int i = 0; i < updatevals.size(); i++){
					//System.out.println(updatevars.get(i) + " "+ updatevals.get(i));
					//pstmt.setCharacterStream(i+1, new StringReader(updatevals.get(i)));
					pstmt.setString(i+1, updatevals.get(i).trim());
				}
				pstmt.setString(updatevals.size()+1, user.getId().toString());
				pstmt.executeUpdate();
				pstmt.close();
			}
		}
		catch(java.sql.SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				connection.close();
			} 
			catch (SQLException ex){ }
		}
		//writeFile(config);
	}
	
	private Vector<String>  modifyColumns(User user, String table){
		Statement st = null;
		Vector<String> available_columns = new Vector<String>();
		try {
			connection = datasource.getConnection();
			Map<String,String> fields = new HashMap<String,String>();
			Vector<String> alter = new Vector<String>();
			st = connection.createStatement();
			ResultSet rs = st.executeQuery("show columns from `" + getTable(table) + "`");
			while(rs.next()){
				fields.put(rs.getString("Field"), rs.getString("Type"));
				available_columns.add(rs.getString("Field"));
			}
			rs.close();
			for(Iterator<String> i = user.toMap().keySet().iterator(); i.hasNext(); ){
				String ke = i.next();
				if(ke.startsWith(SYSTEM_PREFIX)) continue;
				String val = user.getValue(ke).toString();
				if(val != null && fields.containsKey(ke)){
					if(fields.get(ke).equalsIgnoreCase(DB_TYPE_1)){
						if(! ke.startsWith("__") && val.length() > DB_TYPE_LEN_1){
							if(val.length() < DB_TYPE_LEN_2){
								alter.add("modify `" + ke + "` " + DB_TYPE_2);
							}
							else{
								alter.add("modify `" + ke +"` " + DB_TYPE_3);
							}
						}
					}
					else if(fields.get(ke).equalsIgnoreCase(DB_TYPE_2)){
						if(val.length() > DB_TYPE_LEN_2){
							alter.add("modify `" + ke + "` " + DB_TYPE_3);
						}
					}
				}
			}
			if(alter.size() > 0){
				st.executeUpdate("alter table `" + getTable(table) + "` " + joinStatements(alter, ","));
			}
			st.close();
		}
		catch(java.sql.SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				connection.close();
			} 
			catch (SQLException ex) {}
		}
		return available_columns;
	}
	
	private void modifyColumns(HashMap<String, String> data, String table){
		Statement st = null;
		try {
			connection = datasource.getConnection();
			Map<String,String> fields = new HashMap<String,String>();
			Vector<String> alter = new Vector<String>();
			st = connection.createStatement();
			ResultSet rs = st.executeQuery("show columns from `" + getTable(table) + "`");
			while(rs.next()){
				fields.put(rs.getString("Field"), rs.getString("Type"));
			}
			rs.close();
			for(Iterator<String> i = data.keySet().iterator(); i.hasNext(); ){
				String ke = i.next();
				if(ke.startsWith(SYSTEM_PREFIX)) continue;
				String val = data.get(ke);
				if(val != null && fields.containsKey(ke)){
					if(fields.get(ke).equalsIgnoreCase(DB_TYPE_1)){
						if(! ke.startsWith("__") && val.length() > DB_TYPE_LEN_1){
							if(val.length() < DB_TYPE_LEN_2){
								alter.add("modify `" + ke + "` " + DB_TYPE_2);
							}
							else{
								alter.add("modify `" + ke +"` " + DB_TYPE_3);
							}
						}
					}
					else if(fields.get(ke).equalsIgnoreCase(DB_TYPE_2)){
						if(val.length() > DB_TYPE_LEN_2){
							alter.add("modify `" + ke + "` " + DB_TYPE_3);
						}
					}
				}
			}
			if(alter.size() > 0){
				st.executeUpdate("alter table `" + getTable(table) + "` " + joinStatements(alter,","));
			}
			st.close();
		}
		catch(java.sql.SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				connection.close();
			} 
			catch (SQLException ex) { }
		}
	}
	
	public void init(String project){
		Vector<String> fields = new Vector<String>();
		Statement st = null;
		fields.add("__n INT PRIMARY KEY NOT NULL AUTO_INCREMENT");
		fields.add("__pin CHAR(50)");
		fields.add("__id CHAR(100)");
		fields.add("__inited DATETIME");
		fields.add("__lastchange DATETIME");
		fields.add("__lastaccessedpage INT");
		fields.add("__maxaccessedpage INT");
		fields.add("__logins INT");
		fields.add("__updatecount INT");
		fields.add("__language CHAR(10)");
		try {
			connection = datasource.getConnection();
			
			st = connection.createStatement();
			st.executeUpdate("create table if not exists `" + getTable(project) + "` ("+joinStatements(fields, ",")+") CHARACTER SET utf8");
			st.close();
		}
		catch(java.sql.SQLException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				connection.close();
			} 
			catch (SQLException ex){ }
		}
	}
	
	
	private static String joinIntoPreparedStatement(Vector<String> v){
		StringBuffer b = new StringBuffer();
		int len = v.size();
		for(int i = 0; i < len; i++){
			b.append("`"+v.get(i) +"`=?");
			if(i < len-1) b.append(", ");
		}
		return b.toString();
	}
	
	private static String joinAddVariables(Vector<String> v, String delimiter){
		StringBuffer b = new StringBuffer();
		int len = v.size();
		for(int i = 0; i < len; i++){
			b.append("`" + v.get(i) + "`");
			if(i < len-1) b.append(delimiter);
		}
		return b.toString();
	}
	
	private static String joinStatements(Vector<String> v, String delimiter){
		StringBuffer b = new StringBuffer();
		int len = v.size();
		for(int i = 0; i < len; i++){
			b.append( v.get(i) );
			if(i < len-1) b.append(delimiter);
		}
		return b.toString();
	}
	
	
	
	private static boolean isValidColumn(String col){
		return col.matches("\\w{1,30}");
	}
	
}
