/*******************************************************************************
 * Questionnaire runtime
 *
 * Copyright (C) 2015 martin.guist@eoda.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.tinu.qtafi4.postprocessor;

import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import org.w3c.dom.Element;

import de.tinu.qtafi4.ProjectConfiguration;
import de.tinu.qtafi4.QMLRequest;
import de.tinu.qtafi4.data.User;

public class LANGNode extends PostprocessorNode {

	public LANGNode() {
		super("LANG");
	}

	@Override
	public void process(Element processorElement, QMLRequest request, User data) {
		//first search for languages
		ProjectConfiguration config = request.getProjectConfiguration();
		document = processorElement.getOwnerDocument();
		List<String> ls = config.listQuestionnaires();
		
		Element replace = document.createElement("ul");
		replace.setAttribute("class", "qtafi_lang");
		
		String currentfile = request.getRequestfile();
		String currentlang = getLang(currentfile+".qml");
		
		for(Iterator<String> it = ls.iterator(); it.hasNext(); ){
			String l = it.next();
			
			String lang = getLang(l);
			
			if(lang.equals(currentlang)) continue;
			
			Element li = document.createElement("li");
			
			Element link = document.createElement("a");
			link.setAttribute("class", "qtafi_lang");
			
			String label = Locale.forLanguageTag(lang).getDisplayLanguage(request.getLocale());
			if(lang.equals("")) label = "Default";
			
			link.setAttribute("href", request.rootpath + "/" + config.getProjectName() + "/" + getName(l) + "." + request.encodeSession() + ".qml");
			link.appendChild(document.createTextNode(label));
			
			li.appendChild(link);
			replace.appendChild(li);
		}
		
		processorElement.getParentNode().replaceChild(replace, processorElement);
	}

	private String getLang(String l){
		String[] name_ext = l.split("\\.");
		String lang = "";
		if(name_ext.length == 2){
			String name = name_ext[0];
			String[] name_lang = name.split("_");
			if(name_lang.length == 2){
				lang = name_lang[1];
			}
		}
		return lang;
	}
	
	private String getName(String l){
		String[] name_ext = l.split("\\.");
		String name = "";
		if(name_ext.length == 2){
			name = name_ext[0];
		}
		return name;
	}
	
}
