/*******************************************************************************
 * Questionnaire runtime
 *
 * Copyright (C) 2015 martin.guist@eoda.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.tinu.qtafi4.postprocessor;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.traversal.DocumentTraversal;
import org.w3c.dom.traversal.NodeFilter;
import org.w3c.dom.traversal.NodeIterator;

import de.tinu.qtafi4.QMLRequest;
import de.tinu.qtafi4.data.User;


public abstract class PostprocessorNode {

	protected Document document;
	protected final String name;
	protected Postprocessor postprocessor;
	
	public PostprocessorNode(String name){
		this.name = name;
	}
	
	public String getName(){
		return name;
	}
	
	public abstract void process(Element processorElement, QMLRequest request, User data);
	
	public void processSubTree(Node node, QMLRequest request, User data) {
		DocumentTraversal traversal = (DocumentTraversal) document;
		NodeIterator iterator = traversal.createNodeIterator(node, NodeFilter.SHOW_ELEMENT, new NodeFilter(){
			public short acceptNode(Node n) {
				if(n.getNamespaceURI() == null) return NodeFilter.FILTER_SKIP;
				if(n.getNamespaceURI().equals(Postprocessor.POSTPROCESSOR_NAMESPACE)){
					return NodeFilter.FILTER_ACCEPT;
				}
				return NodeFilter.FILTER_SKIP;
			}
		}, false);
		Node n;
		while((n = iterator.nextNode()) != null) postprocessor.processNode(n, request, data);
	}
}