/*******************************************************************************
 * Questionnaire runtime
 *
 * Copyright (C) 2015 martin.guist@eoda.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.tinu.qtafi4.qml;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.w3c.dom.Node;

import bsh.EvalError;
import bsh.Interpreter;

public class CodeInterpreter {
	
	Interpreter interpreter = new Interpreter();
	
	public CodeInterpreter(){
	}
	
	public void setParams(Map<String,Object> params){
		if(params != null){
			try {
				for(Iterator<String> i = params.keySet().iterator(); i.hasNext(); ){
					String k = i.next();
					interpreter.set(k, params.get(k));
					//binding.setVariable(k, params.get(k));
				}
			}catch (EvalError e) {
				e.printStackTrace();
			}
		}
	}
	
	public void setParam(String name, Object param){
		//binding.setVariable(name, param);
		try {
			interpreter.set(name, param);
		} 
		catch (EvalError e) {
			e.printStackTrace();
		}
	}
	
	public Object interpret(String s){
		try{
			ClassLoader bshcl = new ClassLoader(){
				protected Class<?> findClass(String name) throws ClassNotFoundException {
					System.out.println(name);
					throw new ClassNotFoundException(name);
				}
				protected synchronized Class<?> loadClass(String name, boolean resolve) throws ClassNotFoundException {
					System.out.println(name);
					throw new ClassNotFoundException(name);
				}
				public Class<?> loadClass(String name) throws ClassNotFoundException {
					//System.out.println("loadClass "+ name);
					if(name.equals("java.lang.String")) return java.lang.String.class;
					if(name.equals("java.lang.StringBuffer")) return java.lang.StringBuffer.class;
					if(name.equals("java.lang.Integer")) return Integer.class;
					if(name.equals("java.lang.Short")) return Short.class;
					if(name.equals("java.lang.Float")) return Float.class;
					if(name.equals("java.lang.Double")) return Double.class;
					if(name.equals("java.lang.Long")) return Long.class;
					if(name.equals("java.lang.Math")) return java.lang.Math.class;
					if(name.equals("de.qtafi2.jsp.User")) return Long.class;
					if(name.equals("java.util.Vector")) return java.util.Vector.class;
					if(name.equals("java.util.HashMap")) return java.util.HashMap.class;
					if(name.equals("org.w3c.dom.Node")) return org.w3c.dom.Node.class;
					if(name.equals("org.w3c.dom.Element")) return org.w3c.dom.Element.class;
					if(name.equals("org.w3c.dom.Document")) return org.w3c.dom.Document.class;
					if(name.equals("org.w3c.dom.NodeList")) return org.w3c.dom.NodeList.class;
					if(name.startsWith("org.apache")) return super.loadClass(name);
					if(name.startsWith("com.sun.org.apache.xerces")) return super.loadClass(name);
					return Object.class;
				}
				protected String findLibrary(String libname) {
					return "";
				}
			};
			interpreter.setClassLoader(bshcl);
			interpreter.getClassManager().setClassLoader(bshcl);
			Object o = interpreter.eval(s);
			return o;
		} 
		catch (EvalError e) {
			e.printStackTrace();
		}
		return null;
	}
	
	public void interpret(String s, StringBuffer output){
		Object o = interpret(s);
		output.append(o);
	}
	
	public void interpret(String s, Node n){
		Object o = interpret(s);
		if(o instanceof Node){
			//DocumentBuilderFactory domFactory = DocumentBuilderFactory.newInstance();
			//domFactory.setNamespaceAware(true); 
			//DocumentBuilder builder = domFactory.newDocumentBuilder();
			//Document doc = builder.parse(new InputSource(new StringReader(o.toString())));
			Node imp = n.getOwnerDocument().importNode((Node) o, true);
			n.appendChild(imp);
		}
	}
	
	
	public boolean interpret(String s, Boolean output){
		Object o = interpret(s);
		if(o instanceof Boolean) return ((Boolean) o).booleanValue();
		return output;
	}
	
	
//	public void interpret(String s, StringBuffer output){
//		interpret(s, output, null);
//	}
	
//	public void interpret(String s, StringBuffer output, HashMap<String,Object> params){
//		//binding.setVariable("foo", new Integer(2));
//		if(params != null){
//			for(Iterator<String> i = params.keySet().iterator(); i.hasNext(); ){
//				String k = i.next();
//				binding.setVariable(k, params.get(k));
//			}
//		}
//		Object value = shell.evaluate(s);
//		output.append(value);
//	}

	public static void main(String[] a){
		CodeInterpreter interpreter = new CodeInterpreter();
		StringBuffer output = new StringBuffer();
		HashMap<String,Object> hm = new HashMap<String,Object>();

		interpreter.interpret("int i = 45;  return i + 56;", output);
		System.out.println(output.toString() );
	}
	
}
