/*******************************************************************************
 * Questionnaire runtime
 *
 * Copyright (C) 2015 martin.guist@eoda.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.tinu.qtafi4.qml;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import de.tinu.qtafi4.data.User;


public class CodebookFilter {
	
	public static final short IS = 1;
	public static final short IS_IGNORE_CASE = 7;
	public static final short IS_NOT = 2;
	public static final short LESS_THAN = 3;
	public static final short LESS_OR_EQUAL_THAN = 4;
	public static final short GREATER_THAN = 5;
	public static final short GREATER_OR_EQUAL_THAN = 6;
	public static final short STARTS_WITH = 8;
	public static final short ENDS_WITH = 9;
	public static final short FILE_EXISTS = 10;
	
	public static final short AND =  10;
	public static final short OR =   11;
	public static final short NAND = 12;
	public static final short NOR =  13;
	public static final short NOT =  14;
	public static final short XOR =  15;
	
	private static Element getFirstChild(Element e, String s){
		NodeList el = e.getChildNodes();
		if(el == null) return null;
		for(int i = 0; i < el.getLength(); i++){
			Node n = el.item(i);
			if(n.getNodeType() == Node.ELEMENT_NODE && n.getLocalName().equals(s)) return (Element)n;
		}
		return null;
	}
	
	public static int parseOperator(Element ie){
		if(ie.hasAttribute("operator")) return parseOperator(ie.getAttribute("operator"));
		return IS;
	}
	
	public static int parseOperator(String op){
		if(op.equalsIgnoreCase("is")) return IS;
		if(op.equalsIgnoreCase("is_ignore_case")) return IS_IGNORE_CASE;
		if(op.equalsIgnoreCase("is_not")) return IS_NOT;
		if(op.equalsIgnoreCase("less_than")) return LESS_THAN;
		if(op.equalsIgnoreCase("less_or_equal_than")) return LESS_OR_EQUAL_THAN;
		if(op.equalsIgnoreCase("greater_than")) return GREATER_THAN;
		if(op.equalsIgnoreCase("greater_or_equal_than")) return GREATER_OR_EQUAL_THAN;
		if(op.equalsIgnoreCase("starts_with")) return STARTS_WITH;
		if(op.equalsIgnoreCase("ends_width")) return ENDS_WITH;
		return IS;
	}

	public static int parseCondition(String op){
		if(op.equalsIgnoreCase("and")) return AND;
		if(op.equalsIgnoreCase("or")) return OR;
		if(op.equalsIgnoreCase("nand")) return NAND;
		if(op.equalsIgnoreCase("nor")) return NOR;
		if(op.equalsIgnoreCase("not")) return NOT;
		if(op.equalsIgnoreCase("xor")) return XOR;
		return OR;
	}

	private static boolean evalGroup(Element ge, User data){
		int condition = OR;
		//String id = "";
		//if(ge.hasAttribute("id")) id = ge.getAttribute("id");
		if(ge.hasAttribute("condition")) condition = parseCondition(ge.getAttribute("condition"));
		NodeList filterlist = ge.getChildNodes();
		if(filterlist == null) return false;
		
		boolean not = true;
		int count = 0;
		if(condition == NOT){
			for(int ii = 0; ii < filterlist.getLength(); ii++){
				Node nn = filterlist.item(ii);
				if(nn.getNodeType() == Node.ELEMENT_NODE){
					if(nn.getLocalName().equals("use_if") || nn.getLocalName().equals("ignore_if")){
						boolean res = eval((Element)nn, data);
						not = !res;
						count++;
					}
					else if(nn.getLocalName().equals("use_group") || 
							nn.getLocalName().equals("show_group") ||
							nn.getLocalName().equals("hide_group") ||
							nn.getLocalName().equals("ignore_group")|| nn.getLocalName().equals("filter_group")){
						boolean res = evalGroup((Element)nn, data);
						not = !res;
						count++;
					}
				}
			}
			if(count == 1) return not;
			else throw new IllegalArgumentException("Illegal number of operators");
		}
		
		boolean result = true;
		int numresult = -1;
		for(int ii = 0; ii < filterlist.getLength(); ii++){
			Node nn = filterlist.item(ii);
			if(nn.getNodeType() == Node.ELEMENT_NODE){
				if(nn.getLocalName().equals("use_if")|| 
						nn.getLocalName().equals("ignore_if")|| 
						nn.getLocalName().equals("show_if")||
						nn.getLocalName().equals("hide_if") ){
					boolean res = eval((Element)nn, data);
					int thisnum = res?1:0;
					//if(((Element)nn).hasAttribute("id")) System.out.println("filter "+ ((Element)nn).getAttribute("id") + "? "+ res);
					if(condition == AND){//OK
						if(!(result && res)) return false;
					}
					else if(condition == NAND){//OK
						if(! res) return true;
					}
					else if(condition == NOR){//OK
						if(res) return false;
					}
					else if(condition == OR){//OK
						if(res) return true;
					}
					else if(condition == XOR){
						System.out.println(numresult + " <> " + thisnum);
						if(numresult < 0){
							System.out.println("continue");
							result = res;
							numresult = thisnum;
							continue;
						}
						if(numresult != thisnum){
							System.out.println(numresult + " <> " + thisnum + " true");
							return true; 
						}
					}
					result = res;
					numresult = thisnum;
				}
				else if(nn.getLocalName().equals("use_group")|| 
						nn.getLocalName().equals("show_group")||
						nn.getLocalName().equals("hide_group")||
						nn.getLocalName().equals("ignore_group")|| 
						nn.getLocalName().equals("filter_group")){
					boolean res = evalGroup((Element)nn, data);
					int thisnum = res?1:0;
					//if(((Element)nn).hasAttribute("id")) System.out.println("group " + ((Element)nn).getAttribute("id") + "? "+ res);
					if(condition == AND){//OK
						if(!(result && res)) return false;
					}
					else if(condition == NAND){//OK
						if(! res) return true;
					}
					else if(condition == NOR){//OK
						if(res) return false;
					}
					else if(condition == OR){//OK
						if(res) return true;
					}
					else if(condition == XOR){
						if(numresult < 0){
							result = res;
							numresult = thisnum;
							continue;
						}
						if(numresult != thisnum) return true;
					}
					result = res;
					numresult = thisnum;
				}
			}
		}
		if(condition == AND) return true;
		if(condition == NAND) return false;
		if(condition == OR) return false;
		if(condition == NOR) return true;
		if(condition == XOR) return false;
		return false;
	}
	
	public static String parseElementValue(Element ie, User data){
		if(ie.hasAttribute("value")){
			String val = ie.getAttribute("value");
			if(val.startsWith("eval(__maxpage)")) {
				return data.getValue("__maxpage").toString();
			}
			return ie.getAttribute("value");
		}
		return "-9";
	}

	public static boolean eval(Element ie, User data){
		if(ie.hasAttribute("variable")){
			//System.out.println("variable " + ie.getAttribute("variable"));
			boolean re = compare(
					data.getValue(ie.getAttribute("variable")).toString(), 
					parseElementValue(ie, data), 
					parseOperator(ie));
			return re;
		}
		else {
			String sc = ie.getTextContent();
			//System.out.println("interpret " + sc);
			if(sc.trim().length() > 5){
				CodeInterpreter interpreter = new CodeInterpreter();
				interpreter.setParams(data.toMap());
				interpreter.setParam("user", data);
				return interpreter.interpret(sc, Boolean.FALSE);
			}
		}
		return false;
	}

	public static boolean evalFilterOld(Element e, User data){
		//System.out.println("--------");
		Element filter = getFirstChild(e, "filter");
		if(filter == null) return true;
		NodeList filterlist = filter.getChildNodes();
		if(filterlist == null) return true;
		boolean ignore = true;
		for(int i = 0; i < filterlist.getLength(); i++){
			Node n = filterlist.item(i);
			if(n == null) return true;
			if(n.getNodeType() == Node.ELEMENT_NODE){
				Element en = (Element)n;
				if(n.getLocalName().equals("ignore_if")){
					//if condition is true return false;
					boolean res = eval(en, data);
					ignore = true;
					//if(((Element)en).hasAttribute("id")) System.out.println("filter " + ((Element)en).getAttribute("id") + "? "+ res);
					if(res) return false;
				}
				else if(n.getLocalName().equals("ignore_group")){
					//if group condition is true return false;
					//boolean res = evalIgnoreGroup(en, ui);
					boolean res = evalGroup(en, data);
					ignore = true;
					//if(((Element)en).hasAttribute("id")) System.out.println("group " + ((Element)en).getAttribute("id") + "? "+ res);
					if(res) return false;
				}
				else if(n.getLocalName().equals("use_if")){
					//if condition is true return true;
					boolean res = eval(en, data);
					ignore = false;
					if(res) return true;
				}
				else if(n.getLocalName().equals("use_group")){
					//if group condition is true return true;
					boolean res = evalGroup(en, data);
					ignore = false;
					if(res) return true;
				}
			}
		}
		return ignore;
	}
	
	public static boolean evalFilter(Element e, User data){
		//System.out.println("eval filter on " + e );
		Element filter = getFirstChild(e, "filter");
		if(filter == null) {
			//System.out.println("filter==null");
			return true;
		}
		NodeList filterlist = filter.getChildNodes();
		if(filterlist == null){
			//System.out.println("filterlist==null");
			return true;
		}
		boolean ignore = true;
		for(int i = 0; i < filterlist.getLength(); i++){
			Node n = filterlist.item(i);
			if(n == null) return true;
			if(n.getNodeType() == Node.ELEMENT_NODE){
				Element en = (Element)n;
				if(en.hasAttribute("runat")){
					if(en.getAttribute("runat").equalsIgnoreCase("server")){
						if(n.getLocalName().equals("ignore_if") || n.getLocalName().equals("hide_if") ){
							boolean res = eval(en, data);
							ignore = true;
							if(res) return false;
						}
						else if(n.getLocalName().equals("ignore_group") || n.getLocalName().equals("hide_group")){
							boolean res = evalGroup(en, data);
							ignore = true;
							if(res) return false;
						}
						else if(n.getLocalName().equals("use_if") || n.getLocalName().equals("show_if")){
							boolean res = eval(en, data);
							//System.out.println("eval show_if " + data + " -> " + res);
							ignore = false;
							if(res) return true;
						}
						else if(n.getLocalName().equals("use_group") || n.getLocalName().equals("show_group")){
							boolean res = evalGroup(en, data);
							ignore = false;
							if(res) return true;
						}
					}
					else {
						//System.out.println(en.getAttribute("runat"));
					}
				}
				else {
					//System.out.println("runat " + en);
				}
			}
		}
		return ignore;
	}

	public static boolean compare(boolean comp , int operator){
		switch(operator){
			case IS: return comp;
			case IS_IGNORE_CASE: return comp;
			case IS_NOT: return !comp;
			default: return false;
		}
	}
	
	public static boolean compare(String value1, String value2, int operator){
		int val1 = -9;
		int val2 = -9;
		try{
			val1 = Integer.parseInt(value1);
			val2 = Integer.parseInt(value2);
			switch(operator){
				case IS: return val1 == val2;
				case IS_IGNORE_CASE: return val1 == val2;
				case IS_NOT: return val1 != val2;
				case LESS_THAN: return val1 < val2;
				case LESS_OR_EQUAL_THAN: return val1 <= val2;
				case GREATER_THAN: return val1 > val2;
				case GREATER_OR_EQUAL_THAN: return val1 >= val2;
				case STARTS_WITH:{
					return value1.startsWith(value2);
				}
				case ENDS_WITH: return value1.endsWith(value2);
				default: return false;
			}
		}
		catch(NumberFormatException ex){ }
		switch(operator){
			case IS: return value1.equals(value2);
			case IS_IGNORE_CASE: return value1.equalsIgnoreCase(value2);
			case IS_NOT: return !value1.equals(value2);
			case LESS_THAN: return value1.compareTo(value2) < 0;
			case LESS_OR_EQUAL_THAN: return value1.compareTo(value2) <= 0;
			case GREATER_THAN: return value1.compareTo(value2) > 0;
			case GREATER_OR_EQUAL_THAN: return value1.compareTo(value2) >= 0;
			case STARTS_WITH: {
				return value1.startsWith(value2);
			}
			case ENDS_WITH: return value1.endsWith(value2);
			default: return false;
		}
	}
}
