/*******************************************************************************
 * Questionnaire runtime
 *
 * Copyright (C) 2015 martin.guist@eoda.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.tinu.qtafi4.qml;

import java.util.Vector;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class FilterSwitch {

	/**
	 * 
	 * parses a page, detects all filters with no runat and sets runat="client" / runat="server if the variable is on the same page
	 * 
	 * TODO Handle importvars, handle root filters seperatly
	 * 
	 * <filter>
	 *  <importfiltervariable variable="" />
	 * 	<show_if variable="" />
	 * 	<ignore_if />
	 * </filter>
	 * 
	 */
	public static void process(Element root){
		Vector<String> pagevars = searchVariables(root);
		//System.out.println("pagevars " + pagevars);
		traverseFilterElements(pagevars, root);
	}
	
	private static Vector<String> searchVariables(Element el){
		Vector<String> variables = new Vector<String>();
		searchVariables(variables, el);
		return variables;
	}
	
	private static void searchVariables(Vector<String> variables, Element el){
		if(el == null) return;
		NodeList ch = el.getChildNodes();
		if(ch == null) return;
		for(int i = 0; i < ch.getLength(); i++){
			Node n = ch.item(i);
			if(n.getNodeType() == Node.ELEMENT_NODE){
				Element childel = (Element) n;
				if(childel.getNodeName().equals("reportdata")) return;
				else if(childel.getNodeName().equals("variable")){
					if(childel.hasAttribute("name")) variables.add(childel.getAttribute("name"));
				}
				else if(childel.getNodeName().startsWith("qtafi:") && ! childel.getNodeName().equals("qtafi:LINK") && ! childel.getNodeName().equals("qtafi:VAR")){
					if(childel.hasAttribute("variable")) variables.add(childel.getAttribute("variable"));
				}
				else searchVariables(variables, childel);
			}
		}
	}
	
	private static void traverseFilterElements(Vector<String> variables, Element el){
		if(el == null) return;
		NodeList ch = el.getChildNodes();
		if(ch == null) return;
		for(int i = 0; i < ch.getLength(); i++){
			Node n = ch.item(i);
			if(n.getNodeType() == Node.ELEMENT_NODE){
				Element childel = (Element) n;
				if(childel.getNodeName().equals("filter")) applyOnRoot(variables, childel);
				else traverseFilterElements(variables, childel);
			}
		}
	}
	
	private static boolean isFilter(Node filterelement){
		if(filterelement.getNodeType() != Node.ELEMENT_NODE) return false;
		if(filterelement.getNodeName().equals("ignore_if")){
			if(((Element) filterelement).hasAttribute("variable")) return true;
		}
		else if(filterelement.getNodeName().equals("use_if")){
			if(((Element) filterelement).hasAttribute("variable")) return true;
		}
		else if(filterelement.getNodeName().equals("show_if")){
			if(((Element) filterelement).hasAttribute("variable")) return true;
		}
		else if(filterelement.getNodeName().equals("hide_if")){
			if(((Element) filterelement).hasAttribute("variable")) return true;
		}
		return false;
	}
	
	private static boolean isGroup(Node filterelement){
		if(filterelement.getNodeType() != Node.ELEMENT_NODE) return false;
		if(filterelement.getNodeName().equals("ignore_group")) return true;
		if(filterelement.getNodeName().equals("use_group")) return true;
		if(filterelement.getNodeName().equals("show_group")) return true;
		if(filterelement.getNodeName().equals("hide_group")) return true;
		return false;
	}
	
	private static void applyOnRoot(Vector<String> variables, Element rootfilter){
		if(rootfilter == null) return;
		NodeList ch = rootfilter.getChildNodes();
		if(ch == null) return;
		for(int i = 0; i < ch.getLength(); i++){
			Node n = ch.item(i);
			if(n.getNodeType() == Node.ELEMENT_NODE){
				if(isFilter(n)){
					//print("apply", n);
					applyOnFilter(variables, rootfilter, (Element) n);
				}
				else if(isGroup(n)){
					//print("apply", n);
					applyOnFilter(variables, rootfilter, (Element) n);
				}
			}
		}
	}
	
	private static void print(String mess, Node fn){
		Element filter = (Element) fn;
		String var = filter.hasAttribute("variable")?filter.getAttribute("variable"):"no-var";
		System.out.println(mess + " - Filter " + filter.getLocalName() + ": " + var);
	}
	
	private static void applyOnFilter(Vector<String> variables, Element rootfilter, Element filterelement){
		if(filterelement == null) return;
		boolean runatserver = true;
		FilterConfig fg = new FilterConfig();
		fg.rootfilterelement = rootfilter;
		fg.filterelement = filterelement;
		fg.pagevariables = variables;
		fg.detect();
		fg.apply();
	}
	
	private static class FilterConfig {
		Element rootfilterelement;
		Element filterelement;
		String runat = "server";
		Vector<String> pagevariables;
		Vector<String> importvars = new Vector<String>();
		Vector<String> filtervars = new Vector<String>();
		
		void pushVar(String var){
			//System.out.println("push var " + var );
			if(pagevariables.contains(var)) runat = "client";
			filtervars.add(var);
		}
	
		void importVar(){
			if(runat.equals("server")) return;
			for(String v: filtervars){
				if(!pagevariables.contains(v)){
					Element imp = rootfilterelement.getOwnerDocument().createElement("importfiltervariable");
					imp.setAttribute("name", v);
					rootfilterelement.appendChild(imp);
					importvars.add(v);
				}
			}
		}
		
		void detect(){
			detectRecursiv(filterelement);
		}
		
		void detectRecursiv(Element el){
			if(isFilter(el)){
				//print("detect " , el);
				if(el.hasAttribute("variable") && ! el.hasAttribute("runat")) pushVar(el.getAttribute("variable"));
			}
			NodeList ch = el.getChildNodes();
			if(ch == null) return;
			for(int i = 0; i < ch.getLength(); i++){
				Node n = ch.item(i);
				if(n.getNodeType() == Node.ELEMENT_NODE){
					Element childel = (Element) n;
					detectRecursiv(childel);
				}
			}
		}
		void apply(){
			importVar();
			//System.out.println("apply runat=" + runat + " to " + filterelement.getLocalName() + (filterelement.hasAttribute("variable")?" ("+filterelement.getAttribute("variable")+")":""));
			applyRecursiv(filterelement);
		}
		
		void applyRecursiv(Element el){
			if(isFilter(el)){
				if(el.hasAttribute("variable") && ! el.hasAttribute("runat")){
					el.setAttribute("runat", runat);
				}
			}
			else if (isGroup(el)){
				if(! el.hasAttribute("runat")){
					el.setAttribute("runat", runat);
				}
			}
			
			NodeList ch = el.getChildNodes();
			if(ch == null) return;
			for(int i = 0; i < ch.getLength(); i++){
				Node n = ch.item(i);
				if(n.getNodeType() == Node.ELEMENT_NODE){
					applyRecursiv((Element) n);
				}
			}
		}
	}
}
