/*******************************************************************************
 * Questionnaire runtime
 *
 * Copyright (C) 2015 martin.guist@eoda.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.tinu.qtafi4.qml;

import java.util.Vector;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.traversal.DocumentTraversal;
import org.w3c.dom.traversal.NodeFilter;
import org.w3c.dom.traversal.TreeWalker;

import de.tinu.qtafi4.ProjectConfiguration;
import de.tinu.qtafi4.QMLRequest;
import de.tinu.qtafi4.data.AnonymUser;
import de.tinu.qtafi4.data.SurveyUser;
import de.tinu.qtafi4.data.User;
import de.tinu.qtafi4.util.StringHelpers;
import de.tinu.qtafi4.util.xml.XMLUtils;

public class Preprocessor {
	
	
	public void process(QMLRequest request, User data){
		Preprocess pp = new Preprocess(request);
		pp.buildPage(data);
	}
	
	public void buildId(QMLRequest request, User data, String qid){
		Preprocess pp = new Preprocess(request);
		pp.buildId(qid, data);
	}
	
	private class Preprocess {
		
		final ProjectConfiguration configuration;
		final QMLRequest request;
		private Element codebook;
		final Element rootelement;
		//private int page, maxpage;
		
		Preprocess(QMLRequest request){
			this.configuration = request.getProjectConfiguration();
			this.request = request;
			this.rootelement = request.getQMLDocument().getRootelement();
		}
		
//		public int getParameter(String p){
//			String xx = request.getHttpRequest().getParameter(p);
//			if(xx == null) return -9;
//			try{
//				return Integer.parseInt(xx);
//			}
//			catch(Exception e) { 
//				return -9; 
//			}
//		}
		
		private void buildPage(User data){
			int request_page = data.getRequestPage();
			int page = data.getIntValue(SurveyUser.PAGE);
			if(data instanceof AnonymUser) {
				request_page = 1;
				page = 1;
			}
			Vector<Element> v = XMLUtils.getChildNodes(rootelement, "codebook");
			//System.out.println("buildPage request = " + request_page);
	
			if(v.size() > 0){//mehr als ein Codebook
				int readcb = 0;
				int n = 0;
				for(Element cbelement: v){
					boolean read = CodebookFilter.evalFilter(cbelement, data);
					if(read) readcb = n;
					n++;
				}
				
				n = 0;
				//System.out.println("readcb " + readcb);
				for(Element cbelement: v){
					if(n == readcb)	codebook = cbelement;
					else rootelement.removeChild(cbelement); 
					n++;
				}
			}
			
			if(codebook == null){
				applyImportfiles(rootelement);
				applyFilter(rootelement, data);
				return;
			}
			applyImportCodebook(codebook, data);
			String pd = codebook.getAttribute("pagedir");
			if(pd != null){
				this.request.getQMLDocument().pagedir = pd;
			}
			
			
			Vector<Element> pages = getPages(codebook, data);
			if(request_page < 1) request_page = 1;
			if(request_page > pages.size()) request_page = pages.size();
			
			int maxpage = pages.size();
			data.setValue(SurveyUser.MAXPAGE, maxpage);
			int format_page = request_page;
	
			int buildpage = request_page;
			int foundpage = -1;
			if(page < request_page){//vorblättern
				for(int pi = 0; pi < pages.size(); pi++){
					Element pageelement = pages.get(pi);
					if(pi < (request_page-1)){ //davor
						codebook.removeChild(pageelement);
					}
					else { //auf seite oder dahinter
						if(foundpage < 0){//noch keine
							applyImportPages(pageelement);
							//boolean usepage = applyPageFilter(pageelement,ui);
							FilterSwitch.process(pageelement);
							boolean usepage = applyFilter(pageelement, data);
							if(usepage) usepage = XMLUtils.hasBodyElements(pageelement);
							if(usepage){
								foundpage = pi;
								data.setValue(SurveyUser.PAGE, foundpage+1);
								page = foundpage + 1;
							}
							else {
								codebook.removeChild(pageelement);
							}
						}
						else{
							codebook.removeChild(pageelement);
						}
					}
				}
			}
			else if(request_page == page){
				for(int pi = 0; pi < pages.size(); pi++){
					Element pageelement = pages.get(pi);
					if(request_page-1 != pi) codebook.removeChild(pageelement);
				}
			}
			else if(page > request_page){//zurückblättern
				for(int pi = pages.size()-1; pi >= 0; pi--){
					Element pageelement = pages.get(pi);
					if(pi > (request_page-1)){
						codebook.removeChild(pageelement);
					}
					else {
						if(foundpage < 0){
							applyImportPages(pageelement);
							FilterSwitch.process(pageelement);
							boolean usepage = applyFilter(pageelement, data);
							if(usepage) usepage = XMLUtils.hasBodyElements(pageelement);
							if(usepage){
								foundpage = pi;
								data.setValue(SurveyUser.PAGE, foundpage+1);
								page = foundpage+1;
							}
							else {
								codebook.removeChild(pageelement);
							}
						}
						else codebook.removeChild(pageelement);
					}
				}
			}
			
			applyImportQuestions(rootelement);
			applyImportfiles(rootelement);
			FilterSwitch.process(rootelement);
			applyFilter(rootelement, data);
		}
		
		public void buildId(String id, User data){
			//setupCodebook(data);
			XPath xpathinstance = XPathFactory.newInstance().newXPath();
			try{
				String xpath = "//*[@id='"+id+"']";
				System.out.println("xpath  " + xpath);
			    XPathExpression expr = xpathinstance.compile(xpath);
			    Object result = expr.evaluate(rootelement, XPathConstants.NODESET);
			    if(result == null){
			    	System.out.println("[info] not exists " + xpath);
			    }
			    if(result instanceof NodeList){
			    	NodeList nl = (NodeList) result;
			    	if(nl.getLength() > 0){
						Node n = nl.item(0);
						if(n.getNodeType() == Node.ELEMENT_NODE){
							System.out.println("found " + n);
							makeSubtree(n);
							applyImportQuestions(rootelement);
							applyImportfiles(rootelement);
							FilterSwitch.process(rootelement);
							applyFilter(rootelement, data);
						}
						else {
							System.out.println("not found " + id);
						}
				    }
			    	else {
			    		request.getQMLDocument().clear();
			    		System.out.println("not found " + id);
			    	}
			    }
			}
			catch(XPathExpressionException e) {
				e.printStackTrace();
			}
		}
		
		private void setupCodebook(User data){
			Vector<Element> v = XMLUtils.getChildNodes(rootelement, "codebook");
			if(v.size() > 0){//mehr als ein Codebook
				int readcb = 0;
				int n = 0;
				for(Element cbelement: v){
					boolean read = CodebookFilter.evalFilter(cbelement, data);
					if(read) readcb = n;
					n++;
				}
				n = 0;
				//System.out.println("readcb " + readcb);
				for(Element cbelement: v){
					if(n == readcb)	codebook = cbelement;
					else rootelement.removeChild(cbelement); 
					n++;
				}
			}
			if(codebook == null){
				applyImportfiles(rootelement);
				applyFilter(rootelement, data);
				return;
			}
			applyImportCodebook(codebook, data);
		}
		
		private Node makeSubtree(Node n){
			Node node = n;
			Node preparent = node;
			do {
				preparent = node;
				removeAllExceptMine(node);
				node = node.getParentNode();
			}
			while(node != null);
			return preparent;
		}
		
		private void removeAllExceptMine(Node node){
			if(node instanceof Element){
				Node parent = node.getParentNode();
				NodeList l = parent.getChildNodes();
				if(l != null){
					for(int i = 0; i < l.getLength(); i++){
						Node ch = l.item(i);
						if(ch.getNodeType() == Node.ELEMENT_NODE){
							if(ch.equals(node)) continue;
							parent.removeChild(ch);
						}
					}
				}
			}
		}
		
		private Vector<Element> getPages(Element codebook, User data){
			Vector<Element> allpages = XMLUtils.getChildNodes(codebook, "Page");
			Vector<Element> pages = new Vector<Element>();
			for(Element pa: allpages){
				if(XMLUtils.hasChild(pa, "filter")){
					boolean read = CodebookFilter.evalFilter(pa, data);
					if(read) pages.add(pa);
				}
				else pages.add(pa);
			}
			return pages;
		}
		
		private boolean applyPageFilter(Element e, User data){
			Vector<Node> nodes_to_remove = new Vector<Node>();
			NodeList childs = e.getChildNodes();
			if(childs == null) return true;
			for(int i = 0; i < childs.getLength(); i++){
				Node ch = childs.item(i);
				if(ch.getNodeName().equals("filter") && ch.getNodeType() == Node.ELEMENT_NODE){
					Element filter = (Element) ch;
					if(filter.getUserData("applied") == null){
						filter.setUserData("applied", new Boolean(true), null);
						Node element = filter.getParentNode();
						//System.out.println("apply page filter " + e);
						boolean read = CodebookFilter.evalFilter((Element)element, data);
						if(! read){
							if(! nodes_to_remove.contains(element)) nodes_to_remove.add(element);
						}
					}
				}
			}

			for(Node n: nodes_to_remove){
				if(n.equals(e)) return false;
				n.getParentNode().removeChild(n);
			}
			return true;
		}
		
		public boolean applyFilter(Element e, User data){
			if(e == null) return true;
			Vector<Node> nodes_to_remove = new Vector<Node>();
			NodeList filters = e.getElementsByTagName("filter");
			for(int i = 0; i < filters.getLength(); i++){
				Node filter = filters.item(i);
				if(filter.getNodeType() == Node.ELEMENT_NODE){
					if(filter.getUserData("applied") == null){
						filter.setUserData("applied", new Boolean(true), null);
						Node element = filter.getParentNode();
						boolean read = CodebookFilter.evalFilter((Element)element, data);
						if(! read){
							if(! nodes_to_remove.contains(element)) nodes_to_remove.add(element);
						}
					}
				}
			}

			for(Node n: nodes_to_remove){
				if(n.equals(e)) return false;
				n.getParentNode().removeChild(n);
			}
			return true;
		}
		
		/**
		 * TODO: Import path
		 * <codebook importfile="master.xml:/qtafi/codebook"
		 * 
		 * 
		 * @param e
		 * @param data
		 */
		public void applyImportCodebook(Element e, User data){
			if(e == null) return;
			if(e.hasAttribute("importfile")){
				String usecb = e.getAttribute("importfile");
				e.removeAttribute("importfile");
				
				if(configuration.projectResourceExists(usecb)){
					try{
						Document idoc = request.getQMLDocument().parseStream(configuration.getProjectResource(usecb));
						Element importelement = idoc.getDocumentElement();
						if(importelement.getNodeName().equals("qtafi")){
							importelement = XMLUtils.getFirstChild(importelement, "codebook");
						}
						XMLUtils.mergeChildren(request.getQMLDocument().getDocument(), e, importelement);
					}
					catch(Exception ex){ex.printStackTrace();}
				}
			}
		}
		
		public void applyImportfile(Element e){
			if(e == null) return;
			if(e.hasAttribute("importfile")){
				String impf = e.getAttribute("importfile");
				e.removeAttribute("importfile");
				importFile(e, impf);
			}
		}
		
		public void applyImportfiles(Element e){
			if(e == null) return;
			DocumentTraversal traversal = (DocumentTraversal) request.getQMLDocument().getDocument();
			TreeWalker walker = traversal.createTreeWalker(e, NodeFilter.SHOW_ELEMENT, null, false);
			Node n;
			int i = 0;
			while((n = walker.nextNode()) != null){
				if(n.getNodeType()== Node.ELEMENT_NODE){
					Element el = (Element)n;
					if(el.hasAttribute("importfile")){
						String impf = el.getAttribute("importfile");
						el.removeAttribute("importfile");
						importFile(el, impf);
					}
				}
			}
		}

		private void importFile(Element e, String file){
			if(e.hasAttribute("importparam")){
				String[] ts  = e.getAttribute("importparam").split("\\|");
				String[] params = new String[ts.length];
				String[] values =  new String[ts.length];
				for(int i=0; i < ts.length; i++){
					String[] s = ts[i].split("=");
					params[i] = s[0];
					values[i] = s[1];
				}
				try{
					if(configuration.projectResourceExists(file)){
						StringBuffer qmlfilebuf = StringHelpers.readStream(configuration.getProjectResource(file));
						String qmlfile = qmlfilebuf.toString();
						for(int i=0; i < params.length; i++){
							qmlfile = qmlfile.replaceAll(params[i], values[i]);
						}
						Document idoc = request.getQMLDocument().parseStream(configuration.getProjectResource(file));
						XMLUtils.mergeChildren(request.getQMLDocument().getDocument(), e, idoc.getDocumentElement());
					}
				}
				catch(Exception ex){
					ex.printStackTrace();
				}
			}
			else{
				try{
					if(configuration.projectResourceExists(file)){
						Document idoc = request.getQMLDocument().parseStream(configuration.getProjectResource(file));
						XMLUtils.mergeChildren(request.getQMLDocument().getDocument(), e, idoc.getDocumentElement());
					}
				}
				catch(Exception ex){
					ex.printStackTrace();
				}
			}
		}

		/**
		* Xpath style 
		* <CV import="master:://CV[@id='id']" />
		* 
		* <CV import="//CV[@id='id']" />
		*/
		public void applyImportQuestions(Element e){
			DocumentTraversal traversal = (DocumentTraversal) request.getQMLDocument().getDocument();
			TreeWalker walker = traversal.createTreeWalker(rootelement, NodeFilter.SHOW_ELEMENT, null, false);
			Node n;
			int i = 0;
			while((n = walker.nextNode()) != null){
				if(n.getNodeType()== Node.ELEMENT_NODE){
					Element el = (Element)n;
					if(el.hasAttribute("import")) importQuestion(el, el.getAttribute("import"));
				}
			}
		}
		
		private void importQuestion(Element e, String path){
			try{
				String importproject = configuration.getProjectName();
				String importxpath = "";
				String[] pathels = path.split("::");
				if(pathels.length == 2){
					importproject = pathels[0];
					importxpath = pathels[1];
				}
				else {
					importxpath = path;
				}
				if(configuration.projectResourceExists(".questionnaire")){
					Document idoc = request.getQMLDocument().parseStream(configuration.getProjectResource(".questionnaire"));
					Node node = XMLUtils.getNode(idoc.getDocumentElement(), importxpath);
					if(node != null){
						if(node instanceof Element){
							XMLUtils.merge(request.getQMLDocument().getDocument(), e, (Element)node);
						}
					}
				}
			}
			catch(Exception ex){
				ex.printStackTrace();
			}
		}
		
		private void applyImportPages(Element page){
			if(page.hasAttribute("importfile")) {
				String impf = page.getAttribute("importfile");
				page.removeAttribute("importfile");
				importFile(page, impf);
			}
		}
	}
	
}
