/*******************************************************************************
 * Questionnaire runtime
 *
 * Copyright (C) 2015 martin.guist@eoda.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.tinu.qtafi4.qml;

import java.io.FileReader;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Serializable;
import java.io.StringReader;
import java.io.Writer;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.URIResolver;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.InputSource;

/**
 * 
 * <qtafi>
 * 	<codebook pagedir="" formatter="">
 * </qtafi>
 * 
 * 
 * @author maks
 *
 */
public class QMLDocument implements Serializable {
	
	private static final long serialVersionUID = 1L;
	public String pagedir = "ltr";
	public String formatter = "main";
	
	private Element rootelement;
	private Document document;
	private static DocumentBuilder builder;
	
	
	public QMLDocument(){
		try{
			DocumentBuilderFactory builderfactory = DocumentBuilderFactory.newInstance();
			builderfactory.setNamespaceAware(true);
			builderfactory.setSchema(null);
			builderfactory.setValidating(false);
			builder = builderfactory.newDocumentBuilder();
		}
		catch(javax.xml.parsers.ParserConfigurationException ex){
			ex.printStackTrace();
		}
	}
	
	
	public void setRootElement(Element el){
		rootelement = el;
		document = rootelement.getOwnerDocument();
	}
	
	public void parse(InputStream is){
		try {
			document = builder.parse(new InputSource(is));
			rootelement = document.getDocumentElement();
		}
		catch(Exception ex){
			ex.printStackTrace();
		}
	}

	public void parse(StringBuffer buf){
		try{
			document = builder.parse(new InputSource(new StringReader(buf.toString())));
			rootelement = document.getDocumentElement();
		}
		catch(Exception ex){
			ex.printStackTrace();
		}
	}
	
	public void parse(String buf){
		try{
			document = builder.parse(new InputSource(new StringReader(buf)));
			rootelement = document.getDocumentElement();
		}
		catch(Exception ex){
			ex.printStackTrace();
		}
	}
	
	public void clear(){
		document = builder.newDocument();
		rootelement = document.createElement("qtafi");
		document.appendChild(rootelement);
		Element cb = document.createElement("codebook");
		rootelement.appendChild(cb);
	}
	
	public void printDocument(OutputStream out){
		try{
			transform(new OutputStreamWriter(out, "UTF-8"), null);
		}
		catch(java.io.UnsupportedEncodingException ex){
			ex.printStackTrace();
		}
	}

	public void transform(OutputStream os, String xslt, String...parameter){
		try{
			transform(new OutputStreamWriter(os, "UTF-8"), xslt);
		}
		catch(java.io.UnsupportedEncodingException ex){
			ex.printStackTrace();
		}
	}
	
	public void transform(String xslt, String...parameter){
		try{
			TransformerFactory transfac = TransformerFactory.newInstance();
			Transformer trans;
			transfac.setURIResolver(new URIResolver(){
				public Source resolve(String href, String base){
					return new StreamSource(href);
				}
			});
			if(xslt == null) trans = transfac.newTransformer();
			else trans = transfac.newTransformer(new StreamSource(new FileReader(xslt)));
			trans.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION,"yes");
			trans.setOutputProperty(OutputKeys.METHOD,"xml");
			
			trans.setParameter("http-resources-path", "" );
			trans.setParameter("http-projects-path", "");
			trans.setParameter("max-page", "1");
			trans.setParameter("page", "1");
			trans.setParameter("async", "");
			trans.setParameter("project",  "");
			trans.setParameter("style-file", "");
			trans.setParameter("mode",  "");
			trans.setParameter("expiration", "online");
			
			for(int i = 0; i < parameter.length; i+=2){
				trans.setParameter(parameter[i], parameter[i+1]);
			}
			
			DOMSource source = new DOMSource(document);
			Document doc = builder.newDocument();
		    Result result = new DOMResult(doc);
			trans.transform(source, result);
			document = doc;
			rootelement = document.getDocumentElement();
		}
		catch(java.io.FileNotFoundException ex){
			ex.printStackTrace();
		}
		catch(javax.xml.transform.TransformerConfigurationException ex){
			ex.printStackTrace();
		}
		catch(javax.xml.transform.TransformerException ex){
			ex.printStackTrace();
		}
	}
	
	public QMLDocument transformInto(String xslt, String...parameter){
		try{
			TransformerFactory transfac = TransformerFactory.newInstance();
			Transformer trans;
			transfac.setURIResolver(new URIResolver(){
				public Source resolve(String href, String base){
					return new StreamSource(href);
				}
			});
			if(xslt == null) trans = transfac.newTransformer();
			else trans = transfac.newTransformer(new StreamSource(new FileReader(xslt)));
			trans.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION,"yes");
			trans.setOutputProperty(OutputKeys.METHOD,"xml");
			for(int i = 0; i < parameter.length; i+=2){
				trans.setParameter(parameter[i], parameter[i+1]);
			}
			DOMSource source = new DOMSource(document);
			Document doc = builder.newDocument();
			DOMResult result = new DOMResult(doc);
			trans.transform(source, result);
			QMLDocument qq = new QMLDocument();
			qq.setRootElement(doc.getDocumentElement());
			return qq;
		}
		catch(java.io.FileNotFoundException ex){
			ex.printStackTrace();
		}
		catch(javax.xml.transform.TransformerConfigurationException ex){
			ex.printStackTrace();
		}
		catch(javax.xml.transform.TransformerException ex){
			ex.printStackTrace();
		}
		return null;
	}
	
	public void transform(Writer wr, String xslt, String...parameter){
		try{
			TransformerFactory transfac = TransformerFactory.newInstance();
			Transformer trans;
			transfac.setURIResolver(new URIResolver(){
				public Source resolve(String href, String base){
					return new StreamSource(href);
				}
			});
			if(xslt == null) trans = transfac.newTransformer();
			else trans = transfac.newTransformer(new StreamSource(new FileReader(xslt)));
			trans.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION,"yes");
			trans.setOutputProperty(OutputKeys.METHOD,"xml");
			
			trans.setParameter("http-resources-path", "" );
			trans.setParameter("http-projects-path", "");
			trans.setParameter("max-page", "1");
			trans.setParameter("page", "1");
			trans.setParameter("async", "");
			trans.setParameter("project",  "");
			trans.setParameter("expiration", "online");
			
			for(int i = 0; i < parameter.length; i+=2){
				trans.setParameter(parameter[i], parameter[i+1]);
			}
			
			StreamResult result = new StreamResult(wr);
			DOMSource source = new DOMSource(document);
			trans.transform(source, result);
		}
		catch(java.io.FileNotFoundException ex){
			ex.printStackTrace();
		}
		catch(javax.xml.transform.TransformerConfigurationException ex){
		}
		catch(javax.xml.transform.TransformerException ex){
		}
	}
	
	public Document parseStream(InputStream is){
		try{
			return builder.parse(new InputSource(is));
		}
		catch(Exception ex){
			ex.printStackTrace();
		}
		return null;
	}
	
	public Element getRootelement() {
		return rootelement;
	}

	public Document getDocument() {
		return document;
	}
}
