/*******************************************************************************
 * Questionnaire runtime
 *
 * Copyright (C) 2015 martin.guist@eoda.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.tinu.qtafi4.util.xml;

import java.util.Vector;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class XMLUtils {
	
	public static Node getNode(Element doc, String xpathquery){
		try{
			XPath xpath = XPathFactory.newInstance().newXPath();
		    XPathExpression expr = xpath.compile(xpathquery);
		    Object result = expr.evaluate(doc, XPathConstants.NODESET);
		    NodeList nodes = (NodeList) result;
		    if(nodes == null) return null;
		    if(nodes.getLength() == 0) return null;
		    return nodes.item(0);
		}
	    catch(XPathExpressionException e) {
			e.printStackTrace();
		} 
	    return null;
	}
	
	public static boolean hasBodyElements(Element e){
		NodeList el = e.getChildNodes();
		for(int i = 0; i < el.getLength(); i++){
			Node n = el.item(i);
			if(n.getNodeType() == Node.ELEMENT_NODE && n.getLocalName().equals("children")){
				NodeList cel = n.getChildNodes();
				for(int ii = 0; ii < cel.getLength(); ii++){
					if(cel.item(ii).getNodeType() == Node.ELEMENT_NODE) return true;
				}
			}
		}
		return false;
	}
	
	public static void merge(Document document, Element e, Element addelement){
		NodeList iel = addelement.getChildNodes();
		if(iel != null){
			for(int i = 0; i < iel.getLength(); i++){
				Node in = iel.item(i);
				if(! (in.getNodeName().equals("number") && hasChild(e, "number"))){
					e.appendChild(document.importNode(in, true));
				}
			}
		}
		NamedNodeMap nnm = addelement.getAttributes();
		for(int i = 0; i < nnm.getLength(); i++){
			e.setAttribute(nnm.item(i).getLocalName(), nnm.item(i).getNodeValue());
		}
	}
	
	public static void mergeChildren(Document document, Element e, Element ie){
		NodeList iel = ie.getChildNodes();
		for(int i = 0; i < iel.getLength(); i++){
			Node in = iel.item(i);
			e.appendChild(document.importNode(in, true));
		}
	}
	
	public static Element searchId(Element e, String id){
		return searchIdRecursiv(e,id);
	}
	
	public static Element searchIdRecursiv(Element e, String id){
		if(e.hasAttribute("id")){
			if(e.getAttribute("id").equals(id)) return e;
		}
		else {
			NodeList el = e.getChildNodes();
			if(el != null){
				for(int i = 0; i < el.getLength(); i++){
					Node n = el.item(i);
					if(n.getNodeType() == Node.ELEMENT_NODE){
						Element element = (Element) n;
						Element res = searchIdRecursiv(element, id);
						if(res != null) return res;
					}
				}
			}
		}
		return null;
	}
	public static Element getFirstChild(Element e, String s){
		NodeList el = e.getChildNodes();
		for(int i = 0; i < el.getLength(); i++){
			Node n = el.item(i);
			if(n.getNodeType() == Node.ELEMENT_NODE && n.getLocalName().equals(s)){
				return (Element)n;
			}
		}
		return null;
	}

	public static boolean hasChild(Element e, String s){
		NodeList el = e.getChildNodes();
		for(int i = 0; i < el.getLength(); i++){
			Node n = el.item(i);
			if(n.getNodeType() == Node.ELEMENT_NODE && n.getLocalName().equals(s)){
				return true;
			}
		}
		return false;
	}

	public static Vector<Element> getChildNodes(Element e, String s){
		Vector<Element> v = new Vector<Element>();
		if(e == null) return v;
		NodeList el = e.getChildNodes();
		if(el == null) return v;
		for(int i = 0; i < el.getLength(); i++){
			Node n = el.item(i);
			if(n.getNodeType() == Node.ELEMENT_NODE){
				if(n.getLocalName().equals(s)) v.add((Element)n);
			}
		}
		return v;
	}
	
	public static Node searchElement(Element rootelement, String element){
		Node res = searchElementRecursiv(rootelement, element);
		return res;
	}

	public static Node searchElementRecursiv(Node currentnode, String searchid){
		NodeList el = currentnode.getChildNodes();
		for(int i = 0; i < el.getLength(); i++){
			Node n = el.item(i);
			if(n.getNodeType() == Node.ELEMENT_NODE){
				Element ele = (Element) n;
				if(ele.hasAttribute("id")){
					if(ele.getAttribute("id").equals(searchid)) return ele;
				}
			}
		}
		for(int i = 0; i < el.getLength(); i++){
			Node n = el.item(i);
			if(n.getNodeType() == Node.ELEMENT_NODE){
				Node nn = searchElementRecursiv(n, searchid);
				if(nn != null) return nn;
			}
		}
		return null;
	}
}
