/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.tinu.qtafi4.db;

import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.nio.file.FileSystems;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.UUID;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.URIResolver;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.basex.api.dom.BXElem;
import org.basex.api.dom.BXNode;
import org.basex.api.dom.BXText;
import org.basex.core.Context;
import org.basex.core.MainOptions;
import org.basex.core.cmd.Close;
import org.basex.core.cmd.CreateDB;
import org.basex.core.cmd.Export;
import org.basex.core.cmd.XQuery;
import org.basex.io.serial.Serializer;
import org.basex.query.QueryProcessor;
import org.basex.query.iter.Iter;
import org.basex.query.value.item.Item;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import de.tinu.qtafi4.model.Filter;
import de.tinu.qtafi4.model.FilterRoot;
import de.tinu.qtafi4.model.PathInfo;
import de.tinu.qtafi4.model.Question;
import de.tinu.qtafi4.model.QuestionnaireOptions;
import de.tinu.qtafi4.model.Value;
import de.tinu.qtafi4.model.Variable;
import de.tinu.qtafi4.tree.FootNode;
import de.tinu.qtafi4.tree.HeadNode;
import de.tinu.qtafi4.tree.PageNode;
import de.tinu.qtafi4.tree.QuestionNode;
import de.tinu.qtafi4.tree.QuestionnaireNode;
import de.tinu.qtafi4.tree.TreeNode;
/**
 * 
 * 
 * 
 * <CV>
 * 	<text>sdfsfsfsf</text>
 * 	<layout type="" size="" />
 * </CV>
 * 
 * 
 * 
 * @author martin
 *
 */
public class QuestionDBImpl implements QuestionDB {

	//static String DECLARE_CHOP_FALSE = "declare option db:chop 'false'; ";
	
	String qmlfile;
	String project;
	private Document document;
	
	private static DocumentBuilder builder;
	
	static {
		//System.setProperty("org.basex.CHOP", "false");
		try {
			DocumentBuilderFactory builderfactory = DocumentBuilderFactory.newInstance();
			builderfactory.setNamespaceAware(true);
			builderfactory.setSchema(null);
			builderfactory.setValidating(false);
			builder = builderfactory.newDocumentBuilder();
		}
		catch(Exception ex){
		}
	}
	
	public QuestionDBImpl() {
		
	}
	
	@Override
	public void init(String project, String qmlfile) {
		this.qmlfile = qmlfile;
		this.project = project;
		try {
			if(! Files.exists(Paths.get(qmlfile))){
				//Files.createFile(FileSystems.getDefault().getPath(qmlfile) );
				Files.write(FileSystems.getDefault().getPath(qmlfile), "<?xml version=\"1.0\" ?>\n<qtafi xmlns:qtafi=\"urn:qtafi\"/>".getBytes(), StandardOpenOption.CREATE);
			} 
			
			byte[] b = Files.readAllBytes(Paths.get(qmlfile));
			if(b.length == 0){
				Files.write(FileSystems.getDefault().getPath(qmlfile), "<?xml version=\"1.0\" ?>\n<qtafi xmlns:qtafi=\"urn:qtafi\"/>".getBytes(), StandardOpenOption.CREATE);
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		
		this.checkQuestionnaire(qmlfile);
	}
	
	private void checkQuestionnaire(String qmlfile)   {
		Context context = new Context();
		boolean changed = false;
		synchronized(context){
			try {
				new CreateDB("codebook", qmlfile).execute(context);
				String res = new XQuery("exists(/qtafi)").execute(context);
				if(res.equals("false")){
					changed = true;
					new XQuery("insert node <qtafi xmlns:qtafi='urn:qtafi'></qtafi> into /").execute(context);
				}
				
				res = new XQuery("exists(/qtafi/codebook)").execute(context);
				if(res.equals("false")){
					changed = true;
					new XQuery("insert node <codebook></codebook> into /qtafi").execute(context);
				}
				
				res =  new XQuery("exists(/qtafi/codebook/Page)").execute(context);
				if(res.equals("false")){
					changed = true;
					new XQuery("insert node <Page uid='" + UUID.randomUUID().toString() + "'><children/></Page> into /qtafi/codebook").execute(context);
				}
				
				res =  new XQuery("exists(/qtafi/codebook/Head)").execute(context);
				if(res.equals("false")){
					changed = true;
					new XQuery("insert node <Head id='head' uid='" + UUID.randomUUID().toString() + "'><children/></Head> into /qtafi/codebook").execute(context);
				}
				
				res =  new XQuery("exists(/qtafi/codebook/Foot)").execute(context);
				if(res.equals("false")){
					changed = true;
					new XQuery("insert node <Foot id='foot' uid='" + UUID.randomUUID().toString() + "'><children/></Foot> into /qtafi/codebook").execute(context);
				}
				
				if(changed){
					new Export(qmlfile).execute(context);
				}
				new Close().execute(context);
			}
			catch(Exception ex){
				ex.printStackTrace();
			}
			finally {
				context.close();
			}
		}
	}
	
	public TreeNode getTreeNode(TreeNode root, String uid){
		TreeNode res = search(root, uid);
		return res;
	}
	
	private TreeNode search(TreeNode root, String uid){
		if(root.uid.equals(uid)) return root;
		for(TreeNode n: root.children){
			TreeNode x = search(n, uid);
			if(x != null) return x;
		}
		return null;
	}
	
	public TreeNode questionnaireInfo(NodeFilter nodefilter) {
		QuestionnaireNode questionnaire = new QuestionnaireNode();
		Context context = new Context();
		synchronized(context){
			try {
				long start = System.currentTimeMillis();
				new CreateDB("codebook", qmlfile).execute(context);
				//QueryProcessor proc = new QueryProcessor("doc('file:/"+qmlfile+"')/qtafi/codebook/Page", context);
				QueryProcessor proc = new QueryProcessor("/qtafi/codebook/Page", context);
				Iter iter = proc.iter();
				int page = 1;
				for(Item item; (item = iter.next()) != null; ) {
					BXElem ob = (BXElem) item.toJava();
					PageNode pn = new PageNode(ob.getAttribute("uid"));
					pn.type = ob.getNodeName();
					if(nodefilter.accept(pn)){
						pn.page = page;
						pn.id = "p" + (page -1);
						pn.label = ("Page " + page);
						//QueryProcessor qproc = new QueryProcessor("doc('file:/"+qmlfile+"')/qtafi/codebook/Page["+page+"]/children/*", context);
						QueryProcessor qproc = new QueryProcessor("/qtafi/codebook/Page["+page+"]/children/*", context);
						Iter qiter = qproc.iter();
						int qi = 1;
						for(Item qitem; (qitem = qiter.next()) != null;) {
							BXElem qob = (BXElem) qitem.toJava();
							String path = "/qtafi/codebook/Page["+page+"]/children/*["+qi+"]";
							QuestionNode qn = new QuestionNode(qob.getAttribute("uid"));
							qn.type = qob.getNodeName();
							if(nodefilter.accept(qn)){
								qn.page = page;
								qn.id = (qob.getAttribute("id"));
								//qn.number = new XQuery("doc('file:/"+qmlfile+"')"+path + "/number/node()").execute(context);
								qn.number = new XQuery(path + "/number/node()").execute(context);
								//qn.label = (qob.getNodeName() + " - " + qn.number + " (" + qob.getAttribute("id") + ")");
								qn.label = label(qob);
								pn.addNode(qn);
								qi++;
							}
						}
						
						questionnaire.addPage(pn);
						page++;
					}
				}
				
				//proc = new QueryProcessor("doc('file:/"+qmlfile+"')/qtafi/codebook/Head", context);
				proc = new QueryProcessor("/qtafi/codebook/Head", context);
				iter = proc.iter();
				for(Item item; (item = iter.next()) != null;) {
					BXElem ob = (BXElem) item.toJava();
					HeadNode pn = new HeadNode(ob.getAttribute("uid"));
					pn.type = ob.getNodeName();
					if(nodefilter.accept(pn)){
						pn.id = "head";
						//pn.label = "Head";
						pn.label = label(ob);
						QueryProcessor qproc = new QueryProcessor("/qtafi/codebook/Head/children/*", context);
						Iter qiter = qproc.iter();
						int qi = 1;
						for(Item qitem; (qitem = qiter.next()) != null;) {
							BXElem qob = (BXElem) qitem.toJava();
							String path = "/qtafi/codebook/Head/children/*["+qi+"]";
							QuestionNode qn = new QuestionNode(qob.getAttribute("uid"));
							qn.type = qob.getNodeName();
							if(nodefilter.accept(qn)){
								qn.id = (qob.getAttribute("id"));
								//qn.label = qob.getNodeName();
								qn.label = label(qob);
								pn.addNode(qn);
								qi++;
							}
						}
						questionnaire.addPage(pn);
						page++;
					}
				}
				
				//proc = new QueryProcessor("doc('file:/"+qmlfile+"')/qtafi/codebook/Foot", context);
				proc = new QueryProcessor("/qtafi/codebook/Foot", context);
				iter = proc.iter();
				for(Item item; (item = iter.next()) != null;) {
					BXElem ob = (BXElem) item.toJava();
					FootNode pn = new FootNode(ob.getAttribute("uid"));
					pn.type = ob.getNodeName();
					if(nodefilter.accept(pn)){
						pn.id = "foot";
						//pn.label = "Foot";
						pn.label = label(ob);
						QueryProcessor qproc = new QueryProcessor("/qtafi/codebook/Foot/children/*", context);
						Iter qiter = qproc.iter();
						int qi = 1;
						for(Item qitem; (qitem = qiter.next()) != null;) {
							BXElem qob = (BXElem) qitem.toJava();
							String path = "/qtafi/codebook/Foot/children/*["+qi+"]";
							QuestionNode qn = new QuestionNode(qob.getAttribute("uid"));
							qn.type = qob.getNodeName();
							if(nodefilter.accept(qn)){
								qn.id = (qob.getAttribute("id"));
								//qn.label = qob.getNodeName();
								qn.label = label(qob);
								pn.addNode(qn);
								qi++;
							}
						}
						questionnaire.addPage(pn);
						page++;
					}
				}
				new Close().execute(context);
				//System.out.println("overall dur: " + (System.currentTimeMillis()- start));
			}
			catch(Exception ex){
				ex.printStackTrace();
			}
			finally {
				context.close();
			}
		}
		
		return questionnaire;
	}
	
	
	public static String label(Element el){
		switch(el.getNodeName()){
		case "Page": return "Page";
		case "Head": return "Head";
		case "Foot": return "Foot";
		case "CV": return "SC - " + el.getAttribute("id");
		case "MD": return "MC - " + el.getAttribute("id");
		case "OR": return "OR - " + el.getAttribute("id");
		}
		return el.getNodeName() + " - " + el.getAttribute("id");
	}
	
	
	@Override
	public TreeNode questionnaireInfo() {
		Context context = new Context();
		QuestionnaireNode questionnaire = new QuestionnaireNode();
		synchronized(context){
			try {
				new CreateDB("codebook", qmlfile).execute(context);
				long start = System.currentTimeMillis();
				//QueryProcessor proc = new QueryProcessor("doc('file:/"+qmlfile+"')/qtafi/codebook/Page", context);
				QueryProcessor proc = new QueryProcessor("/qtafi/codebook/Page", context);
				Iter iter = proc.iter();
				int page = 1;
				for(Item item; (item = iter.next()) != null;) {
					BXElem ob = (BXElem) item.toJava();
					PageNode pn = new PageNode(ob.getAttribute("uid"));
					pn.page = page;
					pn.type = ob.getNodeName();
					pn.id = "p" + (page -1);
					pn.label = ("Page " + page);
					//QueryProcessor qproc = new QueryProcessor("doc('file:/"+qmlfile+"')/qtafi/codebook/Page["+page+"]/children/*", context);
					QueryProcessor qproc = new QueryProcessor("/qtafi/codebook/Page["+page+"]/children/*", context);
					Iter qiter = qproc.iter();
					int qi = 1;
					for(Item qitem; (qitem = qiter.next()) != null;) {
						BXElem qob = (BXElem) qitem.toJava();
						String path = "/qtafi/codebook/Page["+page+"]/children/*["+qi+"]";
						QuestionNode qn = new QuestionNode(qob.getAttribute("uid"));
						qn.page = page;
						qn.type = qob.getNodeName();
						qn.id = (qob.getAttribute("id"));
						//qn.number = new XQuery("doc('file:/"+qmlfile+"')"+path + "/number/node()").execute(context);
						qn.number = new XQuery(path + "/number/node()").execute(context);
						//qn.label = (qob.getNodeName() + " - " + qn.number + " (" + qob.getAttribute("id") + ")");
						qn.label = label(qob);
						pn.addNode(qn);
						qi++;
					}
					
					questionnaire.addPage(pn);
					page++;
				}
				
				//proc = new QueryProcessor("doc('file:/"+qmlfile+"')/qtafi/codebook/Head", context);
				proc = new QueryProcessor("/qtafi/codebook/Head", context);
				iter = proc.iter();
				for(Item item; (item = iter.next()) != null;) {
					BXElem ob = (BXElem) item.toJava();
					HeadNode pn = new HeadNode(ob.getAttribute("uid"));
					//pn.page = page;
					pn.type = ob.getNodeName();
					pn.id = "head";
					//pn.label = "Head";
					pn.label = label(ob);
					QueryProcessor qproc = new QueryProcessor("/qtafi/codebook/Head/children/*", context);
					Iter qiter = qproc.iter();
					int qi = 1;
					for(Item qitem; (qitem = qiter.next()) != null;) {
						BXElem qob = (BXElem) qitem.toJava();
						String path = "/qtafi/codebook/Head/children/*["+qi+"]";
						QuestionNode qn = new QuestionNode(qob.getAttribute("uid"));
						//qn.page = page;
						qn.type = qob.getNodeName();
						qn.id = (qob.getAttribute("id"));
						//qn.label = qob.getNodeName();
						qn.label = label(qob);
						pn.addNode(qn);
						qi++;
					}
					questionnaire.addPage(pn);
					page++;
				}
				
				//proc = new QueryProcessor("doc('file:/"+qmlfile+"')/qtafi/codebook/Foot", context);
				proc = new QueryProcessor("/qtafi/codebook/Foot", context);
				iter = proc.iter();
				for(Item item; (item = iter.next()) != null;) {
					BXElem ob = (BXElem) item.toJava();
					HeadNode pn = new HeadNode(ob.getAttribute("uid"));
					//pn.page = page;
					pn.type = ob.getNodeName();
					pn.id = "foot";
					//pn.label = "Foot";
					pn.label = label(ob);
					//QueryProcessor qproc = new QueryProcessor("doc('file:/"+qmlfile+"')/qtafi/codebook/Foot/children/*", context);
					QueryProcessor qproc = new QueryProcessor("/qtafi/codebook/Foot/children/*", context);
					Iter qiter = qproc.iter();
					int qi = 1;
					for(Item qitem; (qitem = qiter.next()) != null;) {
						BXElem qob = (BXElem) qitem.toJava();
						String path = "/qtafi/codebook/Foot/children/*["+qi+"]";
						QuestionNode qn = new QuestionNode(qob.getAttribute("uid"));
						//qn.page = page;
						qn.type = qob.getNodeName();
						qn.id = (qob.getAttribute("id"));
						//qn.label = qob.getNodeName();
						qn.label = label(qob);
						pn.addNode(qn);
						qi++;
					}
					questionnaire.addPage(pn);
					page++;
				}
				new Close().execute(context);
				//System.out.println("overall dur: " + (System.currentTimeMillis() - start));
			}
			catch(Exception ex){
				ex.printStackTrace();
			}
			finally {
				context.close();
			}
		}
		return questionnaire;
	}
	
	@Override
	public QuestionnaireOptions getOptions() {
		Context context = new Context();
		QuestionnaireOptions options = new QuestionnaireOptions();
		synchronized(context){
			try {
				options.pagedir = new XQuery("data( doc('file:/"+qmlfile+"')/qtafi/codebook/@pagedir)").execute(context);
				options.formatter = new XQuery("data( doc('file:/"+qmlfile+"')/qtafi/codebook/@formatter)").execute(context);
			}
			catch(Exception ex){
				ex.printStackTrace();
			}
			finally {
				context.close();
			}
		}
		return options;
	}
	
	@Override
	public void setOptions(QuestionnaireOptions opt) {
		Context context = new Context();
		synchronized(context){
			try {
				new CreateDB("codebook", qmlfile).execute(context);
				//pagedir
				String query = "";
				String res =  new XQuery("exists(/qtafi/codebook/@pagedir)").execute(context);
				if(res.equals("false")) {
					query = "insert node (attribute {'pagedir'} {'"+ opt.pagedir + "'}) into /qtafi/codebook" ;
				}
				else {
					query = "replace node  /qtafi/codebook/@pagedir  with (attribute {'pagedir'} {'" + opt.pagedir + "'})";
				}
				new XQuery(query).execute(context);
				
				res =  new XQuery("exists(/qtafi/codebook/@formatter)").execute(context);
				if(res.equals("false")) {
					query = "insert node (attribute {'formatter'} {'"+ opt.formatter + "'}) into /qtafi/codebook" ;
				}
				else {
					query = "replace node  /qtafi/codebook/@formatter  with (attribute {'formatter'} {'" + opt.formatter + "'})";
				}
				new XQuery(query).execute(context);
				new Export(qmlfile).execute(context);
				new Close().execute(context);
			}
			catch(Exception ex){
				ex.printStackTrace();
			}
			finally {
				context.close();
			}
		}
	}
	
	@Override
	public boolean update(Question question) {
		Context context = new Context();
		context.options.set(MainOptions.CHOP, false);
		synchronized (context) {
			try{
				//new Set("CHOP", false).execute(context);
				new CreateDB("codebook", qmlfile).execute(context);
				String query = "";
				String path = "//*[@uid='" + question.uid + "']";
				
				if(question.type.equals("Page")){
					//id
					String res =  new XQuery("exists("+ path + "/@id)").execute(context);
					if(res.equals("false")) {
						query = "insert node (attribute {'id'} {'"+ question.id + "'}) into "+ path;
					}
					else {
						query = "replace node  "+ path + "/@id  with (attribute {'id'} {'" + question.id + "'})";
					}
					new XQuery(query).execute(context);
					
					//protected
					if(question.isQuestionProtected()){
						res =  new XQuery("exists("+ path + "/@protectedby)").execute(context);
						if(res.equals("false")) {
							query = "insert node (attribute {'protectedby'} {'"+ question.getProtectedBy() + "'}) into "+ path;
						}
						else {
							query = "replace node  "+ path + "/@protectedby  with (attribute {'protectedby'} {'" +  question.getProtectedBy() + "'})";
						}
						new XQuery(query).execute(context);
					}
					else {
						new XQuery("delete node " + path + "/@protectedby").execute(context);
					}
					
					//filter
					if(question.filter.filters.size() > 0){
						res =  new XQuery("exists(" + path + "/filter)").execute(context);
						if(res.equals("false")){
							query = "insert node <filter>" + question.filter.toXML() + "</filter> into " + path;
						}
						else {
							query = "replace node " + path + "/filter with <filter>" + question.filter.toXML() + "</filter>";
						}
						new XQuery(query).execute(context);
					}
					else {
						new XQuery("delete node " + path + "/filter").execute(context);
					}
					
					//subject
					if(question.subject != null && question.subject.length() > 0){
						res = new XQuery("exists("+ path + "/subject)").execute(context);
						if(res.equals("false")) query = "insert node <subject>" + question.subject + "</subject> into " + path;
						else query = "replace node " + path + "/subject with <subject>" + question.subject + "</subject>";
						new XQuery(query).execute(context);
					}
					else {
						new XQuery("delete node " + path + "/subject").execute(context);
					}
				}
				else {
					/*
					//type
					String oldtype = new XQuery(path + "/name()").execute(context);
					if(! oldtype.equals(question.getType())){
						String res =  new XQuery("rename node " + path + " as '" + question.type +"'").execute(context);
					}
					*/
					
					//id
					String res =  new XQuery("exists("+ path + "/@id)").execute(context);
					if(res.equals("false")) {
						query = "insert node (attribute {'id'} {'"+ question.id + "'}) into "+ path;
					}
					else {
						query = "replace node  "+ path + "/@id  with (attribute {'id'} {'" + question.id + "'})";
					}
					new XQuery(query).execute(context);
					
					//protected
					if(question.isQuestionProtected()){
						res =  new XQuery("exists("+ path + "/@protectedby)").execute(context);
						if(res.equals("false")) {
							query = "insert node (attribute {'protectedby'} {'"+ question.getProtectedBy() + "'}) into "+ path;
						}
						else {
							query = "replace node  "+ path + "/@protectedby  with (attribute {'protectedby'} {'" +  question.getProtectedBy() + "'})";
						}
						new XQuery(query).execute(context);
					}
					else {
						new XQuery("delete node " + path + "/@protectedby").execute(context);
					}
					
					//subject
					if(question.subject != null && question.subject.length() > 0){
						res = new XQuery("exists("+ path + "/subject)").execute(context);
						if(res.equals("false")) query = "insert node <subject>" + question.subject + "</subject> into " + path;
						else query = "replace node " + path + "/subject with <subject>" + question.subject + "</subject>";
						new XQuery(query).execute(context);
					}
					else {
						new XQuery("delete node " + path + "/subject").execute(context);
					}
					
					//text
					if(question.text.length() > 0){
						System.out.println(question.text);
						res =  new XQuery("exists(" + path + "/text)").execute(context);
						if(res.equals("false")) query = "insert node <text xmlns:qtafi='urn:qtafi'>" + question.text + "</text> into " + path;
						else query = "replace node " + path + "/text with <text xmlns:qtafi='urn:qtafi'>" + question.text + "</text>";
						new XQuery(query).execute(context);
					}
					else {
						new XQuery("delete node " + path + "/text").execute(context);
					}
					
					//number
					if(question.number.length() > 0){
						res =  new XQuery("exists(" + path + "/number)").execute(context);
						if(res.equals("false")){
							query = "insert node <number xmlns:qtafi='urn:qtafi'>" + question.number + "</number> into " + path;
						}
						else {
							query = "replace node " + path + "/number with <number xmlns:qtafi='urn:qtafi'>" + question.number + "</number>";
						}
						new XQuery(query).execute(context);
					}
					else {
						new XQuery("delete node " + path + "/number").execute(context);
					}
					
					
					//moretext
					if(question.moretext.length() > 0){
						res =  new XQuery("exists(" + path + "/moretext)").execute(context);
						if(res.equals("false")){
							query = "insert node <moretext xmlns:qtafi='urn:qtafi'>" + question.moretext + "</moretext> into " + path;
						}
						else {
							query = "replace node " + path + "/moretext with <moretext xmlns:qtafi='urn:qtafi'>" + question.moretext + "</moretext>";
						}
						new XQuery(query).execute(context);
					}
					else {
						new XQuery("delete node " + path + "/moretext").execute(context);
					}
					
					//code
					if(question.code.length() > 0){
						res =  new XQuery("exists(" + path + "/code)").execute(context);
						if(res.equals("false")){
							query = "insert node <code xmlns:qtafi='urn:qtafi'>" + question.code.trim() + "</code> into " + path;
						}
						else {
							query = "replace node " + path + "/code with <code xmlns:qtafi='urn:qtafi'>" + question.code.trim() + "</code>";
						}
						new XQuery(query).execute(context);
					}
					else {
						new XQuery("delete node " + path + "/code").execute(context);
					}
					
					
					//layout
					if(question.layout.attributes.size() > 0){
						res =  new XQuery("exists(" + path + "/layout)").execute(context);
						if(res.equals("false")){
							query = "insert node <layout></layout> into " + path;
							new XQuery(query).execute(context);
						}
						for(String key: question.layout.attributes.keySet()){
							String ex =  new XQuery("exists("+ path + "/layout/@" +key+ ")").execute(context);
							if(ex.equals("false")) {
								query = "insert node (attribute {'"+key+"'} {'"+ question.layout.attributes.get(key) + "'}) into "+ path + "/layout";
							}
							else {
								query = "replace node  "+ path + "/layout/@"+key+"  with (attribute {'"+key+"'} {'" + question.layout.attributes.get(key)  + "'})";
							}
							//System.out.println("query " + query);
							new XQuery(query).execute(context);
						}
					}
					else {
						new XQuery("delete node " + path + "/layout").execute(context);
					}
					
					//filter
					if(question.filter.filters.size() > 0){
						res =  new XQuery("exists(" + path + "/filter)").execute(context);
						if(res.equals("false")){
							query = "insert node <filter>" + question.filter.toXML() + "</filter> into " + path;
						}
						else {
							query = "replace node " + path + "/filter with <filter>" + question.filter.toXML() + "</filter>";
						}
						new XQuery(query).execute(context);
					}
					else {
						new XQuery("delete node " + path + "/filter").execute(context);
					}
					
					
					//values
					if(question.getValues().size() > 0){
						res =  new XQuery("exists(" + path + "/value)").execute(context);
						if(res.equals("false")){
						}
						else {//delete values
							new XQuery("delete node " + path + "/value").execute(context);
						}
						/*
						//set key automatic
						int key = 1;
						for(Iterator<Value> it = question.getValues().iterator(); it.hasNext(); ){
							Value v = it.next();
							new XQuery("insert node <value xmlns:qtafi='urn:qtafi' key='"+key+"'>" + v.getLabel() + "</value> into " + path).execute(context);
							key++;
						}
						*/
						for(Iterator<Value> it = question.getValues().iterator(); it.hasNext(); ){
							Value v = it.next();
							new XQuery("insert node <value key='"+v.key.trim()+"'>" + v.label + "</value> into " + path).execute(context);
						}
					}
					
					//Variables
					if(question.type.equals("CV")){
						if(question.getVariables().size() == 0){
							question.getVariables().add(new Variable(question.id, ""));
						}
					}
					if(question.getVariables().size() > 0){
						res =  new XQuery("exists(" + path + "/variable)").execute(context);
						if(res.equals("false")){
						}
						else {//delete variables
							new XQuery("delete node " + path + "/variable").execute(context);
						}
						/*
						//set name automatic
						int counter = 1;
						String name = "var";
						for(Iterator<Variable> it = question.getVariables().iterator(); it.hasNext(); ){
							Variable v = it.next();
							if(question.getVariables().size() == 1) name = question.getId();
							else name = question.getId() + "_" + counter;
							new XQuery("insert node <variable name='"+name+"'>" + v.getLabel() + "</variable> into " + path).execute(context);
							counter++;
						}
						*/
						for(Iterator<Variable> it = question.getVariables().iterator(); it.hasNext(); ){
							Variable v = it.next();
							new XQuery("insert node <variable name='"+ v.name +"'>" + v.label + "</variable> into " + path).execute(context);
						}
					}
					//Questionnaire.checkQuestionnaire(context, qmlfile);
				}
				new Export(qmlfile).execute(context);
				new Close().execute(context);
			} 
			catch (Exception e) {
				e.printStackTrace();
			}
			finally {
				context.close();
			}
		}
		return true;
	}
	
	private static synchronized String serialize(QueryProcessor proc) throws Exception {
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		Serializer ser = proc.getSerializer(bos);
		Iter iter = proc.iter();
		for(Item item; (item = iter.next()) != null;) {
			bos.write(' ');
			ser.serialize(item);
		}
		return new String(bos.toByteArray());
	}
	
	private static synchronized String serialize(BXNode element) throws TransformerException{
		TransformerFactory transFactory = TransformerFactory.newInstance();
		Transformer transformer = transFactory.newTransformer();
		StringWriter buffer = new StringWriter();
		transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
		transformer.transform(new DOMSource(element), new StreamResult(buffer));
		return buffer.toString();
	}
	
	private static synchronized String serializeChildren(org.w3c.dom.Node element) throws TransformerException{
		TransformerFactory transFactory = TransformerFactory.newInstance();
		Transformer transformer = transFactory.newTransformer();
		StringWriter buffer = new StringWriter();
		transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
		NodeList list = ((org.w3c.dom.Node) element).getChildNodes();
		for(int i = 0; i < list.getLength(); i++){
			transformer.transform(new DOMSource(list.item(i)), new StreamResult(buffer));
			if(i < list.getLength()-1) buffer.append(" ");
		}
		return buffer.toString();
	}
	
	
	public static String serialize(Question question){
		Context context = new Context();
		String qml = "";
		synchronized (context) {
			try{
				new CreateDB("codebook", "<" + question.type + "></" + question.type + ">").execute(context);
				String query = "";
				String path = "/" + question.type;
				
				//id
				query = "insert node (attribute {'id'} {'"+ question.id + "'}) into "+ path;
				new XQuery(query).execute(context);
				
				//subject
				if(question.subject != null && question.subject.length() > 0){
					query = "insert node <subject>" + question.subject + "</subject> into " + path;
					new XQuery(query).execute(context);
				}
		
				//text
				if(question.text.length() > 0){
					query = "insert node <text>" + question.text + "</text> into " + path;
					new XQuery(query).execute(context);
				}
				
				//number
				if(question.number.length() > 0){
					query = "insert node <number>" + question.number + "</number> into " + path;
					new XQuery(query).execute(context);
				}
				
				//moretext
				if(question.moretext.length() > 0){
					query = "insert node <moretext>" + question.moretext + "</moretext> into " + path;
					new XQuery(query).execute(context);
				}
				
				//code
				if(question.code.length() > 0){
					query = "insert node <code>" + question.code + "</code> into " + path;
					new XQuery(query).execute(context);
				}
				
				
				//layout
				if(question.layout.attributes.size() > 0){
					query = "insert node <layout></layout> into " + path;
					new XQuery(query).execute(context);
					for(String key: question.layout.attributes.keySet()){
						query = "insert node (attribute {'"+key+"'} {'"+ question.layout.attributes.get(key) + "'}) into "+ path + "/layout";
						new XQuery(query).execute(context);
					}
				}
				
				//filter
				if(question.filter.filters.size() > 0){
					query = "insert node <filter>" + question.filter.toXML() + "</filter> into " + path;
					new XQuery(query).execute(context);
				}
				
				//values
				if(question.getValues().size() > 0){
					for(Iterator<Value> it = question.getValues().iterator(); it.hasNext(); ){
						Value v = it.next();
						new XQuery("insert node <value key='"+v.key.trim()+"'>" + v.label + "</value> into " + path).execute(context);
					}
				}
				
				//Variables
				if(question.type.equals("CV")){
					if(question.getVariables().size() == 0){
						question.getVariables().add(new Variable(question.id, ""));
					}
				}
				if(question.getVariables().size() > 0){
					for(Iterator<Variable> it = question.getVariables().iterator(); it.hasNext(); ){
						Variable v = it.next();
						new XQuery("insert node <variable name='"+ v.name +"'>" + v.label + "</variable> into " + path).execute(context);
					}
				}
				
				qml = new XQuery("/").execute(context);
				new Close().execute(context);
			} 
			catch (Exception e) {
				e.printStackTrace();
			}
			finally {
				context.close();
			}
		}
		return qml;
	}
	
	public static Question deserialize(String qml, String type){
		//System.out.println("des " + qml);
		Context context = new Context();
		Question q = null;
		long session = System.currentTimeMillis();
		synchronized (context) {
			try {
				new CreateDB("codebook_" + session, "<?xml version=\"1.0\" ?>\n" + qml).execute(context);
				
				//String type = new XQuery("/name()").execute(context);
				String path = "/" + type;
				q = new Question(UUID.randomUUID().toString(), type);
				
				String id = new XQuery("data("+ path + "/@id)").execute(context);
				q.id = id;
				
				String sub = new XQuery(path + "/subject/node()").execute(context);
				q.subject = sub;
				
				String te = new XQuery(path + "/text/node()").execute(context);
				q.text = te;
				
				String num = new XQuery(path + "/number/node()").execute(context);
				q.number = num;
			
				String mt = new XQuery(path + "/moretext/node()").execute(context);
				q.moretext = mt;
			
				String code = new XQuery(path + "/code/node()").execute(context);
				q.code = code;
				
				/*
				proc = new QueryProcessor(path + "/filter/*", context);
				Iter iter = proc.iter();
				for(Item item; (item = iter.next()) != null; ){
					if(item.toJava() instanceof BXElem) {
						BXElem elem = (BXElem) item.toJava();
						addFilter(q.filter, elem);
					}
				}
				*/
				QueryProcessor layoutq = new QueryProcessor(path + "/layout/@*", context);
				Iter liter = layoutq.iter();
				for(Item item; (item = liter.next()) != null; ){
					if(item.toJava() instanceof BXNode) {
						BXNode bx = (BXNode) item.toJava();
						q.layout.attributes.put(bx.getLocalName(), bx.getNodeValue());
					}
				}
				
				QueryProcessor proc = new QueryProcessor(path + "/value", context);
				Iter iter = proc.iter();
				for(Item item; (item = iter.next()) != null; ){
					if(item.toJava() instanceof BXText){
					}
					else if(item.toJava() instanceof BXElem) {
						BXElem elem = (BXElem) item.toJava();
						String valname = serializeChildren(elem);
						q.addValue(new Value(elem.getAttribute("key"), valname));
					}
				}
				
				if(type.equals("MD") && q.values.size() == 0){
					q.addValue(new Value("1", "selected"));
				}
				
				proc = new QueryProcessor(path + "/variable", context);
				iter = proc.iter();
				for(Item item; (item = iter.next()) != null;) {
					if(item.toJava() instanceof BXElem) {
						BXElem elem = (BXElem) item.toJava();
						String val = serializeChildren(elem);
						String name = elem.getAttribute("name");
						if(name == null) name = "x";
						Variable v = new Variable(name, val);
						q.addVariable(v);
					}
				}
				new Close().execute(context);
			} 
			catch (Exception e) {
				e.printStackTrace();
			}
			finally {
				context.close();
			}
		}
		return q;
	}
	
	@Override
	public Question get(PathInfo info) {
		Context context = new Context();
		context.options.set(MainOptions.CHOP, false);
		Question q = null;
		long session = System.currentTimeMillis();
		synchronized (context) {
			try {
				new CreateDB("codebook_" + session, qmlfile).execute(context);
				//new Run("declare option db:chop 'false';").execute(context);
				
				String uid = new XQuery("data("+info.path + "/@uid)").execute(context);
				if(uid.length() == 0) return null;
				
				String type = new XQuery(info.path + "/name()").execute(context);
				
				q = new Question(uid, type);
				
				String id = new XQuery("data("+info.path + "/@id)").execute(context);
				q.id = id;
				
				String prtected = new XQuery("data("+ info.path + "/@protectedby)").execute(context);
				int userid = -1;
				try{
					userid = Integer.parseInt(prtected);
					if(userid >= 0) q.protect(userid);	
				}
				catch(NumberFormatException e){ }
				
				String sub = new XQuery(info.path + "/subject/node()").execute(context);
				q.subject = sub;
				
				String te = new XQuery(info.path + "/text/node()").execute(context);
				q.text = te.replaceAll("xmlns:qtafi=\"urn:qtafi\"", "");
				
				String num = new XQuery(info.path + "/number/node()").execute(context);
				q.number = num;
				
				String mt = new XQuery(info.path + "/moretext/node()").execute(context);
				q.moretext = mt.replaceAll("xmlns:qtafi=\"urn:qtafi\"", "");
				
				String code = new XQuery(info.path + "/code/node()").execute(context);
				q.code = code;
				
				QueryProcessor proc = new QueryProcessor(info.path + "/filter/*", context);
				Iter iter = proc.iter();
				for(Item item; (item = iter.next()) != null; ){
					if(item.toJava() instanceof BXElem) {
						BXElem elem = (BXElem) item.toJava();
						addFilter(q.filter, elem);
					}
				}
				
				QueryProcessor layoutq = new QueryProcessor(info.path + "/layout/@*", context);
				Iter liter = layoutq.iter();
				for(Item item; (item = liter.next()) != null; ){
					if(item.toJava() instanceof BXText){
						System.out.println("BXTEXT");
					}
					else if(item.toJava() instanceof BXNode) {
						BXNode bx = (BXNode) item.toJava();
						q.layout.attributes.put(bx.getLocalName(), bx.getNodeValue());
					}
				}
				
				proc = new QueryProcessor(info.path + "/value", context);
				iter = proc.iter();
				
				for(Item item; (item = iter.next()) != null; ){
					if(item.toJava() instanceof BXText){
					}
					else if(item.toJava() instanceof BXElem) {
						BXElem elem = (BXElem) item.toJava();
						String valname = serializeChildren(elem);
						q.addValue(new Value(elem.getAttribute("key"), valname));
					}
				}
				
				if(type.equals("MD") && q.values.size() == 0){
					q.addValue(new Value("1", "selected"));
				}
				
				proc = new QueryProcessor(info.path + "/variable", context);
				iter = proc.iter();
				for(Item item; (item = iter.next()) != null;) {
					if(item.toJava() instanceof BXElem) {
						BXElem elem = (BXElem) item.toJava();
						String val = serializeChildren(elem);
						String name = elem.getAttribute("name");
						if(name == null) name = "x";
						Variable v = new Variable(name, val);
						q.addVariable(v);
					}
				}
				new Close().execute(context);
			} 
			catch (Exception e) {
				e.printStackTrace();
			}
			finally {
				context.close();
			}
		}
		return q;
	}
	
	public List<Variable> getAllVariables(){
		Context context = new Context();
		List<Variable> variables = new Vector<Variable>();
		long session = System.currentTimeMillis();
		synchronized (context) {
			try {
				new CreateDB("codebook_" + session, qmlfile).execute(context);
				QueryProcessor proc = new QueryProcessor("declare namespace qtafi=\"urn:qtafi\"; //variable | //qtafi:TE | //qtafi:ME | //qtafi:CV | //qtafi:OR | //qtafi:MD", context);
				Iter iter = proc.iter();
				for(Item item; (item = iter.next()) != null;) {
					if(item.toJava() instanceof BXElem) {
						BXElem elem = (BXElem) item.toJava();
						if(elem.getNodeName().startsWith("qtafi:")){
							String name = elem.getAttribute("variable");
							Variable v = new Variable(name, "Inline field");
							variables.add(v);
						}
						else if(elem.getNodeName().equals("variable")){
							String val = serializeChildren(elem);
							String name = elem.getAttribute("name");
							if(name == null) name = "x";
							Variable v = new Variable(name, val);
							variables.add(v);
						}
					}
				}
			}
			catch (Exception e) {
				e.printStackTrace();
			}
			finally {
				context.close();
			}
		}
		return variables;
	}
	
	
	static void addRecursiv(Filter parent, Element element){
		if(element.getNodeName().equals("show_group")){
			Filter group = group(element);
			NodeList nl = element.getChildNodes();
			if(nl != null){
				for(int i = 0; i < nl.getLength(); i++){
					Node n = nl.item(i);
					if(n.getNodeType() == Node.ELEMENT_NODE) addRecursiv(group, (Element)n);
				}
			}
			
			parent.add(group);
		}
		else if(element.getNodeName().equals("hide_group")){
			Filter group = group(element);
			NodeList nl = element.getChildNodes();
			if(nl != null){
				for(int i = 0; i < nl.getLength(); i++){
					Node n = nl.item(i);
					if(n.getNodeType() == Node.ELEMENT_NODE) addRecursiv(group, (Element)n);
				}
			}
			parent.add(group);
		}
		else {
			Filter f = create(element);
			parent.add(f);
		}
	}
	
	static Filter create(Element element){
		//System.out.println("create " + element);
		String var = element.getAttribute("variable");
		String val = element.getAttribute("value");
		String op = element.getAttribute("operator");
		String ref = element.getAttribute("ref");
		String label = element.getAttribute("label");
		
		Filter ff = new Filter(var, val);
		//if(label != null) ff.label = label;
		
		if(ref != null) ff.ref = ref;
		if(op == null) ff.operator = "IS";
		else if(op.equals("")) ff.operator = "IS";
		else ff.operator = op;
		ff.label = var + " " + ff.operator.toLowerCase() + " " + val;
		return ff;
	}
	
	static Filter group(Element element){
		Filter group = null;
		String cond = element.getAttribute("condition");
		if(cond.equalsIgnoreCase("or")) group = new Filter(Filter.TYPE_GROUP_OR);
		else group = new Filter(Filter.TYPE_GROUP_AND);
		return group;
	}
	
	static void addFilter(FilterRoot root, Element element){
		if(element.getNodeName().equals("show_group")){
			Filter group = group(element);
			NodeList nl = element.getChildNodes();
			if(nl != null){
				for(int i = 0; i < nl.getLength(); i++){
					Node n = nl.item(i);
					if(n.getNodeType() == Node.ELEMENT_NODE) addRecursiv(group, (Element)n);
				}
			}
			root.add(group);
		}
		else if(element.getNodeName().equals("hide_group")){
			Filter group = group(element);
			NodeList nl = element.getChildNodes();
			if(nl != null){
				for(int i = 0; i < nl.getLength(); i++){
					Node n = nl.item(i);
					if(n.getNodeType() == Node.ELEMENT_NODE) addRecursiv(group, (Element)n);
				}
			}
			root.add(group);
		}
		else {
			Filter ff = create(element);
			root.add(ff);
		} 
	}
	
	
	@Override
	public Question appendPage() {
		Context context = new Context();
		Question q = null;
		synchronized(context){
			try {
				String uid = UUID.randomUUID().toString();
				q = new Question(uid, "Page");
				new CreateDB("codebook", qmlfile).execute(context);
				new XQuery("insert node <Page uid='" + uid + "'><children/></Page> into /qtafi/codebook").execute(context);
				new Export(qmlfile).execute(context);
				new Close().execute(context);
			}
			catch(Exception ex){
				ex.printStackTrace();
			}
			finally {
				context.close();
			}
		}
		return q;
	}
	
	@Override
	public boolean deletePage(int page) {
		Context context = new Context();
		synchronized(context){
			try {
				new CreateDB("codebook", qmlfile).execute(context);
				String query = "delete node /qtafi/codebook/Page[" + page + "]";
				new XQuery(query).execute(context);
				new Export(qmlfile).execute(context);
				new Close().execute(context);
			}
			catch(Exception e){
				e.printStackTrace();
				return false;
			}
			finally {
				context.close();
			}
		}
		return true;
	}
	
	@Deprecated
	public Question append(PathInfo pos) {
		Context context = new Context();
		Question q = null;
		synchronized (context) {
			try{
				new CreateDB("codebook", qmlfile).execute(context);
				String uid = UUID.randomUUID().toString();
				q = new Question(uid, "NEW");
				String query = "insert node <NEW uid='" + uid +"' />  into " + pos.path + "/children";
				new XQuery(query).execute(context);
				new Export(qmlfile).execute(context);
				new Close().execute(context);
			} 
			catch (Exception e) {
				e.printStackTrace();
			}
			finally {
				context.close();
			}
		}
		return q;
	}
	
	@Override
	public Question append(PathInfo pos, String type) {
		Context context = new Context();
		Question q = null;
		synchronized (context) {
			try{
				new CreateDB("codebook", qmlfile).execute(context);
				String uid = UUID.randomUUID().toString();
				q = new Question(uid, type);
				String query = "insert node <"+type+" uid='" + uid +"' />  into " + pos.path + "/children";
				System.out.println("xquery " + query);
				new XQuery(query).execute(context);
				new Export(qmlfile).execute(context);
				new Close().execute(context);
			} 
			catch (Exception e) {
				e.printStackTrace();
			}
			finally {
				context.close();
			}
		}
		return q;
	}
	
	
	@Override
	public Question prepend(PathInfo pos, String type) {
		Context context = new Context();
		Question q = null;
		synchronized (context) {
			try{
				new CreateDB("codebook", qmlfile).execute(context);
				String uid = UUID.randomUUID().toString();
				q = new Question(uid, type);
				String query = "insert node <"+type+" uid='" + uid +"' /> as first into " + pos.path + "/children";
				new XQuery(query).execute(context);
				new Export(qmlfile).execute(context);
				new Close().execute(context);
			} 
			catch (Exception e) {
				e.printStackTrace();
			}
			finally {
				context.close();
			}
		}
		return q;
	}
	
	
	@Override
	public Question insertBefore(PathInfo pos, String type) {
		Context context = new Context();
		Question q = null;
		synchronized (context) {
			try{
				new CreateDB("codebook", qmlfile).execute(context);
				String uid = UUID.randomUUID().toString();
				q = new Question(uid, type);
				String query = "insert node <" + type + " uid='" + uid +"' />  before " + pos.path;
				new XQuery(query).execute(context);
				new Export(qmlfile).execute(context);
				new Close().execute(context);
			} 
			catch (Exception e) {
				e.printStackTrace();
			}
			finally {
				context.close();
			}
		}
		return q;
	}
	
	@Override
	public Question insertBehind(PathInfo pos, String type) {
		Context context = new Context();
		Question q = null;
		synchronized(context) {
			try{
				new CreateDB("codebook", qmlfile).execute(context);
				String uid = UUID.randomUUID().toString();
				q = new Question(uid, type);
				String query = "insert node <" + type + " uid='" + uid +"' />  after " + pos.path;
				new XQuery(query).execute(context);
				new Export(qmlfile).execute(context);
				new Close().execute(context);
			} 
			catch (Exception e) {
				e.printStackTrace();
			}
			finally {
				context.close();
			}
		}
		return q;
	}
	
	@Override
	public boolean delete(Question question) {
		if(question == null) {
			throw new IllegalArgumentException("Question is null");
		}
		Context context = new Context();
		synchronized(context){
			try {
				new CreateDB("codebook", qmlfile).execute(context);
				String query = "delete node //*[@uid='" + question.uid + "']";
				new XQuery(query).execute(context);
				new Export(qmlfile).execute(context);
				new Close().execute(context);
			}
			catch(Exception e){
				e.printStackTrace();
				return false;
			}
			finally {
				context.close();
			}
		}
		return true;
	}
	
	@Override
	public boolean moveBehind(PathInfo pos, Question question) {
		Context context = new Context();
		synchronized(context){
			try {
				new CreateDB("codebook", qmlfile).execute(context);
				String nodeXML = new XQuery("//*[@uid='" + question.uid + "']").execute(context);
				{
					String query = "insert node (attribute {'toremove'} {'xx"+question.uid+"'}) into //*[@uid='" +question.uid + "']";
					new XQuery(query).execute(context);
				}
				{
					String query = "insert node " + nodeXML + " after " + pos.path;
					//System.out.println(query);
					new XQuery(query).execute(context);
				}
				{
					String source = "//*[@toremove='xx"+question.uid+"']";
					new XQuery("delete node " + source).execute(context);
				}
				new Export(qmlfile).execute(context);
				new Close().execute(context);
				
			}
			catch(Exception e){
				e.printStackTrace();
				return false;
			}
			finally {
				context.close();
			}
		}
		return true;
	}
	
	@Override
	public boolean moveBefore(PathInfo pos, Question question) {
		Context context = new Context();
		synchronized(context){
			try {
				new CreateDB("codebook", qmlfile).execute(context);
				String nodeXML = new XQuery("//*[@uid='" + question.uid + "']").execute(context);
				{
					String query = "insert node (attribute {'toremove'} {'xx"+question.uid+"'}) into //*[@uid='" +question.uid + "']";
					new XQuery(query).execute(context);
				}
				{
					String query = "insert node " + nodeXML + " before " + pos.path;
					new XQuery(query).execute(context);
				}
				{
					String source = "//*[@toremove='xx"+question.uid+"']";
					new XQuery("delete node " + source).execute(context);
				}
				new Export(qmlfile).execute(context);
				new Close().execute(context);
				
			}
			catch(Exception e){
				e.printStackTrace();
				return false;
			}
			finally {
				context.close();
			}
		}
		return true;
	}
	
	@Override
	public boolean moveInsideAsFirst(PathInfo pos, Question question) {
		Context context = new Context();
		synchronized(context){
			try {
				new CreateDB("codebook", qmlfile).execute(context);
				String nodeXML = new XQuery("//*[@uid='" + question.uid + "']").execute(context);
				{
					String query = "insert node (attribute {'toremove'} {'xx"+question.uid+"'}) into //*[@uid='" +question.uid + "']";
					new XQuery(query).execute(context);
				}
				{
					//System.out.println(nodeXML);
					String query = "insert node " + nodeXML + " as first into " + pos.path + "/children";
					new XQuery(query).execute(context);
				}
				{
					String source = "//*[@toremove='xx"+question.uid+"']";
					new XQuery("delete node " + source).execute(context);
				}
				new Export(qmlfile).execute(context);
				new Close().execute(context);
				
			}
			catch(Exception e){
				e.printStackTrace();
				return false;
			}
			finally {
				context.close();
			}
		}
		return true;
	}
	
	@Override
	public boolean moveInsideAsLast(PathInfo pos, Question question) {
		Context context = new Context();
		synchronized(context){
			try {
				new CreateDB("codebook", qmlfile).execute(context);
				String nodeXML = new XQuery("//*[@uid='" + question.uid + "']").execute(context);
				{
					String query = "insert node (attribute {'toremove'} {'xx"+question.uid+"'}) into //*[@uid='" +question.uid + "']";
					new XQuery(query).execute(context);
				}
				{
					String query = "insert node " + nodeXML + " into " + pos.path + "/children";
					new XQuery(query).execute(context);
				}
				{
					String source = "//*[@toremove='xx"+question.uid+"']";
					new XQuery("delete node " + source).execute(context);
				}
				new Export(qmlfile).execute(context);
				new Close().execute(context);
				
			}
			catch(Exception e){
				e.printStackTrace();
				return false;
			}
			finally {
				context.close();
			}
		}
		return true;
	}
	
	@Override
	public boolean movePageBefore(PathInfo pos, PathInfo page) {
		Context context = new Context();
		final String u = UUID.randomUUID().toString();
		synchronized(context){
			try {
				new CreateDB("codebook", qmlfile).execute(context);
				String nodeXML = new XQuery(page.path).execute(context);
				{
					String query = "insert node (attribute {'toremove'} {'xx"+u+"'}) into " + page.path;
					new XQuery(query).execute(context);
				}
				{
					String query = "insert node " + nodeXML + " before " + pos.path;
					//System.out.println("movePageBefor " + query);
					new XQuery(query).execute(context);
				}
				{
					String source = "//*[@toremove='xx"+u+"']";
					new XQuery("delete node " + source).execute(context);
				}
				new Export(qmlfile).execute(context);
				new Close().execute(context);
				
			}
			catch(Exception e){
				e.printStackTrace();
				return false;
			}
			finally {
				context.close();
			}
		}
		return true;
	}
	
	@Override
	public boolean movePageBehind(PathInfo pos, PathInfo page) {
		Context context = new Context();
		final String u = UUID.randomUUID().toString();
		synchronized(context){
			try {
				new CreateDB("codebook", qmlfile).execute(context);
				String nodeXML = new XQuery(page.path).execute(context);
				{
					String query = "insert node (attribute {'toremove'} {'xx"+u+"'}) into " + page.path;
					new XQuery(query).execute(context);
				}
				{
					String query = "insert node " + nodeXML + " after " + pos.path;
					new XQuery(query).execute(context);
				}
				{
					String source = "//*[@toremove='xx"+u+"']";
					new XQuery("delete node " + source).execute(context);
				}
				new Export(qmlfile).execute(context);
				new Close().execute(context);
				
			}
			catch(Exception e){
				e.printStackTrace();
				return false;
			}
			finally {
				context.close();
			}
		}
		return true;
	}
	
	/**
	 * 
	 * 
	 */
	public void transform(StringBuffer buf, String path, InputStream xsltfile, String...params){
		//Context context = new Context();
		//context.options.set(MainOptions.CHOP, false);
		//synchronized(context){
			try {
				//String res = new XQuery("doc('file:/"+qmlfile+"')/qtafi/codebook").execute(context);
				StringWriter result = new StringWriter();
				//StringReader reader = new StringReader(res);
				//transform(result, reader, xsltfile, params);
				transform(result, xsltfile, params);
				postProcessToHtml(result, buf);
			}
			catch(Exception e){
				e.printStackTrace();
			}
			//finally {
				//context.close();
			//}
		//}
	}
	
	private void transform(StringWriter sw, InputStream xsltfile, String...params){
		try {
			//Document document = builder.parse(new InputSource(file));
			TransformerFactory transfac = TransformerFactory.newInstance();
			Transformer trans;
			transfac.setURIResolver(new URIResolver(){
				public Source resolve(String href, String base){
					return new StreamSource(href);
				}
			});

			trans = transfac.newTransformer(new StreamSource(xsltfile));
			trans.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION,"yes");
			trans.setOutputProperty(OutputKeys.METHOD,"xml");
			
			for(int i = 0; i < params.length; i+=2){
				trans.setParameter(params[i], params[i+1]);
			}
			
			StreamResult result = new StreamResult(sw);
			document = builder.parse(new InputSource(new FileInputStream(qmlfile)));
			DOMSource source = new DOMSource(document);
			//Source source = new StreamSource(reader);
			trans.transform(source, result);
		}
		catch(javax.xml.transform.TransformerConfigurationException ex){
			ex.printStackTrace();
		}
		catch(javax.xml.transform.TransformerException ex){
			ex.printStackTrace();
		} 
		catch (FileNotFoundException e) {
			e.printStackTrace();
		} 
		catch (SAXException e) {
			e.printStackTrace();
		} 
		catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	static String postProcessToHtml(final StringWriter xmlinput, StringBuffer buf) {
		final Pattern tagPattern = Pattern.compile("<qtafi:([\\S]+)(\\s+)([^/>]+)/>");
		final Matcher tagMatcher = tagPattern.matcher(xmlinput.toString());
		final Pattern attributesPattern = Pattern.compile("(\\S+)=[\"]?((?:.(?![\"]?\\s+(?:\\S+)=|[>\"]))+.)[\"]?");
		while (tagMatcher.find()) {
			if (tagMatcher.groupCount() == 3) {
				final String tag = tagMatcher.group(1);
				final String attrs = tagMatcher.group(3);
				final Matcher attributesMatcher = attributesPattern.matcher(attrs);
				int len = 0;
				while (attributesMatcher.find()) {
					if (attributesMatcher.groupCount() == 2) {
						final String name = attributesMatcher.group(1);
						final String val = attributesMatcher.group(2);
						if (name.equals("size")) {
							len = Integer.parseInt(val.replaceAll("\"", ""));
						}
					}
				}
				if (tag.equals("TE")) {
					tagMatcher.appendReplacement(buf, "<input type=\"text\" size=\""+ (len) + "\" />");
				} 
				else if (tag.equals("ME")) {
					tagMatcher.appendReplacement(buf, "<input type=\"text\" maxlength=\"" + len + "\" size=\""+ len + "\" style=\"text-align:right\" />");
				}
				else if (tag.equals("CV") || tag.equals("OR")) {
					tagMatcher.appendReplacement(buf, "<input type=\"radio\" />");
				}
				else if (tag.equals("MD")) {
					tagMatcher.appendReplacement(buf, "<input type=\"checkbox\" />");
				}
				else if (tag.equals("PIN")) {
					tagMatcher.appendReplacement(buf, "<input type=\"password\" />");
				} 
				else {
					tagMatcher.appendReplacement(buf, tagMatcher.group());
				}
			} 
			else{
				tagMatcher.appendReplacement(buf, tagMatcher.group());
			}
			
		}
		tagMatcher.appendTail(buf);
		return buf.toString();
	}
	
	static String postProcessToText(final StringWriter xmlinput, StringBuffer buf) {
		final Pattern tagPattern = Pattern.compile("<qtafi:([\\S]+)(\\s+)([^/>]+)/>");
		final Matcher tagMatcher = tagPattern.matcher(xmlinput.toString());
		final Pattern attributesPattern = Pattern.compile("(\\S+)=[\"]?((?:.(?![\"]?\\s+(?:\\S+)=|[>\"]))+.)[\"]?");
		//StringBuffer buf = new StringBuffer();
		while (tagMatcher.find()) {
			if (tagMatcher.groupCount() == 3) {
				final String tag = tagMatcher.group(1);
				final String attrs = tagMatcher.group(3);
				final Matcher attributesMatcher = attributesPattern.matcher(attrs);
				int len = 0;
				while (attributesMatcher.find()) {
					if (attributesMatcher.groupCount() == 2) {
						final String name = attributesMatcher.group(1);
						final String val = attributesMatcher.group(2);
						if (name.equals("size")) {
							len = Integer.parseInt(val.replaceAll("\"", ""));
						}
					}
				}
				if (tag.equals("TE")) {
					char[] xxx = new char[(int) (len / 5)];
					Arrays.fill(xxx, '_');
					tagMatcher.appendReplacement(buf, "[" + new String(xxx)+ "]");
				} 
				else if (tag.equals("ME")) {
					char[] xxx = new char[(int) (len)];
					Arrays.fill(xxx, '_');
					tagMatcher.appendReplacement(buf, "[" + new String(xxx)+ ".]");
				}
				else if (tag.equals("CV") || tag.equals("OR")) {
					tagMatcher.appendReplacement(buf, "()");
				}
				else if (tag.equals("MD")) {
					tagMatcher.appendReplacement(buf, "[]");
				}
				else if (tag.equals("PIN")) {
					tagMatcher.appendReplacement(buf, "[****]");
				} 
				else {
					tagMatcher.appendReplacement(buf, tagMatcher.group());
				}
			} 
			else{
				tagMatcher.appendReplacement(buf, tagMatcher.group());
			}
			
		}
		tagMatcher.appendTail(buf);
		return buf.toString();
	}

}
