/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.tinu.qtafi4.db.test;

import org.junit.Test;

import de.tinu.qtafi4.db.QuestionDB;
import de.tinu.qtafi4.db.QuestionDBImpl;
import de.tinu.qtafi4.model.HeadInfo;
import de.tinu.qtafi4.model.IdInfo;
import de.tinu.qtafi4.model.PageInfo;
import de.tinu.qtafi4.model.PathInfo;
import de.tinu.qtafi4.model.Question;
import de.tinu.qtafi4.model.Variable;
import de.tinu.qtafi4.tree.TreeNode;
import de.tinu.qtafi4.tree.QuestionnaireNode;

public class QuestionDBTest {

	
	@Test
	public void createQuestionnaire(){
		QuestionDB questiondb = new QuestionDBImpl();
		questiondb.init("test","C:/qml/index.qml");
		
		{
			TreeNode node = questiondb.questionnaireInfo();
			if(((QuestionnaireNode)node).getPageCount() == 0) questiondb.appendPage();
			System.out.println(node);
		}
		{
			Question head = questiondb.get(new IdInfo("progressbar"));
			if(head == null){
				head = questiondb.append(new HeadInfo(), "ProgressBar");
				head.id = "progressbar";
				questiondb.update(head);
			}
		}
		
		{
			Question a1 = questiondb.get(new IdInfo("a1"));
			if(a1 == null){
				a1 = questiondb.append(new PageInfo(), "CV");
				
				a1.number = "A1";
				a1.id = "a1";
				a1.text = "Hallo Welt";
				questiondb.update(a1);
			}
		}
		
		{
			Question question = questiondb.get(new PathInfo("//*[@id='a1']"));
			System.out.println(question);
			question.number = "A1";
			question.text = "Hallo Welt  " + System.currentTimeMillis() ;
			questiondb.update(question);
		}
		
		{
			Question question = questiondb.get(new PathInfo("//*[@id='a1']"));
			System.out.println(question);
		}
		
		{
			System.out.println("--appendPage--");
			questiondb.appendPage();
			TreeNode node = questiondb.questionnaireInfo();
			System.out.println(node);
		}
		
		{
			System.out.println("-- create b2--");
			Question q = questiondb.prepend(new PageInfo(2), "TE");
			System.out.println(q);
			q.number = "B1";
			q.id = "b2";
			q.text = "Eine neue Frage";
			q.addVariable(new Variable("sdfsf"));
			
			System.out.println("-- update b2--");
			questiondb.update(q);
			System.out.println(q);
			
			{
				TreeNode node = questiondb.questionnaireInfo();
				System.out.println(node);
			}
		}
		
		{
			//Move 
			questiondb.appendPage();
			System.out.println("-- move b2--");
			Question q = questiondb.get(new PathInfo("//*[@id='b2']"));
			questiondb.moveInsideAsFirst(new PageInfo(3), q);
			
			{
				TreeNode node = questiondb.questionnaireInfo();
				System.out.println(node);
			}
			questiondb.deletePage(3);
			
		}
		
		{
			Question q = questiondb.get(new IdInfo("b2"));
			System.out.println("-- delete b2--");
			//questiondb.delete(q);
			{
				TreeNode node = questiondb.questionnaireInfo();
				System.out.println(node);
			}
		}
		
		{
			System.out.println("-- create behind a2--");
			Question q = questiondb.insertBehind(new PathInfo("//*[@id='a1']"), "MD");
			System.out.println(q);
			q.number = "A2";
			q.id = "a12";
			q.text = "Eine neue Frage hinter a1";
			q.addVariable(new Variable("sdfsf"));
			
			System.out.println("-- update a2--");
			questiondb.update(q);
			System.out.println(q);
			
			{
				TreeNode node = questiondb.questionnaireInfo();
				System.out.println(node);
			}
			
			System.out.println("-- delete a2--");
			questiondb.delete(q);
			{
				TreeNode node = questiondb.questionnaireInfo();
				System.out.println(node);
			}
		}
		
		{
			System.out.println("-- create before a0--");
			Question q = questiondb.insertBefore(new PathInfo("//*[@id='a1']"), "OR");
			System.out.println(q);
			q.number = "AO";
			q.id = "a0";
			q.text = "Eine neue Frage bevor a1";
			q.addVariable(new Variable("sdfsf"));
			
			System.out.println("-- update a0--");
			questiondb.update(q);
			System.out.println(q);
			
			{
				TreeNode node = questiondb.questionnaireInfo();
				System.out.println(node);
			}
			
			System.out.println("-- delete a0--");
			questiondb.delete(q);
			{
				TreeNode node = questiondb.questionnaireInfo();
				System.out.println(node);
			}
		}
		
		
		{
			System.out.println("--deletePage--");
			questiondb.deletePage(2);
			
			TreeNode node = questiondb.questionnaireInfo();
			System.out.println(node);
		}
		
		{
			TreeNode node = questiondb.questionnaireInfo();
			System.out.println(node);
			
		}
		
	}
	
}
