/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.tinu.qtafi4.model;

import java.util.List;
import java.util.Vector;

public class FilterRoot {
	
	public static final int HIDE = 1;
	public static final int SHOW = 2;
	StringBuffer buf = new StringBuffer();
	
	public int type = SHOW;
	public List<Filter> filters = new Vector<Filter>();
	
	public void empty(){
		filters = new Vector<Filter>();
	}
	
	public void add(Filter filter){
		filters.add(filter);
	}
	
	public String toXML(){
		buf = new StringBuffer();
		for(Filter f: filters){
			appendRecursiv(f);
		}
		return buf.toString();
	}
	
	void appendRecursiv(Filter f){
		if(f.type == Filter.TYPE_GROUP_AND) {
			beginGroup(f);
			for(Filter ff: f.children) appendRecursiv(ff);
			endGroup();
		}
		else if(f.type == Filter.TYPE_GROUP_OR){
			beginGroup(f);
			for(Filter ff: f.children) appendRecursiv(ff);
			endGroup();
		}
		else {
			filter(f);
		}
	}
	
	void filter(Filter f){
		if(type == SHOW) buf.append("<show_if ");
		else if(type == HIDE) buf.append("<hide_if ");
		buf.append("variable=\""+f.variable+"\" ");
		buf.append("value=\""+ f.value+"\" ");
		if(f.ref != null) buf.append("ref=\"" + f.ref + "\" ");
		if(f.label != null) buf.append("label=\"" + f.label + "\" ");
		buf.append(" />");
	}
	
	void beginGroup(Filter f){
		if(f.type == Filter.TYPE_GROUP_AND){
			if(type == SHOW) buf.append("<show_group condition=\"AND\">");
			else buf.append("<hide_group condition=\"AND\">");
		}
		else if(f.type == Filter.TYPE_GROUP_OR){
			if(type == SHOW) buf.append("<show_group condition=\"OR\">");
			else buf.append("<hide_group condition=\"OR\">");
		}
	}
	
	void endGroup(){
		if(type == SHOW) buf.append("</show_group>");
		else buf.append("</hide_group>");
	}
	
	public String toString(){
		StringBuffer buf = new StringBuffer();
		buf.append("{\n");
		for(Filter filter: filters){
			buf.append(filter.toString());
		}
		buf.append("}\n");
		
		return buf.toString();
	}
}
