/*******************************************************************************
 * Quest survey management 
 *
 * Copyright (C) 2015 eoda
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package de.tinu.qtafi4.model;

import java.io.Serializable;
import java.util.List;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.jsoup.Jsoup;

/**
 * <type>
 * 	<text>sfsfs</text>
 * <layout />
 * </type>
 * 
 * 
 * 
 * @author maks
 *
 */
public class Question implements Serializable {

	static final long serialVersionUID = 1L;
	
	public final String uid;
	public final String type;
	private int protectedBy = -1;
	public String id = "";
	
	
	public String path = "";
	public String subject = "";
	public String moretext = "";
	public String number = "";
	public String text = "";
	public String code = "";
	public LayoutData layout = new LayoutData();
	public List<Variable> variables = new Vector<Variable>();
	public List<Value> values = new Vector<Value>();
	public FilterRoot filter = new FilterRoot();
	
	public Question(String uid, String type) {
		this.uid = uid;
		this.type = type;
	}
	
	public Question clone(){
		Question cl = new Question(this.uid, this.type);
		cl.protectedBy = this.protectedBy;
		cl.id = this.id;
		cl.subject = this.subject;
		cl.number = this.number;
		cl.text = this.text;
		cl.filter = this.filter;
		cl.code = this.code;
		cl.moretext = this.moretext;
		cl.values = this.values;
		cl.variables = this.variables;
		cl.layout = this.layout;
		
		return cl;
	}
	
	public void set(Question q){
		this.protectedBy = q.protectedBy;
		this.id = q.id;
		this.subject = q.subject;
		this.number = q.number;
		this.text = q.text;
		this.filter = q.filter;
		this.code = q.code;
		this.moretext = q.moretext;
		this.values = q.values;
		this.variables = q.variables;
		this.layout = q.layout;
	}
	
	public String getCleanedText(){
		return Jsoup.parse(text).text();
	}
	
	public String getCleanedSubject(){
		return Jsoup.parse(subject).text();
	}
	
	public void setVariables(List<Variable> variables) {
		this.variables = variables;
	}
	
	public List<Variable> getVariables() {
		return variables;
	}
	
	public void addVariable(Variable var){
		variables.add(var);
	}
	
	public void setValues(List<Value> values) {
		this.values = values;
	}
	
	public void addValue(Value val){
		values.add(val);
	}
	
	public List<Value> getValues() {		
		return values;
	}
	
	public String getMDValue(){
		return "1";
	}
	
	public String getCVVariable(){
		return id;
	}
	
	public boolean validId(){
		if(id == null) return false;
		if(id.length() < 1) return false;
		if(id.contains(" ")) return false;
		if(id.contains("&")) return false;
		if(id.contains("%")) return false;
		if(id.contains("+")) return false;
		for(int i = 0; i < id.length(); i++){
			if(! isAscii(id.charAt(i))) return false;
		}
		return true;
	}
	
	static boolean isAscii(char ch) {
	    return ch < 128;
	}
	
	public boolean validVariables(){
		for(Variable var: variables){
			if(var.name == null) return false;
			if(var.name.length() < 1) return false;
			if(var.name.contains(" ")) return false;
			if(var.name.contains("&")) return false;
			if(var.name.contains("%")) return false;
			if(var.name.contains("+")) return false;
			for(int i = 0; i < id.length(); i++){
				if(! isAscii(id.charAt(i))) return false;
			}
		}
		return true;
	}
	
	
	public boolean isQuestionProtected() {
		return this.protectedBy >= 0;
	}
	
	public int getProtectedBy() {
		return this.protectedBy;
	}
	
	public void protect(int userid) {
		this.protectedBy = userid;
	}
	
	public void unprotect() {
		this.protectedBy = -1;
	}

	public static class StartEndTuple {
		public final int start;
		public final int end;
		
		public StartEndTuple(int start, int end) {
			this.start = start;
			this.end = end;
		}
	}
	
	static List<StartEndTuple> parseTxt(final String txtinput) {
		final Pattern text = Pattern.compile("\\[([_]{1,10})\\]");
		final Matcher textMatcher = text.matcher(txtinput);
		List<StartEndTuple> tps = new Vector<StartEndTuple>();
		while (textMatcher.find()) {
			for (int i = 1; i <= textMatcher.groupCount(); i++) {
				String grs = textMatcher.group(i);
				int st = textMatcher.start(i);
				int e = textMatcher.end(i);
				tps.add(new StartEndTuple(st, e));
				System.out.println("txt len " + (e - st));
			}
		}
		return tps;
	}

	/**
	 * 
	 * @param nrinput
	 */
	static void parseNr(final String nrinput) {
		final Pattern text = Pattern.compile("\\[([_\\.]+)([\\.]){1,1}([_]*)\\]");
		final Matcher textMatcher = text.matcher(nrinput);
		while (textMatcher.find()) {
			if (textMatcher.groupCount() == 3) {
				int prefix = textMatcher.end(1) - textMatcher.start(1);
				int postfix = textMatcher.end(3) - textMatcher.start(3);
				System.out.println("nr len " + prefix + "." + postfix);
			}
		}
	}
	
	static void parseXML(final String xmlinput) {
		final Pattern tagPattern = Pattern.compile("<qtafi:([\\S]+)(\\s+)([^/>]+)");
		final Matcher tagMatcher = tagPattern.matcher(xmlinput);
		final Pattern attributesPattern = Pattern.compile("(\\S+)=[\"]?((?:.(?![\"]?\\s+(?:\\S+)=|[>\"]))+.)[\"]?");
		while (tagMatcher.find()) {
			if (tagMatcher.groupCount() == 3) {
				final String tag = tagMatcher.group(1);
				final String attrs = tagMatcher.group(3);
				final Matcher attributesMatcher = attributesPattern
						.matcher(attrs);
				System.out.println("tag: " + tag);
				while (attributesMatcher.find()) {
					if (attributesMatcher.groupCount() == 2) {
						final String name = attributesMatcher.group(1);
						final String val = attributesMatcher.group(2);
						System.out.println("attribute: " + name + " -> " + val);
					}
				}
			}
		}
	}
	
	String typeLabel(){
		switch(type){
		case "CV": return "SC";
		case "MD": return "MC";
		case "OR": return "OR";
		default: return type;
		}
	}
	
	@Override
	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append(typeLabel());
		buf.append(" - ");
		buf.append(id);
		if(subject.trim().length() > 0){
			buf.append(" - " + subject);
		}
		return buf.toString();
	}
}
